<?php

namespace Plugins\Ccm19Abtests\Controller;

use App\Component\HttpFoundation\JavascriptResponse;
use App\Controller\DomainDependantController;
use App\Event\abtestsDisplayEvent;
use App\Exception\CsrfTokenException;
use App\Model\CsrfToken;
use App\Model\Domain;
use App\Model\Purpose;
use App\Model\Statistic;
use App\Model\SubGroups;
use App\Model\Theme;
use App\Model\Theme as ThemeModel;
use App\Model\PluginState;
use App\Utils;
use http\QueryString;
use Plugins\Ccm19Abtests\Model\ABTestData;
use Plugins\Ccm19Abtests\Model\ABTestModell;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;
use RuntimeException;
use App\Component\Menu\Annotation\Menu;


/**
 * Controller for abtests - domain dependent
 * @Route("/domains/{_domainId}/plugins/abtests/", name="plugin_ccm19_abtests_")
 */
class ABTestPlugin extends DomainDependantController
{
	/**
	 * @Menu("A/B Tests", icon="fa-pie-chart", order=20)
	 * @Route("index", name="index")
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @param PluginState $state
	 * @return Response
	 */
	public function index(Request $request, TranslatorInterface $translator, PluginState $state): Response
	{
		//check if is activated
		if (!$state->isActiveForCurrentDomain()) {
			//Nope - then go to dashboard...
			return $this->redirectToRoute('app_dashboard');
		}

		$abpluginData = ABTestModell::all();
		$abpluginData = array_reverse($abpluginData);

		foreach($abpluginData as $abtest)
		{
			if(!empty(ThemeModel::find($abtest->get('abtest_var1'))))
			{
				$abtest->set("themeName1",ThemeModel::find($abtest->get('abtest_var1'))->getName());
			}

			if(!empty(ThemeModel::find($abtest->get('abtest_var2'))))
			{
				$abtest->set("themeName2",ThemeModel::find($abtest->get('abtest_var2'))->getName());
			}
		}

		$domain = Domain::activeDomain();

		if ($domain->isWidgetCachingEnabled()) {
			$this->addFlash('danger|raw', $translator->trans('As long as the widget cache is active, dynamic output of AB tests cannot occur. Disable the cache in the ":link" section.', [
				':link' => '<a href="'.$this->generateUrl('app_scripts_list', ['_domainId'=>$domain->getId()]).'">'.$translator->trans('Frontend').'</a>',
			]));
		}

		//Render template
		return $this->render('@plugin:Ccm19Abtests/internal/index.html.twig', [
			'csrfToken' => CsrfToken::get(),
			'abtests' => $abpluginData,
			'showmode' => "d",
		]);
	}

	/**
	 * @Route("index_old", name="index2")
	 * @return Response
	 */
	public function oldRoute(): Response
	{
		return $this->redirectToRoute('plugin_ccm19_abtests_index');
	}


	/**
	 * @Route("show", name="show")
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @param PluginState $state
	 * @return Response
	 */
	public function show(Request $request, TranslatorInterface $translator, PluginState $state): Response
	{
		//check if is activated
		if (!$state->isActiveForCurrentDomain()) {
			//Nope - then go to dashboard...
			return $this->redirectToRoute('app_dashboard');
		}
		$statDat['testID']=$request->query->get("id");
		$statDat['valuemode']="p";
		$request->query->set("statdat",urlencode(serialize($statDat)));
		$request->query->set("chartName","all");

		//Get The Winner...
		$all= $this->getJsonDataForBar($request,$translator);
		$first = current($all);
		$second = next($all);

		$winner = " - ";
		if(is_array($first) && is_array($second))
		{
			if($first['0']>$second['0'])
			{
				$winner = array_keys($all)[0];
			}
			else{
				$winner = array_keys($all)[1];
			}
		}


		//Start / End bestimmen und im Zweifel sinnvolle Meldung ausgeben wenn 0
		$abTest = ABTestModell::find($request->query->get("id"));
		$testEnd = "nope";
		if ($abTest->get('abtest_active_stop') > 0) {
			$testEnd = date("d.m.Y", ((int)$abTest->get('abtest_active_stop')));
		}

		$testStart = "nope";
		if ($abTest->get('abtest_active_start') > 0) {
			$testStart = date("d.m.Y", ((int)$abTest->get('abtest_active_start')));
		}

		$changeLog = ABTestModell::getChangeLogs($request->query->get("id"));
		$barCharts = ABTestModell::getBarCharts($translator);


		if ($request->getRealMethod() === 'POST') {

			try {
				CsrfToken::get()->checkRequest($request);
			} catch (CsrfTokenException $e) {
				$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
				return $this->redirectToRoute('plugin_ccm19_abtests_index');
			}
			//print_r($request);
			//Daten rausholen
			$submit = $request->request->all();;
			$daDates = explode(" - ", $submit['statdat']['daterange']);

			return $this->render('@plugin:Ccm19Abtests/internal/index_chart.html.twig', [
				'csrfToken' => CsrfToken::get(),
				'daterange' => htmlentities(serialize($submit['statdat']['daterange'])),
				'statdat' => urlencode(serialize(array_merge($submit['statdat'], array("testID" => $request->query->get('id'))))),
				'startDate' => "startDate: '" . htmlentities(trim($daDates['0'])) . "', ",
				'endDate' => "endDate: '" . htmlentities(trim($daDates['1'])) . "', ",
				'maxDate' => "maxDate: '" . htmlentities(trim(date("d.m.Y"))) . "', ",
				'valuemode' => $submit['statdat']['valuemode'],
				'showmode' => $submit['statdat']['showmode'],
				'abTestData' => $abTest,
				'barCharts' => $barCharts,
				'abTestDataStart' => $testStart,
				'abTestDataEnd' => $testEnd,
				'changeLog' => $changeLog,
				'winner' => $winner,
			]);
		}

		// Nur wenn Import gewollt UND das System json-repos ist - in MongoDB liegen alle Daten schon vor... auÃer den gelÃ¶schten
		if($request->get("import")=="ok" && Utils::getRepositoryStore()==Utils::REPOSITORY_STORE_FILE)
		{
			//Komplettes Archiv einlesen...
			ABTestData::createFullStats(true);

			$this->addFlash('success', $translator->trans('Archivedata are now imported.'));
		}

		//Bei jedem Aufruf die aktuellen Daten aus der Standard Statistik Ã¼bernehmen... damit es immer aktuell ist...
		ABTestData::createFullStats(false);

		if($testStart=="nope")
		{
			$testStart = date("d.m.Y", time() - (60 * 60 * 24 * 30));
		}
		if($testEnd=="nope")
		{
			$testEnd = date("d.m.Y", time());
		}

		//Render template
		return $this->render('@plugin:Ccm19Abtests/internal/index_chart.html.twig', [
			'csrfToken' => CsrfToken::get(),
			'daterange' => htmlentities(trim(date("d.m.Y", time() - (60 * 60 * 24 * 60)))) . " - " . htmlentities(trim(date("d.m.Y"))),
			'statdat' => (urlencode(serialize(array("valuemode" => "p", "showmode" => "d", "testID" => $request->query->get('id'))))),
			'startDate' => "startDate: '" . htmlentities(trim($testStart)). "', ",
			'endDate' => "endDate: '" . htmlentities(trim($testEnd)) . "', ",
			'maxDate' => "maxDate: '" . htmlentities(trim(date("d.m.Y"))) . "', ",
			'valuemode' => "p",
			'showmode' => "d",
			'abTestData' => $abTest,
			'barCharts' => $barCharts,
			'abTestDataStart' => $testStart,
			'abTestDataEnd' => $testEnd,
			'changeLog' => $changeLog,
			'winner' => $winner,
		]);
	}

	/**
	 * @Route("edit", name="edit", methods={"HEAD", "GET"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function editTests(Request $request, TranslatorInterface $translator): Response
	{
		return $this->render('@plugin:Ccm19Abtests/internal/create.html.twig', [
			'csrfToken' => CsrfToken::get(),
			"plugin" => array(),
			'abtestdat' => ABTestModell::find($request->query->get("id")),
			'themes' => ThemeModel::allOrderedByTypeAndName(),
		]);
	}

	/**
	 * @Route("delete", name="delete", methods={"HEAD", "GET"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function deleteTests(Request $request, TranslatorInterface $translator): Response
	{
		return $this->render('@plugin:Ccm19Abtests/internal/delete.html.twig', [
			'csrfToken' => CsrfToken::get(),
			"plugin" => array(),
			'abtestdat' => ABTestModell::find($request->query->get("id")),
			'themes' => ThemeModel::allOrderedByTypeAndName(),
		]);
	}

	/**
	 * @Route("plugin_ccm19_abtests_delete_now", name="deletenow", methods={"POST"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function deleteTestsPost(Request $request, TranslatorInterface $translator): Response
	{
		//check
		try {
			CsrfToken::get()->checkRequest($request);
		} catch (CsrfTokenException $e) {
			$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
			return $this->redirectToRoute('plugin_ccm19_abtests_save');
		}

		//Delete Entry
		ABTestModell::delete($request->get("id"));

		//show delted
		$this->addFlash('success', $translator->trans('The Test Data are now deleted.'));

		//redirect
		return $this->redirectToRoute('plugin_ccm19_abtests_index');
	}


	/**
	 * @Route("new", name="new", methods={"HEAD", "GET"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function newTest(Request $request, TranslatorInterface $translator): Response
	{
		//Layouts rausholen
		$themes = ThemeModel::allOrderedByTypeAndName();

		return $this->render('@plugin:Ccm19Abtests/internal/create.html.twig', [
			'csrfToken' => CsrfToken::get(),
			"plugin" => array(),
			'themes' => $themes,
		]);
	}

	/**
	 * @Route("save", name="save"), methods={"POST"})
	 * @param Request $request
	 * @return Response
	 */
	public function editSettings(Request $request, TranslatorInterface $translator): Response
	{
		//check
		try {
			CsrfToken::get()->checkRequest($request);
		} catch (CsrfTokenException $e) {
			$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
			return $this->redirectToRoute('plugin_ccm19_abtests_save');
		}

		//set
		$postPLuginData = array();

		//get Field Data
		if (is_array(ABTestModell::getFieldNames())) {
			foreach (ABTestModell::getFieldNames() as $v) {
				$postPLuginData[$v] = $request->get($v);
			}
		}

		//Daten checken... wenn aktiv - es kann ja nur einen aktiven Test geben
		if ($postPLuginData['abtest_active'] == "on") {
			//Alle durchgehen
			foreach (ABTestModell::all() as $test) {
				//Wenn aktiv und nicht der gleiche
				if ($test->get('abtest_active') == "on" && $request->get("id") != $test->getId()) {
					//Hups dann Fehker . geht nicht
					$this->addFlash('danger', $translator->trans('There can only be one active Test.'));

					if (empty($request->get("id"))) {
						$postPLuginData['id'] = "";
					}

					return $this->render('@plugin:Ccm19Abtests/internal/create.html.twig', [
						'csrfToken' => CsrfToken::get(),
						"plugin" => array(),
						'abtestdat' => $postPLuginData,
						'themes' => ThemeModel::allOrderedByTypeAndName(),
					]);
				}
			}
		}


		//Es mÃ¼ssen auch Themes ausgewÃ¤hlt sein
		if (empty($postPLuginData['abtest_var1']) || empty($postPLuginData['abtest_var2']) || $postPLuginData['abtest_var1'] == $postPLuginData['abtest_var2']) {
			$this->addFlash('danger', $translator->trans('Please select 2 different Themes.'));

			if (empty($request->get("id"))) {
				$postPLuginData['id'] = "";
			}

			return $this->render('@plugin:Ccm19Abtests/internal/create.html.twig', [
				'csrfToken' => CsrfToken::get(),
				"plugin" => array(),
				'abtestdat' => $postPLuginData,
				'themes' => ThemeModel::allOrderedByTypeAndName(),
			]);
		}
		if ($postPLuginData['abtest_freq1'] + $postPLuginData['abtest_freq2'] < 100) {
			$this->addFlash('danger', $translator->trans('Frequency was corrected.'));
			$postPLuginData['abtest_freq2'] = 100 - $postPLuginData['abtest_freq1'];
		}
		if ($postPLuginData['abtest_freq1'] + $postPLuginData['abtest_freq2'] > 100) {
			$this->addFlash('danger', $translator->trans('Frequency was corrected.'));
			$postPLuginData['abtest_freq2'] = 100 - $postPLuginData['abtest_freq1'];
		}
		//exit();

		//Wenn schon existiert - aber noch nicht aktiv - dann ativier Datum setzen - und hier kommen wir ja nur hin wenn aktiv korrekt gesetzt ist
		if (ABTestModell::exists($request->get("id")) == true) {
			$pluginData = ABTestModell::find($request->get("id"));
			if ($pluginData->get("abtest_active") != "on" && $postPLuginData['abtest_active'] == "on") {
				$postPLuginData['abtest_active_start'] = time();
				$postPLuginData['abtest_active_stop'] = 0;
			}
			if ($pluginData->get("abtest_active") == "on" && $postPLuginData['abtest_active'] != "on") {
				$postPLuginData['abtest_active_stop'] = time();
			}
		} else {
			//neuer aktiver Eintrag - den kann man aber nur 1x aktiv setzen - danach existiert er ja....
			if ($postPLuginData['abtest_active'] == "on") {
				$postPLuginData['abtest_active_start'] = time();
				$postPLuginData['abtest_active_stop'] = 0;
			}
		}

		//Does not exist -> then create...
		if (ABTestModell::exists($request->get("id")) == false) {

			//Create an entry for json file
			$PluginsClass = ABTestModell::create();

			//Set Data from array in new entry
			$PluginsClass->setArray($postPLuginData);

			//finally save the data to the json file
			$PluginsClass->flush();

			//External CSS and JS
			$this->setCssAndJSForExternal();


			//Set Flash Message
			$this->addFlash('success', $translator->trans('A new Entry succesfully created.'));

			return $this->redirectToRoute('plugin_ccm19_abtests_edit', ['id' => $PluginsClass->getId()]);

		}
		//get Plugin Data
		$pluginData = ABTestModell::find($request->get("id"));

		$time = time();
		$changed[$time]= array();

		foreach($postPLuginData as $k=>$item)
		{
			if($item!=$pluginData->get($k) && $k !="id")
			{
				$changed[$time][$k]['old']=$pluginData->get($k);
				$changed[$time][$k]['new']=$item;
			}
		}
		$oldChanged = $pluginData->get("changeLog");
		if(!is_array($oldChanged))
		{
			$oldChanged = array();
		}
		foreach($oldChanged as $k=>$v)
		{
			$changed[$k]=$v;
		}
		$postPLuginData['changeLog']=$changed;
		//print_r($postPLuginData);
		//print_r($pluginData);exit();

		//Set Data and save
		$pluginData->setArray($postPLuginData);
		$pluginData->flush();

		//Set edited Flash Message
		$this->addFlash('success', $translator->trans('Existing Entry succesfully edited.'));

		return $this->redirectToRoute('plugin_ccm19_abtests_edit', ['id' => $request->get("id")]);

	}

	/**
	 * @Route("abtests_basic.js", name="basic_chart_js"),
	 *     methods={"HEAD","GET"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return JavascriptResponse
	 */
	public function makeBasicChart(Request $request, TranslatorInterface $translator): JavascriptResponse
	{
		$response = new JavascriptResponse();
		Utils::resetCacheControl($response);
		$response->setCache([
			'max_age' => 90,
			'public' => false
		]);
		$statDat = unserialize(urldecode($request->get("statdat")));
		if (empty($statDat['showmode'])) {
			$statDat['showmode'] = "d";
		}

		if (empty($statDat['showmode']) || $statDat['showmode'] == "d") {
			$dateForm = "%Y-%m-%d";
		}
		if ($statDat['showmode'] == "w") {
			$dateForm = "%Y - %W";
		}
		if ($statDat['showmode'] == "m") {
			$dateForm = "%Y-%m";
		}
		if ($statDat['showmode'] == "y") {
			$dateForm = "%Y";
		}

		return $this->render('@plugin:Ccm19Abtests/internal/basic_chart.js.twig', array("statdat" => $request->get("statdat"), "dateForm" => $dateForm), $response);
	}

	/**
	 * @Route("abtests_basic2.js", name="basic_chart2_js"),
	 *     methods={"HEAD","GET"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return JavascriptResponse
	 */
	public function makeBasicChart2(Request $request, TranslatorInterface $translator): JavascriptResponse
	{
		$response = new JavascriptResponse();
		Utils::resetCacheControl($response);
		$response->setCache([
			'max_age' => 90,
			'public' => false
		]);
		$statDat = unserialize(urldecode($request->get("statdat")));
		if (empty($statDat['showmode'])) {
			$statDat['showmode'] = "d";
		}

		if (empty($statDat['showmode']) || $statDat['showmode'] == "d") {
			$dateForm = "%Y-%m-%d";
		}
		if ($statDat['showmode'] == "w") {
			$dateForm = "%Y - %W";
		}
		if ($statDat['showmode'] == "m") {
			$dateForm = "%Y-%m";
		}
		if ($statDat['showmode'] == "y") {
			$dateForm = "%Y";
		}
		if ($statDat['valuemode'] == "p") {
			$barHeadline = $translator->trans('Percental View of all Visits');
			$categories ="'".$translator->trans('Accept all')." %','".$translator->trans('Decline')." %','".$translator->trans('Leave')." %','".$translator->trans('All Visits')." %s'";
		}
		else{
			$barHeadline = $translator->trans('Real Counts of all visits');
			$categories ="'".$translator->trans('Accept all')."','".$translator->trans('Decline')."','".$translator->trans('Leave')."','".$translator->trans('All Visits')."'";
		}

		$responseArray = array(
			"statdat" => $request->get("statdat"),
			"chart" => $request->get("chart"),
			"dateForm" => $dateForm,
			"categories"=>$categories,
			"barHeadline" =>$barHeadline,
		);

		return $this->render('@plugin:Ccm19Abtests/internal/basic_chart2.js.twig', $responseArray, $response);
	}

	/**
	 * @Route("basic_data_bar.json", name="basic_data_json_bar",
	 *     methods={"GET"})
	 * @return JSONResponse
	 */
	public function showJsonDataBar(Request $request, TranslatorInterface $translator): JSONResponse
	{
		$all= $this->getJsonDataForBar($request,$translator);
		$response = new JsonResponse($all);

		//For Dev JSON_PRETTY_PRINT
		$response->setEncodingOptions($response->getEncodingOptions() | JSON_PRETTY_PRINT);

		//und raus...
		return $response;
	}

	/**
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return array
	 */
	public function getJsonDataForBar(Request $request, TranslatorInterface $translator)
	{
		//Daten aus Formular - enhÃ¤lt z.B. prozentual
		$statdat = unserialize(urldecode($request->get("statdat")));

		//Die aktuelle AB Test ID
		$testId = (string)$statdat['testID'];
		//AB Test Daten herausholen
		$testData = ABTestModell::find($testId);

		//Die Themes zuweisen
		$themes = Theme::all();
		$themeList = array();
		foreach ($themes as $theme) {
			if($testData->get('abtest_var1')==$theme->getId() || $testData->get('abtest_var2')==$theme->getId())
			{
				$themeList[$theme->getId()] = $theme->getName();
			}
		}

		//Die ButtonListe holen
		$buttonList = ABTestModell::getButtons();



		//Zeitrahmen fixen falls nicht gesetzt
		if ($testData->get('abtest_active_stop') < 1) {
			$testData->set('abtest_active_stop', time());
		}

		try {
			$domain = Domain::activeDomain();
			assert($domain !== null);

			//die gesammentelten Daten kommen hierein
			$sumData = array();

			//Alle Datendurchgehen
			foreach (ABTestData::all() as $dateKey => $stat) {

				//Wenn Datum eines Eintrages innerhalb des gewÃ¼nschten Ranges liegt UND Im Testzeitraum...
				if (strtotime($stat->getId()) >= ($testData->get('abtest_active_start') ) && strtotime($stat->getId()) <= $testData->get('abtest_active_stop')) {

					//die einzelnen Themes durchgehen die im AB test vorliegen
					if (!empty($stat->get('details'))) {
						foreach ($stat->get('details') as $themeID => $themeData) {

							foreach ($buttonList as $button => $buttonName) {
								if (!isset($themeData[$button])) {
									$themeData[$button]['all'] = 0;
								}
							}

							//Alle Consent Typen durchlaufen
							foreach ($buttonList as $button => $buttonName) {

								if (empty($themeList[$themeID])) {
									continue;
								}
								//Daten aufsummieren
								foreach ($themeData[$button] as $type => $value) {
									if (!empty($themeData[$button][$type])) {
										if (empty($sumData[$themeList[$themeID]][$button][$type])) {
											$sumData[$themeList[$themeID]][$button][$type] = 0;
										}
										$sumData[$themeList[$themeID]][$button][$type] += $value;
									}
								}
							}
						}
					}
				}
			}
		}
		catch (RuntimeException $exception) {
		}
		if(empty($sumData))
		{
			$sumData = array(array("0"));
		}
		//Die einzelnen Typen rausholen...
		$barCharts = ABTestModell::getBarCharts($translator);

		//Das Ausgabe Array mit den ProzentDaten
		$sumDatanNeu = array();

		//prozentual wenn so gewÃ¼nscht
		if ($statdat['valuemode'] == "p") {
			foreach ($sumData as $theme => $buttons) {
				foreach ($barCharts as $key => $type) {

					//Anteil accept
					if (!empty($sumData[$theme]['acceptAll'][$type['name']]) && !empty($sumData[$theme]['opened'][$type['name']])) {
						$sumDatanNeu[$theme]['acceptAll'][$type['name']] = round($sumData[$theme]['acceptAll'][$type['name']] / $sumData[$theme]['opened'][$type['name']] * 100, 2);
					} else {
						$sumData[$theme]['acceptAll'][$type['name']] = 0;
						$sumDatanNeu[$theme]['acceptAll'][$type['name']] = 0;
					}

					//prozentualer Anteil decline
					if (!empty($sumData[$theme]['decline'][$type['name']]) && !empty($sumData[$theme]['opened'][$type['name']])) {
						$sumDatanNeu[$theme]['decline'][$type['name']] = round($sumData[$theme]['decline'][$type['name']] / $sumData[$theme]['opened'][$type['name']] * 100, 2);
					} else {
						$sumData[$theme]['decline'][$type['name']] = 0;
						$sumDatanNeu[$theme]['decline'][$type['name']] = 0;
					}
					if(!empty($sumData[$theme]['opened'][$type['name']]))
					{
						//Anteil Leave - der Rest vom SchÃ¼tzenfest...
						$sumDatanNeu[$theme]['opened'][$type['name']] = round(100 - $sumDatanNeu[$theme]['acceptAll'][$type['name']] - $sumDatanNeu[$theme]['decline'][$type['name']], 2);
					}
					else{
						$sumDatanNeu[$theme]['opened'][$type['name']]="";
					}
				}
			}
		} else {
			foreach ($sumData as $theme => $buttons) {
				foreach ($barCharts as $key => $type) {
					//Anteil accept
					if (!empty($sumData[$theme]['acceptAll'][$type['name']]) && !empty($sumData[$theme]['opened'][$type['name']])) {
						$sumDatanNeu[$theme]['acceptAll'][$type['name']] = $sumData[$theme]['acceptAll'][$type['name']];
					} else {
						$sumData[$theme]['acceptAll'][$type['name']] = 0;
						$sumDatanNeu[$theme]['acceptAll'][$type['name']] = 0;
					}

					//prozentualer Anteil decline
					if (!empty($sumData[$theme]['decline'][$type['name']]) && !empty($sumData[$theme]['opened'][$type['name']])) {
						$sumDatanNeu[$theme]['decline'][$type['name']] = $sumData[$theme]['decline'][$type['name']];
					} else {
						$sumData[$theme]['decline'][$type['name']] = 0;
						$sumDatanNeu[$theme]['decline'][$type['name']] = 0;
					}

					if (!empty($sumData[$theme]['opened'][$type['name']])) {
						//Anteil Leave - der Rest vom SchÃ¼tzenfest...
						$sumDatanNeu[$theme]['opened'][$type['name']] = $sumData[$theme]['opened'][$type['name']] - $sumDatanNeu[$theme]['acceptAll'][$type['name']] - $sumDatanNeu[$theme]['decline'][$type['name']];
					} else {
						$sumDatanNeu[$theme]['opened'][$type['name']] = 0;
					}
					if(!empty($sumData[$theme]['opened'][$type['name']]))
					{
						//alle Einblendungen
						$sumDatanNeu[$theme]['leave'][$type['name']] = $sumData[$theme]['opened'][$type['name']];
					}
					else{
						//alle Einblendungen
						$sumDatanNeu[$theme]['leave'][$type['name']] = "";
					}
				}
			}
		}
		//print_r($sumDatanNeu);
		//chartName
		foreach ($sumDatanNeu as $k => $v) {
			foreach ($v as $k1 => $v1) {
				$all[$k][] = $sumDatanNeu[$k][$k1][$request->query->get('chartName')];
			}
		}
		return $all;
	}

	/**
	 * @Route("basic_data.json", name="basic_data_json", methods={"GET"})
	 * @return JSONResponse
	 */
	public function showJsonData(Request $request): JSONResponse
	{
		$abtests = $this->getDataForJson($request);

		// Session schlieÃen
		$request->getSession()->save();

		$response = new JsonResponse($abtests);
		//For Dev JSON_PRETTY_PRINT
		$response->setEncodingOptions($response->getEncodingOptions() | JSON_PRETTY_PRINT);

		//und raus...
		return $response;
	}

	/**
	 * @param Request $request
	 * @return array
	 */
	public function getDataForJson(Request $request)
	{
		$statdat = unserialize(urldecode($request->get("statdat")));
		//Datum immer setzen
		$selDate = array();
		$selDate['from'] = time() - (60 * 60 * 24 * 60);
		$selDate['to'] = time();
		$statdat['showpurpose'] = 0;
		$testId = (string)$statdat['testID'];
		$testData = ABTestModell::find($testId);


		if (!empty($statdat['daterange'])) {
			$daDates = explode(" - ", $statdat['daterange']);
			if (!empty($daDates)) {
				$selDate['from'] = (int)strtotime($daDates['0']);
				$selDate['to'] = (int)(strtotime($daDates['1']) + 86600);
			}
		}
		//print_r($selDate);

		$purposes = Purpose::all();
		$themes = Theme::all();
		$themeList = array();
		foreach ($themes as $theme) {
			if($testData->get('abtest_var1')==$theme->getId() || $testData->get('abtest_var2')==$theme->getId())
			{
				$themeList[$theme->getId()] = $theme->getName();
			}
		}
		$buttonList = array("decline" => "Button abgelehnt", "acceptAll" => "Button alles akzeptiert", "opened" => "Absprung");

		$abtests = array();
		$count = 0;


		if ($testData->get('abtest_active_stop') < 1) {
			$testData->set('abtest_active_stop', time());
		}

		try {
			$domain = Domain::activeDomain();
			assert($domain !== null);

			foreach (ABTestData::all() as $date => $stat) {
				if($testData->get('abtest_active_start') > 0)
				{
					if (strtotime($stat->getId()) >= ($testData->get('abtest_active_start') ) && strtotime($stat->getId()) <= $testData->get('abtest_active_stop')) {
						$abtests["x"][] = $date;
					}
				}

			}
		} catch (RuntimeException $exception) {
		}

		try {
			$domain = Domain::activeDomain();
			assert($domain !== null);
			//print_r(ABTestData::all());


			foreach (ABTestData::all() as $dateKey => $stat) {
				if($testData->get('abtest_active_start') > 0){
					//Wenn Datum eines Eintrages innerhalb des gewÃ¼nschten Ranges liegt...
					if (strtotime($stat->getId()) >= ($testData->get('abtest_active_start') ) && strtotime($stat->getId()) <= $testData->get('abtest_active_stop')) {
						$count += $stat->getViewCount();
						$i = 1;

						//peozentual
						if ($statdat['valuemode'] == "p") {
							$einblendungen = 100;
							if (!empty($stat->get('details'))) {
								foreach ($stat->get('details') as $themeID => $themeData) {

									if ($statdat['showpurpose'] == "1") {
										foreach ($purposes as $purpose) {
											if (empty($themeList[$themeID])) {
												continue;
											}
											$ppID = $purpose->getName() . " (" . $themeList[$themeID] . ")";
											$i++;
											if (empty($abtests[$ppID])) {
												$abtests[$ppID] = array();
											}

											if (!empty($themeData['purposes'][$purpose->getId()])) {
												if ($themeData['purposes'][$purpose->getId()] < 0) {
													$themeData['purposes'][$purpose->getId()] = 0;
												}
												$percent = round($themeData['purposes'][$purpose->getId()] / $themeData['v'] * 100, 2) . "";
												$abtests[$ppID][] = $percent;
											} else {
												if (empty($themeData['purposes'][$purpose->getId()])) {
													$themeData['purposes'][$purpose->getId()] = 0;

												}
											}
										}
									} else {

										foreach ($buttonList as $button => $buttonName) {
											if (!isset($themeData[$button])) {
												$themeData[$button]['all'] = 0;
											}
										}

										//$buttonList

										foreach ($buttonList as $button => $buttonName) {

											if (empty($themeList[$themeID])) {
												continue;//$themeList[$themeID] = $themeID;
												//continue;
											}
											$ppID = $buttonName . " (" . $themeList[$themeID] . ")";
											if (empty($abtests[$ppID])) {
												$abtests[$ppID] = array();
											}

											$percent = 0;
											if ($themeData["opened"]['all'] > 0) {
												if ($button != "opened") {
													//bei opened die Werte korrigieren
													$percent = round($themeData[$button]['all'] / $themeData["opened"]['all'] * 100, 2) . "";
												} else {
													$percent = round(100 - round($themeData['decline']['all'] / $themeData["opened"]['all'] * 100, 2) - round($themeData['acceptAll']['all'] / $themeData["opened"]['all'] * 100, 2), 2);
												}
											}
											$abtests[$ppID][] = $percent;
										}
									}
								}
							}
						} //absolut count
						else {
							if (!empty($stat->get('details'))) {
								foreach ($stat->get('details') as $themeID => $themeData) {
									if ($statdat['showpurpose'] == "1") {
										//print_r($themeData['purposes']);
										foreach ($purposes as $purpose) {
											if (empty($themeList[$themeID])) {
												continue;//$themeList[$themeID] = $themeID;
											}
											$ppID = $purpose->getName() . " (" . $themeList[$themeID] . ")";
											$i++;
											if (empty($abtests[$ppID])) {
												$abtests[$ppID] = array();
											}
											if (!empty($themeData['purposes'][$purpose->getId()])) {
												if ($themeData['purposes'][$purpose->getId()] < 0) {
													$themeData['purposes'][$purpose->getId()] = 0;
												}
											} else {
												if (empty($themeData['purposes'][$purpose->getId()])) {
													$themeData['purposes'][$purpose->getId()] = 0;
												}
											}
											//if($themeData['purposes'][$purpose->getId()]>0)
											{
												$abtests[$ppID][] = $themeData['purposes'][$purpose->getId()];
											}

										}
									} else {

										foreach ($buttonList as $button => $buttonName) {
											if (!isset($themeData[$button])) {
												$themeData[$button]['all'] = 0;
											}
										}

										//$buttonList
										$themeData["leave"]['all'] = $themeData["opened"]['all'] - $themeData['decline']['all'] - $themeData['acceptAll']['all'];

										foreach ($buttonList as $button => $buttonName) {
											if (empty($themeList[$themeID])) {
												continue;//$themeList[$themeID] = $themeID;
											}
											$ppID = $buttonName . " (" . $themeList[$themeID] . ")";
											if (empty($abtests[$ppID])) {
												$abtests[$ppID] = array();
											}

											//bei opened die Werte korrigieren
											if ($button == "opened") {
												$open = $themeData[$button]['all'];
												$themeData[$button]['all'] = $themeData[$button]['all'] - $themeData['decline']['all'] - $themeData['acceptAll']['all'];
												//print_r($themeData);

											}
											$percent = round($themeData[$button]['all'], 2) . "";
											$abtests[$ppID][] = $percent;
										}
									}
									if (!empty($themeList[$themeID])) {
										$abtests["Einblendungen (" . $themeList[$themeID] . ")"][] = $open;
									}
								}
							}
						}
					}
				}


			}
		}
		catch (RuntimeException $exception) {
		}
		//print_r($abtests);exit();

		if (!empty($statdat['showmode'] && $statdat['showmode'] != "d")) {
			if ($statdat['showmode'] == "w") {
				$timePoints = "W";
			}
			if ($statdat['showmode'] == "m") {
				$timePoints = "m";
			}
			if ($statdat['showmode'] == "y") {
				$timePoints = "Y";
			}
			$newXCounter = array();
			//print_r($abtests);
			if (!empty($abtests)) {
				foreach ($abtests as $cat => $dataPerDay) {
					if (!empty($dataPerDay)) {
						foreach ($dataPerDay as $k => $aTimePoint) {
							if (!empty($abtests['x'][$k])) {
								$newXCounter[$cat][$abtests['x'][$k]] = $aTimePoint;
							}
						}
					}
				}
			}
			$newXCounterReal = array();
			foreach ($newXCounter as $cat => $catDat) {
				if ($cat != "x") {
					if (!empty($catDat)) {
						foreach ($catDat as $aDate => $aTimePoint) {
							$daTimeDat = date($timePoints, ((int)strtotime($aDate)));
							if (empty($newXCounterReal[$cat][$daTimeDat])) {
								$newXCounterReal[$cat][$daTimeDat]['value'] = 0;
								$newXCounterReal[$cat][$daTimeDat]['counter'] = 0;
							}
							$newXCounterReal[$cat][$daTimeDat]['value'] += ($aTimePoint);
							$newXCounterReal[$cat][$daTimeDat]['counter'] += 1;
						}
					}
				} else {
					if (!empty($catDat)) {
						foreach ($catDat as $aDate => $aTimePoint) {
							$daTimeDat = date($timePoints, ((int)strtotime($aDate)));
							if (empty($newXCounterReal[$cat][$daTimeDat])) {
								$newXCounterReal[$cat][$daTimeDat] = 0;
							}
							$newXCounterReal[$cat][$daTimeDat] = $aTimePoint;
						}
					}
				}
			}

			//wird nicht benutzt... oder? $newXCounterRealData = array();
			foreach ($newXCounterReal as $cat => $catDat) {
				if ($cat != "x") {
					foreach ($catDat as $k => $dataPoint) {
						if ($statdat['valuemode'] == "p") {
							//print_r($dataPoint);
							$catDat[$k] = round($dataPoint['value'] / $dataPoint['counter'], 2);
						} else {
							$catDat[$k] = round($dataPoint['value'], 2);
						}
					}
				}
				$retData[$cat] = array_values($catDat);
			}
			$abtests = $retData;
		}
		return $abtests;
	}


	/**
	 * @return bool
	 */
	private function setCssAndJSForExternal()
	{
		//Set Plugin CSS for this Domain
		$domain = Domain::activeDomain();
		$cssData = array("plugin_ccm19_abtests_css");
		$domain->addExtraCSS($cssData);

		$jsData = array("plugin_ccm19_abtests_js");
		$domain->addExtraJS($jsData);
		$domain->save();

		return true;
	}

}

?>
