<?php

namespace Plugins\Ccm19Abtests\EventSubscriber;

use App\Event\FilterEvent;
use Symfony\Contracts\EventDispatcher\Event;
use App\Model\PluginState;
use Plugins\Ccm19Abtests\Model\ABTestModell;
use Plugins\Ccm19Abtests\Model\ABTestData;
use Plugins\Ccm19Imex\Event\ExportDomainEvent;
use Plugins\Ccm19Imex\Event\ImportDomainEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class ImExSubscriber implements EventSubscriberInterface
{
	private $pluginState;

	public function __construct(PluginState $state) {
		$this->pluginState = $state;
	}

	/**
	 * @return array<string,array|string>
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			'ccm19.plugin.ccm19imex.exportdomain' => ['onExport'],
			'ccm19.plugin.ccm19imex.importdomain' => ['onImport'],
		];
	}

	/**
	 * Export der Plugin-Daten für die Domain, wenn Themes exportiert werden
	 *
	 * @param ExportDomainEvent $event
	 */
	public function onExport(FilterEvent $event): void
	{
		if (!$event->has('themes')) {
			return;
		}

		$exportTests = array_map(function (AbTestModell $test) {
			return [
				'name' => $test->getStringVar('abtest_name'),
				'active' => (bool)$test->getStringVar('abtest_active'),
				'active_start' => $test->getStringVar('abtest_active_start'),
				'active_stop' => $test->getStringVar('abtest_active_stop'),
				'var1' => $test->getStringVar('abtest_var1'),
				'var2' => $test->getStringVar('abtest_var2'),
				'freq1' => $test->getStringVar('abtest_freq1'),
				'freq2' => $test->getStringVar('abtest_freq2'),
				'comment' => $test->getStringVar('abtest_comment'),
			];
		}, ABTestModell::all());

		if ($exportTests) {
			$event->set('abtests', $exportTests);
		}
	}

	/**
	 * Export der Plugin-Daten für die Domain
	 *
	 * @param ImportDomainEvent $event
	 */
	public function onImport(Event $event): void
	{
		$data = $event->get('abtests');
		$themeMap = $event->getThemeMap();

		if (!$data) {
			return;
		}

		if ($event->isNewDomain()) {
			ABTestData::clear();
			ABTestModell::clear(false);
		}

		$existingMap = [];
		foreach (ABTestModell::all() as $item) {
			$existingMap[$item->getStringVar('abtest_name')] = $item->getId();
		}

		foreach ($data as $testData) {
			$name = $testData['name'] ?? '';
			if (isset($existingMap[$name])) {
				ABTestModell::delete($existingMap[$name]);
			}
			if (!isset($themeMap[$testData['var1']])) {
				continue;
			}
			if (!isset($themeMap[$testData['var2']])) {
				continue;
			}

			$test = ABTestModell::create();
			$test->setArray([
				'abtest_name' => $name,
				'abtest_active' => (isset($testData['active']) and $testData['active']) ? 'on' : '',
				'abtest_active_start' => $testData['active_start'] ?? '',
				'abtest_active_stop' => $testData['active_start'] ?? '',
				'abtest_var1' => $themeMap[$testData['var1']], // TODO: System/User-Themes korrekt behandeln
				'abtest_var2' => $themeMap[$testData['var2']],
				'abtest_freq1' => $testData['freq1'] ?? '',
				'abtest_freq2' => $testData['freq2'] ?? '',
				'abtest_comment' => $testData['comment'] ?? '',
			]);
		}
		ABTestModell::flush();
	}
}
