<?php

namespace Plugins\Ccm19Abtests\EventSubscriber;

use App\Event\ThemeDisplayEvent;
use App\Model\Theme;
use App\Model\PluginState;
use Plugins\Ccm19Abtests\Model\ABTestModell;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class ThemeSwitcher implements EventSubscriberInterface
{
	private $pluginState;
	private $testThemeSelected = false;

	public function __construct(PluginState $state) {
		$this->pluginState = $state;
	}

	/**
	 * @return array<string,array|string>
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			ThemeDisplayEvent::NAME => ['onThemeDisplay'],
			KernelEvents::RESPONSE => ['onKernelResponse', 49],
		];
	}

	/**
	 * Wenn Test aktiv, Theme wechseln
	 */
	public function onThemeDisplay(ThemeDisplayEvent $event): void
	{
		if ($this->pluginState->isActiveForCurrentDomain() and ABTestModell::isRunningTest()) {
			if ($event->isPreview() === false) {
				$this->testThemeSelected = true;
				$themeId = ABTestModell::getTestTheme();
				$theme = Theme::find($themeId);
				if ($theme) {
					$event->setTheme($theme);
				}
			}
		}
	}

	/**
	 * Wenn das Theme im AB-Test gewechselt wurde, Caching verhindern.
	 *
	 * @param ResponseEvent $event
	 */
	public function onKernelResponse(ResponseEvent $event): void
	{
		if ($event->isMainRequest() == false) {
			return;
		}

		if ($this->testThemeSelected) {
			$response = $event->getResponse();
			$response->setVary(['*']);
		}
	}
}
