<?php

namespace Plugins\Ccm19Abtests\Model;

use App\Component\Config\RepositoryInterface;
use App\Model\AccessCountJournal;
use App\Component\Curl;
use App\Component\Tcf\v2\Model\Vendor as TcfVendor;
use App\Component\Tcf\v2\Model\Purpose as TcfPurpose;
use App\Model\Plugins as PluginModel;
use App\Model\Protocol;
use App\Model\Statistic;
use App\Model\User;
use App\Model\Domain;
use App\Utils;
use App\Model\Model;
use DateTimeImmutable;

class ABTestData extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-abtests-data';

	/** @var bool $repositoryWithCustomIds */
	protected static $repositoryWithCustomIds = true;

	/** @var bool $domainSpecificRepository */
	protected static $domainSpecificRepository = true;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
		$this->assets = [];
	}

	/**
	 * @param $name
	 * @param $arguments
	 * @return string
	 */
	public function __call($name, $arguments)
	{
		return (string)$this->get($name,"");
	}


	/**
	 * @param bool $includeArchiv
	 * @return bool
	 */
	public static function createFullStats(bool $includeArchiv=false) :bool
	{
		$domain = Domain::activeDomain();

		$currentDate = null;

		// Nach Unique-ID gruppieren; immer neuesten Eintrag nehmen
		$byUcid = [];
		$sumsPerDay = [];
		$date = null;
		$user = User::loggedInUser();
		//$includeArchiv = true;
		$sytems = array("Linux", "Windows", "Mac", "iphone", "android");


		//Wenn Archiv mitlaufen soll...
		if($includeArchiv==true)
		{
			set_time_limit(300);
			//Einmal alles zurücksetzen
			self::clear();

			//Verzeichnis der Logfiles und Archie
			$verz = Utils::getBundledVarDir()."/user_data/".$user->getId()."/domains/".$domain->getId()."/";

			// Alles aus dem Verzeichnis holen an Dateien
			$files = scandir($verz, SCANDIR_SORT_ASCENDING);

			//wenn es welche gibt... könnte ja sein dass nicht...
			if(!empty($files))
			{
				//Dateien durchloopen
				foreach($files as $file)
				{
					//Es handelt sich um eine consent Log Datei
					if(stristr($file,"cm-consent"))
					{
						//Und um ein Arhciv
						if(stristr($file,".gz")) {
							//Alle Zeilen holen
							$lines = gzfile($verz . $file);
							if(!empty($lines))
							{
								//zeilen loopen
								foreach ($lines as $jitem) {
									//in ein array umwandeln
									$item = json_decode($jitem, true);
									//print_r($item);exit();

									//wenn leer - weiter
									if (empty($item))
									{
										continue;
									}

									//Hossa könnte tatsähclich passieren... - auch dann raus
									if (empty($item['ucid'])) {
										continue;
									}

									//wenn Embedding leer ist - kommt oft vor - weil preembedding Zeit verwendet werden kann, dann setzen
									if (empty($item['embeddings'])) {
										$item['embeddings'] = array();
									}

									$date = date("Y-m-d", $item['timestamp'] ?? 0);
									$ucid = $item['ucid'];
									unset($item['ucid']);
									unset($item['manipulationPrevention']);
									$item['date'] = $date;

									if ($currentDate !== $date) {
										$sumsPerDay[$date] = [
											'purposes' => [],
											'embeddings' => []
										];
										$currentDate = $date;
									}

									if (isset($byUcid[$ucid])) {
										$olditem = $byUcid[$ucid];
										$item['first_date'] = $olditem['first_date'] ?? $olditem['date'];
									} else {
										$olditem = null;
									}



									// Purposes zählen


									if (isset($item['clickedButton']))
									{
										if (!Protocol::isClickedButtonValid((string)$item['clickedButton'])) {
											$item['clickedButton'] = "opened";
										}
										if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all'])) {
											$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all'] = 0;
										}
										$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all']++;

										//Systemdaten
										if (isset($item['clientUserAgent']))
										{
											if(strlen($item['clientUserAgent'])< 3)
											{
												$item['clientUserAgent'] = "none";
											}
											foreach($sytems as $system)
											{
												if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system])) {
													$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system] = 0;
												}
												if(stristr($item['clientUserAgent'],$system))
												{
													$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system]++;
												}
											}
											if(stristr($item['clientUserAgent'],"iphone") || stristr($item['clientUserAgent'],"android"))
											{
												if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"])) {
													$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"] = 0;
												}
												$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"]++;
											}
											else{
												if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"])) {
													$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"] = 0;
												}
												$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"]++;
											}
										}

									}


									//Hier ab Datum 17022021
									if(strtotime($date) > 1613430098)
									{
										if(empty($sumsPerDay[$date]['v']))
										{
											$sumsPerDay[$date]['v']=0;
										}
										if(!empty($item['actualThemeID'])) {
											if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']]['v'])) {
												$sumsPerDay[$date]['details'][$item['actualThemeID']]['v']=0;
											}
											else{
												$sumsPerDay[$date]['details'][$item['actualThemeID']]['v']++;
											}
										}
										$sumsPerDay[$date]['v']++;
									}


									// Cookies zählen - machen wir nicht mehr...

									$byUcid[$ucid] = $item;
									unset($item);
								}
							}
						}
					}
					// Speicher freigeben
					foreach ($byUcid as $ucid => &$item) {
						$item = null;
					}
					unset($item);
				}

			}
		}
		//print_r($sumsPerDay);exit();


		//aktuelles Protokoll, auch hier die Daten holen...
		$protocol = Protocol::byDomain($domain);

		$allStat = self::all();
		if(!empty($allStat))
		{
			$lastTimeStamp = (strtotime(end($allStat)->getId()));
		}
		else{
			$lastTimeStamp = 0;
		}
		//print_r($protocol);
		//Eigentlich nicht zwingend notwendig da die Daten ja schon in Org Statistik drin stehen... aber sicher ist sicher aus den ORg Daten erstellen...
		if(!empty($protocol))
		{
			foreach ($protocol->allFromDate($lastTimeStamp) as $item) {
				//
				//print_r($item);

				$date = date("Y-m-d", $item['timestamp'] ?? 0);
				$ucid = $item['ucid'];
				unset($item['ucid']);
				unset($item['manipulationPrevention']);
				$item['date'] = $date;

				if ($currentDate !== $date) {
					$sumsPerDay[$date] = [
						'purposes'=>[],
						'embeddings'=>[]
					];
					$currentDate = $date;
				}

				if (isset($byUcid[$ucid])) {
					$olditem = $byUcid[$ucid];
					$item['first_date'] = $olditem['first_date'] ?? $olditem['date'];
				} else {
					$olditem = null;
				}



				// Purposes zählen


				if (isset($item['clickedButton']))
				{
					if (!Protocol::isClickedButtonValid((string)$item['clickedButton'])) {
						$item['clickedButton'] = "opened";
					}
					if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all'])) {
						$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all'] = 0;
					}
					$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]['all']++;

					//Systemdaten
					if (isset($item['clientUserAgent']))
					{
						if(strlen($item['clientUserAgent'])< 3)
						{
							$item['clientUserAgent'] = "none";
						}
						foreach($sytems as $system)
						{
							if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system])) {
								$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system] = 0;
							}
							if(stristr($item['clientUserAgent'],$system))
							{
								$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']][$system]++;
							}
						}
						if(stristr($item['clientUserAgent'],"iphone") || stristr($item['clientUserAgent'],"android"))
						{
							if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"])) {
								$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"] = 0;
							}
							$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["mobile"]++;
						}
						else{
							if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"])) {
								$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"] = 0;
							}
							$sumsPerDay[$date]['details'][$item['actualThemeID']][$item['clickedButton']]["desktop"]++;
						}
					}

				}


				//Hier ab Datum 17022021
				if(strtotime($date) > 1613430098)
				{
					if(empty($sumsPerDay[$date]['v']))
					{
						$sumsPerDay[$date]['v']=0;
					}
					if(!empty($item['actualThemeID'])) {
						if (!isset($sumsPerDay[$date]['details'][$item['actualThemeID']]['v'])) {
							$sumsPerDay[$date]['details'][$item['actualThemeID']]['v']=0;
						}
						else{
							$sumsPerDay[$date]['details'][$item['actualThemeID']]['v']++;
						}
					}
					$sumsPerDay[$date]['v']++;
				}

				$byUcid[$ucid] = $item;
				unset($item);
			}

			// Speicher freigeben
			foreach ($byUcid as $ucid => &$item) {
				$item = null;
			}
			unset($item);
		}
		//exit();
		//print_r($sumsPerDay);exit();

		//Jetzt noch die views übertragen...
		foreach(Statistic::all() as $key=>$statEntry)
		{
			//print_r($statEntry);
			if(empty($sumsPerDay[$key]["v"]))
			{
				//Nur view übertragen wenn es geht - ein Eintrag vorhanden ist
				if(!empty($statEntry->get("v")) && !empty($sumsPerDay[$key]))
				{
					$sumsPerDay[$key]["v"]=$statEntry->get("v");
					$sumsPerDay[$key]["c"]=$statEntry->get("c");
				}
				//Kein EIntrag aus Logfile - dann zusammenstellen... aus original Stats
				if(!empty($statEntry->get("v")) && empty($sumsPerDay[$key]))
				{
					$sumsPerDay[$key]["v"]=$statEntry->get("v");
					$sumsPerDay[$key]["c"]=$statEntry->get("c");
					$sumsPerDay[$key]["purposes"]=$statEntry->get("p");
					$sumsPerDay[$key]["embeddings"]=$statEntry->get("e");
				}
			}
			$sumsPerDay[$key]["c"]=$statEntry->get("c");
		}
		//print_r($sumsPerDay);
		//exit();



		//Nur wenn auch Einträge da sind...
		if(!empty($sumsPerDay))
		{
			// Statistiken speichern
			self::beginWrite();
			foreach ($sumsPerDay as $date => $row) {
				$stat = self::findOrCreate($date);
				//print_r($row);
				if(!empty($row['v']))
				{
					$stat->set("v",$row['v']);
				}

				if(!empty($row['c']))
				{
					$stat->set("c",$row['c']);
				}

				if(!empty($row['purposes']))
				{
					foreach ($row['purposes'] as $purposeId => $count) {
						$stat->setPurposeAcceptCount($purposeId, $count);
					}
				}

				if(!empty($row['details']))
				{
					//print_r($row['details']);
					$stat->set("details",$row['details']);
				}

				if(!empty($row['embeddings']))
				{
					foreach ($row['embeddings'] as $embeddingId => $count) {
						$stat->setEmbeddingAcceptCount($embeddingId, $count);
					}
				}

				if(!empty($row['cookies']))
				{
					foreach ($row['cookies'] as $cookieId => $count) {
						if($count>1)
						{
							$stat->setCookiecceptCount($cookieId, $count);
						}
					}
				}
				if(!empty($row['details']))
				{
					//print_r($stat);exit();
					self::flush();
				}
			}


			//yeah - done
			return true;
		}
		//nope - nix...
		return false;
	}

	/**
	 * @param string $cookieId
	 * @param int $count
	 * @return $this
	 */
	public function setCookiecceptCount(string $cookieId, int $count): self
	{
		$this->set('c.'.$cookieId, $count);
		return $this;
	}

	/**
	 * @param string $embeddingId
	 * @param int $count
	 * @return $this
	 */
	public function setEmbeddingAcceptCount(string $embeddingId, int $count): self
	{
		$this->set('e.'.$embeddingId, $count);
		return $this;
	}

	/**
	 * @param string $purposeId
	 * @param int $count
	 * @return $this
	 */
	public function setPurposeAcceptCount(string $purposeId, int $count): self
	{
		$this->set('p.'.$purposeId, $count);
		return $this;
	}

	/**
	 * @param string $date Y-m-d
	 * @return static
	 */
	public static function findOrCreate(string $date): self
	{
		return self::find($date) ?? self::create($date);
	}

	/**
	 * Diese Methode erstellt ein Objekt dieser Klasse
	 * @param string|null $date Y-m-d (default: heute)
	 * @return static
	 * @throws \LogicException
	 */
	public static function create(?string $date=null): Model
	{
		if ($date === null) {
			$date = (new DateTimeImmutable('now'))->format('Y-m-d');
		}
		self::beginWrite();
		return new static($date);
	}

	/**
	 * @param string $date Y-m-d
	 * @return static|null
	 */
	public static function find(string $date): ?Model
	{
		return self::exists($date) ? new static($date) : null;
	}
	/**
	 * @return int
	 */
	public function getPurposeAcceptCount(string $purposeId, bool $includeArchived=true): int
	{
		$result = $this->get('p.'.$purposeId, 0);
		if ($includeArchived) {
			$result += $this->get('ap.'.$purposeId, 0);
		}
		return $result;
	}

	/**
	 * @return int
	 */
	public function getViewCount(): int
	{
		return $this->get('v') ?? 0;
	}

	/**
	 * @return int
	 */
	public function getCallCount(): int
	{
		$calls = $this->get('c');
		if(is_array($calls))
		{
			return 1;
		}
		else{
			if($calls>=1)
			{
				return $calls;
			}
			return 1;
		}
	}

}
