<?php

namespace Plugins\Ccm19Abtests\Model;

use App\Component\Config\RepositoryInterface;
use App\Model\Domain;
use App\Component\Curl;
use App\Component\Tcf\v2\Model\Vendor as TcfVendor;
use App\Component\Tcf\v2\Model\Purpose as TcfPurpose;
use App\Model\Plugins as PluginModel;
use App\Model\Protocol;
use App\Model\Statistic;
use App\Model\Theme;
use App\Utils;
use App\Model\Model;
use Symfony\Contracts\Translation\TranslatorInterface;

class ABTestModell extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-abtests';

	/** @var bool $domainSpecificRepository */
	protected static $domainSpecificRepository = true;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
		$this->assets = [];
	}

	/**
	 * @param $name
	 * @param $arguments
	 * @return string
	 */
	public function __call($name, $arguments)
	{
		return (string)$this->get($name,"");
	}

	/**
	 * Generic String Catcher...
	 *
	 * @param string $varname
	 * @return string
	 */
	public function getStringVar(string $varname = "")
	{
		if(!empty($varname))
		{
			return (string)$this->get($varname,"");
		}

	}

	/**
	 * @param int $abId
	 * @return string
	 */
	public static function getChangeLogs($abId=0)
	{
		//logtext
		$log ="";

		//Eigene Daten
		$testData = self::find($abId);
		$abLog = $testData->get("changeLog") ?? array();

		//theme Daten
		$t1 = Theme::find( $testData->get("abtest_var1"));
		if(!empty($t1))
		{
			$t1log = $t1->get("changeLog") ?? array();
		}
		else{
			$t1log=array();
		}


		$t2 = Theme::find( $testData->get("abtest_var2"));
		if(!empty($t2))
		{
			$t2log = $t2->get("changeLog") ?? array();
		}
		else{
			$t2log=array();
		}

		$completeLog = $abLog + $t1log + $t2log;

		foreach($completeLog as $time => $logentry)
		{
			if(!empty($logentry))
			{
				foreach ($logentry as $type => $entry)
				{
					if(empty($entry['old']))
					{
						$entry['old'] = "0";
					}
					if(empty($entry['new']))
					{
						$entry['new'] = "0";
					}
					if (is_array($entry['old'])) {
						$entry['old'] = serialize($entry['old']);
					}
					if (is_array($entry['new'])) {
						$entry['new'] = serialize($entry['new']);
					}
					$log.=date("Y-m-d H:i:s",(int)$time)." - ".$type.": ".$entry['old']." -> ".$entry['new']." \n";
				}
			}
		}
		$pattern = '/#(\S*?) +/';
		$replacement ='<span style="border-left:20px solid #$1;margin-right:10px;">&nbsp;&nbsp;#$1#</span>';
		$log = preg_replace($pattern, $replacement,$log);
		//print_r($log);exit();

		$log = nl2br($log);
		return $log;
	}

	/**
	 * @return array
	 */
	public static function getBarCharts(TranslatorInterface $translator)
	{
		$barCharts = array();
		$barCharts['0']['name'] = "all";
		$barCharts['1']['name'] = "mobile";
		$barCharts['2']['name'] = "desktop";
		$barCharts['3']['name'] = "iphone";
		$barCharts['4']['name'] = "android";
		$barCharts['5']['name'] = "Windows";
		$barCharts['6']['name'] = "Mac";
		$barCharts['7']['name'] = "Linux";

		$barCharts['0']['description'] = $translator->trans('All consent data.');
		$barCharts['1']['description'] = $translator->trans('Only mobile consent data.');
		$barCharts['2']['description'] = $translator->trans('Only desktop consent data.');
		$barCharts['3']['description'] = $translator->trans('Only consent data from iphone.');
		$barCharts['4']['description'] = $translator->trans('Only consent data from android phones.');
		$barCharts['5']['description'] = $translator->trans('Only consent data from Windows.');
		$barCharts['6']['description'] = $translator->trans('Only consent data from Mac.');
		$barCharts['7']['description'] = $translator->trans('Only consent data from Linux.');
		return $barCharts;
	}

	/**
	 * @return string[]
	 */
	public static function getButtons()
	{
		return array("decline" => "Button abgelehnt", "acceptAll" => "Button alles akzeptiert", "opened" => "Absprung");
	}


	/**
	 * @return string[]
	 */
	public static function getFieldNames(): array
	{
		return array(
			"abtest_name",
			"abtest_active",
			"abtest_var1",
			"abtest_var2",
			"abtest_freq1",
			"abtest_freq2",
			"abtest_comment",
			"id"
		);

	}

	/**
	 * @return bool
	 */
	public static function isRunningTest()
	{
		$tests = self::all();
		foreach($tests as $test)
		{
			if ($test->get("abtest_active")=="on")
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * Get Theme zufällig anhand der Freqenz die eingetragen ist
	 * @return mixed
	 */
	public static function getTestTheme()
	{
		$tests = self::all();
		foreach($tests as $test)
		{
			if ($test->get("abtest_active")=="on")
			{
				$rand = rand(0,100);
				if ($rand <=$test->get("abtest_freq1"))
				{
					return $test->get("abtest_var1");
				}
				else{
					return $test->get("abtest_var2");
				}
			}
		}

	}

	/**
	 * @param array $postData
	 * @return true
	 */
	public function setArray(array $postData): self
	{
		foreach($postData as $k=>$v)
		{
			$this->set($k, $v);
		}
		return $this;
	}

}
