<?php

namespace Plugins\Ccm19CBImport\Controller;

use App\Component\LocaleService;
use App\Component\Menu\Annotation\Menu;
use App\Controller\UniversalController;
use App\Model\CsrfToken;
use App\Model\Domain;
use App\Model\PluginState;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;
use Plugins\Ccm19CBImport\Model\CBImportModel;

/**
 * Controller for CBImport - domain dependent
 * @Route("/cbimport/", name="plugin_ccm19_cookieimport_")
 */
class CBImport extends AbstractController
{
	/**
	 * @Menu("Cookie Import", icon="fa-coffee",  order=60, group="Plugins", navigation="main", editions={"extended", "multi"})
	 * @Route("index", name="index", methods={"HEAD", "GET"})
	 * @param PluginState $state
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function index(PluginState $state, TranslatorInterface $translator): Response
	{
		//ZUgriff checken...
		if(!$state->isAllowedForCurrentUser())
		{
			return $this->redirectToRoute('app_domains_index');
		}

        //wenn KEIN Access aufgrund von Tarif besteht und Trial Expired ist
        if($state->trialIsExpired())
        {
            return $this->renderTrialExpired();
        }

        //Einträge erzeugen falls noch nicht vorhanden
        $this->createEntriesIfNotExists();

        //Render template with all Data
        return $this->render('@plugin:Ccm19CBImport/internal/index.html.twig', [
            'csrfToken' => CsrfToken::get(),
            'cmps' => CBImportModel::getImportVorlagen($translator),
        ]);

    }

    /**
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function renderTrialExpired(): Response
    {
        return $this->render('@plugin:Ccm19CBImport/internal/trial_end.html.twig', [
            'csrfToken' => CsrfToken::get(),
        ]);
    }

	/**
	 * Most basic creation of an entry...
	 *
	 * @return bool
	 * Create a new entry with senseless data
	 */
	public function createEntriesIfNotExists(): bool
	{
		$all = CBImportModel::all();

		if(count($all) < 1)
		{
			//Set Some Data
			$postPLuginData = array(
				array(
					"name"=>"CookieBot",
					"description"=>"Import der Domain und Cookie / Skriptdaten direkt aus Cookiebot",
					"path"=>"/cbimport/cbbot"
				)
			);

			foreach($postPLuginData as $singleSet)
			{
				//Create an entry for json file
				$PluginsClass = CBImportModel::create();

				//Set Data from array in new entry
				$PluginsClass->setArray($singleSet);

				//finally save the data to the json file
				$PluginsClass->flush();
			}
			return true;
		}

		return false;
	}
}
