<?php

namespace Plugins\Ccm19CBImport\Model;

use App\Component\Config\RepositoryInterface;
use App\Component\Curl;
use App\Component\Tcf\v2\Model\Vendor as TcfVendor;
use App\Utils;
use App\Model\Model;
use Symfony\Contracts\Translation\TranslatorInterface;

class CBImportModel extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-plugin-cbimport';

	/** @var bool $userSpecificRepository */
	protected static $userSpecificRepository = true;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
		$this->assets = [];
	}

	/**
	 * @param $name
	 * @param $arguments
	 * @return string
	 */
	public function __call($name, $arguments)
	{
		return (string)$this->get($name,"");
	}

	/**
	 * @param string $name
	 * @return self|null
	 * @phan-side-effect-free
	 */
	public static function byName(string $name): ?self
	{
		$name = strtolower($name);
		return array_reduce(self::all(), function ($carry, $lang) use ($name) {
			return $carry ?? (strtolower($lang->getName()) == $name ? $lang : $carry);
		}, null);
	}

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name',"cookiebot");
	}

	/**
	 * @return string
	 */
	public function getApiKey(): string
	{
		return trim((string)$this->get('apikey',""));
	}

	/**
	 * @return string
	 */
	public function getCulture(): string
	{
		return trim((string)$this->get('culture',""));
	}

	/**
	 * @return string
	 */
	public function getSerial(): string
	{
		return trim((string)$this->get('serial',""));
	}

	/**
	 * @return string
	 */
	public function getDomains(): string
	{
		return (string)implode("\n",$this->get('domains',array()));
	}

	/**
	 * @return array
	 */
	public function getDomainsArray(): array
	{
		return (array)$this->get('domains',array());
	}

	/**
	 * @param string $text
	 * @return $this
	 */
	public function setName(string $text): self
	{
		$this->set('name', $text);
		return $this;
	}

	/**
	 * @param string $text
	 * @return $this
	 */
	public function setApiKey(string $text): self
	{
		$this->set('apikey', $text);
		return $this;
	}



	/**
	 * @param string $text
	 * @return $this
	 */
	public function setCulture(string $text): self
	{
		$this->set('culture', $text);
		return $this;
	}

	/**
	 * @param string $text
	 * @return $this
	 */
	public function setSerial(string $text): self
	{
		$this->set('serial', $text);
		return $this;
	}

	/**
	 * @param array $text
	 * @return $this
	 */
	public function setDomains(array $text): self
	{
		$this->set('domains', $text);
		return $this;
	}

	/**
	 * Generic String Catcher...
	 *
	 * @param string $varname
	 * @return string
	 */
	public function getStringVar(string $varname)
	{
		if (!empty($varname)) {
			return (string)$this->get($varname,"");
		}
	}

	/**
	 * @return string[]
	 */
	public function getFieldNames(): array
	{
		return array(
			"exampleInputName",
			"expampleCheckboxData",
			"exampleSelectBox"
		);
	}

	/**
	 * @param TranslatorInterface $translator
	 * @return array[]
	 */
	public static function getImportVorlagen(TranslatorInterface $translator): array
	{
		return array(
			"123456"=>array(
				"id" => "1",
				"name"=>"cookiebot",
				"description" =>$translator->trans("Import der Domain und Cookie Skriptdaten direkt aus Cookiebot"),
				"path"=>"plugin_ccm19_cookieimport_cbbot_index"
			)
		);
	}
	/**
	 * @param array $postData
	 * @return true
	 */
	public function setArray(array $postData): self
	{
		foreach($postData as $k=>$v)
		{
			$this->set($k, $v);
		}
		return $this;
	}


}
