<?php
namespace Plugins\Ccm19Imex\Command;

use App\Model\User;
use App\Utils;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Filesystem\Filesystem;
use Plugins\Ccm19Imex\Component\Export;

/**
 *
 */
class ExportCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'plugins:imex:export';

	/**
	 *
	 */
	protected function configure(): void
	{
		$this
			->setDescription('Export all data To Files.')
		;
	}

	/**
	 * @param InputInterface $input
	 * @param OutputInterface $output
	 * @return int
	 */
	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		//Export Verzeichnis erstellen
		$exportDir = $this->getExportDir();
		$time = time();
		$ucount = 0;
		$store = Utils::getRepositoryStore();
		$varDir = Utils::getBundledVarDir();
		$filesystem = new Filesystem();
		$io = new SymfonyStyle($input, $output);

		//Clients durchloopen und zips erstellen - alle einzeln am besten
		foreach (User::getClients() as $client)
		{
			$io->text("Client: ".$client->getId());
			if($store=="json")
			{
				$io->text("Archiv-Dateien klonen (JSON)");
				//Archiv Verzeichnis erstellen
				$targetArchivDir = $exportDir."/".$client->getId()."_loggzs";
				$filesystem->mkdir($targetArchivDir);

				//Archiv Dateien holen und verarbeiten
				try {
					$filesystem->mirror($varDir."/user_data/".basename($client->getId())."/domains",$targetArchivDir);
				}
				catch (\Exception $ex)
				{
					//ignorieren
					$io->text("Keine Archiv Dateien vorhanden");
				}
			}
			else{
				$io->text("Archiv-Dateien klonen (MongoDB)");
				//Archiv Verzeichnis erstellen
				$targetArchivDir = $exportDir."/".$client->getId()."_loggzs";
				$filesystem->mkdir($targetArchivDir);

				//Archiv Dateien holen und verarbeiten
				try {
					$filesystem->mirror($varDir."/user_data/consentarchiv/".basename($client->getId())."/domains",$targetArchivDir);
				}
				catch (\Exception $ex)
				{
					//ignorieren
					$io->text("Keine Archiv-Dateien vorhanden");
				}
			}
			$io->text("Repo-Daten speichern und zippen ");
			$this->writeToGzipFile(json_encode(Export::exportClient($client),JSON_PRETTY_PRINT),$exportDir."/".$client->getId().".gz");
			if (json_last_error()) {
				$io->error(json_last_error_msg());
			}
			$ucount++;
		}
		$io->text("System-Themes zippen ");
		//SystemThemes noch hinterher exportSystemThemes
		$this->writeToGzipFile(json_encode(Export::exportSystemThemes(),JSON_PRETTY_PRINT),$exportDir."/systemthemes.gz");

		$io->text("Alles in eine Datei ");
		//Yeah - plain Commandline alles zippen und löschen
		echo exec('cd '.Utils::getVarDir().'; tar zcf ./export_complete.tar.gz export/*; rm -R ./export/;');


		$io->success("Export-Datei erzeugt - Dauer: ".(time()-$time)." Sekunden für ".$ucount." User, Filesize: ".round(filesize(Utils::getVarDir()."/export_complete.tar.gz")/(1024*1024),2)." MB");
		$io->note("Pfad: ".Utils::getVarDir()."/export_complete.tar.gz");
		//Als zip wegspeichern
		return 0;
	}

	/**
	 * Liefert das Export Verzeichnis und erstellt falls nicht vorhanden
	 * @return string
	 */
	protected function getExportDir()
	{
		$exportDir = Utils::getVarDir()."/export/";
		if(!is_dir($exportDir))
		{
			mkdir($exportDir);
		}

		return $exportDir;
	}


	/**
	 * @param string $string
	 * @param string $path
	 * @return bool
	 */
	protected function writeToGzipFile(string $string, string $path)
	{
		if(!empty($path) && !empty($string))
		{
			$gz = gzopen($path,'w9');
			gzwrite($gz, $string);
			gzclose($gz);
			return true;
		}
		return false;
	}

}
