<?php

namespace Plugins\Ccm19Imex\Command;

use App\Exception\ApiKeyAlreadyExistsException;
use App\Exception\UserAlreadyExistsException;
use App\Model\Domain;
use App\Model\Model;
use App\Model\SubGroups;
use App\Model\User;
use App\Model\UserEmbedding;
use App\Utils;
use Exception;
use RuntimeException;
use Plugins\Ccm19Imex\Component\Import;
use Plugins\Ccm19Imex\Event\ImportClientEvent;
use Plugins\Ccm19Imex\Event\ImportClientPreDomainsEvent;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

use Plugins\Ccm19Imex\Component\Export;
use Symfony\Component\Filesystem\Filesystem;

/**
 *
 */
class ImportCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'plugins:imex:import';

	/**
	 *
	 */
	protected function configure(): void
	{
		$this
			->setDefinition([
				new InputOption('duplicate-user-prefix', 'p', InputOption::VALUE_REQUIRED, 'Don\'t merge users with the same name but a different API key. If a user would overwrite another user with a different API key, the imported user gets this prefix in his user name.'),
			])
			->setDescription('Import everything from <info>var/export_complete.tar.gz</info> to current database / JSON storage. May overwrite existing data.');
	}

	/**
	 * @param InputInterface $input
	 * @param OutputInterface $output
	 * @return int
	 * @throws ApiKeyAlreadyExistsException
	 */
	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		//Export Verzeichnis erstellen
		$importDir = $this->getImportDir();
		$time = time();
		$ucount = 0;
		$alreadyExists = array();
		$alreadyExistsDom = array();
		$excepts = array();
		$archivDir = Utils::getBundledVarDir()."/user_data/consentarchiv/";
		$filesystem = new Filesystem();
		$io = new SymfonyStyle($input, $output);
		$filesystem->mkdir($archivDir);
		$duplUserPrefix = (string)$input->getOption('duplicate-user-prefix');
		//exit();

		//Einmal den Admin User einloggen
		$userDatOne = User::byRole("admin");
		$uDatOne = array_shift($userDatOne);
		User::loginByGeneralApiKey($uDatOne->getGeneralApiKey());


		$io->text("GZ Files entpacken und ins Import Verzeichnis");
		//Yeah - plain Commandline alles entzippen und ins Import Verzeichnis
		echo exec('cd ' . escapeshellarg(Utils::getVarDir()) . ' && tar -zxf  ./export_complete.tar.gz  --directory ./import --strip-components=1; ');

		//Dateien Liste einlesen
		$gzFiles = scandir($importDir);
		if(!empty($gzFiles)) {


			$sysfile = $this->getDataFromFile($importDir, "systemthemes.gz");
			$systemThemeMap = [];
			if (isset($sysfile['systemThemes']) && $sysfile['systemThemes']) {
				$io->text("MAke Theme Import");
				$systemThemeMap = Import::makeThemesImport('system', $sysfile['systemThemes'], NULL, true);
			}
			unset($sysfile);

			//loopen
			$gzFiles = array_values(array_filter($gzFiles, function ($file) {
				return stristr($file, ".gz") && !stristr($file, "system");
			}));
			$count = count($gzFiles);
			foreach ($gzFiles as $i => $file) {
				if (stristr($file, ".gz") && !stristr($file, "system")) {
					$ucount++;
					$io->text("Import old Client: <info>".$file."</info> ($i/$count)");
					//Daten eines Clients
					$fileClient = $this->getDataFromFile($importDir, $file);
					$fileClient['clientData']['id']=str_ireplace(".gz","",$file);

					try {
						$client = User::create($fileClient['clientData']['username']);
						$client->setApiKey($fileClient['clientData']['apiKey']);
						if (!empty($fileClient['clientData']['generalApiKey'])) {
							$client->setGeneralApiKey($fileClient['clientData']['generalApiKey']);
						}
					} catch (UserAlreadyExistsException $e) {
						$alreadyExists[] = $fileClient['clientData']['username'];
						$client = User::find($fileClient['clientData']['username']);

						if ($duplUserPrefix !== '' and $client->getApiKey() !== $fileClient['clientData']['apiKey']) {
							$fileClient['clientData']['username'] = $duplUserPrefix.$fileClient['clientData']['username'];

							$io->text("Doppelung: Benutzername auf <info>".$duplUserPrefix.$fileClient['clientData']['username']."</info> geändert.");

							try {
								$client = User::create($fileClient['clientData']['username']);
							} catch (UserAlreadyExistsException $e) {
								$alreadyExists[] = $fileClient['clientData']['username'];
								$client = User::find($fileClient['clientData']['username']);
							}
						}

						//Den ApiKey des neuen Accounts übernehmen, wenn es Domains gibt
						if ($client->getApiKey() !== $fileClient['clientData']['apiKey']) {
							if (!empty($fileClient['domains'])) {
								$client->setApiKey($fileClient['clientData']['apiKey']);
								$client->setGeneralApiKey($fileClient['clientData']['generalApiKey']);
								$io->note("Doppelter User hat anderen API-KEY: ".$fileClient['clientData']['username'].". Neuen Key übernommen.");
							} else {
								$io->note("Doppelter User hat anderen API-KEY: ".$fileClient['clientData']['username'].". Alten Key behalten.");
							}
						}
						//continue;
					}

					$io->text("Import Client: ".$client->getId());

					// Writes changes of the client to prevent accidental deadlocks
					$client->save();

					if (User::find($fileClient['clientData']['username']) !== NULL) {
						$client = User::find($fileClient['clientData']['username']);
					}

					// Client importieren - Grunddaten
					$io->text("Client importieren - Grunddaten ");
					Import::makeClientImport($client, $fileClient['clientData']);
					$maxDomainCount = $client->getMaxDomainCount();
					$client->setMaxDomainCount(-1);
					$client->save();
					// Switch to newly created user
					User::setApiUser($client->getApiKey());
					Model::closeUserModels();
					Domain::switchToUser($client);

					// Themes
					$io->text("Client importieren - Themes ");
					$userThemeMap = Import::makeThemesImport('user', $fileClient['userThemes'], NULL, true);
					// User Locales
					$io->text("Client importieren - Locales ");
					Import::makeLocalesImport('user', $fileClient['standardTexts']);
					User::flush();

					$idMappings = [
						'systemTheme' => $systemThemeMap,
						'userTheme' => $userThemeMap,
						'systemEmbedding' => [],
						'userEmbedding' => [],
					];
					$io->text("Client importieren - Embeddings ");
					UserEmbedding::clear(true);
					$idMappings['userEmbedding'] = Import::makeEmbeddingsImport('user', $fileClient['userEmbeddings'] ?? [], $idMappings);

					Utils::dispatchEvent(ImportClientPreDomainsEvent::NAME, new ImportClientPreDomainsEvent($client, $fileClient, $idMappings));

					$io->text("Client importieren - Domains ");
					if (isset($fileClient['domains']) && is_array($fileClient['domains'])) {
						foreach ($fileClient['domains'] as $domainId => $fileDomain) {
							$io->text("Domain erstellen ");
							//DomID mit übernehmen...
							$newDomainID =  Domain::canonicalizeId($client->getId(), $fileDomain['domainData']['id']);
							$existOldDom = Domain::byName($fileDomain['domainData']['name']);
							if ($existOldDom) {
								$alreadyExistsDom[] = $fileDomain['domainData']['name'];
								Domain::delete($existOldDom->getId());
							}
							elseif (Domain::find($domainId)) {
								// Domain-ID-Kollision im gleichen Benutzer
								$io->error("Domain mit gleicher ID aber anderem Namen existiert schon: $domainId (ALT: ".Domain::find($domainId)->getName()." NEU: ".$fileDomain['domainData']['name'].")");
								if ($io->confirm("Alte Domain überschreiben?", false)) {
									Domain::delete($domainId);
								} else {
									continue;
								}
							}
							$domain = Domain::createForUser($client, false, $newDomainID,false)->setName($fileDomain['domainData']['name']);
							$io->text("Domain Speichern ");
							// Writes changes of the domain to prevent accidental deadlocks
							$domain->save();
							// Switch to domain context
							Model::closeDomainModels();
							Domain::setDomainForRequest($domain->getId());

							$io->text("Archiv Consent Daten übertragen ");
							//Archiv Consent Daten übertragen
							$this->copyArchivConsentFiles($fileClient['clientData']['id'],$client->getId(),$fileDomain['domainData']['id'],$domain->getId(),$archivDir,$importDir,$filesystem,$io);

							// Grant access to newly created domain(s) to subuser
							if ($client->isSubUser()) {
								$client
									->grantDomainAccess($domain)
									->save();
							}

							$io->text("Domain-Inhalte übertragen ");
							try {
								Import::importDomain($domain, $fileDomain, true, $idMappings);
								$domainMap[$domainId] = $domain;
							} catch (Exception $exception) {
								$io->error("Domain-Import fehlgeschlagen: ".$exception->getMessage());
								$excepts[] = array("msg" => $exception->getMessage(), "user" => $client->getUsername());
							}
						}
						Domain::flush();
						// Close domain context
						Model::closeDomainModels();
						Domain::setDomainForRequest('');
					}

					$client->setMaxDomainCount($maxDomainCount);
					$client->updateDomainInfo();
					$client->save();

					Utils::dispatchEvent(ImportClientEvent::NAME, new ImportClientEvent($client, $fileClient, $idMappings));

					if (isset($fileClient['subuserGroups']) && $fileClient['subuserGroups']) {
						$groupMap = [];

						SubGroups::switchRepositoryContext($client);
						foreach ($fileClient['subuserGroups'] as $groupId => $fileGroup) {
							$subgroup = Import::importSubUserGroup($fileGroup);
							$groupMap[$groupId] = $subgroup;
						}
						SubGroups::flush();

						if (isset($fileClient['subusers']) && $fileClient['subusers']) {
							foreach ($fileClient['subusers'] as $subuserData) {
								try {
									$subuser = User::create($subuserData['username'])
										->setSubUserOf($client->getId());
								} catch (UserAlreadyExistsException $exception) {
									$io->note("Subuser schon vorhanden: ".$subuserData['username']);
									$excepts[] = array("msg" => $exception->getMessage(), "user" => $fileClient['clientData']['username']);
									continue;
								}

								$realGroups = [];
								$realDomains = [];
								foreach ($subuserData['groups'] as $key => $group) {
									if (isset($groupMap[$key])) {
										$realGroups[$groupMap[$key]->getId()] = $group ? "on" : "off";
									}
								}
								foreach ($subuserData['domains'] as $key => $domain) {
									if (isset($domainMap[$key])) {
										$realDomains[$domainMap[$key]->getId()] = $domain ? "on" : "off";
									}
								}
								$subuser->setGroupActive($realGroups);
								$subuser->setDomActive($realDomains);
								$subuser->save();

								Import::makeClientImport($subuser, $subuserData);
							}
							User::flush();
						}
					}
				}
				unset($fileClient);
			}

			echo exec('cd '.escapeshellarg(Utils::getVarDir()).'; rm -R ./import/;');

			$io->success("Clienten importiert - Dauer: " . (time() - $time) . " Sekunden für " . $ucount . " User ");
			$io->error("Diese existieren schon: ");
			$io->listing($alreadyExists);
			$io->error("Diese Doms wurden durch neue Einträge ersetzt");
			$io->listing($alreadyExistsDom);

			if (!empty($excepts)) {
				$io->error("Ups - es gab Exceptions");
				$io->listing(array_map(function ($entry) {
					return sprintf("User <comment>%s</comment>: %s", htmlspecialchars($entry['user']), $entry['msg']);
				}, $excepts));
			}
		}
		else{
			$io = new SymfonyStyle($input, $output);
			$io->error("Ups - Keine Datei zum importieren vorhanden");
		}

		//Als zip wegspeichern
		return 0;
	}

	/**
	 * @param string $oldUserId
	 * @param string $newUserID
	 * @param string $oldDomId
	 * @param string $newDomId
	 * @param string $archivDir
	 * @param string $importDir
	 * @param Filesystem $filesystem
	 * @param $io
	 */
	protected function copyArchivConsentFiles(string $oldUserId, string $newUserID, string $oldDomId,string  $newDomId, string  $archivDir, string $importDir, Filesystem $filesystem,$io)
	{
		//Archiv Verzeichnis bestimmen aus den IDs
		$importArchivDir = $importDir."/".$oldUserId."_loggzs/".$oldDomId;
		$targetArchivDir = $archivDir."/".$newUserID."/domains/".$newDomId."/";

		//Alte Archivdateien suchen
		$archivFiles = @scandir($importArchivDir);
		if(!empty($archivFiles)) {
			//es gibt evtl. noch kein Archiv Dir - sicherheitshalber mal erstellen
			$filesystem->mkdir($targetArchivDir);

			//Archiv durchloopen und kopieren
			foreach($archivFiles as $file)
			{
				if(stristr($file,".gz"))
				{
					$orgFile = $importArchivDir."/".$file;
					$destFile = $targetArchivDir."/".$file;

					try {
						$filesystem->copy($orgFile,$destFile);
						$io->text("File kopiert: ".$orgFile);
					}
					catch (Exception $ex)
					{
						$io->error("File nicht kopiert: ".$orgFile);
						//ignorieren
						continue;
					}

				}
			}
		}

	}

	/**
	 * @param $importDir
	 * @param string $file
	 * @return array
	 */
	protected function getDataFromFile(string $importDir, string $file): array
	{
		$f = gzopen($importDir . "/" . $file, 'rb', false);
		if (!$f) {
			throw new RuntimeException("File not found: ".escapeshellarg($file));
		}
		$json = stream_get_contents($f);
		gzclose($f);
		$clientDataArray = json_decode($json, true);

		return $clientDataArray;
	}

	/**
	 * Liefert das Export Verzeichnis und erstellt falls nicht vorhanden
	 * @return string
	 */
	protected function getImportDir()
	{
		$importDir = Utils::getVarDir() . "/import";
		if (!is_dir($importDir)) {
			mkdir($importDir);
		}
		return $importDir;
	}


}
