<?php

namespace Plugins\Ccm19Imex\Event;

use App\Event\FilterEvent;
use App\Model\Domain;

/**
 * Event beim Exportieren einer Domain, nachdem die Daten der
 * Kern-Models zusammengetragen wurden.
 *
 * Wird im Kontext der zu exportierenden Domain ausgeführt.
 *
 * @phan-inherits FilterEvent<array>
 */
final class ExportDomainEvent extends FilterEvent
{
	const NAME = 'ccm19.plugin.ccm19imex.exportdomain';

	/** @var Domain $domain */
	private $domain;

	public function __construct(Domain $domain, array $data)
	{
		$this->domain = $domain;
		parent::__construct($data);
	}

	/**
	 * @param string $key
	 * @return bool
	 */
	public function has(string $key): bool
	{
		return isset($this->returnValue[$key]);
	}

	/**
	 * @param string $key
	 * @param mixed $value
	 * @return $this
	 */
	public function set(string $key, $value): self
	{
		$this->returnValue[$key] = $value;
		return $this;
	}

	/**
	 * @return Domain
	 */
	public function getDomain(): Domain
	{
		return $this->domain;
	}
}
