<?php

namespace Plugins\Ccm19Imex\Event;

use Symfony\Contracts\EventDispatcher\Event;
use App\Model\User;

/**
 * Event beim Import eines Users nach dem Laden der Daten der
 * Kern-Models und vor dem Laden der Domains.
 *
 * Wird im Kontext des zu importierenden Users ausgeführt.
 */
final class ImportClientPreDomainsEvent extends Event
{
	const NAME = 'ccm19.plugin.ccm19imex.importclientpredomains';

	/** @var User $client */
	private $client;

	/** @var array $data */
	private $data;

	/** @var array<string,array> $idMappings */
	private $idMappings;

	public function __construct(User $client, array $data, array $idMappings)
	{
		$this->client = $client;
		$this->data = $data;
		$this->idMappings = $idMappings;
	}

	/**
	 * @return User
	 */
	public function getClient(): User
	{
		return $this->client;
	}

	/**
	 * @return array
	 */
	public function getImportData(): array
	{
		return $this->data;
	}

	/**
	 * @return null|mixed
	 */
	public function get($key)
	{
		return $this->data[$key] ?? null;
	}

	public function getIdMapping(string $kind): array
	{
		return $this->idMappings[$kind] ?? [];
	}

	public function getSystemThemeMap(): array
	{
		return $this->getIdMapping('systemTheme');
	}

	public function getUserThemeMap(): array
	{
		return $this->getIdMapping('userTheme');
	}
}
