<?php

namespace Plugins\Ccm19Imex\Event;

use Symfony\Contracts\EventDispatcher\Event;
use App\Model\Domain;

/**
 * Event beim Importieren einer Domain, nachdem die Daten der
 * Kern-Models geladen wurden.
 *
 * Wird im Kontext der zu importierenden Domain ausgeführt.
 */
final class ImportDomainEvent extends Event
{
	const NAME = 'ccm19.plugin.ccm19imex.importdomain';

	/** @var Domain $domain */
	private $domain;

	/** @var array $data */
	private $data;

	/** @var bool $isNewDomain */
	private $isNewDomain;

	/** @var array<string,array> $idMappings */
	private $idMappings;

	public function __construct(Domain $domain, array $data, bool $isNewDomain, array $idMappings)
	{
		$this->domain = $domain;
		$this->data = $data;
		$this->isNewDomain = $isNewDomain;
		$this->idMappings = $idMappings;
	}

	/**
	 * @return Domain
	 */
	public function getDomain(): Domain
	{
		return $this->domain;
	}

	/**
	 * @return array
	 */
	public function getImportData(): array
	{
		return $this->data;
	}

	/**
	 * @return null|mixed
	 */
	public function get($key)
	{
		return $this->data[$key] ?? null;
	}

	public function isNewDomain(): bool
	{
		return $this->isNewDomain;
	}

	public function getIdMapping(string $kind): array
	{
		return $this->idMappings[$kind] ?? [];
	}

	public function getSystemThemeMap(): array
	{
		return $this->getIdMapping('systemTheme');
	}

	public function getUserThemeMap(): array
	{
		return $this->getIdMapping('userTheme');
	}

	public function getThemeMap(): array
	{
		return $this->getIdMapping('theme');
	}

	public function getUserEmbeddingMap(): array
	{
		return $this->getIdMapping('userEmbedding');
	}

	public function getEmbeddingMap(): array
	{
		return $this->getIdMapping('embedding');
	}
}
