<?php

namespace Plugins\Ccm19Payment\Controller;

use App\Component\Curl;
use App\Controller\UniversalController;
use App\Model\CsrfToken;
use App\Model\User;
use App\Utils;
use App\Model\PluginState;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Controller for payment
 * @Route("/plugins/payment/", name="app_ccm19payment_")
 */
class Payment extends UniversalController
{
	/**
	 * @Route("index", name="admin")
	 * @param PluginState $state
	 * @return Response
	 */
	public function index(PluginState $state): Response
	{
		//CK Instanz - nix anzeigen
		if(Utils::isClickskeksCloud())
		{
			return $this->redirectToRoute('app_domains_index');
		}

		$user = User::loggedInUser();
		$laufzeit = $user->getCreateTimeStamp();
		$maxCallCount = $user->getMaxCallCount();
		$maxWLCount = $user->getMaxWLCount();
		$actualCallCount = $user->getActualCallCount();
		$actualDomainCount = $user->getActualDomainCount();
		$dayOfMonth = (int)date("d");
		$isNotHBClient = 1;

		//wenn noch keine 30 Tage dann dayofMonth auf Anzahl der Tage Laufzeit...
		if ((time()-$laufzeit) < (86400*$dayOfMonth))
		{
			$dayOfMonth = round((time()-$laufzeit)/86400.,0)-1;
			if($dayOfMonth<1)
			{
				$dayOfMonth =1;
			}
		}

		//30% onTop um Spitzen abzudecken und auf Monat hochrechnen
		$actualCallCount = round(($actualCallCount * 1.3 / $dayOfMonth * 31),0);
		$bestTarif = $this->getBestTarif($actualCallCount,$actualDomainCount);
		$maxDomains = $user->getMaxDomainCount();

		//ExternalMeta Data
		$addData = $user->getExternalMetadata();	//Array
		if(!empty($addData['hbclient']))
		{
			$isNotHBClient = 0;
			if($bestTarif['besttarif']=="Free")
			{
				$bestTarif['besttarif'] = "Starter";
			}
		}

		//Render template with all Data
		return $this->render('@plugin:Ccm19Payment/internal/index.html.twig', [
			"plugin" =>"",
			'csrfToken' => CsrfToken::get(),
			'_metaSidebar' => true,
			'slidetariff' => $user->get("SlidingTarif"),
			'tarifBlock' => self::getTarifBlock($maxCallCount,$maxWLCount,$maxDomains),
			'bestTarif' => $bestTarif,
			'domanz' => $maxDomains,
			'actualCallCount' =>$actualCallCount,
			'actualEdition' => Utils::getEdition(),
			'isNotHBClient' => $isNotHBClient,
		]);
	}


	/**
	 * @Route("cancel", name="admin_cancel")
	 * @param PluginState $state
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function cancelSub(PluginState $state,Request $request,TranslatorInterface $translator): Response
	{
		// Agency-Edition: Buchung nur mit User-ID und geheimem API-Schlüssel möglich
		if (Utils::isExtendedEdition()) {
			$userId = User::loggedInUser()->getId();
			$tokenFetchParams = [
				'getSecret'=>1,
				'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'] ?? '',
				'userid' => $userId,
			];
		}
		// Andere Editionen (insb. Multidomain): Buchung mit Lizenzschlüssel möglich
		else {
			$userId = Utils::getLicenseKey();
			$tokenFetchParams = [
				'getSecret'=>1,
				'userid' => $userId,
			];
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api.php', $tokenFetchParams);

		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			$this->addFlash('danger', $translator->trans('No Curl Extension installed.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->get($url);
		$secret = $curl->getResponse();
		$curl->close();

		if (!$secret) {
			$this->addFlash('danger', $translator->trans('User does not exist - please contact Support.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}


		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api.php', [
				'kuendigen'=>1,
				'si'=>$secret,
				'uid' =>$userId,
			]);

		//307 redirect zur Buchungsmaske damit der Kunde direkt eingeloggt werden kann...
		return $this->redirect($url, 307);
	}


	/**
	 * @param int $actualCallCount
	 * @return array
	 */
	public function getBestTarif($actualCallCount=0,$actualDomainCount=0)
	{
		$bestTarif = "";
		//Die Tarife
		$tarif = array(
			"5000" => "Free",
			"20000" => "Starter",
			"100000" => "Business",
			"250000" => "Business",
			"1000000" => "Business",
			"2500000" => "Business",
			"5000000" => "Business",
		);

		$optiOption = array(
			"5000"=>1,
			"20000"=>1,
			"100000"=>5,
			"250000"=>10,
			"1000000"=>20,
			"2500000"=>50,
			"5000000"=>100,
		);

		//Finde besten Tarif
		foreach ($tarif as $count => $name)
		{
			if($actualCallCount < $count && empty($bestTarif) && $actualDomainCount <= $optiOption[$count])
			{
				$bestTarif = $name;
				$bestOPtion = $optiOption[$count];
			}
		}
		//Oha - Enterprise... dann E
		if(empty($bestTarif))
		{
			$bestTarif = "Enterprise";
			$bestOPtion = "";
		}

		$bestData = array("besttarif"=> $bestTarif, "bestoption"=>$bestOPtion);

		return $bestData;
	}

	/**
	 * @Route("slide_form", name="slide_form")
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function appPaymentSlideForm(Request $request, TranslatorInterface $translator): Response
	{
		$user = User::loggedInUser();
		$this->checkTarifYearly($translator);
		//Free Tarif? Dann nix machen
		if(self::getTarifBlock( $user->getMaxCallCount(),$user->getMaxWLCount(),$user->getMaxDomainCount())==="Free")
		{
			if($user->get("SlidingTarif")===true)
			{
				$user->set("SlidingTarif",false);
				$user->save();
			}
			else{
				$this->addFlash('danger', $translator->trans('Sliding Tariff is not possible in Free Tarif.'));
			}

			return $this->redirectToRoute('app_domains_index');
		}

		//Hier Check einbauen, ob Tarif auch monatlich läuft.
		if($this->checkTarifYearly($translator))
		{
			$user->set("SlidingTarif",false);
			$user->save();
			//oha - yearly
			$this->addFlash('danger', $translator->trans('Sliding Tariff is not possible in Yearly Tarifs.'));
			return $this->redirectToRoute('app_domains_index');
		};

		/**
		//überprüft ob als Pament Kreditkarte vorliegt, falls ja nix machen, kommt später
		if($this->checkTarifCC($translator))
		{
			$user->set("SlidingTarif",false);
			$user->save();
			//oha - Kreditkarte
			$this->addFlash('danger', $translator->trans('Sliding Tariff is not possible with Credit Card at the moment, please use Lastschrift or Paypal or Billing.'));
			return $this->redirectToRoute('app_domains_index');
		};
		*/

		//überprüft ob als Pament Kreditkarte vorliegt, falls ja nix machen, kommt später
		if($this->checkTarifPaypal($translator))
		{
			$user->set("SlidingTarif",false);
			$user->save();
			//oha - Kreditkarte
			$this->addFlash('danger', $translator->trans('Sliding Tariff is not possible with Paypal at the moment, please use Lastschrift or Credidcard or Billing.'));
			return $this->redirectToRoute('app_domains_index');
		};

		$user->set("SlidingTarif",(bool)$request->request->get('slidetariff'));
		$user->save();
		if((bool)$request->request->get('slidetariff'))
		{
			$this->addFlash('success', $translator->trans('Sliding Tariff ist active.'));
		}
		else{
			$this->addFlash('danger', $translator->trans('Sliding Tariff ist inactive.'));
		}

		return $this->redirectToRoute('app_domains_index');

	}

	/**
	 * @param TranslatorInterface $translator
	 * @return bool|RedirectResponse
	 */
	public function checkTarifYearly(TranslatorInterface $translator)
	{
		// Agency-Edition: Buchung nur mit User-ID und geheimem API-Schlüssel möglich
		if (Utils::isExtendedEdition()) {
			$userId = User::loggedInUser()->getId();
			$tokenFetchParams = [
				'getSecret'=>1,
				'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'] ?? '',
				'userid' => $userId,
				'getTarifYearly' => 1
			];
		}
		// Andere Editionen (insb. Multidomain): Buchung mit Lizenzschlüssel möglich
		else {
			$userId = Utils::getLicenseKey();
			$tokenFetchParams = [
				'getSecret'=>1,
				'userid' => $userId,
				'getTarifYearly' => 1
			];
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api2.php', $tokenFetchParams);

		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			$this->addFlash('danger', $translator->trans('No Curl Extension installed.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->get($url);
		$secret = (int)$curl->getResponse();
		$curl->close();
		return $secret === 1;
	}

	/**
	 * @param TranslatorInterface $translator
	 * @return bool|RedirectResponse
	 */
	public function checkTarifCC(TranslatorInterface $translator)
	{
		// Agency-Edition: Buchung nur mit User-ID und geheimem API-Schlüssel möglich
		if (Utils::isExtendedEdition()) {
			$userId = User::loggedInUser()->getId();
			$tokenFetchParams = [
				'getSecret'=>1,
				'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'] ?? '',
				'userid' => $userId,
				'getTarifCC' => 1
			];
		}
		// Andere Editionen (insb. Multidomain): Buchung mit Lizenzschlüssel möglich
		else {
			$userId = Utils::getLicenseKey();
			$tokenFetchParams = [
				'getSecret'=>1,
				'userid' => $userId,
				'getTarifCC' => 1
			];
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api2.php', $tokenFetchParams);

		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			$this->addFlash('danger', $translator->trans('No Curl Extension installed.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->get($url);
		$secret = (int)$curl->getResponse();
		$curl->close();

		return $secret === 1;
	}

	/**
	 * @param TranslatorInterface $translator
	 * @return bool|RedirectResponse
	 */
	public function checkTarifPaypal(TranslatorInterface $translator)
	{
		// Agency-Edition: Buchung nur mit User-ID und geheimem API-Schlüssel möglich
		if (Utils::isExtendedEdition()) {
			$userId = User::loggedInUser()->getId();
			$tokenFetchParams = [
				'getSecret'=>1,
				'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'] ?? '',
				'userid' => $userId,
				'getTarifPaypal' => 1
			];
		}
		// Andere Editionen (insb. Multidomain): Buchung mit Lizenzschlüssel möglich
		else {
			$userId = Utils::getLicenseKey();
			$tokenFetchParams = [
				'getSecret'=>1,
				'userid' => $userId,
				'getTarifPaypal' => 1
			];
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api2.php', $tokenFetchParams);

		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			$this->addFlash('danger', $translator->trans('No Curl Extension installed.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->get($url);
		$secret = (int)$curl->getResponse();
		$curl->close();

		return $secret === 1;
	}

	/**
	 * @Route("book", name="book")
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function goBookBookBook(Request $request, TranslatorInterface $translator): Response
	{
		if(!empty($hcCLientID=self::getHbClientID()))
		{
			//print_r($request);
			//Das ist erstmal provisorisch, da wir ja noch nicht wissen, wie die Zielurl lautet
			$params['tarif'] = $request->get("tarif");
			$params['anzdom'] = (int) $request->get("anzdom");
			$params['pay'] = 	$request->get("bpay");
			$params['clientid'] = $hcCLientID;
			if($params['tarif']==='starter')
			{
				$url = 'https://hb-marketplace.com/products/ccm19-starter-paket';
				if($params['pay']=="y")
				{
					$url = 'https://hb-marketplace.com/products/ccm19-starter-paket?variant=46880224018765';
				}
			}
			if($params['tarif']==='business')
			{
				$url = 'https://hb-marketplace.com/products/ccm19-business-paket';

				if($params['pay']==="y")
				{
					if($params['anzdom']===5)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241779021';
					}
					if($params['anzdom']===10)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241844557';
					}
					if($params['anzdom']===20)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241910093';
					}
					if($params['anzdom']===50)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241975629';
					}
					if($params['anzdom']===100)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880242041165';
					}
				}

				if($params['pay']==="m")
				{
					if($params['anzdom']===5)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241811789';
					}
					if($params['anzdom']===10)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241877325';
					}
					if($params['anzdom']===20)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880241942861';
					}
					if($params['anzdom']===50)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880242008397';
					}
					if($params['anzdom']===100)
					{
						$url = 'https://hb-marketplace.com/products/ccm19-business-paket?variant=46880242073933';
					}
				}
			}

			return $this->redirect($url, 307);
		}

		// Agency-Edition: Buchung nur mit User-ID und geheimem API-Schlüssel möglich
		if (Utils::isExtendedEdition()) {
			$userId = User::loggedInUser()->getId();
			$tokenFetchParams = [
				'getSecret'=>1,
				'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'] ?? '',
				'userid' => $userId,
			];
		}
		// Andere Editionen (insb. Multidomain): Buchung mit Lizenzschlüssel möglich
		else {
			$userId = Utils::getLicenseKey();
			$tokenFetchParams = [
				'getSecret'=>1,
				'userid' => $userId,
			];
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api.php', $tokenFetchParams);

		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			$this->addFlash('danger', $translator->trans('No Curl Extension installed.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->get($url);
		$secret = $curl->getResponse();
		$curl->close();

		if (!$secret) {
			$this->addFlash('danger', $translator->trans('User does not exist - please contact Support.'));
			return $this->redirectToRoute('app_ccm19payment_admin');
		}
		if ($request->get("bpay")) {
			$payment = $request->get("bpay");
		}
		else {
			$payment = $request->get("pay");
		}

		if ($request->get("anzdom")) {
			$anzdom = $request->get("anzdom");
		}
		else {
			$anzdom = '';
		}

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api.php', [
			'redirectToShop'=>1,
			'si'=>$secret,
			'uid' =>$userId,
			"selTarif" =>$request->get("tarif"),
			"payment" =>$payment,
			"download" =>Utils::getEdition(),
			"anzdom" =>$anzdom,
			"lang" => $request->getSession()->get("locale") ?? "de",
		]);

		//307 redirect zur Buchungsmaske damit der Kunde direkt eingeloggt werden kann...
		return $this->redirect($url, 307);
	}

	/**
	 * @return string
	 */
	public static function getHbClientID() :string
	{
		$user = User::loggedInUser();
		$addData = $user->getExternalMetadata();	//Array
		return $addData['hbclient'] ?? "";
	}

	/**
	 * @param $maxCallCount
	 * @param $maxWLCount
	 * @param $maxDomains
	 * @return string
	 */
	public static function getTarifBlock($maxCallCount,$maxWLCount,$maxDomains) : string
	{
		//DL Tarif unlimited...
		if($maxCallCount<1)
		{
			//$maxDomains
			$tarif = array(
				"0" => "Free",
				"2" => "Starter",
				"5" => "Business",
				"10" => "Business",
				"20" => "Business",
				"50" => "Business",
				"100" => "Business",
				//"25000000" => "XXXL"
			);
			if(array_key_exists(trim((string)$maxDomains),$tarif))
			{
				return $tarif[$maxDomains];
			}
			else{
				if ($maxDomains > 100 || !is_numeric($maxDomains))
				{
					return "Enterprise";
				}
				return "Special";
			}
		}
		else{

			$tarif = array(
				"5000" => "Free",
				"20000" => "Starter",
				"50000" => "Agency Cloud",
				"100000" => "Business",
				"250000" => "Business",
				"1000000" => "Business",
				"2500000" => "Business",
				"5000000" => "Business",
				//"25000000" => "XXXL"
			);

			//print_R($maxCallCount);
			if(array_key_exists(trim((string)$maxCallCount),$tarif))
			{
				return $tarif[$maxCallCount];
			}
			else{
				if ($maxCallCount > 5000000 || !is_numeric($maxCallCount))
				{
					return "Enterprise";
				}
				return "Special";
			}
		}
	}
}
?>
