<?php

namespace Plugins\Ccm19Payment\Controller;

use App\Component\HttpFoundation\CssResponse;
use App\Controller\ExternalController;
use App\Model\PluginState;
use App\Utils;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;


/**
 * Controller for payment - domain dependent
 * @Route("/payment/", name="app_ccm19payment_")
 */
class PaymentExternalCSS extends ExternalController
{
	/**
	 * @Route("payment.css", name="css")
	 * @return CssResponse
	 */
	public function index(PluginState $state): CssResponse
	{

		//check if is activated
		if (!$state->isActiveForCurrentDomain()) {
			//Nope - then go to dashboard...
			return new Response('CssFile not active.', 404);
		}

		$response = new CssResponse();

		// Workaround: HTTP-Header Cache-Control würde sonst auf private, must-revalidate etc. gesetzt
		Utils::resetCacheControl($response);

		// Lebensdauer der CSS-Datei auf ein Jahr setzen, da ein Zeitstempel an die CSS-Route angehängt wird
		$response->headers->remove('Cache-Control');
		$response->setCache([
			'max_age' => 31536000,
			'immutable' => true,
			'public' => true,
		]);

		//Render CSS template with all Data
		return $this->render('@plugin:Ccm19Payment/external/ExternalCss.css.twig', [], $response);
	}
}
?>
