<?php

namespace Plugins\Ccm19Payment\CronJob;

use App\Component\Curl;
use App\Model\SystemLog;
use App\Model\User;
use App\Component\Cron\Job;
use App\Utils;
use Symfony\Component\HttpFoundation\Request;

class SlideTarif extends Job
{
	//const MAX_COUNT = 10;
	const COMPONENT = 'Slide Tarif';

	public function __construct()
	{
		//alle 1h
		parent::__construct(60*60*1);
	}

	/**
	 * Regelmäßig den Gleittarif anpassen
	 */
	public function run(Request $request): void
	{
		$this->touchLastExecutionTimestamp();

		if (empty($_ENV['PLUGIN_PAYMENT_SLIDETARIF_APIKEY']) || Utils::isMultiDomainEdition() || Utils::isClickskeksCloud())  {
			return;
		}

		//$translator = Utils::getTranslator();
		$logger = SystemLog::current();
		$logger->logWarning(self::COMPONENT, "Gleittarif Check");

		$clients = User::byRole(User::ROLE_CLIENT);
		$client = null;
		$restDate = (strtotime('last day of this month'));
		$restDay = ((int)date("d",$restDate)) - ((int)date("d"));
		$curl = new Curl();


		foreach ($clients as $client) {
			$callCount 		= $client->getActualCallCount();
			$callMaxCount 	= $client->getMaxCallCount();
			$actualDomCount = $client->getActualDomainCount();

			// Wenn die Anzahl der Widgetausgabe nicht limitiert ist, sofort raus
			if ($callMaxCount == -1) {
				continue;
			}

			//Settings Call Limits...
			$callLimit = 95;
			$callLimitDowngrade = 25;

			//Match Call Limits from Settings
			$quota = $callMaxCount > 0
				? min((int)floor($callCount / $callMaxCount * 100), 100)
				: 100;

			//Das hier ist ein Array mit den DomCounts des jeweils niedrigeren Tarifes
			//So kann gecheckt werden ob ei Downgrade möglich ist
			$downGradeOptions = array(
				"250000"=>5,
				"1000000"=>10,
				"2500000"=>20,
				"5000000"=>50
			);

			//Wenn z.B. Tarif mit 250k besteht, muss der aktuelle Verbrauch unterhalb des Limits vom Tarif darunter liegen
			//dann also unter 100k - und zwar deutlich - eben die 25% Regel
			$downGradeOptionsTraffc = array(
				"250000"=>25000,
				"1000000"=>62500,
				"2500000"=>250000,
				"5000000"=>500000
			);

			//Wenn bei dem User Call Limit größer ist und es ist ein Pay Tarif
			if ($quota > $callLimit  && $callMaxCount > 5000) {
				//Und er nutzt Gleittarife... und Monat hat noch mind. 1 Tage
				if ($client->get("SlidingTarif") == true && $restDay >=1) {
					$logMessage = 'automatic Tarif Upgrade for user: '.$client->getId();
					$logger->logSuccess(self::COMPONENT, $logMessage);
					//API Call mit Userid to upgrade
					$curl->get(Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/update_api.php', [
						'apikey' => $_ENV['PLUGIN_PAYMENT_SLIDETARIF_APIKEY'],
						'do' => 'upgrade',
						'updateuser' => $client->getId()
					]));
				}
			}
			//Kunde nutzt Gleitarif und Restday max 1, d.h. Ende des Monats... UND mindestens Business Tarif
			elseif ($client->get("SlidingTarif") == true && $restDay<=1 && $callMaxCount >= 250000) {
				//DOmCounts checken
				//ok - also weniger als 25% - dann downgraden
				if(($downGradeOptions[$callMaxCount] > $actualDomCount) && $downGradeOptionsTraffc[$callMaxCount] > $callCount) {
					$logMessage = 'automatic Tarif Downgrade for user: '.$client->getId();
					$logger->logInfo(self::COMPONENT, $logMessage);
					//API Call mit Userid to Downgrade
					$curl->get(Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/update_api.php', [
						'apikey' => $_ENV['PLUGIN_PAYMENT_SLIDETARIF_APIKEY'],
						'do' => 'downgrade',
						'updateuser' => $client->getId()
					]));
				}
			}
		}
		// Wir sind fertig, wenn wir beim letzten Kunden angekommen waren
		$done = (array_pop($clients) === $client);


		// Wenn noch nicht fertig mit allen Kunden, dann gleich fortsetzen
		if (!$done) {
			$this->scheduleDeferred();
		}

		return;
	}
}
