<?php

namespace Plugins\Ccm19Payment\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use App\Component\Curl;
use App\Event\AccountUpdateEvent;
use App\Model\User;
use App\Utils;
use Plugins\Ccm19Payment\Controller\Payment;

class AccountUpdate implements EventSubscriberInterface
{
	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			AccountUpdateEvent::NAME_POST_SAVE => ['onAccountUpdated', 100],
		];
	}

	/**
	 * Daten in der Faktura aktualisieren
	 * @return void
	 */
	public function onAccountUpdated(AccountUpdateEvent $event)
	{
		if (!Utils::isExtendedEdition() or (!Utils::isCCMCloud() and !Utils::isClickskeksCloud()) or !isset($_ENV['PLUGIN_PAYMENT_APIKEY'])) {
			return;
		}
		$user = $event->getUser();
		$cause = $event->getCause();
		// Nicht auslösen, wenn es ein Update durch die API ist
		if ($cause === AccountUpdateEvent::CAUSE_API) {
			return;
		}
		// Nur bei Kundenaccounts auslösen
		if (!$user->isClient() or $user->isSubUser()) {
			return;
		}
		$tokenFetchParams = [
			'setClientData' => 1,
			'apiKeyGetSecret' => $_ENV['PLUGIN_PAYMENT_APIKEY'],
			'userid' => $user->getId(),
		];
		$postData = [
			'email' => $user->getEmailAddress(),
			'firstName' => $user->getFirstName(),
			'lastName' => $user->getLastName(),
			'company' => $user->getCompany(),
			'country' => $user->getCountry() ? $user->getCountry()->getCode() : '',
			'plz' => (string)$user->getPlz(),
			'city' => (string)$user->getCity(),
			'streetNr' => (string)$user->getStrNr(),
			'vatId' => (string)$user->getVatId(),
			'invoiceEmail' => $user->getInvoiceEmailAddress(),
			'dunningEmail' => $user->getDunningEmailAddress(),
			'username' => $user->getUsername(),
			'pwHash' => $user->getPasswordHash(),
		];

		//Daten zum Buchen bereitstellen
		$postDataSend = array("postDataClient" => json_encode($postData));

		$url = Utils::getCmpServiceUrl('/plugins/ccmtarifverwaltung/api.php', $tokenFetchParams);

		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 3);
		$curl->setOpt(CURLOPT_TIMEOUT, 5);
		$curl->post($url, $postDataSend);
		$curl->close();
	}
}
