<?php

namespace Plugins\Ccm19ScriptPlaceholder\EventSubscriber;

use App\Controller\AbstractController;
use App\Controller\ExternalController;
use App\Model\PluginState;
use App\Utils;
use ReflectionClass;
use ReflectionException;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Contracts\Translation\TranslatorInterface;

class TrialPeriod implements EventSubscriberInterface
{
	/** @var PluginState $pluginState */
	private $pluginState;

	/** @var TranslatorInterface $translator */
	private $translator;

	public function __construct(PluginState $pluginState, TranslatorInterface $translator)
	{
		$this->pluginState = $pluginState;
		$this->translator = $translator;
	}

	public static function getSubscribedEvents(): array
	{
		return [
			KernelEvents::CONTROLLER => ['onKernelController', 7000],
		];
	}

	/**
	 * @phan-suppress PhanTypeArraySuspicious
	 */
	public function onKernelController(ControllerEvent $event): void
	{
		if ($event->isMainRequest() == false) {
			return;
		}
		// There is nothing to do if the trial period is neither running nor expired
		elseif ($this->pluginState->trialIsRunningOrExpired() == false) {
			return;
		}

		/** @var AbstractController $controller */
		$controller = $event->getController()[0];
		$request = $event->getRequest();

		try {
			$class = new ReflectionClass($controller);
		} catch (ReflectionException $_) {
			return;
		}

		// Disregard controllers that aren't part of this plugin
		if (strpos($class->getNamespaceName(), $this->pluginState->getNamespace()) !== 0) {
			return;
		}

		// Ignore frontend resources
		if ($controller instanceof ExternalController) {
			return;
		}

		// Inject "trial period expired" message into backend resources
		if ($request->getMethod() == 'GET') {
			if ($this->pluginState->trialIsExpired()) {
				Utils::addFlash('danger', $this->translator->trans('Your trial period has expired. If you want to continue using this plugin, consider upgrading your plan.'));
			}
			elseif ($this->pluginState->trialIsRunning()) {
				Utils::addFlash('info', $this->translator->trans(
					'You are currently using this plugin in a free trial period. If you want to continue using it after the trial period ends, consider upgrading your plan. Your trial period ends in :x days.',
					[':x'=>$this->pluginState->trialDaysRemaining()]
				));
			}
		}
	}
}
