<?php

namespace Plugins\Ccm19ScriptPlaceholder\Model;

use App\Model\LocaleBase;
use App\Model\Model;

abstract class SnippetBase extends Model
{
	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'plugin-ccm19ScriptPlaceholder-snippets';

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @param string $name
	 * @return $this
	 */
	public function setName(string $name): self
	{
		$this->set('name', trim(preg_replace('~[^a-z0-9_-]~i', '', $name)));
		return $this;
	}

	/**
	 * @return string[]
	 */
	public function getTranslations(): array
	{
		return $this->get('translations', []);
	}

	/**
	 * @param LocaleBase $locale
	 * @return string
	 */
	public function getTranslation(LocaleBase $locale): string
	{
		return $this->getTranslations()[$locale->getName()] ?? '';
	}

	/**
	 * @param LocaleBase $locale
	 * @param string $translation
	 * @return $this
	 */
	public function setTranslation(LocaleBase $locale, string $translation): self
	{
		$translations = $this->getTranslations();
		$translations[$locale->getName()] = $translation;
		$this->set('translations', $translations);
		return $this;
	}

	/**
	 * @param string[] $translations
	 * @return $this
	 */
	public function setTranslations(array $translations): self
	{
		$this->set('translations', $translations);
		return $this;
	}
}
