<?php

namespace Plugins\Ccm19ScriptPlaceholder\Model;

use App\Component\Config\RepositoryInterface;
use App\Model\ImmutableModel;

class Template extends ImmutableModel
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'templates.json';

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @return string
	 */
	public function getPlacement(): string
	{
		return (string)$this->get('placement');
	}

	/**
	 * @return string
	 */
	public function getPlacementCssSelector(): string
	{
		return (string)$this->get('placementCssSelector');
	}

	/**
	 * @return string
	 */
	public function getPlacementScriptNeedle(): string
	{
		return (string)$this->get('placementScriptNeedle');
	}

	/**
	 * @return string
	 */
	public function getHtmlCode(): string
	{
		return (string)$this->get('htmlCode');
	}

	/**
	 * @return string
	 */
	public function getCssCode(): string
	{
		return (string)$this->get('cssCode');
	}

	/**
	 * @return string
	 */
	public function getJsCode(): string
	{
		return (string)$this->get('jsCode');
	}

	/**
	 * @return array[]
	 */
	public function getSnippets(): array
	{
		$snippets = $this->get('snippets');
		return is_array($snippets) ? array_values($snippets) : [];
	}

	/**
	 * @return array
	 */
	public function getTimeConstraints(): array
	{
		$constraints = $this->get('timeConstraints');
		return is_array($constraints) ? array_values($constraints) : [];
	}

	/**
	 * @return string
	 */
	public function getCssCodeWithPlaceholderScope(): string
	{
		$cssCode = $this->getCssCode();

		$hideout = [];

		// Mask css comments to facilitate further changes
		$cssCode = preg_replace_callback(
			'~/\*((?:[^*]*)\*(?(?=/)/|(?1)))~',
			function ($match) use (&$hideout) {
				$needle = '@comment--'.uniqid().';';
				$hideout[$needle] = $match[0];
				return $needle;
			},
			$cssCode
		);

		// Mask keyframes to prevent inner rules from being modified
		$cssCode = preg_replace_callback(
			'~@keyframes\s[^{]+(?<rule>(?>[^{}]*+)(?(?={){(?&rule)*}))~i',
			function ($match) use (&$hideout) {
				$needle = '@at-rule--'.uniqid().';';
				$hideout[$needle] = $match[0];
				return $needle;
			},
			$cssCode
		);

		// Modify every css selector to achieve a scope limited to the current placeholder
		$scope = '.ccm-plugin--script-placeholder[data-placeholder="'.addcslashes($this->getId(), '"').'"]';
		$cssCode = preg_replace_callback(
			'~(?<pre>(?<mq>@media\s[^{]++)?(?(mq){|(?:^|\*/|}))\s*+)(?<sel>(?(R&sel),|)\s*+(?!\*/)[a-z.#+\~:*[][^,{}]*+(?(?=})!|)(?&sel)*)~i',
			function ($match) use ($scope) {
				return $match['pre'].$scope.' '.str_replace(',', ", $scope ", $match['sel']);
			},
			$cssCode
		);

		// Restore masked code to the previous state
		return str_replace(array_reverse(array_keys($hideout)), array_reverse($hideout), $cssCode);
	}
}
