<?php

namespace Plugins\Ccm19ScriptPlaceholder\Model;

use App\Component\Config\RepositoryInterface;

class UserPlaceholder extends PlaceholderBase
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var bool $userSpecificRepository */
	protected static $userSpecificRepository = true;

	/** @var bool $accountSingleton */
	protected static $accountSingleton = true;

	/** @var UserSnippet[] $snippets */
	private $snippets;

	/**
	 * @return UserSnippet[]
	 */
	public function getSnippets(): array
	{
		if (empty($this->snippets)) {
			$this->snippets = array_values(array_filter(array_map(function ($id) {
				return UserSnippet::find($id);
			}, $this->get('snippets', []))));
		}

		return $this->snippets;
	}

	/**
	 * @param UserSnippet[] $snippets
	 * @return $this
	 */
	public function setSnippets(array $snippets): self
	{
		// Remove duplicates
		$snippets = array_values(array_reduce($snippets, function (array $snippets, UserSnippet $snippet) {
			$snippets[$snippet->getId()] = $snippet;
			return $snippets;
		}, []));

		$ids = array_map(function (UserSnippet $snippet) {
			return $snippet->getId();
		}, $this->snippets = $snippets);

		$this->set('snippets', $ids);
		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getPublishedPlacementCssSelector(): ?string
	{
		return $this->get('publishedPlacementCssSelector');
	}

	/**
	 * @param string|null $selector
	 * @return $this
	 */
	public function setPublishedPlacementCssSelector(?string $selector): self
	{
		$this->set('publishedPlacementCssSelector', $selector);
		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getPublishedPlacementScriptNeedle(): ?string
	{
		return $this->get('publishedPlacementScriptNeedle');
	}

	/**
	 * @param string|null $needle
	 * @return $this
	 */
	public function setPublishedPlacementScriptNeedle(?string $needle): self
	{
		$this->set('publishedPlacementScriptNeedle', $needle);
		return $this;
	}

	/**
	 * @return array|null
	 */
	public function getPublishedTimeConstraints(): ?array
	{
		// Use a fallback if published because this "published" property was added after the others
		return $this->get('publishedTimeConstraints') ?? ($this->isPublished() ? $this->getTimeConstraints() : null);
	}

	/**
	 * @param array|null $constraints
	 * @return $this
	 */
	public function setPublishedTimeConstraints(?array $constraints): self
	{
		$this->set('publishedTimeConstraints', $constraints);
		return $this;
	}

	/**
	 * @return bool
	 */
	public function isPublished(): bool
	{
		return $this->getPublishedPlacementCssSelector() !== null;
	}

	/**
	 * @param string $name
	 * @return self|null
	 * @phan-side-effect-free
	 */
	public static function byName(string $name): ?self
	{
		$name = mb_strtolower($name);
		return array_reduce(self::all(), function ($carry, $userPlaceholder) use ($name) {
			return $carry ?? (mb_strtolower($userPlaceholder->getName()) == $name ? $userPlaceholder : $carry);
		}, null);
	}

	/**
	 * Übernimmt Eigenschaften des ursprünglichen Objekts aus dem Repository.
	 * Fordert exklusiven Lock an und gibt abschließend wieder frei.
	 */
	public function __clone()
	{
		// Generiere neue ID und stelle ursprünglichen Platzhalter in $this->source bereit
		parent::__clone();

		// Generiere aufsteigende Nummerierung für neuen Namen
		for ($i = 2; ; $i++) {
			if (!self::byName($newUserPlaceholderName = "{$this->source->getName()} #$i")) {
				break;
			}
		}

		// Klone Snippets für das neue Platzhalter-Objekt
		$newSnippets = array_map(static function (UserSnippet $snippet) {
			return UserSnippet::create()
				->setName($snippet->getName())
				->setTranslations($snippet->getTranslations());
		}, $this->source->getSnippets());
		UserSnippet::flush();

		// Neue Ausangswerte vorgeben und Platzhalter speichern
		$this
			->setName($newUserPlaceholderName)
			->setSnippets($newSnippets)
			->save();

	}
}
