<?php

namespace Plugins\Ccm19SingleSignOn\Controller;

use App\Component\LoggerInterface;
use App\Controller\UniversalController;
use App\Controller\Login as LoginController;
use App\Exception\CsrfTokenException;
use App\Model\CsrfToken;
use App\Model\PluginState;
use App\Model\User;
use App\Utils;
use Plugins\Ccm19SingleSignOn\Component\OpenId;
use Plugins\Ccm19SingleSignOn\Model\Provider;
use Plugins\Ccm19SingleSignOn\Model\AuthLink;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;

/**
 * Controller for TOTP 2FA Backend
 * @Route("/plugins/sso/", name="plugin_ccm19_singlesignon_")
 */
class Backend extends UniversalController
{
	/**
	 * @Route("", name="index", methods={"HEAD", "GET"})
	 * @return Response
	 */
	public function indexRedirect(): Response
	{
		return $this->redirectToRoute('app_account');
	}

	/**
	 * @Route("", name="action", methods={"POST"})
	 * @return Response
	 */
	public function actionDispatch(Request $request, PluginState $state, OpenId $openId): Response
	{
		$user = User::loggedInUser();

		try {
			CsrfToken::get()->checkRequest($request);
		}
		catch (CsrfTokenException $exception) {
			$this->addFlash('danger', 'CSRF token is missing or invalid. Please try again.');
			return $this->redirectToRoute('app_account');
		}

		// Verbindungen trennen
		if ($request->request->has('disconnect')) {
			$success = false;
			$disconnects = (array)$request->request->get('disconnect');
			foreach ($disconnects as $providerId => $subjectId) {
				$provider = Provider::find($providerId);
				if ($provider) {
					$authLink = AuthLink::bySubject($provider, $subjectId);
					if ($authLink->getUserId() === $user->getId()) {
						$openId->setProvider($provider);
						$openId->revokeToken($authLink->getToken());
						AuthLink::delete($authLink->getId());
						$success = true;
					}
				}
			}
			if ($success) {
				$this->addFlash('success', 'Account connection was successfully removed.');
			} else {
				$this->addFlash('danger', 'No account connection could be removed.');
			}
		}

		// Verbindung herstellen
		if ($request->request->has('connect')) {
			$providerId = (string)$request->request->get('connect');
			$provider = Provider::find($providerId);
			$extraData = Utils::encodeReturnTo('app_account').'..'.$user->getId();

			$openId->reset();
			$openId->setProvider($provider);
			$redirectUrl = $this->generateUrl('app_login_plugin_ccm19_singlesignon_continue', [], UrlGeneratorInterface::ABSOLUTE_URL);
			$redirect = $openId->authenticate($request, $redirectUrl, $extraData, 'login', $user->getEmailAddress());
			if ($redirect) {
				return $redirect;
			} else {
				$this->addFlash('danger', 'Account connection failed.');
			}
		}

		return $this->redirectToRoute('app_account');
	}
}
