<?php

namespace Plugins\Ccm19SingleSignOn\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use App\Event\TemplateRenderEvent;
use App\Event\TemplateResolveEvent;
use App\Model\PluginState;
use App\Model\User;
use App\Utils;
use Plugins\Ccm19SingleSignOn\Model\Provider;
use Plugins\Ccm19SingleSignOn\Model\AuthLink;
use Symfony\Component\HttpFoundation\RequestStack;

class BackendInjection implements EventSubscriberInterface
{
	private $pluginState;
	private $requests;

	public function __construct(PluginState $state, RequestStack $requests)
	{
		$this->pluginState = $state;
		$this->requests = $requests;
	}

	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			TemplateRenderEvent::nameForView('account/index.html.twig') => ['onRenderAccount', 900],
			TemplateRenderEvent::nameForView('_components/relogin_popup.html.twig') => ['onRenderRelogin', 900],
			TemplateResolveEvent::nameForView('subuser/_general_form.html.twig') => ['onTemplateSubuser', 900],
			TemplateResolveEvent::nameForView('hosting/client/edit.html.twig') => ['onTemplateClientEdit', 900],
			TemplateResolveEvent::nameForView('hosting/client/new.html.twig') => ['onTemplateClientNew', 900],
			TemplateResolveEvent::nameForView('hosting/manager/edit.html.twig') => ['onTemplateManagerEdit', 900],
			TemplateResolveEvent::nameForView('hosting/manager/new.html.twig') => ['onTemplateManagerNew', 900],
		];
	}

	/**
	 * @return void
	 */
	public function onRenderAccount(TemplateRenderEvent $event)
	{
		$user = User::loggedInUser();
		if (!$user) {
			return;
		}
		$event->set('ssoProviders', Provider::allActive());
		$event->set('ssoAuthLinks', AuthLink::byUser($user));

		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/account_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onRenderRelogin(TemplateRenderEvent $event)
	{
		$user = User::loggedInUser();
		$origUser = User::originalLoggedInUser();
		$loginUser = $origUser ?? $user;

		if (!$user or !$this->pluginState->isAllowedForUser($user)) {
			return;
		}

		// Ermitteln, ob diese Session mit einem SSO-Anbieter angemeldet wurde
		$request = $this->requests->getCurrentRequest();
		$session = $request->getSession();
		$loginViaId = $session->get('loginViaSSO');
		$loginVia = null;
		if ($loginViaId !== null) {
			$loginVia = AuthLink::find($loginViaId);
			if ($loginVia) {
				$provider = $loginVia->getProvider();
				if (!$loginVia->isActive() or $loginVia->getUserId() !== $loginUser->getId() or !$provider or !$provider->isActive()) {
					$loginVia = null;
				}
			}
		}

		$authLinks = AuthLink::byUser($loginUser);
		if ($origUser) {
			$authLinks = array_merge($authLinks, AuthLink::byUser($user));
		}

		$result = [];
		foreach ($authLinks as $link) {
			if (!isset($result[$link->getProviderId()]) and $link->getId() !== $loginViaId) {
				$provider = $link->getProvider();
				if ($link->isActive() and $provider and $provider->isActive()) {
					$result[$link->getProviderId()] = $link;
				}
			}
		}

		$event->set('ssoCurrentLink', $loginVia);
		$event->set('ssoLinks', $result);

		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/relogin_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onTemplateSubuser(TemplateResolveEvent $event)
	{
		$user = User::loggedInUser();
		if (!$user or !$this->pluginState->isAllowedForUser($user)) {
			return;
		}

		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/subuser/form_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onTemplateClientEdit(TemplateResolveEvent $event)
	{
		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/client/edit_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onTemplateClientNew(TemplateResolveEvent $event)
	{
		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/client/new_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onTemplateManagerEdit(TemplateResolveEvent $event)
	{
		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/manager/edit_extension.html.twig');
	}

	/**
	 * @return void
	 */
	public function onTemplateManagerNew(TemplateResolveEvent $event)
	{
		$event->extendTemplate('@plugin:Ccm19SingleSignOn/internal/manager/new_extension.html.twig');
	}
}
