<?php

namespace Plugins\Ccm19SingleSignOn\Model;

use App\Component\Config\RepositoryInterface;
use App\Component\Color;
use App\Utils;
use App\Model\Model;
use DateTimeImmutable;
use UnexpectedValueException;

class Provider extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'plugin-ccm19-sso-providers';

	/**
	 * @return self[]
	 */
	public static function allActive(): array
	{
		return array_filter(self::all(), function ($provider) {
			/** @var Provider $provider */
			return $provider->isActive();
		});
	}

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setName(string $value): self
	{
		$this->set('name', $value);
		return $this;
	}

	/**
	 * @return bool
	 */
	public function isActive(): bool
	{
		return (bool)$this->get('active');
	}

	/**
	 * @param bool $value
	 * @return $this
	 */
	public function setActive(bool $value): self
	{
		$this->set('active', $value);
		return $this;
	}

	/**
	 * @return string
	 */
	public function getClientId(): string
	{
		return (string)$this->get('clientId');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setClientId(string $value): self
	{
		$this->set('clientId', $value);
		return $this;
	}

	/**
	 * @return string
	 */
	public function getClientSecret(): string
	{
		return (string)$this->get('clientSecret');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setClientSecret(string $value): self
	{
		$this->set('clientSecret', $value);
		return $this;
	}

	/*
	 * @return string
	 */
	public function getAuthorityUrl(): string
	{
		return (string)$this->get('authority');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setAuthorityUrl(string $value): self
	{
		$this->set('authority', $value);
		return $this;
	}

	/**
	 * @return string
	 */
	public function getAuthEndpoint(): string
	{
		return (string)$this->get('authEndpoint');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setAuthEndpoint(string $value): self
	{
		$this->set('authEndpoint', $value);
		return $this;
	}

	/**
	 * @return string
	 */
	public function getTokenEndpoint(): string
	{
		return (string)$this->get('tokenEndpoint');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setTokenEndpoint(string $value): self
	{
		$this->set('tokenEndpoint', $value);
		return $this;
	}

	/*
	 * @return string
	 */
	public function getProfileEndpoint(): string
	{
		return (string)$this->get('profileEndpoint');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setProfileEndpoint(string $value): self
	{
		$this->set('profileEndpoint', $value);
		return $this;
	}

	/*
	 * @return string
	 */
	public function getEndSessionEndpoint(): string
	{
		return (string)$this->get('endSessionEndpoint');
	}

	/**
	 * @param string $value
	 * @return $this
	 */
	public function setEndSessionEndpoint(string $value): self
	{
		$this->set('endSessionEndpoint', $value);
		return $this;
	}

	/*
	 * @return string[]
	 */
	public function getScopes(): array
	{
		return (array)$this->get('scopes');
	}

	/**
	 * @param string[] $value
	 * @return $this
	 */
	public function setScopes(array $values): self
	{
		$this->set('scopes', array_values($values));
		return $this;
	}

	/*
	 * @return Color
	 */
	public function getButtonTextColor(): Color
	{
		try {
			return Color::fromCssString((string)$this->get('btnTextColor'));
		} catch (UnexpectedValueException $e) {
			return Color::fromRGB(0x243140);
		}
	}

	/**
	 * @param Color $value
	 * @return $this
	 */
	public function setButtonTextColor(Color $value): self
	{
		$this->set('btnTextColor', (string)$value);
		return $this;
	}

	/*
	 * @return Color
	 */
	public function getButtonBackgroundColor(): Color
	{
		try {
			return Color::fromCssString((string)$this->get('btnBgColor'));
		} catch (UnexpectedValueException $e) {
			return Color::fromRGB(0xDBDBDB);
		}
	}

	/**
	 * @param Color $value
	 * @return $this
	 */
	public function setButtonBackgroundColor(Color $value): self
	{
		$this->set('btnBgColor', (string)$value);
		return $this;
	}

	/*
	 * @return Color
	 */
	public function getIcon(): string
	{
		return (string)$this->get('icon');
	}

	/**
	 * @param Color $value
	 * @return $this
	 */
	public function setIcon(string $value): self
	{
		$this->set('icon', $value);
		return $this;
	}
}
