<?php

namespace Plugins\Ccm19Statistics\EventSubscriber;

use App\Component\Menu\Model\MenuItem;
use App\Controller\Settings\Translate;
use App\Event\FilterEvent;
use App\Model\PluginState;
use App\Model\User;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class DashboardChart implements EventSubscriberInterface
{
	private $pluginState;

	public function __construct(PluginState $state) {
		$this->pluginState = $state;
	}

	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			'ccm19.dashboard.chart.results' => ['modifyDashboardChart', 10],
		];
	}

	/**
	 * Bearbeitet den ViewCount im Dashboard-Diagramm
	 *
	 * TODO: ist das noch nötig?
	 *
	 * @param FilterEvent<array> $event
	 * @return void
	 */
	public function modifyDashboardChart(FilterEvent $event)
	{
		$user = User::loggedInUser();
		if ($user and $this->pluginState->isActiveForCurrentDomain() and $user->hasAccountFeature('canUseAnalytics')) {
			$statisticData = $event->getReturn();
			$start = $statisticData['startDate']->format('Y-m-d');
			$end = $statisticData['endDate']->format('Y-m-d');

			$allP = StatisticsData::all();
			$viewCountData = [];
			if(!empty($allP))
			{
				foreach($allP as $k=>$v)
				{
					//Nur aktueller Monat
					if ($v->getId() < $start || $v->getId() > $end) {
						continue;
					}
					$data = $v->get("details");
					$viewCountData[$v->getId()] = $data['opened']['all'] ?? 0;
				}
			}

			$count = 0;
			foreach (Statistic::inDateRange($statisticData['startDate'], $statisticData['endDate']) as $date => $stat) {
				$count += $viewCountData[$date] ?? 0;
			}

			$statisticData['viewCountTotal'] = $count;
			$event->setReturn($statisticData);
		}
	}

}
