<?php

namespace Plugins\Ccm19Statistics\EventSubscriber;

use App\Event\ProtocolRotateEvent;
use App\Model\Domain;
use App\Model\PluginState;
use App\Model\User;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use App\Model\StatisticsData;

class ProtocolRotate implements EventSubscriberInterface
{
	private $pluginState;

	public function __construct(PluginState $state) {
		$this->pluginState = $state;
	}

	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			ProtocolRotateEvent::NAME => ['onProtocolRotate', 20],
		];
	}

	/**
	 * Sichert die Statistik bevor das Protokoll rotiert wird
	 *
	 * @param ProtocolRotateEvent $event
	 * @return void
	 */
	public function onProtocolRotate(ProtocolRotateEvent $event)
	{
		$user = User::loggedInUser();
		$domain = Domain::activeDomain();
		if ($user and $domain and $this->pluginState->isActiveForCurrentDomain() and $user->hasAccountFeature('canUseAnalytics')) {
			StatisticsData::createFullStats(false);
			$today = StatisticsData::findToday();
			if ($today) {
				$today->archiveCounts();
			}
			StatisticsData::flush();
		}
	}

}
