<?php

namespace Plugins\Ccm19Statistics\Model;

use App\Component\Config\RepositoryInterface;
use App\Model\Domain;
use App\Component\Curl;
use App\Component\Tcf\v2\Model\Vendor as TcfVendor;
use App\Component\Tcf\v2\Model\Purpose as TcfPurpose;
use App\Model\Plugins as PluginModel;
use App\Model\Protocol;
use App\Model\Statistic;
use App\Utils;
use App\Model\Model;
use Symfony\Contracts\Translation\TranslatorInterface;

class StatisticsModell extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-statistics';

	/** @var bool $domainSpecificRepository */
	protected static $domainSpecificRepository = true;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
		$this->assets = [];
	}

	/**
	 * @param $name
	 * @param $arguments
	 * @return string
	 */
	public function __call($name, $arguments)
	{
		return (string)$this->get($name,"");
	}

	/**
	 * Generic String Catcher...
	 *
	 * @param string $varname
	 * @return string
	 */
	public function getStringVar(string $varname = "")
	{
		if(!empty($varname))
		{
			return (string)$this->get($varname,"");
		}

	}

	/**
	 * @return array
	 */
	public static function getBarCharts(TranslatorInterface $translator)
	{
		$barCharts = array();
		$barCharts['0']['name'] = "all";
		$barCharts['1']['name'] = "mobile";
		$barCharts['2']['name'] = "desktop";
		$barCharts['3']['name'] = "iphone";
		$barCharts['4']['name'] = "android";
		$barCharts['5']['name'] = "Windows";
		$barCharts['6']['name'] = "Mac";
		$barCharts['7']['name'] = "Linux";

		$barCharts['0']['description'] = $translator->trans('All consent data.');
		$barCharts['1']['description'] = $translator->trans('Only mobile consent data.');
		$barCharts['2']['description'] = $translator->trans('Only desktop consent data.');
		$barCharts['3']['description'] = $translator->trans('Only consent data from iphone.');
		$barCharts['4']['description'] = $translator->trans('Only consent data from android phones.');
		$barCharts['5']['description'] = $translator->trans('Only consent data from Windows.');
		$barCharts['6']['description'] = $translator->trans('Only consent data from Mac.');
		$barCharts['7']['description'] = $translator->trans('Only consent data from Linux.');
		return $barCharts;
	}

	/**
	 * @return string[]
	 */
	public static function getButtons()
	{
		return array("decline" => "Button abgelehnt", "acceptAll" => "Button alles akzeptiert", "opened" => "Absprung");
	}

	/**
	 * @return string[]
	 */
	public function getFieldNames(): array
	{
		return array(
			"exampleInputName",
			"expampleCheckboxData",
			"exampleSelectBox"
		);

	}

	/**
	 * @param array $postData
	 * @return true
	 */
	public function setArray(array $postData): self
	{
		foreach($postData as $k=>$v)
		{
			$this->set($k, $v);
		}
		return $this;
	}

}
