(function () {
	"use strict";

	function bytesToBase64(bytes) {
		bytes = new Uint8Array(bytes);
		return btoa(Array.from(bytes, function (byte) { return String.fromCodePoint(byte); }).join(""));
	}
	function bytesFromBase64(base64) {
		const binString = atob(base64.replaceAll('-', '+').replaceAll('_', '/'));
		return Uint8Array.from(binString, function (m) { return m.codePointAt(0); });
	}
	function showInProgress() {
		Array.prototype.forEach.call(document.getElementsByClassName("login-button"), function (el) {
			el.disabled = true;
		});
	}
	function showDone() {
		Array.prototype.forEach.call(document.getElementsByClassName("login-button"), function (el) {
			el.disabled = false;
		});
	}

	async function doPasskeyCheck(config, credentials) {
		var csrf_token = $('meta[name="X-CSRF-Token"]').prop('content');

		// Challenge erzeugen
		var challenge = await fetch(config.loginChallengeUrl, {
			cache: 'no-store',
			credentials: 'include',
			headers: {"X-CSRF-Token": csrf_token, 'Accept': 'application/octet-stream'}
		});
		if (!challenge.ok) {
			var e = new Error("could not generate challenge", { cause: challenge });
			e.name = "ChallengeError";
			throw e;
		}

		// WebAuthn-API anfragen
		var credential = await navigator.credentials.get({
			publicKey: {
				challenge: await challenge.arrayBuffer(),
				rpId: config.rpId,
				allowCredentials: credentials,
			}
		});

		return {
			type: credential.type,
			id: credential.id,
			clientData: bytesToBase64(credential.response.clientDataJSON),
			authenticatorData: bytesToBase64(credential.response.authenticatorData),
			signature: bytesToBase64(credential.response.signature),
		};
	}

	async function passkeyLogin() {
		showInProgress();
		var messageContainer = document.getElementById('passkey-error-message');
		messageContainer.innerText = '';
		messageContainer.classList.add('hidden');

		Array.prototype.forEach.call(document.getElementsByClassName("alert-danger"), function (el) {
			if (el != messageContainer) {
				el.parentNode.removeChild(el);
			}
		});

		try {
			var config = JSON.parse(document.getElementById('passkey-config').innerHTML);
			var csrf_token = $('meta[name="X-CSRF-Token"]').prop('content');

			if (!('registeredCredentials' in config) || this.tagName.toLowerCase() !== 'form') {
				return;
			}

			var credentials = config.registeredCredentials.map(function (entry) {
				var result = {
					type: entry.type,
					id: bytesFromBase64(entry.id),
				};
				if (entry.transports) {
					result.transports = entry.transports;
				}
				return result;
			});

			var data = await doPasskeyCheck(config, credentials);
			document.getElementsByName('passkeyId')[0].value = data.id;
			document.getElementsByName('authenticatorData')[0].value = data.authenticatorData;
			document.getElementsByName('clientData')[0].value = data.clientData;
			document.getElementsByName('signature')[0].value = data.signature;
			setTimeout(showDone, 5000);
			this.submit();
		} catch (e) {
			showDone();
			if ('name' in e && config.messages.login[e.name] !== undefined) {
				if (e.name == 'NotAllowedError') {
					return;
				}
				messageContainer.innerText = config.messages.login[e.name];
			} else {
				messageContainer.innerText = config.messages.login.error.replace("%ERROR%", e);
			}
			messageContainer.classList.remove("hidden");
		}
	}

	Array.prototype.forEach.call(document.getElementsByClassName("passkey-form"), function (el) {
		el.addEventListener('submit', async function (evt) {
			evt.preventDefault();
			await passkeyLogin.call(this);
		});
	});

	Array.prototype.some.call(document.getElementsByClassName("passkey-immediate-form"), function (el) {
		setTimeout(passkeyLogin.bind(el), 50);
		return true;
	});
})();
