<?php

namespace Plugins\Ccm19TwoFactorAuth\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use App\Event\FilterEvent;
use App\Event\TemplateRenderEvent;
use App\Event\TemplateResolveEvent;
use App\Model\PluginState;
use App\Model\User;
use App\Utils;

class BackendContent implements EventSubscriberInterface
{
	/** @var string $pluginId */
	private $pluginId;
	/** @var PluginState $state */
	private $state;

	public function __construct(PluginState $state)
	{
		$this->pluginId = $state->getPluginUID();
		$this->state = $state;
	}

	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			TemplateRenderEvent::nameForView('account/index.html.twig') => ['onRenderAccount', 100],
			TemplateRenderEvent::nameForView('hosting/client/edit.html.twig') => ['onRenderClientEdit', 100],
			TemplateRenderEvent::nameForView('hosting/manager/edit.html.twig') => ['onRenderManagerEdit', 100],
			TemplateRenderEvent::nameForView('subuser/edit_user.html.twig') => ['onRenderSubuserEdit', 100],
			TemplateResolveEvent::nameForView('subuser/_general_form.html.twig') => ['onResolveSubuserForm', 100],
			Utils::EVENT_FILTER_USER_RIGHTS => ["onAccessCheck", 100],
		];
	}

	/**
	 * @return void
	 */
	public function onRenderAccount(TemplateRenderEvent $event)
	{
		$user = User::loggedInUser();
		$active = ($user and $user->hasTwoFactorAuthParameters());

		$event->set('any_2fa_active', $active);
		$event->set('plugin_2fa_on_tariff_upgrade', !$this->state->isFullAccessForAllAccounts());

		$event->extendTemplate('@plugin:Ccm19TwoFactorAuth/internal/_account.html.twig');
	}

	/**
	 * @return void
	 */
	public function onRenderClientEdit(TemplateRenderEvent $event)
	{
		$user = $event->get('client');
		$active = ($user and $user->hasTwoFactorAuthParameters());

		$event->set('any_2fa_active', $active);

		$event->extendTemplate('@plugin:Ccm19TwoFactorAuth/internal/_client_edit.html.twig');
	}

	/**
	 * @return void
	 */
	public function onRenderManagerEdit(TemplateRenderEvent $event)
	{
		$user = $event->get('manager');
		$active = ($user and $user->hasTwoFactorAuthParameters());

		$event->set('any_2fa_active', $active);

		$event->extendTemplate('@plugin:Ccm19TwoFactorAuth/internal/_manager_edit.html.twig');
	}

	/**
	 * @return void
	 */
	public function onRenderSubuserEdit(TemplateRenderEvent $event)
	{
		$user = $event->get('subuser');
		$active = ($user and is_object($user) and $user->hasTwoFactorAuthParameters());
		$event->set('any_2fa_active', $active);
	}

	/**
	 * @return void
	 */
	public function onResolveSubuserForm(TemplateResolveEvent $event)
	{
		$event->extendTemplate('@plugin:Ccm19TwoFactorAuth/internal/_subuser_form.html.twig');
	}

	/**
	 * @return void
	 */
	public function onAccessCheck(FilterEvent $event)
	{
		if (!$this->state->isFullAccessForAllAccounts()) {
			$data = $event->getReturn();
			// Menüpunkt auch für User im falschen Tarif immer anzeigen
			if (($data['rights']['plugin_ccm19_twofactorauth_'] ?? false) === false) {
				$data['rights']['plugin_ccm19_twofactorauth_index'] = ['read'=>true, 'write'=>false];
			}
			$event->setReturn($data);
		}
	}
}
