<?php

namespace Plugins\Ccm19TwoFactorAuth\EventSubscriber;

use App\Event\LoginEvent;
use App\Utils;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use App\Model\PluginState;

class LoginInjection implements EventSubscriberInterface
{
	/** @var PluginState $pluginState */
	private $pluginState;

	public function __construct(PluginState $state)
	{
		$this->pluginState = $state;
	}

	/**
	 * @return array
	 */
	public static function getSubscribedEvents()
	{
		return [
			LoginEvent::NAME => ['onLogin', 100],
		];
	}

	/**
	 * @return void
	 */
	public function onLogin(LoginEvent $event)
	{
		$user = $event->getUser();
		$totpSettings = $user->getTwoFactorAuthParameters($this->pluginState->getPluginUID());
		$passkeySettings = $user->getTwoFactorAuthParameters($this->pluginState->getPluginUID().'--passkey');
		if (!$totpSettings and !$passkeySettings) {
			return;
		} elseif (!$this->pluginState->isAllowedForUser($user)) {
			if (!$this->pluginState->isFullAccessForAllAccounts()) {
				Utils::addFlash('danger', 'Two-factor authentication is not included in your current plan. Consider upgrading to continue using it.');
			}
			return;
		} else {
			if ($totpSettings) {
				$event->addSecondFactorAuth('Smartphone App', 'app_login_plugin_ccm19_twofactorauth_totp', []);
			}
			if ($passkeySettings) {
				$event->addSecondFactorAuth('Passkey', 'app_login_plugin_ccm19_twofactorauth_passkey', []);
			}
		}

	}

}
