"use strict";
$(function () {
	// Checkboxen mit data-disable-confirm="Hinweis…" warnen vor dem Deaktivieren
	$('input[data-disable-confirm]').on('click', function () {
		var message = this.getAttribute('data-disable-confirm').replace(/\\n/g, '\n');
		if (!this.checked) {
			return confirm(message);
		}
	});

	// Checkboxen mit data-enable-confirm="Hinweis…" warnen vor dem Aktivieren
	$('input[data-enable-confirm]').on('click', function () {
		var message = this.getAttribute('data-enable-confirm').replace(/\\n/g, '\n');
		if (this.checked) {
			return confirm(message);
		}
	});

	// Buttons mit data-confirm-msg="Hinweis…" warnen vor dem Aktivieren
	$('button[data-confirm-msg], input[data-confirm-msg]').on('click', function () {
		var message = this.getAttribute('data-confirm-msg').replace(/\\n/g, '\n');
		return confirm(message);
	});

	// Zu Tab wechseln, wenn Tabname im Fragment der URL
	// und zu Input wechseln und Inhalt markieren, wenn ID im Fragment der URL
	if (location.hash && location.hash.length > 1) {
		$('a[data-toggle="tab"][href="#'+$.escapeSelector(location.hash.substring(1))+'"]').each(function () {
			var $this = $(this);
			if (!$this.hasClass('hidden') && !$this.parent().hasClass('hidden')) {
				// Vorbedingte Tabs öffnen
				$this.parents('[data-binds-to]').each(function () {
					var selector = this.getAttribute("data-binds-to");
					$(selector).filter(".tab-pane[id]").each(function () {
						$('a[data-toggle="tab"][href="#'+$.escapeSelector(this.id)+'"]').tab('show');
					});
				});
				// Übergeordnete Tabs öffnen
				$this.parents('.tab-pane[id]').not('.active').each(function () {
					$('a[data-toggle="tab"][href="#'+$.escapeSelector(this.id)+'"]').tab('show');
				});
				// Tab selbst öffnen
				$this.tab('show');
			}
		});
		$('input#'+$.escapeSelector(location.hash.substring(1))).each(function () {
			this.focus();
			this.select();
			try {
				this.setSelectionRange(0, this.value.length);
			} catch (e) {}
		});
	}

	// Ausgewählten Tab in URL speichern
	$('a[data-toggle="tab"]').on('shown.bs.tab', function () {
		var $tab = $(this.href.replace(/^[^#]*#/, '#'));

		// Innersten offenen Tab suchen
		var $subtab = $tab.first();
		while ($subtab.length) {
			$subtab = $subtab.find('.tab-pane.active[id]').first();
			if ($subtab.length) {
				$tab = $subtab;
			}
		}

		var state = ($tab.prop('id')) ? '#'+$tab.prop('id') : this.href;
		history.replaceState(null, "", state);
	});

	// Trägt bei Tab-Navigation ARIA-Attribute nach
	$('a[data-toggle="tab"]').each(function () {
		var match = this.href.match(/#(.*)$/, '');
		if (match && !this.hasAttribute('aria-controls')) {
			this.setAttribute('aria-controls', match[1]);
			var pane = document.getElementById(match[1]);
			if (pane) {
				if (this.id) {
					pane.setAttribute("aria-labelledby", this.id);
				} else {
					pane.setAttribute("aria-label", this.innerText.trim());
				}
			}
		}
	});

	// Barrierefreiheit bei Panel-Tabs
	$('.tab-pane:not([role])').attr('role', 'tabpanel');
	$('.tab-pane.active').attr('aria-expanded', 'true');
	var tablists = $('ul.panel-tabs:not([role])').attr('role', 'tablist');
	tablists.find('a, button').attr('role', 'tab').prop('tabIndex', -1).on('hidden.bs.tab', function () {
		this.setAttribute('aria-selected', 'false');
		this.tabIndex = -1;
		(this.getAttribute('aria-controls') +'').split(' ').forEach(function (id) {
			var el = document.getElementById(id);
			if (el) { el.setAttribute('aria-expanded', 'false'); }
		});
	}).on('shown.bs.tab', function () {
		this.setAttribute('aria-selected', 'true');
		this.tabIndex = 0;
		(this.getAttribute('aria-controls') + '').split(' ').forEach(function (id) {
			var el = document.getElementById(id);
			if (el) { el.setAttribute('aria-expanded', 'true'); }
		});
	}).on('keydown', function (evt) {
		if (evt.isComposing || evt.shiftKey || evt.altKey || evt.metaKey || evt.ctrlKey) {
			return;
		}
		// Move
		if (evt.keyCode === 39 || evt.keyCode === 37) {
			var $li = $(this).closest('li');
			this.tabIndex = -1;
			evt.preventDefault();

			// Move right
			if (evt.keyCode === 39) {
				var $nextLi = $li.nextAll('li:visible').first();
				if ($nextLi.length == 0) {
					$nextLi = $li.parent().children('li:visible').first();
				}
				$nextLi.find('a, button').prop('tabIndex', 0).focus();
			}
			// Move left
			else if (evt.keyCode === 37) {
				var $nextLi = $li.prevAll('li:visible').first();
				if ($nextLi.length == 0) {
					$nextLi = $li.parent().children('li:visible').last();
				}
				$nextLi.find('a, button').prop('tabIndex', 0).focus();
			}
		}
		if (evt.keyCode === 13 || evt.keyCode === 32) {
			evt.preventDefault();
			this.click();
		}
	});
	tablists.find('li.active a, li.active button').attr('aria-selected', 'true').prop('tabIndex', 0);

	// Beliebiges Element mit Tab oder Checkbox/Radio mitsteuern
	// data-binds-to="#selector" gibt an, welches Element kontrolliert
	// data-bind-mode="mute"|"class"|"disable"|"hide" (Standard: "hide") steuert, was passiert
	// data-bind-invert kehrt das Verhalten für Checkbox/Radio um
	// data-bind-values kommaseparierte Werte bei select-Feldern, die das Element steuern
	$('[data-binds-to]').each(function () {
		var self = this;
		var selector = this.getAttribute('data-binds-to');
		var action = this.hasAttribute('data-bind-mode') ? this.getAttribute('data-bind-mode') : 'hide';
		var invert = this.hasAttribute('data-bind-invert');
		var values = this.dataset.bindValues ? this.dataset.bindValues.split(',') : [];
		var $bound_to = $(selector);
		var $controller = $('[data-toggle][href="'+selector+'"');
		var type = $bound_to.prop('type');

		var show, hide;
		if (action == 'mute') {
			show = function () {
				self.classList.remove('text-muted');
				self.ariaDisabled = "false";
				$(self).find('[data-required]').each(function () {
					this.required = true;
				});
			};
			hide = function () {
				self.classList.add('text-muted');
				self.ariaDisabled = "true";
				$(self).find('[data-required]').each(function () {
					this.required = false;
				});
			};
		}
		else if (action == 'class') {
			show = function () {
				String(self.dataset.classIfOff || '').split(' ').forEach(function (className) {
					self.classList.remove(className);
				});
				String(self.dataset.classIfOn || '').split(' ').forEach(function (className) {
					self.classList.add(className);
				});
			};
			hide = function () {
				String(self.dataset.classIfOn || '').split(' ').forEach(function (className) {
					self.classList.remove(className);
				});
				String(self.dataset.classIfOff || '').split(' ').forEach(function (className) {
					self.classList.add(className);
				});
			};
		}
		else if (action == 'disable') {
			show = function () {
				self.disabled = false;
				$(self).find('[data-required]').each(function () {
					this.required = true;
				});
			};
			hide = function () {
				self.disabled = true;
				$(self).find('[data-required]').each(function () {
					this.required = false;
				});
			};
		}
		else /* 'hide' */ {
			show = function () {
				self.classList.remove('hidden');
				self.setAttribute('aria-hidden', 'false');
				$(self).find('[data-required]').each(function () {
					this.required = true;
				});

				// Breite für sichtbare DataTables-Spalten berechnen
				$.fn.dataTable.tables({visible: true, api: true}).columns.adjust();
			}
			hide = function () {
				self.classList.add('hidden');
				self.setAttribute('aria-hidden', 'true');
				$(self).find('[data-required]').each(function () {
					this.required = false;
				});
			}
		}

		var updateFunction;
		if (type === 'checkbox') {
			$controller = $bound_to;
			$controller.on('change', function () {
				var checked = false;
				$bound_to.each(function () { if (this.checked) { checked = true; } });
				if (checked ^ invert) {
					show();
				} else {
					hide();
				}
			});
			var checked = false;
			$bound_to.each(function () { if (this.checked) { checked = true; } });
			if (checked ^ invert) {
				show();
			} else {
				hide();
			}
		}
		else if (type === 'radio') {
			var names = $bound_to.map(function () { return this.name; }).get();
			$controller = $();
			names.forEach(function (name) {
				$controller = $controller.add(document.getElementsByName(name));
			});
			$controller.on('click', function () {
				var checked = false;
				$bound_to.each(function () { if (this.checked) { checked = true; } });
				if (checked ^ invert) {
					show();
				} else {
					hide();
				}
			});
			var checked = false;
			$bound_to.each(function () { if (this.checked) { checked = true; } });
			if (checked ^ invert) {
				show();
			} else {
				hide();
			}
		}
		else if (type === 'select-one') {
			$controller = $bound_to;
			updateFunction = function () {
				if (values.indexOf($bound_to.val()) > -1) {
					show();
				} else {
					hide();
				}
			};
			$controller.on('change', updateFunction);
			updateFunction();
		}
		else {
			$controller.on('hide.bs.tab', function () {
				hide();
			});
			$controller.on('show.bs.tab', function () {
				show();
			});

			if ($bound_to.hasClass('active')) {
				show();
			} else {
				hide();
			}
		}

		if (this.id) {
			var id = this.id;
			$controller.each(function () {
				var controls = this.getAttribute('aria-controls');
				controls = (controls) ? (controls + ' ' + id) : id;
				this.setAttribute('aria-controls', controls);
			});

			if (!invert && action == "hide" && this.classList.contains("panel") && $bound_to.length == 1) {
				$bound_to.filter('.tab-pane').first().each(function () {
					var owns = this.getAttribute('aria-owns');
					owns = (owns) ? (owns + ' ' + id) : id;
					this.setAttribute('aria-owns', owns);
				})
			}
		}

	});

	// data-dismiss=li soll umschließendes li löschen
	$("main").on('click', 'button[data-dismiss="li"]', function () {
		$(this).closest('li').fadeOut("fast", function() {
			$(this).trigger('remove');
		});
	});

	// Typeahead.js
	var datalistMatcher = function(datalist) {
		return function findMatches(q, cb) {
			var matches = {};

			var pattern = q.replace(/[.+*^${}()|[\]\\]/g, '\\$&').replace(/\?/g, '.').replace(/\\\\\.\*/g, '\?');
			var prefixRegex = new RegExp("(^|\b|[ ,.-])"+pattern, "i");
			var substrRegex = new RegExp(pattern, "i");
			var count = 0;

			$(datalist).children('option').each(function() {
				if (prefixRegex.test(this.value)) {
					matches[this.value] = this;
					count++;
				}
			});
			$(datalist).children('option').each(function() {
				if (substrRegex.test(this.value) && count < 20 && matches[this.value] === undefined) {
					matches[this.value] = this;
					count++;
				}
			});

			cb(Object.values(matches));
		};
    };
    // Init Typeahead Plugin
    $('.typeahead[list]').each(function () {
		var $this = $(this).typeahead(
			{
				hint: true,
				highlight: true,
				minLength: 1
			}, {
				name: this.getAttribute('list'),
				displayKey: 'value',
				source: datalistMatcher(document.getElementById(this.getAttribute('list')))
			}
		);
		var ta = $this.data("ttTypeahead");

		this.removeAttribute('list');
		$(this).on('keypress', function(evt) {
			if (evt.key == 'Enter' || evt.keyCode == 13) {
				if (ta.input.getHint()) {
					ta.input.setInputValue(ta.input.getHint());
					evt.preventDefault();
					$(this).trigger('change');
				}
				if (ta.dropdown.isOpen) {
					ta.close();
					evt.preventDefault();
				}
			}
		});

	});

	// Sammelauswahl-Checkboxen in Tabellenheadern
	$('thead input.select-all[type="checkbox"]').each(function () {
		var self = this;
		var n = $(this).closest('td,th').prevAll('td,th').length + 1;
		var $tbody = $(this).closest('table').children('tbody');
		var $checkboxes = $tbody.find('tr :nth-child('+n+') input[type="checkbox"]')

		function updateState() {
			var checked = $checkboxes.filter(':checked');
			var unchecked = $checkboxes.filter(':not(:checked)');
			self.indeterminate = (checked.length > 0 && unchecked.length > 0);
			self.checked = (checked.length > 0 && unchecked.length == 0);
		}

		updateState();
		$tbody.on('click', 'input[type="checkbox"]', updateState);
		$(self).on('click', function () {
			var checked = self.checked;
			$checkboxes.each(function () {
				this.checked = checked;
			});
		});

	});

	// Platzhalter von Value eines anderen Inputs füllen
	$('input[data-placeholder-from]').each(function () {
		var input = this;
		var $selector = $(this.getAttribute('data-placeholder-from'));
		$selector.on('change', function () {
			input.placeholder = this.value;
		});
		this.placeholder = $selector.prop('value');
	});

	// Platzhalter übernehmbar machen, wenn Fieldset/Form data-inherit-buttons gesetzt hat
	$('[data-inherit-buttons] .bs-component input[placeholder], [data-inherit-buttons] .bs-component textarea[placeholder]').each(function () {
		var input = this;

		if (input.type == 'checkbox' || input.type == 'radio' || input.type == 'button') {
			return;
		}

		var button = document.createElement('button');
		button.type = 'button';
		button.className = 'inheritPlaceholder';
		button.innerText = '';
		input.parentNode.appendChild(button);

		function updateButton() {
			if (input.value == input.placeholder) {
				button.classList.add('overwrite');
				button.disabled = true;
			}
			else if (input.value != '') {
				button.classList.add('overwrite');
				button.disabled = false;
			} else {
				button.classList.remove('overwrite');
				button.disabled = false;
			}
		}

		$(input).on('change keyup', updateButton);
		$(button).on('click', function () { input.value = input.placeholder; updateButton(); });
		updateButton();
	});

	/**
	 * @param {boolean} [dark=false]
	 */
	function switchTinyMceSkin(dark) {
		if (!window.tinymce || !tinymce.editors.length) {
			return;
		}

		dark = typeof dark == 'boolean' ? dark : false;

		var skin = dark ? 'oxide-dark' : 'oxide';
		var contentDir = dark ? 'dark' : 'default';

		var linkSelector = 'link[href*="/js/tinymce/skins/"]';
		var linkElements = Array.prototype.slice.call(document.querySelectorAll(linkSelector));

		tinymce.editors.forEach(function (editor) {
			if (editor.contentDocument) {
				linkElements = linkElements.concat(
					Array.prototype.slice.call(editor.contentDocument.querySelectorAll(linkSelector))
				);
			}
		});

		linkElements.forEach(function (link) {
			link.href = link.href.replace(/(js\/tinymce\/skins\/ui\/)[^\/]+/, '$1' + skin);
			link.href = link.href.replace(/(js\/tinymce\/skins\/content\/)[^\/]+/, '$1' + contentDir);
		});
	};

	/**
	 * Stylesheets für Darkmode/Lightmode wechseln
	 *
	 * @param {boolean} [dark=false]
	 */
	function switchStylesheets(dark) {
		"use strict";
		Array.prototype.forEach.call(document.getElementsByClassName(dark ? 'darkmode-only' : 'lightmode-only'), function (e) {
			if ('relList' in e) {
				e.relList.add('stylesheet');
				e.relList.remove('prefetch');
			}
		});

		Array.prototype.forEach.call(document.getElementsByClassName(dark ? 'lightmode-only' : 'darkmode-only'), function (e) {
			if ('relList' in e) {
				e.relList.remove('stylesheet');
			}
		});

		switchTinyMceSkin(dark);
	}

	// Dark-Mode-Schalter
	$('#darkmode-toggle').click(function() {
		var baseLink = $('head link[rel="x-logincheck-url"]').attr('href').replace("login/check", "");

		var postData = {
			"action": "saveColorScheme", "preferredColorScheme": null
		}

		if (this.checked) {
			$(':root').removeClass("color-scheme--light").addClass("color-scheme--dark");
			postData.preferredColorScheme = "dark";
			$.post( baseLink+"account", postData, function (data) {
				console.log("darkModeSet");
			});
			switchStylesheets(true);
		}
		else {
			$(':root').removeClass("color-scheme--dark").addClass("color-scheme--light");
			postData.preferredColorScheme = "light";
			$.post( baseLink+"account", postData, function (data) {
				console.log("lightModeSet");
			});
			switchStylesheets(false);
		}
	});
	switchStylesheets($(':root').hasClass("color-scheme--dark"));

	// Label mit Button-Role sind auch mit Tastatur bedienbar
	$('label[role="button"], label span[role="button"]').on('keypress', function (evt) {
		var control = (this.tagName.toUpperCase() == 'LABEL') ? this.control : $(this).closest('label').prop('control');
		switch (evt.key.toLowerCase()) {
			case 'enter':
			case ' ':
			case 'return':
				if (control) {
					control.click();
					evt.preventDefault();
				}
				break;
			default:
		}
	}).each(function () {
		var label = this;
		var control = (this.tagName.toUpperCase() == 'LABEL') ? this.control : $(this).closest('label').prop('control');
		function updateButtonState() {
			label.setAttribute('aria-pressed', label.control.checked ? 'true' : 'false');
			if (label.hasAttribute('aria-controls')) {
				label.setAttribute('aria-expanded', label.control.checked ? 'true' : 'false');
			}
		}

		if (control && control.hasAttribute('aria-controls')) {
			this.setAttribute('aria-controls', control.getAttribute('aria-controls'));
			if (control.type == 'checkbox') {
				updateButtonState();
				$(control).on('click', updateButtonState);
			}
			else if (control.type == 'radio' && control.name) {
				updateButtonState();
				$('input[name="'+$.escapeSelector(control.name)+'"').on('click', updateButtonState);
			}
		}
	});

	// JSON-Kodieren von Formularen
	if ('entries' in FormData.prototype) {
		$('form[data-pack]').on('formdata', function (evt) {
			var formData = evt.originalEvent.formData;
			var result = {};
			var prefixes = this.getAttribute('data-pack').split(',').map(function (key) { return key+'['; });

			// Hilfsfunktion: Fügt Daten rekursiv in ein Result-Object ein
			function appendRecursive(base, baseKey, keyParts, finalValue) {
				var currentKey = keyParts.shift();
				var currentValue = currentKey === undefined ? finalValue : undefined;
				// Direktes Zuweisen
				if (currentKey === undefined) {
					base[baseKey] = currentValue;
				}
				else {
					// Ggf. Array/Objekt anlegen
					if (base[baseKey] === undefined || !(currentKey in base[baseKey])) {
						// Append-Keys (foo[])
						if (currentKey === '') {
							if (base[baseKey] === undefined) {
								base[baseKey] = [currentValue];
								currentKey = 0;
							}
							else if (Array.isArray(base[baseKey])) {
								currentKey = base[baseKey].push(currentValue) - 1;
							}
							else {
								var currentBase = base[baseKey];
								for (var i = 0; i < 65535; ++i) {
									if (!(i in currentBase)) {
										currentBase[i] = currentValue;
										currentKey = i;
										break;
									}
								}
								if (!(i in currentBase)) {
									console.error("failed to complete JSON post data: too many numeric keys in associative array");
									return;
								}
							}
						}
						// Nummerische Keys (foo[0])
						else if (/^[0-9]*$/.test(currentKey)) {
							if (base[baseKey] === undefined) {
								if (currentKey === "0") {
									base[baseKey] = [currentValue];
								} else {
									base[baseKey] = {};
									base[baseKey][currentKey] = currentValue;
								}
							}
							else if (Array.isArray(base[baseKey])) {
								if (base[baseKey].length == currentKey) {
									base[baseKey].push(currentValue);
								} else if (base[baseKey].length > currentKey) {
									base[baseKey][currentKey] = currentValue;
								} else {
									var currentBase = base[baseKey];
									var replacementBase = {};
									for (var [key, value] of currentBase.entries()) {
										replacementBase[key] = value;
									}
									replacementBase[currentKey] = currentValue;
									base[baseKey] = replacementBase;
								}
							}
							else {
								base[baseKey][currentKey] = currentValue;
							}
						}
						else {
							if (base[baseKey] === undefined) {
								base[baseKey] = {};
							}
							else if (Array.isArray(base[baseKey])) {
								var currentBase = base[baseKey];
								var replacementBase = {};
								for (var [key, value] of currentBase.entries()) {
									replacementBase[key] = value;
								}
								base[baseKey] = replacementBase;
							}
							base[baseKey][currentKey] = currentValue;
						}
					}
					// Rekursiv weitergehen
					appendRecursive(base[baseKey], currentKey, keyParts, finalValue);
				}
			}

			// Parst ein Key-Value-Paar und fügt das Ergebnis in `result` ein.
			function appendToResult(key, value) {
				var keyParts = key.split('[').map(function (s) { return s.replace(/\]$/, ''); });
				var firstKey = keyParts.shift();
				appendRecursive(result, firstKey, keyParts, value)
			}

			// Betroffene Eingabewerte zusammensuchen und konvertieren
			var toDelete = [];
			for (var [key, value] of formData.entries()) {
				if (prefixes.reduce(function (acc, prefix) { return acc || key.startsWith(prefix);  }, false) && typeof value === 'string') {
					appendToResult(key, value);
					toDelete.push(key);
				}
			}

			// Betroffene Original-Daten aus FormData löschen
			for (var key of toDelete) {
				formData.delete(key);
			}

			// JSON-Daten in FormData einfügen
			for (var key of Object.keys(result)) {
				formData.set(key, JSON.stringify(result[key]));
			}

		});
	}

	// Check, ob beim Wegnavigieren ungespeicherte Änderungen vorliegen
	//
	// Ungespeicherte Änderungen sind alle Änderungen von Inputs, Selects und Textareas, in <form>s mit
	// der Klasse "confirm-discard", die vom ursprünglichen Zustand beim Laden der Seite abweichen.

	// Aktivierungsfunktion: Sollte erst mit der ersten Änderung ausgeführt werden, weil Browser ihren Forward-Back-Cache
	// deaktivieren, wenn dieses Event genutzt wird und damit die Performance beim Hin- und Herspringen leiden kann.
	function activateBeforeUnloadCheck() {
		if (activateBeforeUnloadCheck.done) {
			return;
		}
		activateBeforeUnloadCheck.done = true;
		window.addEventListener('beforeunload', function (evt) {
			// Wenn das relevante Formular abgeschickt wird, ist alles ok.
			if (inSubmitTimeout) {
				return;
			}

			// Prüfen, ob ein relevantes Form-Element geändert wurde
			var isDirty = false;
			$('form.confirm-discard').each(function () {
				Array.prototype.forEach.call(this.elements, function (e) {
					if (!e.name || e.disabled || e.readOnly) {
						return;
					}

					if (e.type === 'checkbox' || e.type === 'radio') {
						if ('defaultChecked' in e && e.defaultChecked !== e.checked) {
							e.classList.add('is-unsaved');
							isDirty = true;
							if (e.parentNode.classList.contains("input-group")) {
								e.parentNode.classList.add('is-unsaved');
							}
						}
					}
					else if (e.type === 'select-one' || e.type === 'select-multi') {
						var selectDirty = false;
						var someDefaultSelection = false;
						Array.prototype.forEach.call(e.options, function (o) {
							if ('defaultSelected' in o && o.defaultSelected) {
								someDefaultSelection = true;
							}
						});
						if (someDefaultSelection) {
							Array.prototype.forEach.call(e.options, function (o) {
								if ('defaultSelected' in o && o.defaultSelected != o.selected) {
									isDirty = true;
									selectDirty = true;
								}
							});
							if (selectDirty) {
								e.classList.add('is-unsaved');
								if (e.parentNode.classList.contains("input-group")) {
									e.parentNode.classList.add('is-unsaved');
								}
							}
						}
					}
					else if (
						'value' in e
						&& e.type !== 'fieldset' && e.type !== 'object' && e.type !== 'output' && e.type !== 'file'
						&& e.type !== "submit" && e.type !== "reset" && e.type !== "button" && e.type !== 'hidden'
					) {
						if ('defaultValue' in e && e.defaultValue !== e.value) {
							isDirty = true;
							e.classList.add('is-unsaved');
							if (e.parentNode.classList.contains("input-group")) {
								e.parentNode.classList.add('is-unsaved');
							}
						}
					}
				});

			});
			if (isDirty) {
				evt.preventDefault();
				evt.returnValue = "unsaved changes";

				$(document.body).off('focusin', 'input, textarea, select', resetUnsavedMarkers);
				window.setTimeout(function () {
					window.setTimeout(function () {
						$(document.body).one('focusin', 'input, textarea, select', resetUnsavedMarkers);
					}, 50);
				}, 1);
			}

		});
	}
	activateBeforeUnloadCheck.done = false;

	function resetUnsavedMarkers() {
		$('form .is-unsaved').removeClass('is-unsaved');
	}
	var inSubmitTimeout = null;
	let forms = $('form.confirm-discard').on('submit', function (evt) {
		var submitter = evt.submitter;
		if (!submitter) {
			submitter = evt.originalEvent.submitter;
		}
		if (submitter && submitter.type == "submit") {
			submitter.classList.add('disabled');
		}
		if (inSubmitTimeout) {
			window.clearTimeout(inSubmitTimeout);
		}
		inSubmitTimeout = window.setTimeout(function () {
			if (submitter && submitter.type == "submit") {
				submitter.classList.remove('disabled');
			}
			inSubmitTimeout = null;
		}, 900);
	})
	.one('change', activateBeforeUnloadCheck);

	// Sticky Save-Bars
	if ($('fieldset#save, div#save, section#save').length > 0) {
		var stickyDisabled = false;
		var stickyScrollShowAtTop = false;
		var stickyScrollThrottle = false;
		var stickyScrollTimeout = null;
		var saveBar = document.getElementById('save');
		var senitel = document.createElement('div');
		senitel.id = 'savebar-senitel';
		senitel.style.clear = 'left';
		senitel.setAttribute("role", "none");
		saveBar.parentNode.insertBefore(senitel, saveBar);
		var closeBtn = document.createElement('button');
		closeBtn.type = 'button';
		closeBtn.className = 'close sticky-close';
		closeBtn.innerHTML = '<i class="fa fa-times"></i>';
		saveBar.firstElementChild.appendChild(closeBtn);
		closeBtn.addEventListener('click', function () {
			if (closeBtn.classList.contains("toggled")) {
				closeBtn.classList.remove("toggled");
				stickyDisabled = false;
				onScrollSticky();
			} else {
				closeBtn.classList.add("toggled");
				saveBar.classList.remove('is-floating');
				saveBar.classList.remove('bottom-sticky');
				saveBar.classList.add('bottom-sticky-out');
				if (stickyScrollTimeout) {
					clearTimeout(stickyScrollTimeout);
				}
				stickyScrollTimeout = window.setTimeout(function () {
					saveBar.classList.remove('bottom-sticky-out');
					saveBar.classList.remove('is-floating');
					stickyScrollTimeout = null;
				}, 350);
				stickyDisabled = true;
			}
		})

		function onStickyFormInputChange(evt) {
			if (('defaultValue' in this && this.defaultValue !== this.value)
				|| this.type === 'select-one' || this.type === 'select-multi'
				|| ('defaultChecked' in this && this.defaultChecked !== this.checked)
			)
			{
				stickyScrollShowAtTop = true;
				onScrollSticky();
				$('form').off('change input', '**', onStickyFormInputChange);
			}

		}
		$('form').on('change input', ':input[name][type != "hidden"],.vendor-switch', onStickyFormInputChange);

		function onScrollSticky() {
			if (window.scrollY === 0 && stickyScrollShowAtTop === false && saveBar.classList.contains('bottom-sticky')) {
				saveBar.classList.remove('bottom-sticky');
				saveBar.classList.add('bottom-sticky-out');
				if (stickyScrollTimeout) {
					clearTimeout(stickyScrollTimeout);
				}
				stickyScrollTimeout = window.setTimeout(function () {
					saveBar.classList.remove('bottom-sticky-out');
					saveBar.classList.remove('is-floating');
					stickyScrollTimeout = null;
				}, 350);
			}
			else if (!stickyDisabled && (window.scrollY > 10 || stickyScrollShowAtTop == true)) {
				if (stickyScrollTimeout) {
					clearTimeout(stickyScrollTimeout);
					stickyScrollTimeout = null;
				}

				if (window.scrollY + window.innerHeight - 44 >= senitel.offsetTop + save.offsetHeight) {
					if (saveBar.classList.contains('is-floating')) {
						saveBar.classList.remove('is-floating');
					}
				} else if (!saveBar.classList.contains('bottom-sticky') || !saveBar.classList.contains('is-floating')) {
					saveBar.classList.remove('bottom-sticky-out');
					saveBar.classList.add('bottom-sticky');
					saveBar.classList.add('is-floating');
				}
			}
		}

		function onScrollStickyThrottled() {
			if (!stickyScrollThrottle) {
				window.requestAnimationFrame(function () {
					stickyScrollThrottle = false;
					onScrollSticky();
				});
				stickyScrollThrottle = true;
			}
		}
		window.addEventListener('scroll', onScrollStickyThrottled, {'passive': true});
		window.addEventListener('resize', onScrollStickyThrottled, {'passive': true});

		$(window).on('shown.bs.tab', onScrollSticky);
	}

	function saveDatatableSetting(name, key, value) {
		var data = localStorage.getItem('ccm19-datatable-settings');
		if (data) {
			data = JSON.parse(data);
		}
		if (!data || typeof data !== 'object') {
			data = {};
		}
		var itemData = data[name];
		if (!itemData || typeof itemData !== 'object') {
			itemData = {};
		}
		itemData[key] = value;
		data[name] = itemData;
		try {
			localStorage.setItem('ccm19-datatable-settings', JSON.stringify(data));
		} catch (e) {
			// ignore exceeded quota
			console.error("could not save datatable configuration in localStorage", e);
		}

	}

	$(window).on('order.dt', function (evt, dt) {
		var settings = dt.oInit;
		var order = dt.aaSorting;
		if (!('ccm19Name' in settings)) {
			return;
		}

		if (JSON.stringify(order) !== settings.ccm19SavedOrder) {
			saveDatatableSetting(settings.ccm19Name, 'order', order);
		}
	});

	$(window).on('length.dt', function (evt, dt, pageLength) {
		var settings = dt.oInit;
		if (!('ccm19Name' in settings)) {
			return;
		}

		if (pageLength !== settings.ccm19SavedPageLength) {
			saveDatatableSetting(settings.ccm19Name, 'pageLength', pageLength);
		}
	});

	// Barrierefreiheit von Datatables
	$(window).on('init.dt', function(evt) {
		var tableDataTable = evt.target;
		var tableWrapper = tableDataTable.parentNode.parentNode;
		if (!tableWrapper.classList.contains('dataTables_scroll')) {
			// keine scrolling Datatable. Abbrechen.
			return;
		}
		tableWrapper.role = "table";
		tableWrapper.parentNode.role = "group";
		// Tabellenkopf mit ARIA taggen
		var tableHead = tableWrapper.getElementsByClassName('dataTables_scrollHead')[0];
		var tableHeadTable = tableHead.getElementsByTagName('table')[0];
		var tableHeadTHead = tableHeadTable.getElementsByTagName('thead')[0];
		tableHead.role = "rowgroup";
		tableHead.getElementsByClassName('dataTables_scrollHeadInner')[0].role = "none";
		tableHeadTable.role = "none";
		tableHeadTHead.role = "none";
		var i = 1;
		for (var tr of tableHeadTHead.children)  {
			tr.role = "row";
			tr.ariaRowIndex = i++;
			for (var th of tr.children)  {
				th.role = "columnheader";
			}
		}
		// Tabellenkörper mit ARIA taggen
		tableDataTable.parentNode.role = "none";
		tableDataTable.role = "rowgroup";
		for (var tgroup of tableDataTable.children)  {
			if (tgroup.tagName.toUpperCase() === "THEAD") {
				tgroup.ariaHidden = true;
			} else {
				tgroup.role = "none";
			}
		}
	});
	$(window).on('draw.dt', function(evt, settings) {
		var tableDataTable = evt.target;
		var tableWrapper = tableDataTable.parentNode.parentNode;
		if (!tableWrapper.classList.contains('dataTables_scroll')) {
			// keine scrolling Datatable. Abbrechen.
			return;
		}
		var tableHead = tableWrapper.getElementsByClassName('dataTables_scrollHead')[0];
		var offset = settings._iDisplayStart + tableHead.getElementsByTagName('tr').length + 1;
		for (var tgroup of tableDataTable.children)  {
			if (tgroup.tagName === "THEAD") {
				tgroup.ariaHidden = true;
			} else {
				tgroup.role = "none";

				for (var tr of tgroup.children) {
					tr.role = "row";
					tr.ariaRowIndex = offset++;
					for (var td of tr.children) {
						td.role = (tgroup.tagName === "TH") ? 'rowheader' : 'cell';
					}
				}
			}
		}
		var tableWrapper = tableDataTable.parentNode.parentNode;
		tableWrapper.ariaRowCount = settings._iRecordsDisplay;
	});
});

// Toggle linked switches depending on their state (input[type="checkbox"][data-partial])
(function () {
	/** @type {Array.<HTMLInputElement>} */
	var partials = [];
	var recursionLevel = 0;

	function partial(values)
	{
		return values.reduce(function (newState, value) {
			if (value === 'partial') {
				newState = 'partial';
			}
			else if (newState === true) {
				newState = !!value || 'partial';
			}
			else {
				newState = !!value ? 'partial' : newState;
			}

			return newState;
		}, values.length ? !!values[0] : false);
	}

	function onStateChanged()
	{
		var self = this;

		var parent = document.getElementById(this.dataset.partial || '');
		var siblings = this.dataset.partial ? partials.filter(function (element) {
			return element.dataset.partial == self.dataset.partial;
		}) : [];
		var children = this.id ? partials.filter(function (element) {
			return element.dataset.partial == self.id;
		}) : [];

		// Depending on other scripts, track if checkboxes are switched back
		var oldChildrenState = partial(children.map(function (checkbox) {
			return checkbox.indeterminate ? 'partial' : checkbox.checked;
		}));

		// Set new state for all children
		var updatedChildren = [];
		if (!this.indeterminate) {
			children.forEach(function (checkbox) {
				if (checkbox.checked != self.checked || checkbox.indeterminate) {
					checkbox.checked = self.checked;
					checkbox.indeterminate = false;
					updatedChildren.push(checkbox);
				}
			});
		}

		// Intentionally dispatch event after all children have been updated
		updatedChildren.forEach(function (checkbox) {
			recursionLevel++;
			checkbox.dispatchEvent(new Event('change'));
			recursionLevel--;
		});

		// Allow all checkboxes to be enabled if there are mandatory child elements
		if (recursionLevel == 0 && (this.indeterminate && oldChildrenState !== true)) {
			this.checked = true;
			this.indeterminate = false;
			recursionLevel++;
			this.dispatchEvent(new Event('change'));
			recursionLevel--;
			return;
		}

		// Set new state for parent
		var newGroupState = partial(siblings.map(function (checkbox) {
			return checkbox.indeterminate ? 'partial' : checkbox.checked;
		}));

		if (parent && (parent.checked != !!newGroupState || parent.indeterminate != (newGroupState == 'partial'))) {
			parent.checked = !!newGroupState;
			parent.indeterminate = newGroupState == 'partial';
			recursionLevel++;
			parent.dispatchEvent(new Event('change'));
			recursionLevel--;
		}
	}

	function initPartials()
	{
		partials = Array.prototype.slice.call(document.querySelectorAll('input[type="checkbox"][data-partial]'));
		partials.forEach(function (checkbox) {
			var parent = document.getElementById(checkbox.dataset.partial);
			if (parent) {
				checkbox.addEventListener('change', onStateChanged);
				parent.addEventListener('change', onStateChanged);
			}
		});

		// Trigger update for checked elements on load
		partials.forEach(function (checkbox) {
			if (checkbox.checked) {
				checkbox.dispatchEvent(new Event('change'));
			}
		});
	}

	window.addEventListener('DOMContentLoaded', initPartials);
})();

// Force checked state for switches (input[type="checkbox"][data-mandatory="true"])
(function () {
	function checkCheckbox(checkbox)
	{
		if (!checkbox.checked || checkbox.indeterminate) {
			checkbox.checked = true;
			checkbox.indeterminate = false;
			checkbox.dispatchEvent(new Event('change'));
		}
	}

	function onMandatoryStateChanged(event)
	{
		event.preventDefault();
		checkCheckbox(this);
	}

	window.addEventListener('DOMContentLoaded', function () {
		Array.prototype.slice.call(document.querySelectorAll('input[type="checkbox"][data-mandatory="true"]')).forEach(function (checkbox) {
			checkbox.disabled = false;
			checkbox.required = true;
			checkbox.addEventListener('change', onMandatoryStateChanged, true);
			checkCheckbox(checkbox);
		});
	});
})();

// Handle checkboxes with multiple values (e.g. name="foo[]"); stores concatenated values in specified form field
(function () {
	var handlers = {};

	/**
	 * This method registers the `change` event for the specified checkbox.
	 * @param {HTMLInputElement} checkbox
	 */
	function registerCheckbox(checkbox) {
		if (checkbox.name in handlers) {
			handlers[checkbox.name].registerCheckbox(checkbox);
		}
		else {
			throw new Error('Checkbox handler for "' + checkbox.name + '" not found, call MultiValueConcat.handle first.');
		}
	}

	/**
	 * This method updates the state of the checkbox based on the selected values.
	 * @param {HTMLInputElement} checkbox
	 */
	function updateCheckbox(checkbox) {
		if (checkbox.name in handlers) {
			handlers[checkbox.name].updateCheckbox(checkbox);
		}
		else {
			throw new Error('Checkbox handler for "' + checkbox.name + '" not found, call MultiValueConcat.handle first.');
		}
	}

	/**
	 * This method initializes the state namespace for the specified checkbox name.
	 * Selected values are stored in the hidden field with the specified ID.
	 * The hidden field is updated when the state of a checkbox changes.
	 * Subsequently add checkboxes for the same name by calling `registerCheckbox`.
	 * Manually apply the current state to checkboxes by calling `updateCheckbox`.
	 * @param {string} checkboxName e.g. "foo[]"
	 * @param {string} hiddenFieldId e.g. "foo-values-hidden-field-id"
	 * @return {void}
	 */
	function handleCheckboxState(checkboxName, hiddenFieldId) {
		// Throw an error if the hidden field does not exist
		if (document.getElementById(hiddenFieldId) === null) {
			throw new Error('The hidden field with the ID "' + hiddenFieldId + '" does not exist.');
		}

		var selectedValues = [];

		/**
		 * @param {HTMLInputElement} checkbox
		 */
		function recordCheckedState(checkbox) {
			if (checkbox.checked) {
				// Add the value to the list of selected values if it is not already in the list
				if (selectedValues.indexOf(checkbox.value) === -1) {
					selectedValues.push(checkbox.value);
				}
			}
			else {
				// Remove the value from the list of selected values
				selectedValues = selectedValues.filter(function (value) {
					return value !== checkbox.value;
				});
			}

			// Update the hidden field with the selected values
			document.getElementById(hiddenFieldId).value = selectedValues.join(',');
		}

		/**
		 * This function updates the state of the checkbox based on the selected values.
		 * @param {HTMLInputElement} checkbox
		 */
		function updateCheckbox(checkbox) {
			checkbox.checked = selectedValues.indexOf(checkbox.value) !== -1;
		}

		/**
		 * This function is called when the state of a checkbox changes.
		 * @return {void}
		 */
		function onCheckboxStateChanged() {
			if (this.name === checkboxName) {
				recordCheckedState(this);
			}
		}

		/**
		 * This function registers the `change` event for the specified checkbox.
		 * @param {HTMLInputElement} checkbox
		 */
		function registerCheckbox(checkbox) {
			checkbox.addEventListener('change', onCheckboxStateChanged);
		}

		// Register all checkboxes that match the specified name
		Array.prototype.slice.call(document.querySelectorAll('input[type="checkbox"][name="' + checkboxName + '"]'))
			.forEach(function (checkbox) {
				registerCheckbox(checkbox);
			});

		handlers[checkboxName] = {
			registerCheckbox: registerCheckbox,
			updateCheckbox: updateCheckbox,
		};
	}

	// Export the function to the global scope
	window.MultiValueConcat = window.MultiValueConcat || {
		handle: handleCheckboxState,
		registerCheckbox: registerCheckbox,
		updateCheckbox: updateCheckbox,
	};
})();

// Code der ausgeführt werden kann/soll, bevor die Seite fertig geladen ist
(function () {

	// Hosting-Login
	var loginForm;
	var tokenMatch;
	if ((tokenMatch = location.hash.match(/(?:^#|&)user=([^&]+)&token=([^&]+)(&?)/)) && (loginForm = document.getElementById('login-form'))) {
		// URL bereinigen
		var newHash = (tokenMatch[0][0] == '#') ? location.hash.replace(tokenMatch[0], '#') : location.hash.replace(tokenMatch[0], tokenMatch[3]);
		if ('replaceState' in history) {
			history.replaceState(null, '', newHash);
		} else {
			location.hash = newHash;
		}

		// Token rausholen
		var username = decodeURIComponent(tokenMatch[1]);
		var token = tokenMatch[2];

		// Username setzen
		document.getElementsByName('username')[0].value = username;

		// Passwort-Feld leeren
		document.getElementsByName('password')[0].value = '';

		// Login-Button deaktivieren
		var button = loginForm.getElementsByClassName('login-button')[0];
		button.disabled = true;

		try {
			// Hidden Token-Feld hinzufügen
			var tokenInput = document.createElement('input');
			tokenInput.type = 'hidden';
			tokenInput.name = 'loginToken';
			tokenInput.value = token;
			loginForm.appendChild(tokenInput);

			loginForm.submit();

			window.setTimeout(function () {
				loginForm.removeChild(tokenInput);
			}, 100);

			window.setTimeout(function () {
				button.disabled = false;
			}, 10000);
		} catch (e) {
			button.disabled = false;
			loginForm.removeChild(tokenInput);
			throw e;
		}
	}


})();
