'use strict';

(function () {
	var defaultTimeout = 58000;
	var channel = null;
	var handle = null;
	var modalVisible = false;
	var wasHidden = false;
	// ID zur Unterscheidung verschiedener CCM19-Installationen auf der selben Domain
	var appId = document.querySelector('head meta[name="X-App-Id"]').content;
	if (!appId) {
		return;
	}

	// Inter-Tab-Kommunikationskanal aufbauen
	if ('BroadcastChannel' in window) {
		channel = new BroadcastChannel('ccm19.backend.auth.state');

		channel.addEventListener('message', function (event) {
			var message = (event.data || {});
			if (message.appId !== appId) {
				return; // Nachrichten von anderen Installationen ignorieren
			}
			if (message.type == 'heartbeat') {
				onHeartbeat(message);
			}
			else if (message.type == 'logout') {
				planLoginCheck(1750);
			}
			else if (message.type == 'login') {
				planLoginCheck(2000);
			}
		});

		// Bei Klick auf Login-/Logout-Buttons andere Tabs informieren
		$('.logout-button').on('click', function () {
			channel.postMessage({'type':'logout', 'appId': appId});
		});
		$('.login-button').on('click', function () {
			channel.postMessage({'type':'login', 'appId': appId});
		});
	}

	// Check-URL und erwarteten Benutzernamen auslesen
	var url = document.querySelector('head link[rel="x-logincheck-url"]').href;
	var username = document.querySelector('head meta[name="X-Username"]').content;
	var userid = document.querySelector('head meta[name="X-UserID"]').content;
	var resumeTokenMeta = document.querySelector('head meta[name="X-ResumeToken"]');
	var resumeToken = (resumeTokenMeta) ? document.querySelector('head meta[name="X-ResumeToken"]').content : null;

	if (!userid) {
		return;
	}

	function doLoginCheck () {
		$.ajax(url, { dataType: 'json' }).done(function (data) {
			if (channel) {
				data.type = 'heartbeat';
				data.appId = appId;
				channel.postMessage(data);
			}
			onHeartbeat(data);
		}).fail(function (xhr, status, error) {
			planLoginCheck(defaultTimeout);
		});
	}

	function planLoginCheck(timeout) {
		var timeout = timeout + (Math.random()*(timeout/4)-(timeout/8));
		if (handle) {
			window.clearTimeout(handle);
			handle = null;
		}
		handle = window.setTimeout(doLoginCheck, timeout);
	}

	// Dialog anzeigen/verstecken
	function onHeartbeat(data)
	{
		if (modalVisible) {
			if (userid === data.userid) {
				$('#modal-session-expired').modal('hide');
				modalVisible = false;
			}
		}
		else {
			if (userid !== data.userid) {
				$('#modal-session-expired').modal('show');
				modalVisible = true;
			}
		}

		// Re-Login: neues CSRF-Token
		if ('csrfToken' in data && userid === data.userid) {
			$('head meta[name="X-CSRF-Token"]').prop('content', data.csrfToken);
			$('input[type="hidden"][name="csrf_token"]').prop('value', data.csrfToken);
		}

		// Username geändert, aber ID gleich
		if (userid === data.userid && data.user !== username) {
			username = data.user;
			$('.current-username').text(data.user);
		}

		// nächsten Check planen
		planLoginCheck(defaultTimeout);
	}

	// Beim Tab-Wechsel/Fenster-Fokussieren direkt neu prüfen, ob die Sitzung noch aktiv ist
	window.addEventListener('blur', function (event) {
		wasHidden = true;
		// Während der Tab im Hintergrund ist, Session nicht offenhalten
		window.clearTimeout(handle);
		handle = null;
	});
	window.addEventListener('focus', function (event) {
		if (wasHidden) {
			planLoginCheck(500);
			wasHidden = false;
		}
	});

	// Ersten Check planen
	planLoginCheck(defaultTimeout);

	function setReloginError($form, message)
	{
		$form.find('.modal-body > .alert').remove();
		if (message) {
			var div = document.createElement('div');
			div.className = "alert alert-danger alert-dismissable";
			var btn = document.createElement('button');
			btn.type = 'button';
			btn.className = 'close';
			btn.setAttribute('data-dismiss', 'alert');
			btn.appendChild(document.createTextNode('x'));
			var i = document.createElement('i');
			i.className = 'fa fa-info pr10';
			var text = document.createTextNode(message);

			div.appendChild(btn);
			div.appendChild(i);
			div.appendChild(text);
			$form.find('.modal-body').first().prepend(div);
		}
	}

	// Erneut-Anmelden-Formular
	$('#relogin-form').on('submit', function (evt) {
		var $currentUser = $('meta[name="X-UserID"]');

		var $form = $(this);
		evt.preventDefault();
		$form.find('button').prop('disabled', true);
		$.ajax(this.action, {
			method: 'POST',
			dataType: 'json',
			data: {
				'username': $form.find('#relogin-username').prop('value'),
				'password': $form.find('#relogin-password').prop('value'),
				'switchTo': $form.find('#relogin-switchto').prop('value'),
				'locale_name': $form.find('#relogin-lang').prop('value'),
				'resumeToken': resumeToken,
				'format': 'json',
			}
		}).done(function (data, textStatus, xhr) {
			setReloginError($form, null);
			$('#modal-session-expired').modal('hide');
			if ($currentUser.prop('content') !== data.userid) {
				location.reload();
			}
			$('head meta[name="X-Original-Username"]').prop('content', (data.originalUsername !== null) ? data.originalUsername : '');
			$('header .dropdown-menu .login-button').css('display', (data.originalUsername !== null) ? 'block' : 'none');

			var heartbeat = { type: 'heartbeat', userid: data.userid, user: data.username, origUser: data.originalUsername, csrfToken: data.csrfToken, appId: appId };
			if (channel) {
				channel.postMessage(heartbeat);
			}
			onHeartbeat(heartbeat);

		}).fail(function (xhr, textStatus, error) {
			if (xhr.responseJSON && 'error' in xhr.responseJSON) {
				if ('nextStep' in xhr.responseJSON) {
					window.location = xhr.responseJSON.nextStep;
				}
				setReloginError($form, xhr.responseJSON.error);
			} else {
				setReloginError($form, error);
			}
		}).always(function () {
			$form.find('button').prop('disabled', false);
		});
	});
})();
