'use strict';

(function () {
	var CCM_DICT = ['ccm19', 'ccm', 'keks', 'cookie'];

	if (!('PWSTRENGTH_I18N' in window)) {
		window.PWSTRENGTH_I18N = {};
	}
	if (!('PWSTRENGTH_FORCE' in window)) {
		window.PWSTRENGTH_FORCE = true;
	}
	function _(s) {
		if (s in window.PWSTRENGTH_I18N) {
			return window.PWSTRENGTH_I18N[s];
		} else {
			return s;
		}
	}

	var SCORES = [
		_('Too weak'),
		_('Weak'),
		_('Medium'),
		_('Strong'),
		_('Very strong'),
	]

	function checkPasswordStrength() {
		var input = this;
		var meter = this.nextSibling;
		// Wörterbuch mit CCM19-Begriffen und Formularfeldern ergänzen
		var dictionary = CCM_DICT.slice();
		Array.prototype.forEach.call(document.querySelectorAll('input[type=text], input[type=email]'), function (el) { if (el.value && el != input) { dictionary.push(el.value); } });

		// Eigentlicher Check
		var result = zxcvbn(input.value.substr(0,60), dictionary);
		// Mindestlänge von 8 Zeichen, wenn Bewertung nicht extrem gut
		if (result.score < 2 && input.value.length < 8) {
			result.score = 0;
			if (!result.feedback.warning) {
				result.feedback.warning = 'Please use at least 8 characters';
			}
		}
		// Daten anzeigen
		meter.value = (input.value.length > 0) ? result.score : -1;
		meter.title = SCORES[result.score];
		if (PWSTRENGTH_FORCE && result.score < 1) {
			input.setCustomValidity(_('Password is too weak.')+'\n'+_(result.feedback.warning));
		} else {
			input.setCustomValidity('');
		}
	}

	Array.prototype.forEach.call(document.querySelectorAll('input[data-pwstrength]'), function (input) {
		var container = input.parentNode;
		var meter = document.createElement('meter');
		meter.className = 'password-strength';
		meter.setAttribute('aria-label', _('Password strength'));
		meter.min = -1;
		meter.max = 4;
		meter.low = 2;
		meter.high = 3;
		meter.optimum = 4;
		meter.value = -1;
		if (input.nextSibling) {
			container.insertBefore(meter, input.nextSibling);
		} else {
			container.appendChild(meter);
		}
		input.classList.add('has-pwstrength-meter');

		input.addEventListener('keyup', checkPasswordStrength);
		input.addEventListener('change', checkPasswordStrength);
		checkPasswordStrength.call(input);
	});
})();
