"use strict";

/**
 * @param {Color} color
 * @return {number}
 */
function blackWhiteLuminance(color) {
	var rgb = color.toRGB();
    var mapped = ['r', 'g', 'b'].map(function (key) {
		var v = rgb[key];
		v /= 255;
		return (v <= 0.03928) ? v / 12.92 : Math.pow((v + 0.055) / 1.055, 2.4);
	});
    return mapped[0] * 0.2126 + mapped[1] * 0.7152 + mapped[2] * 0.0722;
}

/**
 * @param {Color} color1
 * @param {Color} color2
 * @return {number}
 */
function contrastRatio(color1, color2) {
    var luminances = [
		blackWhiteLuminance(color1),
		blackWhiteLuminance(color2),
	];
    var brightest = Math.max.apply(null, luminances);
    var darkest = Math.min.apply(null, luminances);
    return (brightest + 0.05) / (darkest + 0.05);
}

/**
 * @param {object} $dropdownBtn
 * @return {}
 */
function updateDropdownVisibility($dropdownBtn)
{
	var $dropdown = $dropdownBtn.next('.dropdown-menu');
	if ($dropdown.find('li:not(.hidden):not(.divider)').length > 0) {
		$dropdownBtn.removeClass('hidden');
	} else {
		$dropdownBtn.addClass('hidden');
	}
	$dropdown.children('li.divider').each(function () {
		if ($(this).nextUntil('.divider').filter('li').filter(':not(.hidden)').length > 0) {
			this.classList.remove('hidden');
		} else {
			this.classList.add('hidden');
		}
	})
}

$('.custom-file-upload input[type="file"]').on('change', function(){
	var reader = new FileReader(),
		$previewContainer = $(this).closest('.custom-file-upload').find('img'),
		$dropdownBtn = $(this).parent().find('.dropdown-toggle');
	if ($previewContainer.length == 0) {
		$previewContainer = $('.custom-file-upload-preview');
	}

	if (this.files[0]) {
		reader.onload = function (e) {
			if (/^data:image\//.test(e.target.result)) {
				$previewContainer.attr('src', e.target.result);
				$('.settings-icon--color-picker').addClass('disabled').attr('aria-disabled', 'true');
			}
			else {
				$previewContainer.attr('src', '').attr('alt', '{% trans %}Invalid file{% endtrans %}');
				$('.settings-icon--color-picker').removeClass('disabled').attr('aria-disabled', 'false');
			}
		};
		reader.readAsDataURL(this.files[0]);
	}
});

$('.custom-file-upload-delete').on('click', function () {
	var $container = $(this).closest('.custom-file-upload');

	var $upload = $container.find('input[type="file"]');
	$upload.prop('value', '');
	var $preview = $container.find('img.custom-file-upload-preview');
	var src = $preview.attr('data-original');
	if (src) {
		$preview.prop('src', src);
	} else {
		$preview.prop('src', '#');
	}
	var $input = $('<input type="hidden" />').appendTo($container);
	$input.prop('name', $upload.prop('name')+'__delete');
	$input.prop('value', '1');

	this.disabled = true;
	this.parentNode.classList.add('hidden');
	var $dropdownBtn = $container.find('.dropdown-toggle');
	updateDropdownVisibility($dropdownBtn);
	$dropdownBtn.dropdown('toggle');

	$('.settings-icon--color-picker').removeClass('disabled').attr('aria-disabled', 'false');
})

function updateDependentElements() {
	setTimeout((function () {
		var id = this.id;
		var isChecked = this.checked;

		if (isChecked) {
			$('.show-if-'+id).removeClass('hidden').attr('aria-hidden', 'false');
			$('.hide-if-'+id).addClass('hidden').attr('aria-hidden', 'true');

		} else {
			$('.show-if-'+id).addClass('hidden').attr('aria-hidden', 'true');
			$('.hide-if-'+id).removeClass('hidden').attr('aria-hidden', 'false');
		}
	}).bind(this), 0);
}
$('#content input[type=checkbox][id]').on('click', updateDependentElements).each(updateDependentElements);

$(function () {
	$('.color-settings--group').each(function () {
		var $group = $(this);
		var $heading = $group.prev('.panel-body-heading');
		var $pickers = $group.find('.colorpicker-component');
		var $bgPicker = $pickers.eq(0),
			$fgPicker = $pickers.eq(1);

		var $indicator = $heading.find('.contrast-indication');
		var $indicatorText = $indicator.find('.contrast-indication--text');
		var $indicatorIcon = $indicator.children('i');

		var bgPickerId = $bgPicker.find('input').prop('id');
		var fgPickerId = $fgPicker.find('input').prop('id');

		$indicatorText.prop("for", (bgPickerId ? bgPickerId : '')+" "+(fgPickerId ? fgPickerId : ''));
		$indicator.tooltip({
			title: $indicatorIcon.attr('title'),
			placement: 'bottom',
			container: $group,
		});

		var limitGood = ($group.attr('data-contrast-good') || 4.5);
		var limitBest = ($group.attr('data-contrast-best') || 7);
		var warn = !!$group.attr('data-contrast-warn');
		var enforce = $group.attr('data-contrast-enforce');

		function checkContrast() {
			var ratio = contrastRatio(
				$bgPicker.data('colorpicker').color,
				$fgPicker.data('colorpicker').color
			);
			ratio = Math.round(ratio*10)/10;
			if (ratio >= limitBest) {
				$indicator.removeClass('too-low').addClass('text-success').removeClass('text-muted').removeClass('text-danger');
				if (enforce) {
					$fgPicker.find('input')[0].setCustomValidity('');
				}
			}
			else if (ratio >= limitGood) {
				$indicator.removeClass('too-low').removeClass('text-success').addClass('text-muted').removeClass('text-danger');
				if (enforce) {
					$fgPicker.find('input')[0].setCustomValidity('');
				}
			}
			else {
				$indicator.removeClass('text-success').removeClass('text-muted').addClass('text-danger');
				if (warn) {
					$indicator.addClass('too-low');
				}
				if (enforce) {
					$fgPicker.find('input')[0].setCustomValidity(enforce);
					$indicator.addClass('too-low');
				}
			}
			$indicatorText.text(ratio.toLocaleString(undefined, {minimumFractionDigits: 1, maximumFractionDigits: 1}) + ' : 1');

			$('.contrast-warning').toggle($('.contrast-indication.too-low').length > 0);
		}
		if ($bgPicker.length && $fgPicker.length) {
			$pickers.on('changeColor', checkContrast);
			checkContrast();
		}

	});
});

(function () {
	/**
	 * Bestimmt, ob ein Event das Element verlässt
	 * @param {MouseEvent} evt
	 * @param {Element} element
	 * @return {boolean}
	 */
	function checkElementLeave(evt, element) {
		var rect = element.getBoundingClientRect();
		if (evt.clientX >= rect.x && evt.clientX <= rect.x + rect.width) {
			if (evt.clientY >= rect.y && evt.clientY <= rect.y + rect.height) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Berechnet den X-Offset des Events relativ zu `element`
	 * @param {MouseEvent} evt
	 * @param {Element} element
	 * @return {number}
	 */
	function calcOffsetX(evt, element) {
		var offset = evt.offsetX;
		var target = evt.target;
		while (target && target !== element) {
			offset += target.offsetLeft;
			target = target.parentNode;
		}
		if (target === element) {
			return offset;
		} else {
			return evt.offsetX;
		}
	}

	/**
	 * Der vorherige Slot links von `slot`
	 * @param {HTMLElement} slot
	 * @return {?HTMLElement}
	 */
	function nextSlot(slot) {
		if (slot.classList.contains("drop-slot")) {
			while ((slot = slot.nextElementSibling)) {
				if (slot.classList.contains("drop-slot")) {
					return slot;
				}
			}
		}
		return null;
	}

	/**
	 * Der nächste Slot rechts von `slot`
	 * @param {HTMLElement} slot
	 * @return {?HTMLElement}
	 */
	function prevSlot(slot) {
		if (slot.classList.contains("drop-slot")) {
			while ((slot = slot.previousElementSibling)) {
				if (slot.classList.contains("drop-slot")) {
					return slot;
				}
			}
		}
		return null;
	}

	/**
	 * Alle Slots rechts von `slot` bis zur ersten Lücke, aber nur wenn es eine Lücke gibt
	 * @param {HTMLElement} slot
	 * @return {HTMLElement[]}
	 */
	function nextSlotsUntilGap(slot) {
		var nextSlots = [];
		var occupant = null;
		while (slot = nextSlot(slot)) {
			nextSlots.push(slot);
			var occupant = getSlotOccupant(slot);
			if (!occupant || occupant.hasAttribute("data-dragging")) {
				return nextSlots;
			}
		}
		return [];
	}

	/**
	 * Alle Slots links von `slot` bis zur ersten Lücke, aber nur wenn es eine Lücke gibt
	 * @param {HTMLElement} slot
	 * @return {HTMLElement[]}
	 */
	function prevSlotsUntilGap(slot) {
		var prevSlots = [];
		var occupant = null;
		while (slot = prevSlot(slot)) {
			prevSlots.push(slot);
			var occupant = getSlotOccupant(slot);
			if (!occupant || occupant.hasAttribute("data-dragging")) {
				return prevSlots;
			}
		}
		return [];
	}

	/**
	 * Gibt das Drag-Element zurück, das in `slot` liegt, falls vorhanden
	 * @param {HTMLElement} slot
	 * @return {?HTMLElement}
	 */
	function getSlotOccupant(slot) {
		return slot.querySelector(':scope > [draggable="true"]');
	}

	/**
	 * Verschiebt `draggable` in `slot`
	 * @param {HTMLElement} slot
	 * @param {HTMLElement} draggable
	 * @return {}
	 */
	function moveInto(slot, draggable) {
		if (draggable.parentNode.classList.contains("btn-drag-area")) {
			if (draggable.parentNode.children.length == 1) {
				draggable.parentNode.classList.add("empty");
			}
		}
		slot.classList.remove("empty");
		var old_input = draggable.parentNode.getElementsByTagName("input")[0];
		var new_input = slot.getElementsByTagName("input")[0];
		var value = draggable.getAttribute("data-value");
		slot.appendChild(draggable);

		if (old_input && old_input.value == value) {
			old_input.value = "";
			old_input.dispatchEvent(new CustomEvent("change", { bubbles: true }));
		}
		if (new_input) {
			new_input.value = draggable.getAttribute("data-value");
			new_input.dispatchEvent(new CustomEvent("change", { bubbles: true }));
		}
	}

	/**
	 * Verschiebt ein Drag-Element einen Slot nach rechts, wenn möglich
	 * @param {HTMLElement} draggable
	 * @param {boolean} shouldSwap
	 * @return {boolean} Erfolg
	 */
	function moveRight(draggable, shouldSwap) {
		var slot = nextSlot(draggable.parentNode);
		if (slot) {
			var occupant = getSlotOccupant(slot);
			if (!occupant) {
				moveInto(slot, draggable);
				return true;
			} else if (shouldSwap) {
				moveInto(draggable.parentNode, occupant);
				moveInto(slot, draggable);
				return true;
			}
		}
		return false;
	}

	/**
	 * Verschiebt ein Drag-Element einen Slot nach links, wenn möglich
	 * @param {HTMLElement} draggable
	 * @param {boolean} shouldSwap
	 * @return {boolean} Erfolg
	 */
	function moveLeft(draggable, shouldSwap) {
		var slot = prevSlot(draggable.parentNode);
		if (slot) {
			var occupant = getSlotOccupant(slot);
			if (!occupant) {
				moveInto(slot, draggable);
				return true;
			} else if (shouldSwap) {
				moveInto(draggable.parentNode, occupant);
				moveInto(slot, draggable);
				return true;
			}
		}
		return false;
	}

	/**
	 * Array.prototype.forEach(), nur rückwärts
	 * @param {function} callback(element, thisarg, array)
	 * @param {Node=} thisarg
	 * @return {boolean}
	 */
	Array.prototype.forEachRev = function forEachRev (callback, thisarg) {
		for (var i = this.length - 1; i >= 0; i--) {
			callback.call(thisarg, this[i], i, this);
		}
	};

	/**
	 * @param {HTMLElement} draggable
	 * @param {string} targetArea ('drag', 'drop')
	 * @return {boolean}
	 */
	function isDropAllowed(draggable, targetArea) {
		if (!enforceDeclineButton) {
			return true;
		}

		var draggingAcceptButton = draggable.dataset.value == 'accept';
		var draggingDeclineButton = draggable.dataset.value == 'decline';

		var isAcceptButtonInDropArea = $('.btn-drop-area .btn-draggable[data-value="accept"]').length > 0;
		var isDeclineButtonInDragArea = $('.btn-drag-area .btn-draggable[data-value="decline"]').length > 0;

		// Not enforcing the decline button
		// or moving the accept button to the drop area without the decline button
		// or moving the decline button to the drag area without the accept button
		return !(
			enforceDeclineButton && (
				draggingAcceptButton && draggingFromDragArea && isDeclineButtonInDragArea && targetArea == 'drop'
				|| draggingDeclineButton && draggingFromDropArea && isAcceptButtonInDropArea && targetArea == 'drag'
			)
		);
	}

	var enforceDeclineButton = !!$('.btn-drop-area').data('decline-button-enforce');
	var dragAndDropAllowed, draggingFromDragArea, draggingFromDropArea;
	$('[draggable="true"][data-value]').each(function () {
		if (!this.id) {
			// Zufällige ID setzen, falls keine manuell gesetzt
			while (true) {
				var id = 'x'+((Math.random()*(1<<30))|0);
				if (!document.getElementById(id)) {
					this.id = id;
					break;
				}
			}
		}
		if (this.tabIndex === -1) {
			this.tabIndex = 0;
		}
		if (this.childNodes.length == 1 || this.children.length == 0) {
			this.setAttribute("aria-atomic", true);
		}

		var $self = $(this);
		$self.find(".dropdown").prop("draggable", true).on("dragstart", function (evt) {
			evt.preventDefault();
			evt.stopPropagation();
		}).on("keydown", function (evt) {
			evt.stopPropagation();
		});

		function updateExtraText() {
			var $checkboxes = $self.find('input[type="checkbox"]');
			var $checked = $checkboxes.filter(":checked");
			var $unchecked = $checkboxes.filter(":not(:checked)");
			var worldIsChecked = $checked.filter('[value="world"]').length == 1;
			if ($checkboxes.length == $checked.length) {
				this.innerText = '';
			} else {
				if (worldIsChecked) {
					var template = this.getAttribute('data-template-inverse');
					var values = $unchecked.map(function () { return this.getAttribute('data-extra'); });
					this.innerText = template.replace('%s', Array.prototype.join.call(values, ", "));
				} else {
					var template = this.getAttribute('data-template');
					var values = $checked.map(function () { return this.getAttribute('data-extra'); });
					this.innerText = template.replace('%s', Array.prototype.join.call(values, ", "));
				}
			}
		}

		$self.find(".extra").each(updateExtraText);
		$self.find('input[type="checkbox"]').on("click", function () {
			$self.find(".extra").each(updateExtraText);
		})
	}).on("dragstart", function (evt) {
		if ('originalEvent' in evt) {
			evt = evt.originalEvent;
		}
		var self = this;
		evt.dataTransfer.setData("text/x-html-id", self.id);
		evt.dataTransfer.setData("text/plain", self.getAttribute("data-value"));
		evt.dataTransfer.effectAllowed = "move";

		dragAndDropAllowed = true;
		draggingFromDragArea = $(this).closest('.btn-drag-area').length > 0;
		draggingFromDropArea = !draggingFromDragArea;

		self.setAttribute("data-dragging", "");

		setTimeout(function () {
			self.classList.add("dragging");
		}, 76)
	}).on("dragend", function (evt) {
		this.removeAttribute("data-dragging");
		this.classList.remove("dragging");
		if (!dragAndDropAllowed) {
			var message = $(this).closest('.panel-body').data('alert-msg').replace(/\\n/g, '\n');
			alert(message);
		}
	}).on("keydown", function (evt) {
		if (evt.metaKey || evt.shiftKey || evt.altKey || evt.ctrlKey) {
			return;
		}
		if (evt.key == "ArrowUp" || evt.key == "Enter") {
			if (this.parentNode.classList.contains("btn-drag-area")) {
				for (var slot of document.getElementsByClassName("drop-slot")) {
					if (!slot.querySelector(':scope > [draggable="true"]')) {
						moveInto(slot, this);
						evt.preventDefault();
						this.focus();
						break;
					}
				}
			}
		}
		else if (evt.key == "ArrowDown" || evt.key == "Delete") {
			if (this.parentNode.classList.contains("drop-slot")) {
				moveInto(document.getElementsByClassName("btn-drag-area")[0], this);
				evt.preventDefault();
				this.focus();
			}
		}
		else if (evt.key == "ArrowLeft") {
			if (moveLeft(this, true)) {
				evt.preventDefault();
				this.focus();
			}
		}
		else if (evt.key == "ArrowRight") {
			if (moveRight(this, true)) {
				evt.preventDefault();
				this.focus();
			}
		}
	}).on("transitionstart", function () {
		this.setAttribute("data-transitioning", "");
	}).on("transitionend", function () {
		this.removeAttribute("data-transitioning");
	});

	$(".btn-drop-area").each(function () {
		var dropArea = this;
		this.setAttribute("role", "group");
		if (this.hasAttribute("data-decline-button-enforce")) {
			$(this).find("input").on("change", function (evt) {
				var firstEmpty = null;
				var hasAccept = false;
				var hasDecline = false;
				for (var element of $(dropArea).find("input")) {
					if (element.value == '') {
						if (!firstEmpty) {
							firstEmpty = element;
						} else {
							this.setCustomValidity('');
						}
					} else {
						if (element.value == 'accept') {
							hasAccept = true;
						}
						else if (element.value == 'decline') {
							hasDecline = true;
						}
						this.setCustomValidity('');
					}
				}
				if (firstEmpty) {
					var text = dropArea.getAttribute("data-decline-button-enforce");
					text = text ? text : '';
					firstEmpty.setCustomValidity((hasAccept && !hasDecline) ? text : '');
				}
			});
		}
	})

	$('.drop-slot').each(function () {
		this.setAttribute("aria-live", "true");
		var input = this.getElementsByTagName("input")[0];
		if (!input) {
			return;
		}
		input.setAttribute("aria-hidden", "true");
		input.tabIndex = -1;
		var value = input.value;
		for (var item of document.querySelectorAll('[draggable="true"]')) {
			if (item.getAttribute("data-value") == value) {
				this.appendChild(item);
				break;
			}
		}
	}).on("dragover", function (evt) {
		if ('originalEvent' in evt) {
			evt = evt.originalEvent;
		}
		// Drag über animiertes Kind-Element ignorieren. Verhindert Bounces
		var target = evt.target;
		while (target && target !== this) {
			if (target.hasAttribute("data-transitioning")) {
				evt.preventDefault();
				evt.stopPropagation();
				return;
			}
			target = target.parentNode;
		}
		if (evt.dataTransfer.types.includes("text/x-html-id")) {
			var occupant = this.querySelector(':scope > [draggable="true"]');
			if (occupant && !occupant.hasAttribute("data-dragging")) {
				var offsetX = calcOffsetX(evt, this);
				if (offsetX < (this.offsetWidth / 2 - 8)) {
					var nextSlots = nextSlotsUntilGap(this);
					if (nextSlots.length > 0) {
						this.setAttribute("data-drop-side", "left");
						occupant.style.translate = (nextSlots[0].offsetLeft - this.offsetLeft) + "px";
						nextSlots.forEach(function (slot, index) {
							var occupant = getSlotOccupant(slot);
							if (occupant && (index+1) in nextSlots) {
								var otherSlot = nextSlots[index+1];
								occupant.style.translate = (otherSlot.offsetLeft - slot.offsetLeft) + "px";
							}
						});
						evt.preventDefault();
						evt.dataTransfer.dropEffect = "move";
						this.classList.add("dragover");
					} else {
						this.removeAttribute("data-drop-side");
						occupant.style.translate = "";
						nextSlots.forEach(function (slot) {
							var occupant = getSlotOccupant(slot);
							if (occupant) {
								occupant.style.translate = "";
							}
						});
					}
				}
				else if (offsetX > (this.offsetWidth / 2 + 8)) {
					var prevSlots = prevSlotsUntilGap(this);
					if (prevSlots.length > 0) {
						this.setAttribute("data-drop-side", "right");
						occupant.style.translate = (prevSlots[0].offsetLeft - this.offsetLeft) + "px";
						prevSlots.forEach(function (slot, index) {
							var occupant = getSlotOccupant(slot);
							if (occupant && (index+1) in prevSlots) {
								var otherSlot = prevSlots[index+1];
								occupant.style.translate = (otherSlot.offsetLeft - slot.offsetLeft) + "px";
							}
						});
						evt.preventDefault();
						evt.dataTransfer.dropEffect = "move";
						this.classList.add("dragover");
					} else {
						this.removeAttribute("data-drop-side");
						occupant.style.translate = "";
						prevSlots.forEach(function (slot) {
							var occupant = getSlotOccupant(slot);
							if (occupant) {
								occupant.style.translate = "";
							}
						});
					}
				}
				else {
					if (this.hasAttribute("data-drop-side")) {
						evt.preventDefault();
						evt.dataTransfer.dropEffect = "move";
						this.classList.add("dragover");
					}
				}

			} else {
				evt.preventDefault();
				evt.dataTransfer.dropEffect = "move";
				this.removeAttribute("data-drop-side");
				this.classList.add("dragover");
			}
		}
	}).on("dragleave", function (evt) {
		if (!checkElementLeave(evt, this)) {
			return;
		}
		this.removeAttribute("data-drop-side");
		this.classList.remove("dragover");
		Array.prototype.forEach.call(this.parentNode.children, function (node) {
			var occupant = getSlotOccupant(node);
			if (occupant) {
				occupant.style.translate = "";
			}
		});
	}).on("drop", function (evt) {
		this.classList.remove("dragover");
		Array.prototype.forEach.call(this.parentNode.children, function (node) {
			var occupant = getSlotOccupant(node);
			if (occupant) {
				occupant.style.translate = "";
			}
		});
		if ('originalEvent' in evt) {
			evt = evt.originalEvent;
		}
		if (!evt.dataTransfer.types.includes("text/x-html-id")) {
			return;
		}
		evt.preventDefault();
		var id = evt.dataTransfer.getData("text/x-html-id");
		var item = document.getElementById(id);

		if (!isDropAllowed(item, 'drop')) {
			dragAndDropAllowed = false;
			return;
		}

		var dropSide = this.getAttribute("data-drop-side");
		var occupant = this.querySelector(':scope > [draggable="true"]');
		if (occupant && dropSide) {
			occupant.style.translate = "";
			if (dropSide == 'left') {
				var nextSlots = nextSlotsUntilGap(this);
				if (nextSlots.length === 0) {
					return;
				}
				nextSlots.unshift(this);
				nextSlots.forEachRev(function (slot, index) {
					if (index+1 < nextSlots.length) {
						var occupant = getSlotOccupant(slot);
						if (occupant) {
							moveRight(occupant, true);
						}
					}
				});
			} else {
				var prevSlots = prevSlotsUntilGap(this);
				if (prevSlots.length === 0) {
					return;
				}
				prevSlots.unshift(this);
				prevSlots.forEachRev(function (slot, index) {
					if (index+1 < prevSlots.length) {
						var occupant = getSlotOccupant(slot);
						if (occupant) {
							moveLeft(occupant, true);
						}
					}
				});
			}
		}

		moveInto(this, item);
	});

	$(".btn-drag-area").each(function () {
		this.setAttribute("role", "group");
		if (this.children.length == 0) {
			this.classList.add("empty");
		}
	}).on("dragover", function (evt) {
		if ('originalEvent' in evt) {
			evt = evt.originalEvent;
		}
		if (evt.dataTransfer.types.includes("text/x-html-id")) {
			evt.preventDefault();
			evt.dataTransfer.dropEffect = "move";
			this.classList.add("dragover");
		}
	}).on("dragleave", function (evt) {
		if (checkElementLeave(evt, this)) {
			this.classList.remove("dragover");
		}
	}).on("drop", function (evt) {
		this.classList.remove("dragover");
		if ('originalEvent' in evt) {
			evt = evt.originalEvent;
		}
		if (!evt.dataTransfer.types.includes("text/x-html-id")) {
			return;
		}
		var id = evt.dataTransfer.getData("text/x-html-id");
		var element = document.getElementById(id);

		if (!isDropAllowed(element, 'drag')) {
			dragAndDropAllowed = false;
			return;
		}

		this.classList.remove('empty');
		evt.preventDefault();

		if (element.parentNode != this) {
			moveInto(this, element);
		}
	});
})();
