<?php

use App\Utils;
use Symfony\Component\Dotenv\Dotenv;

function render_header($title, $headline='Rescue')
{
	?><!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" lang="en">
<head>
	<meta charset="UTF-8" />
	<title><?php echo htmlspecialchars($title); ?></title>
	<link rel="stylesheet" href="css/theme.css" type="text/css" />
</head>
<body>
	<div id="main">
		<header class="navbar navbar-fixed-top navbar-shadow bg-dark">
			<h1 class="navbar-branding bg-dark dark">
				<a class="navbar-brand" href="./">
					<img src="logo/backend" alt="" width="158" height="40" />
				</a>
				<span id="toggle_sidemenu_l" class="ad ad-lines"></span>
			</h1>
			<ul class="nav navbar-nav navbar-left">
				<li>
					<ol class="breadcrumb hidden-xs">
						<li class="crumb-active crumb-trail">
							<h1 style="font: inherit; display: inline;"><?php echo htmlspecialchars($headline); ?></h1>
							<span class="fa fa-cogs"></span>
						</li>
					</ol>
				</li>
			</ul>
		</header>
		<nav id="sidebar_left" class="nano nano-light affix">
			<div class="sidebar-left-content nano-content">
				<ul class="nav sidebar-menu">
					<li>
						<a href="./">
							<span class="fa fa-arrow-left"></span>
							<span class="sidebar-title">Back</span>
						</a>
					</li>
				</ul>
			</div>
		</nav>
		<div id="content_wrapper">
			<main id="content" class="animated fadeIn container">
				<div class="bounded-width"><?php
}

function render_footer()
{
	?></div></main>
	</div></div>
	<!-- jQuery -->
	<script src="js/jquery-3.6.1.min.js"></script>
	<script src="js/jquery-ui.min.js"></script>

	<!-- Theme Javascript -->
	<script src="js/utility.js"></script>
	<script src="js/main.js"></script>
	<script src="js/helpers.js"></script>

	<!-- Widget Javascript -->
	<script type="text/javascript">
		jQuery(function() {
			'use strict';
			// Init Theme Core
			Core.init();
		});
	</script>
</body>
</html>
	<?php
}

function clearCaches()
{
	$cacheBaseDir = dirname(__DIR__).'/var/cache/';
	foreach (scandir($cacheBaseDir, SCANDIR_SORT_NONE) as $filename) {
		$path = $cacheBaseDir.$filename;
		if (substr($filename, 0, 1) === '.' or $filename === 'CACHEDIR.TAG' or !is_dir($path)) {
			continue;
		}
		if (strpos($filename, '.tmp') !== false) {
			continue;
		}
		if (rename($path, $path.'.tmp')) {
			Utils::removeDirectory($path.'.tmp');
		}
	}
}

include dirname(__DIR__).'/vendor/autoload.php';

(new Dotenv())->bootEnv(dirname(__DIR__).'/.env');

include dirname(__DIR__).'/src/Utils.php';

$baseKey = hash_pbkdf2("sha256", $_SERVER['APP_SECRET'], 'rescue', 1000);
$ts = (int)(time()/600);
$keys = [
	substr(hash_hmac('sha256', $ts-1, $baseKey), 0, 24),
	substr(hash_hmac('sha256', $ts, $baseKey), 0, 24),
	substr(hash_hmac('sha256', $ts+1, $baseKey), 0, 24)
];

$providedKey = $_REQUEST['token'] ?? '';
$keyOk = array_reduce($keys, function ($carry, $key) use ($providedKey) {
	return hash_equals($key, $providedKey) || $carry;
}, false);

if (!$keyOk) {
	http_response_code(403);
	header("Refresh: 3;url=./");
	render_header('Access denied');
	echo '<div class="alert alert-danger">Access denied.</div>';
	render_footer();
	exit();
}

$action = $_POST['action'] ?? '';

switch ($action) {

	case 'clear_caches':
		clearCaches();
		header("Refresh: 3;url=./");
		render_header('Rescue mode');
		echo '<div class="alert alert-success">Caches were successfully cleared.</div>';
		render_footer();

		break;

	default:
		render_header('Rescue mode');
		echo '<form action="" method="POST"><button type="submit" name="action" value="clear_caches" class="btn btn-danger"><i class="fa fa-fw fa-trash"></i> Clear all caches</button></form>';
		render_footer();
}
