<?php
namespace App\Command\Env;

use App\Utils;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class SetCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:env:set';

	protected function configure(): void
	{
		$this
			->setDescription('Set an environment variable.')
			->setDefinition([
				new InputArgument('key', InputArgument::REQUIRED),
				new InputArgument('value', InputArgument::OPTIONAL),
				new InputOption('unset', 'u', InputOption::VALUE_NONE, 'Unset variable'),
			])
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);

		$key = $input->getArgument('key');
		$value = $input->getArgument('value');
		$unset = $input->getOption('unset');
		if ($value === null and !$unset) {
			$io->error('Not enough arguments (missing: "value" or "--unset").');
			return 1;
		}

		if (strtoupper($key) !== $key) {
			$io->error('Environment variable keys must be uppercase.');
			return 1;
		}

		Utils::setLocalEnvironment([$key => $value]);

		return 0;
	}
}
