<?php
namespace App\Command\Ip;

use App\Model\LoginAttemptsLog;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Datetime;

class FailedLoginsCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:ip:failed-logins';

	protected function configure(): void
	{
		$this
			->setDefinition([
				new InputOption('recovery', 'r', InputOption::VALUE_NONE, 'Use password recovery attempts instead of login attempts.'),
				new InputOption('clear', 'c', InputOption::VALUE_NONE, 'Clear log and reset blockades.'),
			])
			->setDescription('Show IP addresses that entered a wrong password in the last few minutes.')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);

		$kind = ($input->getOption('recovery')) ? 'recovery' : 'login';
		$ips = $this->makeIPList($kind);

		$io->table(['IP address', 'Attempts', 'Blocked until'], array_map(function (array $row) {
			return [
				$row['ip'],
				$row['attempts'],
				$row['blocked_until'] ? $row['blocked_until']->format('Y-m-d H:i:s') : '',
			];
		}, $ips));

		if ($input->getOption('clear')) {
			LoginAttemptsLog::clear($kind);
		}

		return 0;
	}

	public static function makeIPList($kind): array
	{
		$attempts = LoginAttemptsLog::forKind($kind)->all();

		$start = (new Datetime('- '.LoginAttemptsLog::TIMEWINDOW_MINUTES.' minutes'));

		$result = [];

		foreach ($attempts as $row) {
			$ip = $row['ip'];
			$date = new Datetime();
			$date->setTimestamp($row['ts']);
			if (isset($result[$ip])) {
				$result[$ip]['attempts']++;
				$result[$ip]['last'] = $date;
				if ($result[$ip]['attempts'] > 5) {
					$date = (clone $result[$ip]['first']);
					$date->modify('+ '.LoginAttemptsLog::TIMEWINDOW_MINUTES.' minutes');
					$result[$ip]['blocked_until'] = $date;
				}
			} else {
				$result[$ip] = [
					'ip' => $ip,
					'first' => $date,
					'last' => $date,
					'attempts' => 1,
					'blocked_until' => null,
				];
			}
		}
		return array_values($result);
	}
}
