<?php
namespace App\Command\Plugins;

use App\Component\Plugin\Manager;
use App\Model\PluginState;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputArgument;


class PluginsInstallCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:plugins:install';

	protected function configure(): void
	{
		$this
			->addArgument("pluginName",InputArgument::REQUIRED, 'PluginName muss angegeben werden.')
			->setDescription('Installiert ein Plugin')
			->setHelp("Diese Command installiert ein Plugin das schon im Pluginsverzeichnis liegt.")
		;
	}

	/**
	 * @param InputInterface $input
	 * @param OutputInterface $output
	 * @return int
	 */
	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);
		$InstalledPlugins = PluginState::all();
		$isInstalled = false;
		$isNowInstalled = false;
		foreach($InstalledPlugins as $onePlugin)
		{
			if ($input->getArgument('pluginName') == $onePlugin->getDirname())
			{
				$io->error('Das Plugin wurde schon installiert');
				$isInstalled = true;
			};
		}

		//Noch nicht installiert
		if($isInstalled==false) {
			//$io->text("Noch nicht installiert - also los");
			$manager = new Manager();
			$available = $manager->installedPlugins();
			//$io->text("Available Plugins durchloopen");
			foreach ($available as $k => $v) {
				//$io->text("PLugin: ".$v->getDirname());
				//Ok - das Plugin liegt tatsächlich vor UND es gibt Daten dazu
				if ($v->getDirname() == $input->getArgument('pluginName')) {
					//Jetzt installieren
					$state = PluginState::create();
					$state->setName($v->getLabel());
					$state->setType($v->getType());
					$state->setVersion($v->getVersion());
					$state->setDirname($v->getDirname());
					$state->setPluginUID($v->getName());
					$state->setIcon($v->getIcon());
					if (!$v->isActivatablePerDomain()) {
						$state->setActivatedForAllAccounts(true);
					}
					$state->save();
					$isNowInstalled = true;
					$io->success('Das Plugin wurde ' . $v->getDirname() . ' installiert');
				}
			}
		}
		if($isNowInstalled)
		{
			$io->success("Installation hat geklappt.");
		}
		else
		{
			if($isInstalled==false)
			{
				$io->error("Installation hat Nicht geklappt - das Plugin gibt es so nicht.");
			}
		}
		$io->text("THE END MY FRIEND");

		return 0;
	}
}
