<?php
namespace App\Command\Update;

use App\Component\Update;
use App\Utils;
use App\Model\VersionLog;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class DowngradeCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:downgrade';

	/** @var Update $updater */
	private $updater;

	public function __construct(Update $updater)
	{
		$this->updater = $updater;
		parent::__construct();
	}

	protected function configure(): void
	{
		$this
			->setDefinition([
				new InputOption('previous', '1', InputOption::VALUE_NONE, 'Downgrade to the previously installed version without any user interaction.'),
			])
			->setDescription('Downgrade to a previously installed version')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);

		// Versionslog öffnen
		$versionId = Utils::getVersionId();
		$previousVersions = VersionLog::all();
		usort($previousVersions, function (VersionLog $a, VersionLog $b) {
			return $b->getFirstInstallDate() <=> $a->getFirstInstallDate();
		});
		$previousVersions = array_values(array_filter($previousVersions, function (VersionLog $item) use ($versionId) {
			return $item->getId() != $versionId;
		}));
		$versionLabels = array_map(function (VersionLog $v) {
			return $v->getVersionName().' ('.$v->getId().')';
		}, $previousVersions);

		if (!$previousVersions) {
			$choice = $io->error('Downgrade not possible. No previously installed version found.');
			return 1;
		}

		if ($input->getOption('previous')) {
			$versionId = $previousVersions[0]->getId();
		} else {
			$choice = $io->choice('Please select a version to downgrade to', $versionLabels, 0);
			$index = array_search($choice, $versionLabels, true);
			$versionId = $previousVersions[$index]->getId();
		}

		$io->comment('Downgrading to version '.$versionId);

		$updateCmd = new UpdateCommand($this->updater);
		return $updateCmd->perform($io, $versionId, false, false);
	}
}
