<?php
namespace App\Command\User;

use App\Model\User;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class DeleteAllUserCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:user:deleteall';

	protected function configure(): void
	{
		$this
			->setDefinition([
				new InputOption('i-know-what-i-am-doing', '', InputOption::VALUE_NONE, 'This option has to be set for this command to work.'),
			])
			->setDescription('Delete all client users irrecovably from the database.')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);

		if (!$input->getOption('i-know-what-i-am-doing')){
			$io->error("Safety option --i-know-what-i-am-doing wasn't set. Aborting.");
			return 1;
		}

		if ($io->confirm("Wirklich alle User löschen?", false)) {
			if ($io->confirm("Wirklich WIRKLICH alle User löschen? Das kann nicht rückgängig gemacht werden. Am besten einmal vorher alles exportieren!!", false)) {

				$users = $this->makeSortedUserList();
				$io->text(" User löschen ");

				User::beginWrite();

				foreach ($users as $oneUser) {
					/** @var User|null $client */
					$client = User::byId($oneUser->getId());

					if ($client == null || $client->getRole() != User::ROLE_CLIENT || $client->isAccountOwner() == false) {
						continue;
					}
					$username = $client->getUsername();

					//Subsuser vorher löschen...
					$allUSer = $client->getSubusers();
					foreach ($allUSer as $k => $subuser) {
						// Repo nicht zwischenspeichern, um keine Performance zu verlieren
						User::delete($subuser->getId(), false);
					}

					User::delete($client->getId());
					$io->text("<info>".$username . "</info> wurde gelöscht.");
				}
			}
		}

		return 0;
	}

	public static function makeSortedUserList(): array
	{
		$users = User::all();

		usort($users, function (User $a, User $b) {
			$aOwner = $a->getAccountOwner();
			$bOwner = $b->getAccountOwner();
			$aName = ($aOwner->getId() != $a->getId()) ? $aOwner->getUsername().'!'.$a->getUsername() : $a->getUsername();
			$bName = ($bOwner->getId() != $b->getId()) ? $bOwner->getUsername().'!'.$b->getUsername() : $b->getUsername();
			return ($a->getRole() <=> $b->getRole()) ?: ($aName <=> $bName) ?: ($a->getId() <=> $b->getId());
		});

		return $users;
	}
}
