<?php
namespace App\Command\User;

use App\Model\User;
use App\Utils;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Filesystem\Filesystem;

class JsonCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:user:json';

	protected function configure(): void
	{
		$this
			->setDescription('Create a Json of User Data, same as API Export User.')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);
		$filesystem = new Filesystem();
		$users = $this->makeSortedUserList();
		$progressBar = new ProgressBar($output, count($users) );

		$io->comment("Exporting User Data to Json File.");
		$progressBar->start();
		$json = json_encode(array_map(static function (User $user) use ($progressBar) {
			$progressBar->advance();
			return [
				$user->getId(),
				$user->getUsername(),
				$user->isSubuser() ? 'subuser' : $user->getRole(),
				($user->isActive() or $user->getRole() == User::ROLE_ADMIN) ? 'x' : ' ',
				$user->isSubuser() ? $user->getAccountOwner()->getId() : '',
				$user->getEmailAddress(),
				$user->getFirstName(),
				$user->getLastName(),
				$user->getCompany(),
				$user->getApiKey(),
			];
		}, $users));
		$progressBar->finish();
		$io->newLine(2);
		$exportDir = Utils::getVarDir().'/export';
		$filesystem->mkdir($exportDir);
		$filesystem->dumpFile($exportDir.'/userExport.json', $json);
		$io->success("Exporting User Data to Json File - DONE - Location: ".$exportDir."/userExport.json");


		return 0;
	}

	public static function makeSortedUserList(): array
	{
		$users = User::all();

		usort($users, function (User $a, User $b) {
			$aOwner = $a->getAccountOwner();
			$bOwner = $b->getAccountOwner();
			$aName = ($aOwner->getId() != $a->getId()) ? $aOwner->getUsername().'!'.$a->getUsername() : $a->getUsername();
			$bName = ($bOwner->getId() != $b->getId()) ? $bOwner->getUsername().'!'.$b->getUsername() : $b->getUsername();
			return ($a->getRole() <=> $b->getRole()) ?: ($aName <=> $bName) ?: ($a->getId() <=> $b->getId());
		});

		return $users;
	}
}
