<?php
namespace App\Command\User;

use App\Model\User;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

class ListCommand extends Command
{
	// the name of the command (the part after "bin/console")
	protected static $defaultName = 'app:user:list';

	protected function configure(): void
	{
		$this
			->setDescription('List users.')
		;
	}

	protected function execute(InputInterface $input, OutputInterface $output): int
	{
		$io = new SymfonyStyle($input, $output);

		$users = $this->makeSortedUserList();

		$io->table(['ID', 'Name', 'Role', 'Active', 'Owner', 'Email', 'First name', 'Last name', 'Company'], array_map(function (User $user) {
			return [
				$user->getId(),
				$user->getUsername(),
				$user->isSubuser() ? 'subuser' : $user->getRole(),
				($user->isActive() or $user->getRole() == User::ROLE_ADMIN) ? 'x' : ' ',
				$user->isSubuser() ? $user->getAccountOwner()->getId() : '',
				$user->getEmailAddress(),
				$user->getFirstName(),
				$user->getLastName(),
				$user->getCompany(),
			];
		}, $users));


		return 0;
	}

	public static function makeSortedUserList(): array
	{
		$users = User::all();

		usort($users, function (User $a, User $b) {
			$aOwner = $a->getAccountOwner();
			$bOwner = $b->getAccountOwner();
			$aName = ($aOwner->getId() != $a->getId()) ? $aOwner->getUsername().'!'.$a->getUsername() : $a->getUsername();
			$bName = ($bOwner->getId() != $b->getId()) ? $bOwner->getUsername().'!'.$b->getUsername() : $b->getUsername();
			return ($a->getRole() <=> $b->getRole()) ?: ($aName <=> $bName) ?: ($a->getId() <=> $b->getId());
		});

		return $users;
	}
}
