<?php

namespace App\Component\Config;

use Symfony\Component\HttpKernel\Controller\ArgumentValueResolverInterface;
use Symfony\Component\HttpKernel\ControllerMetadata\ArgumentMetadata;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Convert Model instances from request attribute variable.
 */
class ModelParamConverterService implements ArgumentValueResolverInterface
{
	public function supports(Request $request, ArgumentMetadata $metadata): bool
	{
		$argumentType = $metadata->getType();
		return $argumentType
			&& !$metadata->isVariadic()
			&& $request->attributes->has($metadata->getName())
			&& is_subclass_of($argumentType, ModelInterface::class, true);
	}

	private function resolveInner(Request $request, ArgumentMetadata $metadata): ?ModelInterface
	{
		$argumentType = $metadata->getType();
		assert(is_subclass_of($argumentType, ModelInterface::class));
		$param = $metadata->getName();
        $value = $request->attributes->get($param);

		if ($value === '' or $value === null) {
			if ($metadata->hasDefaultValue()) {
				return $metadata->getDefaultValue();
			}
			return null;
		}

		if (!is_string($value)) {
			return null;
		}
		return $argumentType::find($value);
	}

	/**
	 * @return array{0:ModelInterface|null}
	 */
	public function resolve(Request $request, ArgumentMetadata $metadata): iterable
	{
		$result = $this->resolveInner($request, $metadata);

		if ($result === null and !$metadata->isNullable()) {
			$param = $metadata->getName();
			throw new NotFoundHttpException("No such $param found");
		}
		return [$result];
	}
}
