<?php

namespace App\Component\Cron;

use App\Config;

abstract class Job extends JobBase
{
	/** @var string $namespace */
	private $namespace;

	public function __construct(int $intervalSeconds)
	{
		parent::__construct($intervalSeconds);
		$this->namespace = 'cron.jobs.'.lcfirst($this->getName());
	}

	/** @override */
	public function reinitialize(): void
	{}

	/**
	 * @override
	 * @param string $identifier
	 * @return mixed
	 */
	protected final function readConfig(string $identifier)
	{
		return Config::getInstance()->get($this->namespace.'.'.$identifier);
	}

	/**
	 * @override
	 * @param string $identifier
	 * @param mixed $data
	 */
	protected final function writeConfig(string $identifier, $data): void
	{
		Config::getInstance()->set($this->namespace.'.'.$identifier, $data);
	}

	/**
	 * Diese Methode aktualisiert den Zeitstempel der letzten Ausführung des Jobs.
	 * @override
	 */
	protected final function touchLastExecutionTimestamp(): void
	{
		$this->writeConfig('lastExecutionTimestamp', (int)time());
		Config::getInstance()->flush();
	}

	/**
	 * @override
	 * @return int
	 */
	public final function lastExecutionTimestamp(): int
	{
		return (int)$this->readConfig('lastExecutionTimestamp');
	}

}
