<?php

namespace App\Component\Cron\Job;

use App\Component\Cron\DomainJob;
use App\Component\EmbeddingUpdateService;
use App\Model\Domain;
use App\Model\Embedding;
use App\Model\EmbeddingAsset;
use App\Model\User;
use App\Component\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;

class DomainEmbeddingUpdate extends DomainJob
{
	const COMPONENT = 'EmbeddingUpdate';

	/** @var LoggerInterface */
	private $logger;

	/** @var EmbeddingUpdateService */
	private $updater;

	public function __construct(LoggerInterface $logger, EmbeddingUpdateService $updater)
	{
		parent::__construct(60 * 60 * 6);
		$this->logger = $logger;
		$this->updater = $updater;
	}

	public function run(Request $request): void
	{
		$this->touchLastExecutionTimestamp();

		$user = User::loggedInUser();
		assert($user instanceof User);

		if ($user->isAllowAutoUpdateOfEmbeddings() == false) {
			return;
		}

		$counter = 0;
		foreach (Embedding::all() as $embedding) {
			if ($embedding->isAutoUpdated()) {
				if ($this->updater->updateEmbedding($embedding)) {
					$counter++;
				}
			}
		}
		EmbeddingAsset::flush();
		Embedding::flush();

		if ($counter) {
			$this->logger->logInfo(self::COMPONENT, sprintf('Updated %d embedding(s) in domain "%s" of user "%s"', $counter, Domain::activeDomain()->getName(), $user->getUsername()));
		}
	}
}
