<?php

namespace App\Component\Cron\Job;

use App\Component\Cron\Job;
use App\Model\Domain;
use App\Model\User;
use App\Utils;
use Symfony\Component\HttpFoundation\Request;

class PlanIntegrity extends Job
{
	public function __construct()
	{
		parent::__construct(60 * 20);
	}

	public function run(Request $request): void
	{
		$this->touchLastExecutionTimestamp();

		// Wenn kein Lizenzschlüssel hinterlegt ist oder nicht in Hostingversion, Cronjob als erledigt betrachten
		if (Utils::hasLicenseKey() == false or Utils::isBaseEdition()) {
			return;
		}

		if (Utils::isClickskeksCloud()) {
			return;
		}

		$clients = User::getClients();

		if(Utils::isMultiDomainEdition())
		{
			$clients = User::byRole(User::ROLE_ADMIN);
		}

		$this->checkIntegrity($clients);
	}

	private function checkIntegrity($clients): void
	{
		try {
			array_walk($clients, function ($user) {
				/** @var User $user */
				Domain::switchToUser($user, false);

				// Frontend-Widget nur für so viele Domains aktivieren, wie im Tarif vereinbart
				$widgetCount = Domain::frontendWidgetCountForAllDomains();
				$widgetMaxCount = $user->getMaxDomainCount();

				/** @var Domain[] $domains */
				$domains = array_filter(Domain::all(), function ($domain) {
					/** @var Domain $domain */
					return $domain->isFrontendWidgetEnabled();
				});

				while (count($domains) > 0 && $widgetCount > $widgetMaxCount && $widgetMaxCount > -1) {
					array_pop($domains)->setFrontendWidgetEnabled(false);
					$widgetCount--;
				}

				// Anzahl verwendeter Whitelabel-Lizenzen überprüfen; ggf. streichen
				$whitelabelCount = Domain::whitelabelCountForAllDomains();
				$whitelabelMaxCount = $user->getMaxWLCount();

				/** @var Domain[] $domains */
				$domains = array_filter(Domain::all(), function ($domain) {
					/** @var Domain $domain */
					return $domain->hasWhitelabel();
				});

				while (count($domains) > 0 && $whitelabelCount > $whitelabelMaxCount && $whitelabelMaxCount > -1) {
					array_pop($domains)->setWhitelabel(false);
					$whitelabelCount--;
				}

				// Änderungen abschließend schreiben
				Domain::flush();
			});
		} finally {
			if ($user = User::loggedInUser()) {
				Domain::switchToUser($user, false);
			}
		}
	}
}
