<?php

namespace App\Component\Cron\Job;

use App\Component\Cron\Job;
use App\Component\EmbeddingUpdateService;
use App\Model\SystemEmbedding;
use App\Model\SystemEmbeddingAsset;
use App\Model\User;
use App\Component\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;

class SystemEmbeddingUpdate extends Job
{
	const COMPONENT = 'EmbeddingUpdate';

	/** @var LoggerInterface */
	private $logger;

	/** @var EmbeddingUpdateService */
	private $updater;

	public function __construct(LoggerInterface $logger, EmbeddingUpdateService $updater)
	{
		parent::__construct(60 * 60 * 12);
		$this->logger = $logger;
		$this->updater = $updater;
	}

	public function run(Request $request): void
	{
		$this->touchLastExecutionTimestamp();

		$user = User::loggedInUser();
		assert($user instanceof User);

		if ($user->isAllowAutoUpdateOfEmbeddings() == false) {
			return;
		}

		$updated = [];

		foreach (SystemEmbedding::all() as $embedding) {
			if ($embedding->isAutoUpdated()) {
				$success = $this->updater->updateEmbedding($embedding);
				if ($success) {
					$updated[] = ['id'=>$embedding->getId(), 'name'=>$embedding->getName(), 'ts'=>$embedding->getSourceTimestamp()];
				}
			}
		}
		SystemEmbeddingAsset::flush();
		SystemEmbedding::flush();

		foreach ($updated as $row) {
			$this->logger->logInfo(self::COMPONENT, sprintf('Updated system embedding %s "%s" from embedding database.', $row['id'], $row['name']));
		}
	}
}
