<?php

namespace App\Component\Cron;

use Psr\Container\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

abstract class JobBase
{
	/** @var int $intervalSeconds */
	protected $intervalSeconds;

	/** @var ?ContainerInterface $container */
	protected $container;

	/** @var ?Cron $cron */
	protected $cron;

	public function __construct(int $intervalSeconds)
	{
		$this->container = null;
		$this->cron = null;
		$this->intervalSeconds = $intervalSeconds;
	}

	/**
	 * @required
	 * @param ContainerInterface $container
	 */
	public function setContainer(ContainerInterface $container)
	{
		$this->container = $container;
	}

	/**
	 * @required
	 * @param Cron $cron
	 */
	public function setCronManager(Cron $cron)
	{
		$this->cron = $cron;
	}

	/**
	 * Zurücksetzen aller Benutzer-/Domainspezifischen Daten im Job-Objekt
	 */
	abstract public function reinitialize(): void;

	/**
	 * @param string $identifier
	 * @return mixed
	 */
	abstract protected function readConfig(string $identifier);

	/**
	 * @param string $identifier
	 * @param mixed $data
	 */
	abstract protected function writeConfig(string $identifier, $data): void;

	/**
	 * @return string
	 */
	public static function getName(): string
	{
		return array_reverse(explode('\\', static::class))[0];
	}

	/**
	 * @return int
	 */
	public final function getIntervalSeconds(): int
	{
		return $this->intervalSeconds;
	}

	/**
	 * Diese Methode aktualisiert den Zeitstempel der letzten Ausführung des Jobs.
	 */
	abstract protected function touchLastExecutionTimestamp();

	/**
	 * @return int
	 */
	abstract public function lastExecutionTimestamp();

	/**
	 * @return bool
	 */
	public final function isDue(): bool
	{
		return time() > $this->lastExecutionTimestamp() + $this->intervalSeconds;
	}

	/**
	 * Führt diesen Job später über einen Curl-Aufruf aus.
	 */
	public final function scheduleDeferred(): void
	{
		if ($this->cron) {
			$this->cron->addDeferredJob($this);
		}
	}

	abstract public function run(Request $request): void;
}
