<?php

namespace App\Component\Cron;

use App\Model\UserCronState;

abstract class UserJob extends JobBase
{
	/** @var string $namespace */
	private $namespace;

	/** @var UserCronState|null $state */
	private $state = null;

	public function __construct(int $intervalSeconds)
	{
		parent::__construct($intervalSeconds);
		$this->namespace = lcfirst($this->getName());
	}

	protected final function state(): UserCronState
	{
		if (!$this->state) {
			$this->state = UserCronState::findOrCreate($this->namespace);
		}
		return $this->state;
	}

	/** @override */
	public function reinitialize(): void
	{
		$this->state = null;
	}

	/**
	 * @override
	 * @param string $identifier
	 * @return mixed
	 */
	protected final function readConfig(string $identifier)
	{
		return $this->state()->getExtra($identifier);
	}

	/**
	 * @override
	 * @param string $identifier
	 * @param mixed $data
	 */
	protected final function writeConfig(string $identifier, $data): void
	{
		$this->state()->setExtra($identifier, $data)->save();
	}

	/**
	 * Diese Methode aktualisiert den Zeitstempel der letzten Ausführung des Jobs.
	 * @override
	 */
	protected final function touchLastExecutionTimestamp(): void
	{
		$this->writeConfig('lastExecutionTimestamp', (int)time());
		$this->state()->save();
	}

	/**
	 * @override
	 * @return int
	 */
	public final function lastExecutionTimestamp(): int
	{
		return (int)$this->readConfig('lastExecutionTimestamp');
	}
}
