<?php

namespace App\Component;

use Symfony\Component\HttpFoundation\Response;

class CurlResponse extends Response
{
	/** @var mixed|null $jsonData */
	private $jsonData = null;
	/** @var bool $jsonValid */
	private $jsonValid = false;
	/** @var bool $verified */
	private $verified = false;
	/** @var int $jsonError */
	private $jsonError = JSON_ERROR_NONE;

	private function update()
	{
		$type = trim(explode(';', $this->headers->get('Content-Type', ''))[0]);
		if ($type === 'application/json' or $type === 'text/json') {
			$this->jsonData = json_decode($this->content, true);
			$this->jsonError = json_last_error();
			$this->jsonValid = ($this->jsonError === JSON_ERROR_NONE);
		} else {
			$this->jsonData = null;
			$this->jsonError = JSON_ERROR_NONE;
			$this->jsonValid = false;
		}
		$this->verified = false;
	}

	/**
	 * {@inheritdoc}
	 * @override
	 */
	public function __construct($content = '', int $status = 200, array $headers = [])
	{
		parent::__construct($content, $status, $headers);
		$this->update();
	}

	/**
	 * {@inheritdoc}
	 * @override
	 * @return static
	 */
	#[\ReturnTypeWillChange]
	public function setContent(?string $content)
	{
		parent::setContent($content);
		$this->update();
		return $this;
	}

	/**
	 * Set if the response was verified (e.g. with Hawk)
	 *
	 * @return $this
	 */
	public function setVerified(bool $verified)
	{
		$this->verified = $verified;
		return $this;
	}

	/**
	 * If the response was verified (e.g. with Hawk)
	 */
	public function isVerified(): bool
	{
		return $this->verified;
	}

	/**
	 * If the content type is JSON and the data is decodeable
	 */
	public function isJson(): bool
	{
		return $this->jsonValid;
	}

	/**
	 * json_decode() error code
	 */
	public function getDecodeErrorCode(): int
	{
		return $this->jsonError;
	}

	/**
	 * Decoded JSON data
	 *
	 * @return mixed
	 */
	public function getData()
	{
		return $this->jsonData;
	}
}
