<?php

namespace App\Component\EmbeddingDb;
use App\Component\Curl;
use App\Component\CurlResponse;
use App\Model\EmbeddingAssetBase;
use App\Model\Language;
use App\Utils;
use DateTimeImmutable;
use DateTimeInterface;

class EmbeddingDb
{
	protected const APIURL = 'https://cookiedb.ccm19.de/v2/';
	protected const APIKEY = 'jkn487gbf67igvfg6f2fhfbzg96ibjgbvzug';
	protected const CACHE_FILE = 'embedcachev2_2.json';

	/** @var array|null $upgradeSource */
	private static $upgradeSource = null;


	/**
	 * @param string $path
	 * @param array $query
	 * @param array $headers
	 * @return CurlResponse
	 */
	protected static function fetchApi(string $path, array $query = [], array $headers = [])
	{
		$query['key'] = static::APIKEY;
		$url = static::APIURL . ltrim($path, '/') . '?'.http_build_query($query);
		return static::fetchUrl($url, $headers);
	}

	/**
	 * @param string $url
	 * @param array $headers
	 * @return CurlResponse
	 */
	protected static function fetchUrl(string $url, array $headers = []): CurlResponse
	{
		$curl = new Curl();
		$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 10);
		$curl->setOpt(CURLOPT_TIMEOUT, 30);
		foreach ($headers as $headerkey => $headervalue) {
			if ($headervalue === null) {
				continue;
			} elseif ($headervalue instanceof DateTimeInterface) {
				$headervalue = $headervalue->format('D, d M Y H:i:s').' GMT';
			}
			$curl->setHeader($headerkey, $headervalue);
		}
		$curl->get($url);
		$result = $curl->getResponseObject();
		$curl->close();

		if (($src = $result->headers->get('X-Upgrade-Source'))) {
			self::$upgradeSource = explode(':', $src, 2);
		} else {
			self::$upgradeSource = null;
		}

		return $result;
	}


	/**
	 * @param string $path
	 * @param string $cacheFile
	 * @param array $query
	 * @return bool
	 */
	protected static function fetchApiFile(string $path, string $cacheFile, array $query=[]): bool
	{
		$query['key'] = static::APIKEY;
		$url = static::APIURL . ltrim($path, '/') . '?'.http_build_query($query);
		return static::fetchUrlFile($url, $cacheFile);
	}


	/**
	 * Holt die Daten von der API und speichert sie in einer Datei ohne Umwege damit der Speicherverbrauch möglichst gering ist
	 * Wird dann direkt da reingestreamt
	 *
	 * @param string $url
	 * @param string $cacheFile
	 * @return bool
	 */
	protected static function fetchUrlFile(string $url, string $cacheFile): bool
	{
		$fp = fopen($cacheFile, 'w');
		if($fp)
		{
			$curl = new Curl();
			//CURLOPT_URL
			$curl->setOpt(CURLOPT_URL, $url);
			$curl->setOpt(CURLOPT_CONNECTTIMEOUT, 10);
			$curl->setOpt(CURLOPT_TIMEOUT, 60);
			$curl->setOpt(CURLOPT_FILE, $fp);
			$curl->exec();
			$curl->close();

			//Falls leer oder sonstiger Fehler
			if(empty($curl->error)){
				fclose($fp);
				return true;
			}
		}
		//auf jeden Fall schließen
		@fclose($fp);
		return false;
	}

	/**
	 * @param int|string $purposeId
	 */
	protected static function convertPurposeId($purposeId): string
	{
		if ($purposeId == 1 || $purposeId == 8) {
			return "41ba25c"; //Technisch notwendig
		}
		if ($purposeId == 3) {
			return "cdcbd7c"; //Anzeigen / Ads
		}
		if ($purposeId == 2 || $purposeId == 7) {
			return "a717ff5"; //Analyse / Statistiken
		}
		if ($purposeId == 5) {
			return "7c19e32"; //Personalisierung
		}
		if ($purposeId == 6) {
			return "6cd2721"; //Social Media
		}
		if ($purposeId == 4) {
			return "15c61c3"; //Sonstiges
		}
		return (string)$purposeId;
	}

	/**
	 * @param int|string $storageId
	 */
	protected static function convertStorageType($storageId): string
	{
		switch ((int)$storageId) {
			case 1:
				return EmbeddingAssetBase::STORAGE_TYPE_COOKIE;
			case 2:
				return EmbeddingAssetBase::STORAGE_TYPE_LOCAL_STORAGE;
			case 4:
				return EmbeddingAssetBase::STORAGE_TYPE_SESSION_STORAGE;
			default:
				return EmbeddingAssetBase::STORAGE_TYPE_OTHER;
		}
	}

	/**
	 * Purpose-IDs und Storage-Types übertragen sowie Angaben beim Consent-Cookie anpassen
	 */
	protected static function postprocessEmbedding(array $embedding): array
	{
		$embedding['purpose'] = static::convertPurposeId($embedding['purpose']);
		foreach ($embedding['assets'] ?? [] as $assetKey => $asset) {
			$embedding['assets'][$assetKey]['storageType'] = static::convertStorageType($asset['storageType']);
		}
		//13308
		// Sonderfall CCM19-Consent-Cookie: Wenn nicht Full-Service, dann Daten vom Kunden eintragen
		if ($embedding['id'] == 13308) {
			$embedding['ccm_special'] = true;
			if (Utils::isClickskeksCloud() || stripos($_SERVER['HTTP_HOST'], ".ccm19.de") === false) {
				$embedding['embedding_vendor'] = '{Firma}';
				foreach ($embedding['translations'] as $lang => $translation) {
					$embedding['translations'][$lang]['privacy_policy_url'] = '{Firma_Datenschutz}';
					$embedding['translations'][$lang]['place_of_processing'] = '{Firma_Ort}';
				}
			}
			if (Utils::isClickskeksCloud()) {
				$embedding['name'] = 'Cookie Consent Manager Clickskeks';
				foreach ($embedding['translations'] as $lang => $translation) {
					$embedding['translations'][$lang]['label'] = 'Cookie Consent Manager Clickskeks';
				}
			}
		}
		return $embedding;
	}

	/**
	 * Gibt ein Embedding anhand einer ID aus der DB zurück, falls es seit $timestamp
	 * verändert wurde oder eine der Sprachen in $missingLanguages bereitgestellt wird.
	 *
	 * Wenn es unverändert ist, wird NULL zurückgegeben.
	 * Fehler liefern ein leeres Array.
	 *
	 * @param int $id Embedding-Datenbank-ID
	 * @param DateTimeInterface|null $timestamp
	 * @param string[] $missingLanguages Short-Codes
	 * @return array|null
	 */
	public static function fetchEmbeddingIfModified(int $id, ?DateTimeInterface $timestamp, array $missingLanguages = []): ?array
	{
		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			return [];
		}

		$response = static::fetchApi('/embeddings/'.$id, [], ['If-Modified-Since' => $timestamp]);

		if ($response->getStatusCode() == 304) {
			// Unverändert, Sprachen checken
			$foundLanguages = explode(', ', $response->headers->get('Content-Language', 'de'));
			$fetchableLanguages = array_intersect($missingLanguages, $foundLanguages);

			// Wenn eine verfügbare Sprache fehlt, komplettes Embedding doch laden
			if ($timestamp !== null and $fetchableLanguages) {
				return self::fetchEmbedding($id);
			}
			// Ansonsten unverändert -> NULL
			else {
				return null;
			}
		}

		$result = $response->getData();
		if (!is_array($result) or !$result) {
			return [];
		}

		// Postprocessing
		$result = static::postprocessEmbedding($result);

		// DateTime erstellen
		try {
			$result['timestamp'] = empty($result['lastModified']) ? null : new DateTimeImmutable($result['lastModified']);
		} catch (\Exception $ignore) {
			$result['timestamp'] = null;
		}

		return $result;
	}

	/**
	 * Gibt ein Embedding anhand einer ID aus der DB zurück
	 *
	 * @param int $id Embedding-Datenbank-ID
	 * @return array
	 */
	public static function fetchEmbedding(int $id): array
	{
		return (array)self::fetchEmbeddingIfModified($id, null);
	}

	/**
	 * Gibt Embeddings anhand übergebener IDs aus der DB zurück.
	 *
	 * @param int[] $ids Embedding-Datenbank-IDs
	 * @return array[]
	 */
	public static function fetchEmbeddings(array $ids): array
	{
		$embeddings = array_filter(self::fetchEmbeddingList(), function (array $embedding) use ($ids) {
			return in_array($embedding['id'], $ids);
		});

		return array_combine(
			array_map(function (array $embedding) { return $embedding['id']; }, $embeddings),
			$embeddings
		);
	}

	/**
	 * Gibt Embeddings anhand übergebener Namen aus der DB zurück.
	 *
	 * @param string[] $names Embedding-Datenbank-Namen
	 * @return array[]
	 */
	public static function fetchEmbeddingsByNames(array $names): array
	{
		$embeddings = array_filter(self::fetchEmbeddingList(), static function (array $embedding) use ($names) {
			return in_array(strtolower($embedding['name']), array_map('strtolower', $names),false);
		});

		return array_combine(
			array_map(static function (array $embedding) { return $embedding['name']; }, $embeddings),
			$embeddings
		);
	}

	/**
	 * Gibt die Daten von Embeddings in der DB zurück, die bestimmte Cookies enthalten
	 *
	 * Dabei wird ggf. unscharf gesucht.
	 *
	 * @param string[] $cookienames
	 * @return array[]
	 */
	public static function fetchEmbeddingsByCookieNames(array $cookienames): array
	{
		//Es steht kein curl zur Verfügung... dann nix machen
		if (!Curl::installed()) {
			return [];
		}

		$response = static::fetchApi('/embeddings/', [
			'cookiename'=>implode(',',$cookienames)
		]);

		if (!$response->isSuccessful()) {
			return [];
		}

		$result = $response->getData();

		if (is_array($result)) {
			// Postprocessing
			foreach ($result as $key => $embedding) {
				$result[$key] = static::postprocessEmbedding($embedding);
			}
			return $result;
		}
		else {
			return [];
		}
	}

	/**
	 * Gibt die Daten aller Embeddings in der DB zurück (ggf. gefiltert)
	 *
	 * @param string|null $search
	 * @param Language|null $language Abgleich auf eine bestimmte Übersetzung beschränken.
	 * @return array[]
	 */
	public static function fetchEmbeddingList(?string $search=null, ?Language $language=null): array
	{
		//Es steht kein curl zur Verfügung - dann nichts machen
		if (!Curl::installed()) {
			return [];
		}

		//ini
		$result = array();

		// Cache auslesen
		$cacheFile = Utils::getCacheDir().'/'.static::CACHE_FILE;

		if (file_exists($cacheFile)) {
			$mtime = filemtime($cacheFile);
			// erstmal eine Stunde cachen...
			if (!$mtime || $mtime < (time()-3600)) {
				// alte Datei löschen
				unlink($cacheFile);
			} else {
				$result = json_decode(file_get_contents($cacheFile), true);

				// Fallback, falls was nicht stimmt
				if (!is_array($result) || count($result) < 1300) {
					unlink($cacheFile);
					$result = [];
				}
			}
		}

		if (empty($result)) {

			//Daten runterladen und direkt speichern
			$response = static::fetchApiFile('/embeddings/',$cacheFile);

			// hat nicht geklappt, dann leeres Array
			if (!$response) {
				return [];
			}

			//Jetzt die Daten auslesen
			$result = json_decode(file_get_contents($cacheFile), true);

			// Fallback, falls was nicht stimmt
			if (!is_array($result) || count($result) < 1300) {
				unlink($cacheFile);
				return [];
			}
		}

		// Es wird gesucht.
		if ($search !== null && trim($search) !== '') {
			assert(is_array($result));
			$result = static::filterEmbeddings($result, $search, $language);
		}
		else {
			// Scoring anhand der Reihenfolge der Datenbankeinträge bestimmen
			$score = count($result);
			array_walk($result, static function (array &$embedding) use (&$score) {
				$embedding['@score'] = $score--;
			});
		}

		// Verhindere PhanTypeMismatchReturnNullable
		if (!isset($result) || !is_array($result)) {
			$result = [];
		}

		// Postprocessing
		$result = array_map([static::class, 'postprocessEmbedding'], $result);

		return $result;
	}

	/**
	 * This method filters `$embeddings` by the given `$search` string.
	 * @param array[] $embeddings A list provided by this component. (e.g. `fetchEmbeddingList`)
	 * @param string $search
	 * @param Language|null $language
	 * @return array
	 */
	public static function filterEmbeddings(array $embeddings, string $search, ?Language $language=null): array
	{
		array_walk($embeddings, function (array &$embedding) use ($search, $language) {
			$scoringTable = [
				'name' => 1000,
				'embedding_vendor' => 100,
				'translations.description' => 10,
				'translations.label' => 1000,
				'assets.name' => 100,
			];
			$searchInsideWords = array_fill_keys([
				'name',
				'translations.label',
				'assets.name',
			], true);

			// Übereinstimmungsgrad ermitteln, um das Suchergebnis zu sortieren
			$score = 0;

			// Basisdaten durchsuchen
			foreach ($embedding as $key=>$value) {
				if (is_string($value) and $key !== 'lastModified') {
					$scoringKey = $key;
					if (($searchInsideWords[$scoringKey] ?? false) ? mb_stristr($value, $search) : preg_match('~\b'.preg_quote($search, '~').'\b~i', $value)) {
						$score += ($scoringTable[$scoringKey] ?? 1) * (mb_strtolower($search) == mb_strtolower($value) ? 10 : 1);
					}
				}
			}
			// Übersetzungen durchsuchen
			$translations = $language
				? array_filter([$embedding['translations'][$language->getShortCode()] ?? null])
				: $embedding['translations'];
			foreach ($translations as $lang=>$translation) {
				foreach ($translation as $key=>$value) {
					if (is_string($value)) {
						$scoringKey = implode('.', ['translations', $key]);
						if (($searchInsideWords[$scoringKey] ?? false) ? mb_stristr($value, $search) : preg_match('~\b'.preg_quote($search, '~').'\b~i', $value)) {
							$score += ($scoringTable[$scoringKey] ?? 1) * (mb_strtolower($search) == mb_strtolower($value) ? 10 : 1);
						}
					}
				}
			}
			// Cookies/Assets durchsuchen
			foreach ($embedding['assets'] as $asset) {
				foreach ($asset as $key=>$value) {
					if (is_string($value)) {
						$scoringKey = implode('.', ['assets', $key]);
						if (($searchInsideWords[$scoringKey] ?? false) ? mb_stristr($value, $search) : preg_match('~\b'.preg_quote($search, '~').'\b~i', $value)) {
							$score += ($scoringTable[$scoringKey] ?? 1) * (mb_strtolower($search) == mb_strtolower($value) ? 10 : 1);
						}
					}
				}
				// Übersetzungen durchsuchen
				$translations = $language
					? array_filter([$asset['translations'][$language->getShortCode()] ?? null])
					: $asset['translations'];
				foreach ($translations as $lang=>$translation) {
					foreach ($translation as $key=>$value) {
						if (is_string($value)) {
							$scoringKey = implode('.', ['assets.translations', $key]);
							if (($searchInsideWords[$scoringKey] ?? false) ? mb_stristr($value, $search) : preg_match('~\b'.preg_quote($search, '~').'\b~i', $value)) {
								$score += ($scoringTable[$scoringKey] ?? 1) * (mb_strtolower($search) == mb_strtolower($value) ? 10 : 1);
							}
						}
					}
				}
			}

			$embedding['@score'] = $score;
		});

		// Elemente herausfiltern, die keinerlei Übereinstimmung zeigen
		$result = array_filter($embeddings, function (array $embedding) { return $embedding['@score'] > 0; });

		// Elemente nach Relevanz sortieren
		usort($result, function (array $a, array $b) {
			return $b['@score'] <=> $a['@score'];
		});

		return $result;
	}

	public static function getLastUpgradeSource(): ?array
	{
		return self::$upgradeSource;
	}
}
