<?php

namespace App\Component\EmbeddingDb;
use App\Utils;

final class EmbeddingDbLegacy extends EmbeddingDb
{
	protected const APIURL = 'https://www.ccm19.de/plugins/flexapi/v2/';
	protected const APIURL_CK = 'https://cookiedb.ccm19.de/v2/';
	protected const CACHE_FILE = 'embedtmp.json';

	/**
	 * @override
	 */
	protected static function fetchApi(string $path, array $query = [], array $headers = [])
	{
		$apiUrl = (Utils::isClickskeksCloud()) ? static::APIURL_CK : static::APIURL;
		$query['key'] = static::APIKEY;
		$url = $apiUrl . ltrim($path, '/') . '?'.http_build_query($query);
		return static::fetchUrl($url, $headers);
	}

	/**
	 * Purpose-IDs und Storage-Types übertragen sowie Angaben beim Consent-Cookie anpassen
	 * @override
	 */
	protected static function postprocessEmbedding(array $embedding): array
	{
		$embedding['purpose'] = static::convertPurposeId($embedding['purpose']);
		foreach ($embedding['assets'] ?? [] as $assetKey => $asset) {
			$embedding['assets'][$assetKey]['storageType'] = static::convertStorageType($asset['storageType']);
		}
		// Sonderfall CCM19-Consent-Cookie: Wenn nicht Full-Service, dann Daten vom Kunden eintragen
		if ($embedding['id'] == (Utils::isClickskeksCloud() ? 13308 : 207)) {
			$embedding['ccm_special'] = true;
			if (Utils::isClickskeksCloud() || stripos($_SERVER['HTTP_HOST'], ".ccm19.de") === false) {
				$embedding['embedding_vendor'] = '{Firma}';
				foreach ($embedding['translations'] as $lang => $translation) {
					$embedding['translations'][$lang]['privacy_policy_url'] = '{Firma_Datenschutz}';
					$embedding['translations'][$lang]['place_of_processing'] = '{Firma_Ort}';
				}
			}
			if (Utils::isClickskeksCloud()) {
				$embedding['name'] = 'Cookie Consent Manager Clickskeks';
				foreach ($embedding['translations'] as $lang => $translation) {
					$embedding['translations'][$lang]['label'] = 'Cookie Consent Manager Clickskeks';
				}
			}
		}
		return $embedding;
	}


}
