<?php

namespace App\Component;

/**
 *  * Crawler für Firmen-Impressen
 * $impcrawler = new ImpressumCrawler();
 * $res = $impcrawler->extract_impressum_data("https://www.spiegel.de");
 * print_r($res);
 *
 * */

class ImpressumCrawler
{
	private $prenames;

	private $umlaut_replace_pairs = [
		'ä'=>'ae', 'ö'=>'oe', 'u'=>'ue', 'ß'=>'ss',
		'Ä'=>'Ae', 'Ö'=>'Oe', 'Ü'=>'ue',
		'á' => 'a', 'é' => 'e', 'í' => 'i', 'ó' => 'o', 'ú' => 'u',
		'à' => 'a', 'è' => 'e', 'ì' => 'i', 'ò' => 'o', 'ù' => 'u',
		'â' => 'a', 'ê' => 'e', 'î' => 'i', 'ô' => 'o', 'û' => 'u',
		'ç' => 'c', 'Ç' => 'C',
		'Á' => 'A', 'É' => 'E', 'Í' => 'I', 'Ó' => 'O', 'Ú' => 'U',
		'À' => 'A', 'È' => 'E', 'Ì' => 'I', 'Ò' => 'O', 'Ù' => 'U',
		'Â' => 'A', 'Ê' => 'E', 'Î' => 'I', 'Ô' => 'O', 'Û' => 'U',
		'Ă' => 'A', 'Ș' => 'S', 'Ş' => 'S', 'Ț' => 'T', 'Ţ' => 'T',
		'ă' => 'a', 'ș' => 's', 'ş' => 's', 'ț' => 't', 'ţ' => 't',
		'Đ'=>'Dj', 'đ'=>'dj', 'Ž'=>'Z', 'ž'=>'z', 'Ã'=>'A', 'Æ'=>'AE',
		'Ø'=>'O', 'Þ'=>'B', 'ÿ'=>'y', 'æ'=>'ae'];

	public function __construct()
	{
		$this->prenames = Lang_names::getPreNames();
	}

	/**
	 * Relative URL zu absoluter machen
	 *
	 * @param string $rel Relative URL
	 * @param string $base URL auf der der Link stand bzw. Link-Basis
	 * @return string Absolute URL
	 */
	private function rel2abs($rel, $base)
	{
		/* Wenn schon absolut so lassen */
		if( parse_url($rel, PHP_URL_SCHEME) != '' )
			return $rel;

		/* Query-String/Hash? */
		if( $rel[0]=='#' || $rel[0]=='?' )
			return $base.$rel;

		/* Basis-URL parsen und in lokale Variablen packen */
		$urldata = parse_url($base);
		$path = isset($urldata['path']) ? $urldata['path'] : '';
		$host = $urldata['host'];
		$scheme = $urldata['scheme'];

		/* Pfad säubern */
		$path = preg_replace( '#/[^/]*$#', '', $path );

		/* URL mit nur fehlendem Schema? */
		if( $rel[0] == '/' and $rel[1] == '/' ) {
			$path = '';
			return $scheme.':'.$rel;
		}
		/* absoluter pfad? */
		elseif( $rel[0] == '/' )
			$path = '';

		$abs = $host;

		/* port? */
		if (!empty($urldata['port']))
			$abs.= ':'.$urldata['port'];

		$abs.=$path.'/'.$rel;

		/* vereinfachen */
		$re = array('#(/\.?/)#', '#/(?!\.\.)[^/]+/\.\./#');
		for( $n=1; $n>0; $abs=preg_replace( $re, '/', $abs, -1, $n ) ) {}

		return $scheme.'://'.$abs;
	}

	/**
	 * @param string $url URL
	 * @return array|null Impressums-Daten
	 */
	public function extract_impressum_data($url)
	{
		global $log;
		error_reporting(0);
		// Impressum finden
		$datenschutz_url = null;
		$impressum_url = null;
		$result = array();

		$ch = curl_init( $url );
		curl_setopt($ch, CURLOPT_TIMEOUT, 5);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		//curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt( $ch, CURLOPT_USERAGENT, "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:48.0) Gecko/20100101 Firefox/72.0" );
		curl_setopt($ch, CURLOPT_FORBID_REUSE, false);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		$page_content = curl_exec($ch);

		$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		//print_r(curl_getinfo($ch));
		if ($httpcode >= 300 and $httpcode < 400) {
			$url = curl_getinfo($ch, CURLINFO_REDIRECT_URL);
			if ($url) {
				curl_setopt($ch, CURLOPT_URL, curl_getinfo($ch, CURLINFO_REDIRECT_URL));
				$page_content = curl_exec($ch);
				$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			}
		}

		if ($httpcode >= 300 and $httpcode < 400) {
			$url = curl_getinfo($ch, CURLINFO_REDIRECT_URL);
			if ($url) {
				curl_setopt($ch, CURLOPT_URL, curl_getinfo($ch, CURLINFO_REDIRECT_URL));
				$page_content = curl_exec($ch);
				$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			}
		}

		if ($httpcode >= 300 and $httpcode < 400) {
			$url = curl_getinfo($ch, CURLINFO_REDIRECT_URL);
			if ($url) {
				curl_setopt($ch, CURLOPT_URL, curl_getinfo($ch, CURLINFO_REDIRECT_URL));
				$page_content = curl_exec($ch);
				$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			}
		}

		if ($httpcode >= 400) {
			return null;
		}

		$url = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);
		// Base-Tag auswerten, wenn vorhanden
		$matches = [];
		if (preg_match('/<base[^>]*href=["\']([^\'"]*)["\'][^>]*>/is', $page_content, $matches)) {
			$url = $this->rel2abs(html_entity_decode($matches[1]), $url);
		}

		// Link mit "Impressum" suchen
		$page_content = str_replace('<A', '<a', $page_content);
		$page_content = str_replace('</A', '</a', $page_content);
		foreach (explode('</a>', $page_content) as $part) {
			$matches = array();
			if (preg_match('/<a[^>]*href=["\']([^\'"]*)["\'][^>]*>.*impressum/is', $part, $matches)) {
				$impressum_url = $this->rel2abs(html_entity_decode($matches[1]), $url);
				break;
			}
		}

		//Datenschutz Link suchen...
		foreach (explode('</a>', $page_content) as $part) {
			$matches = array();
			if (preg_match('/<a[^>]*href=["\']([^\'"]*)["\'][^>]*>.*datenschutz/is', $part, $matches)) {
				$datenschutz_url = $this->rel2abs(html_entity_decode($matches[1]), $url);
				break;
			}
		}
		// Sonst Link mit "Kontakt"/"Contact" suchen
		if (!$impressum_url) {
			foreach (explode('</a>', $page_content) as $part) {
				$matches = array();
				if (preg_match('/<a[^>]*href=["\']([^\'"]*)["\'][^>]*>.*(?:kontakt|contact)/is', $part, $matches)) {
					$impressum_url = $this->rel2abs(html_entity_decode($matches[1]), $url);
					break;
				}
			}
		}
		$index_length = strlen($page_content);

		// Wenn Impressum gefunden, aufrufen, sonst Hauptseite angucken
		if ($impressum_url) {
			// Impressum laden
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt($ch, CURLOPT_URL, $impressum_url);
			$page_content = curl_exec($ch);
			$result = array("impressum" => $impressum_url, 'index_length'=>$index_length, 'httpcode'=>$httpcode);
		} else {
			$result = array('httpcode'=>$httpcode, 'index_length'=>$index_length);
			//print_r(curl_getinfo($ch));
		}
		$result['datenschutz_url'] = $datenschutz_url;
		// Bei Nicht-Erfolg abbrechen
		$httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if ($httpcode >= 300) {
			//$log->warn('### EXIT - > ERROR: '.$httpcode);
			//return null;
		}

		// Zeichensatz zu UTF-8
		$mimetype = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
		// Zeichensatz rausfinden
		$matches = array();
		if ($mimetype and preg_match('/charset\s?=\s?([a-zA-Z0-9-]+)/i', $mimetype, $matches)) {
			$charset = strtolower($matches[1]);
		} else {
			$charset = null;
		}
		if (!$charset) {
			// Ausprobieren
			$page_content = mb_convert_encoding($page_content, 'UTF-8', 'ASCII,UTF-8,ISO-8859-1,ISO-8859-15');
		} elseif ($charset == 'utf-8' or $charset == 'utf8') {
			// Alles OK
		} else {
			$page_content = mb_convert_encoding($page_content, 'UTF-8', $charset);
		}
		curl_close( $ch );

		// HTML to Text
		$page_content = strtr($page_content, array("\r\n"=>' ', "\n"=>' ', "\r"=>' '));
		$page_content = preg_replace('/.*<body/i', "<body", $page_content);
		$page_content = preg_replace('/<br ?\/?'.'>/i', "\n", $page_content);
		$page_content = preg_replace('/<\/(?:p|pre|div|h[1-6]|aside|article|section|main|address|nav|ul|ol|li|blockquote|form|tr|dd)>/i', "\n", $page_content);
		$page_content = preg_replace('/<!--.?-->/i', "\n", $page_content);
		$page_content = preg_replace('/<script.*?<\/script>/i', "\n", $page_content);
		$page_content = preg_replace('/<style.*?<\/style>/i', "\n", $page_content);
		$page_content = preg_replace('/<\/(?:td|th|dt)>/i', " ", $page_content);
		$page_content = preg_replace('/(="[^"]*")"([\s>])/', '\1\2', $page_content);

		$page_content = strip_tags($page_content);
		$page_content = html_entity_decode($page_content);

		// Alles vor dem Wort "Impressum:" löschen, wenn vorhanden
		if (strpos($page_content, 'Impressum:') !== false) {
			$page_content = preg_replace('/.*\bImpressum:/s', '', $page_content);
		}

		// NBSP, etc. umwandeln
		$page_content = preg_replace('/\h/u', " ", $page_content);
		// " | " zu Newline
		$page_content = preg_replace('/ \| /', "\n", $page_content);
		// Newlines zusammenziehen
		$page_content = preg_replace('/\n\n+/is', "\n", $page_content);
		// Leading Spaces auf jeder Zeile entfernen
		$page_content = preg_replace('/^\s+/um', "", $page_content);
		// Trailing Spaces auf jeder Zeile entfernen
		$page_content = preg_replace('/\s+$/um', "", $page_content);

		// Postprocessing: Doppelpunkt am Zeilenende -> Zeilen zusammenziehen, außer bei "Angaben gemäß §5 TMG", "ist:"
		$page_content = str_replace(":\n", ': ', $page_content);
		$page_content = str_replace("TMG: ", "TMG:\n", $page_content);
		$page_content = str_replace("ist: ", "ist:\n", $page_content);
		// "Adresse:" löschen
		$page_content = str_replace('Adresse: ', '', $page_content);

		// Telefonnummern suchen
		$matches = array();
		if (preg_match_all('/^[^\n]*?((?:\b0|0049 ?|\+49 ?)\h?[0-9][0-9 ]{1,5}(?: ?[\/–-] ?|\h?)[0-9 –-]{4,}|(?:\+49 )?\([0-9][0-9 ]{1,7}\)\h[0-9 –-]{4,}|\+49 \(0\) ?[0-9][0-9 ]{1,7}[\/-]?[0-9 –-]{4,}|\b0[1-9][0-9]{1,4}[0-9-]{3,9})/um', $page_content, $matches, PREG_SET_ORDER)) {
			foreach ($matches as $match) {
				$match[1] = preg_replace('/[[:punct:]\h]+$/um', '', $match[1]);
				if (stripos($match[0], 'fax') !== false) {
					// Faxnummer gefunden
					if (!isset($result['fax']))
						$result['fax'] = $match[1];
				} elseif (stripos($match[0], 'tel') !== false) {
					// Telefonnummer gefunden
					if (!isset($result['tel']))
						$result['tel'] = $match[1];
				} elseif (stripos($match[0], 'fon') !== false) {
					// Telefonnummer gefunden
					if (!isset($result['tel']))
						$result['tel'] = $match[1];
				} elseif (count($matches) === 1) {
					$result['tel'] = $match[1];
				}
			}
		}

		// Adresse suchen (Format Name\nAdresse\nPLZ Ort)
		$matches = array();
		if (preg_match_all('/^([[:alnum:]\s.,\/§&®@+:()\'-]+?)\n([[:alpha:] .-]+[0-9]{1,3}(?: ?[a-hA-H])?(?: ?[+\/-] ?[0-9]{1,3}(?: ?[a-hA-H])?)?(?:\s+Gebäude [[:alnum:] ]*)?)(?:,[^\n]*)?\s*\n(?:D ?- ?|D )?([0-9]{5}|[0-9]{4}[A-Z]{2}) ([[:alpha:] .,\/()-]+)$/mu', $page_content, $matches, PREG_SET_ORDER)) {
			foreach ($matches as $match) {
				// Zeilen vor der Adresse aufsplitten
				$namedata = array();
				foreach (explode("\n", preg_replace('/^\s+/mu', '', $match[1])) as $line) {
					if (trim($line)) {
						$namedata[] = trim($line);
					}
					// Wenn hier schon GmbH, AG, ... als einzelnes Wort auftaucht, ist das wohl der Firmenname
					if (!isset($result['name']) and preg_match('/\b(?:GmbH|GbR|AG$|UG|KG|Limited|Ltd\.?|e\.K\.)(?:\s|\n|$)/iu', $line)) {
						$result['name'] = trim($line);
					}
				}
				$namedata = array_reverse($namedata);
				// c/o, Geschäftsführer, etc. rausholen
				foreach ($namedata as $line) {
					$match2 = [];
					// c/o
					if (preg_match('/(?:^|:\s*)(?:℅|c\/o\b|p\. Adr\.|p\. A\.)\s*(.*)$/u', $line, $match2)) {
						$result['c_o'] = $match2[0];
					}
					// vertreten durch, Geschäftsführer/Gesellschafter...
					elseif (preg_match('/\b(?:vertreten durch (?:(?:den|die) [A-Z][[:alpha:]]+)?|Vertretungsberechtigte [A-Z][[:alpha:]]+\b|Geschäftsführer(?:in)?\b|GF\b|Direktor(?:in)?\b|Inh\.|(?:Firmen)?Inhaber(?:in)?\b|Ansprechpartner(?:in)?\b)(?::?\s*)(.*)/iu', $line, $match2)) {
						$result['vertreten_durch_raw'] = $match2[1];

					}
					// Personennamen mit Titel
					elseif (preg_match('/^(?:(?>Dr\.|Mag\.|Prof\.|Dipl\.(-Ing\.)?|Herr|Frau|Mr\.|Ms\.|Mrs\.)\s*)+(?:[[:upper:]][[:alpha:]-]*[.,]?\s*)+$/imu', $line, $match2)) {
						if (!isset($result['vertreten_durch_raw'])) {
							$result['vertreten_durch_raw'] = $match2[0];
						}
					}
					// Zeilen mit nur "Fachhandel für ..." oder "...niederlassung" ignorieren
					elseif (preg_match('/^(?:Fachhandel\sfür\b[^:]*|[[:upper:]][[:alpha:]]+niederlassung|.*\bGewerbepark .*|Innungs-Ingenieurbetrieb|Niederlassung [[:alpha:]].*|UST-ID.*|Einzelunternehme[nr]|Rechtsform:.*|Freiberufler|.* Büro und Lager)$/u', $line)) {
						continue;
					}
					elseif (empty($result['name']) and $line) {
						$result['name'] = $line;
						break;
					}
				}

				$result['addr'] = $match[2];
				$result['plz'] = $match[3];
				$result['stadt'] = $match[4];
				break;
			}
		}

		// Wenn hier noch kein Geschäftsführer gefunden, auf ganzer Seite suchen
		$match2 = [];
		if (empty($result['vertreten_durch_raw']) and preg_match('/\b(?:Geschäftsführer(?:in)?\b|GF\b|Direktor(?:in)?\b|Inh\.|(?:Firmen)?Inhaber(?:in)?\b):\s*(.*)/imu', $page_content, $match2)) {
			$result['vertreten_durch_raw'] = $match2[1];
		}

		// Adressformat Name, Adresse, PLZ Ort
		if (!isset($result['plz'])) {
			if (preg_match_all('/^([[:alpha:]\s.,\/&:-]+?), ([[:alpha:] .-]+[0-9]{1,3}(?: ?[a-hA-H])?(?: ?- ?[0-9]{1,3}(?: ?[a-hA-H])?)?)\s*, (?:D ?- ?)?([0-9]{5}) ([[:alpha:] .,\/()-]+)$/mu', $page_content, $matches, PREG_SET_ORDER)) {
				foreach ($matches as $match) {
					$tmp = array();
					foreach (explode("\n", preg_replace('/^\s+/mu', '', $match[1])) as $line) {
						if (trim($line))
							$tmp[] = $line;
					}
					$result['name'] = $tmp[count($tmp)-1];
					$result['addr'] = $match[2];
					$result['plz'] = $match[3];
					$result['stadt'] = $match[4];
					break;
				}
			}
		}

		//$result['raw_content'] = $page_content;

		if (!isset($result['name'])) {
			$result['name'] = '';
		}

		// Postprocessing Name: Übliche Präfixe rausholen
		$match2 = [];
		if (preg_match('/\b(?:Gesetzliche Anbieterkennung|Anbieterkennzeichnung|Inh\.|\(Impressum\)|Kontaktdaten|Kontaktadresse|Büroadresse|Geschäftsadresse|Hausanschrift|[Vv]erantwortlich|Firmenname|Firma|Firmenangaben|Firmensitz|[Vv]erantwortlich für alle Inhalte|Dienste?anbieter|Anbieter|Adresse|Lieferadresse|Geschäftssitz|Impressum|Onlineshopbetreiber|Betreiber des Online-Shops|Betreiber der Website|Betreiber dieser Seite|Shopbetreiber|Anschrift|Postanschrift|Kontakt|[Vv]erantwortlich für (?:die Inhalte|den Inhalt)(?: ist)?|richten an|ist eine? (?:eingetragene )?(?:Marke|Angebot|Projekt|Division) (?:von|der)(?:\sFirma)?|unterhalten von(?: der)|betrieben von(?: der)?|Inhaber|Herausgeber|HERAUSGEBER|ist die|Herausgeber [^:]* gemäß [^:]+|nach §5\s+TMG(?:\s+\(TeleMedienGesetz\))?|§\s*6\s+TDG|§ 55 RStV|Kundenbetreuung|Verkäufer|Sitz der Gesellschaft|Authorized Distributor [^:]*):?\s+(.+)/u', $result['name'], $match2)) {
			$result['name'] = $match2[1];
		}
		elseif (preg_match('/^by\s+(.*)/u', $result['name'], $match2)) {
			$result['name'] = $match2[1];
		}
		if (isset($result['vertreten_durch_raw']) and $result['name'] === $result['vertreten_durch_raw']) {
			unset($result['vertreten_durch_raw']);
		}
		// Vertretungsberechtigte Person weiter aufsplitten
		if (isset($result['vertreten_durch_raw'])) {
			$match2 = [];
			if (preg_match('/^(?:(?P<anrede>Herrn?|Frau|Fr\.|Mr\.|Ms\.|Mrs\.)\s+)?(?P<titel>(?:(?:Dr\.|PD|Univ\.-Prof\.|Prof\.|Dipl\. ?[[:alpha:].-]*|Mag\.[[:alpha:].-]*|Pfr\.|phil\.|oec\.|me\.|medic\.|pol\.|habil\.|biol\.|hum\.|curt\.|med\.?|vet\.|univ\.|rer\.|dent\.|nat\.|vet\.|Ass\.|Ök\.|Ökonomin|Ökonom|Agr\.|Bw\.|Bibl\.|chem\.|Wirtsch\.|Ing\.|iur\.|Jur\.|Vw\.|Kfm\.|RA|h\. ?[cj]\.)\s+)*)(?P<namen>(?:[[:upper:]][[:alpha:]-]*[.]?-?\s*)+)/iu', $result['vertreten_durch_raw'], $match2)) {
				$vresult = ['titel'=>trim($match2['titel']), 'anrede'=>null, 'vornamen'=>null, 'nachname'=>null];
				$match2['namen'] = trim(preg_replace('/\bund .*/u', '', $match2['namen']));
				// Nachgestellte Titel o.ä. abschneiden
				$match3 = [];
				while (preg_match('/\b(e\. ?K\.|E\.K\.|eK|e\.k\.|e\. ?Kfm\.|e\. ?Kfr\.|Kfr\.|jr\.|sr\.|mult\.|jun\.|sen\.|und|mit|email|Fax|Dipl\. ?[[:alpha:].-]*|Diplom-[[:alpha:].-]*|Dr\.|einzelvertretungsberechtigt|BSc|Tel\.|BA|MBA|MSc|Msc\.|MA|M\. ?A\.|B\.V\.|M\. ?Sc\.|AG|RB|HRB|VDI|Inh\.|I{1,3}|Ph\. ?D\.|Md[BL]|MdEP|B\.M\.|CEO|Sitz|et\. ?al\.|USt\.|S[tT]\.? ?Nr\.)$/u', $match2['namen'], $match3, PREG_OFFSET_CAPTURE)) {
					$match2['namen'] = trim(substr($match2['namen'], 0, $match3[1][1]));
				}
				$nameparts = explode(' ', trim($match2['namen']));
				$vresult['nachname'] = trim(array_pop($nameparts));
				if ($nameparts) {
					// von/van/... zum Nachnamen dazupacken
					$match3 = [];
					if (preg_match('/\b(van(?: der?)?|v\.|Graf von|von|vom|de la|den|de|du|im)$/ui', implode(' ', $nameparts), $match3, PREG_OFFSET_CAPTURE)) {
						$vresult['nachname'] = trim(substr(implode(' ', $nameparts), $match3[1][1])) . ' '. trim($vresult['nachname']);
						$nameparts = explode(' ', trim(substr(implode(' ', $nameparts), 0, $match3[1][1])));
					}
				}
				// Wenn eindeutige Anrede vorhanden, daraus Geschlecht bestimmen
				if (in_array($match2['anrede'], ['Herr', 'Mr.'])) {
					$vresult['anrede'] = 'Herr';
				} elseif ($match2['anrede']) {
					$vresult['anrede'] = 'Frau';
				}

				// Sonst versuchen Geschlecht aus erstem Vornamen zu bestimmen
				$vorname1 = ($nameparts) ? $nameparts[0] : null;
				if (!$vresult['anrede'] and $vorname1) {
					// Für Vergleich vorbereiten
					$vorname1 = strtolower(strtr($vorname1, $this->umlaut_replace_pairs));
					if (in_array($vorname1, $this->prenames['fem'])) {
						$vresult['anrede'] = 'Frau';
					} elseif (in_array($vorname1, $this->prenames['male'])) {
						$vresult['anrede'] = 'Herr';
					} elseif (strpos($vorname1, '-') !== false) {
						$vorname1 = explode('-', $vorname1)[0];
						if (in_array($vorname1, $this->prenames['fem'])) {
							$vresult['anrede'] = 'Frau';
						} elseif (in_array($vorname1, $this->prenames['male'])) {
							$vresult['anrede'] = 'Herr';
						}
					}
				}
				// Vornamen wieder zusammensetzen
				if ($nameparts) {
					$vresult['vornamen'] = trim(implode(' ', $nameparts));
				}

				// Sonderfälle raus...
				if (substr($vresult['nachname'], -1) == '.' or in_array(strtolower($vresult['nachname'] ?? ''), ['gmbh', 'agbs', 'mbh', 'ag', 'des', 'der', 'von', 'die', 'website', 'de', 'den', 'für', 'fh', 'ort', 'uns.', 'v.', 'e.v', 'ev', 'tdg', 'str.', 'ist', 'inhalte', 'aktualität', 'ansprechpartner', 'gbr', 'karriere', 'finanzamt', 'downloads', 'internetauftritt', 'kontaktformular', 'tv-', 'ein', 'hifi', 'sind', 'ug', 'site', 'id'])) {
					// Vertreten durch weglassen...
				} else {
					$result['vertreten_durch'] = $vresult;
				}
			}
		}
		return $result;
	}
}
