<?php

namespace App\Component\LogStream;

use Countable;
use Iterator;

interface RepositoryInterface extends Countable
{
	/**
	 * Durchsuchen des Protokolls.
	 *
	 * @param array $query MongoDB-Query-Array. Je nach Repository wird nicht
	 *                      alles unterstützt.
	 * @param bool $reversed
	 * @return Iterator<array>
	 * @see https://docs.mongodb.com/manual/reference/operator/query/
	 */
	public function filter(array $query, bool $reversed=false): Iterator;

	/**
	 * Iteriert über alle Einträge im Log
	 *
	 * @param bool $reversed
	 * @return Iterator<array>
	 */
	public function all(bool $reversed=false): Iterator;

	/**
	 * {@inheritdoc}
	 */
	public function count(): int;

	/**
	 * @param array $data
	 */
	public function append(array $data): void;

	/**
	 * @param iterable<array> $datas
	 */
	public function appendMulti(iterable $datas): void;

	/**
	 * Protokoll rotieren/archivieren
	 *
	 * @param array|null $keepQueryHint MongoDB-style Query (optional)
	 *
	 * Vor dem eigentlichen Prozess wird ein ProtocolRotateEvent ausgelöst.
	 *
	 * Wenn $keepQueryHint angegeben ist und der Storage das unterstützt, wird
	 * nur alles was mindestens so alt ist, wie der erste Eintrag der nicht mehr
	 * das Query matcht, archiviert.
	 *
	 * @see ProtocolRotateEvent
	 */
	public function rotateProtocol(?array $keepQueryHint = null): void;

	/**
	 * Löscht alle Einträge ab dem ersten der nicht mehr auf $query matcht
	 *
	 * @param array $query
	 * @return int Anzahl gelöschter Einträge
	 */
	public function trimBefore(array $query): int;

	/**
	 * Leeren des Repositorys.
	 */
	public function clear(): void;

	/**
	 * Größe des Repositories auf der Festplatte in Bytes, ohne ungespeicherte Änderungen
	 *
	 * @return int Größe in Bytes (-1 wenn nicht vom Repository unterstützt)
	 */
	public function getPersistedSize(): int;
}
