<?php

namespace App\Component\Media;

use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\HttpFoundation\File\Exception\FileNotFoundException;

class CachedFile extends File
{
	private $nameInfo;

	/**
	 * @param string $path The path to the file
	 * @param string $name The original filename
	 *
	 * @throws FileNotFoundException If the given path is not a file
	 */
	public function __construct(string $path, string $name)
	{
		$this->nameInfo = new \SplFileInfo($name);
		parent::__construct($path, true);
	}

	/**
	 * @override
	 */
	public function getFilename(): string
	{
		return $this->nameInfo->getFilename();
	}

	/**
	 * @override
	 */
	public function getBasename($suffix=null): string
	{
		if ($suffix !== null) {
			return $this->nameInfo->getBasename($suffix);
		} else {
			return $this->nameInfo->getBasename();
		}
	}

	/**
	 * @override
	 */
	public function getExtension(): string
	{
		return $this->nameInfo->getExtension();
	}

	/**
	 * @override
	 */
	protected function getName(string $name): string
	{
		return (string)$this->nameInfo;
	}
}
