<?php

namespace App\Component\Media;

use Symfony\Component\HttpFoundation\File\File;

interface RepositoryInterface
{
	/**
	 * Leeren des Repositorys.
	 */
	public function clear(): void;

	/**
	 * Datei aus Repository holen.
	 * @param string $identifier
	 * @return File|null
	 */
	public function getFile(string $identifier): ?File;

	/**
	 * Datei zu Repository hinzufügen.
	 * @param File $file
	 * @param string|null $filename Gewünschter Dateiname, falls abweichend von Quelldatei
	 * @param bool $overwrite Wenn TRUE, wird überschrieben. Ansonsten wird die Datei ggf. umbenannt.
	 * @return string Identifier
	 */
	public function addFile(File $file, ?string $filename=null, bool $overwrite=false): string;

	/**
	 * Datei zu Repository hinzufügen und dabei eine alte Datei löschen. Soweit technisch möglich
	 * wird der alte Identifier beibehalten, aber dafür gibt es keine Garantie.
	 *
	 * Wenn $oldIdentifier == null, dann verhält sich die Funktion wie addFile().
	 * Wenn $file === null, dann verhält sich die Funktion wie removeFile() und
	 * gibt einen leeren Identifier zurück.
	 *
	 * @param string|null $oldIdentifier
	 * @param File|null $file
	 * @param string|null $filename Gewünschter Dateiname, falls abweichend von Quelldatei
	 * @return string Identifier
	 */
	public function replaceFile(?string $oldIdentifier, ?File $file, ?string $filename=null): string;

	/**
	 * Diese Methode entfernt eine Datei.
	 * @param string $identifier
	 * @return $this
	 */
	public function removeFile(string $identifier);
}
