<?php

namespace App\Component\Menu\Annotation;

use App\Component\Menu\Model\MenuItem;

/**
 * Annotation für Menüpunkte.
 *
 * @Annotation
 * @Target({"METHOD","CLASS"})
 */
final class Menu
{
	private $app;
	private $name;
	private $group;
	private $route;
	private $routeGroup;
	private $icon;
	private $access = [];
	private $navigation;
	private $editions;
	private $order;
	private $envs;
	private $pluginStateId;
	private $submenu;

	/**
	 * @param array $data An array of key/value parameters
	 *
	 * @throws \BadMethodCallException
	 */
	public function __construct(array $data)
	{
		if (isset($data['value'])) {
			$data['name'] = $data['value'];
			unset($data['value']);
		}

		foreach ($data as $key => $value) {
			$method = 'set'.str_replace('_', '', $key);
			if (strtolower($method) === 'setpluginstateid' or !method_exists($this, $method)) {
				throw new \BadMethodCallException(sprintf('Unknown property "%s" on annotation "%s".', $key, static::class));
			}
			$this->$method($value);
		}
	}

	public function setApp(?string $app)
	{
		$this->app = $app;
	}

	public function getApp(): ?string
	{
		return $this->app;
	}

	public function setName(?string $name)
	{
		$this->name = $name;
	}

	public function getName(): ?string
	{
		return $this->name;
	}

	public function setGroup(?string $groupname)
	{
		$this->group = $groupname;
	}

	public function getGroup()
	{
		return $this->group;
	}

	public function setRoute(?string $route)
	{
		$this->route = $route;
	}

	public function getRoute(): ?string
	{
		return $this->route;
	}

	public function setRouteGroup(?string $routeGroup)
	{
		$this->routeGroup = $routeGroup;
	}

	public function getRouteGroup(): ?string
	{
		return $this->routeGroup;
	}

	public function setIcon(?string $icon)
	{
		$this->icon = $icon;
	}

	public function getIcon(): ?string
	{
		return $this->icon;
	}

	/**
	 * Zugriffsrechte mit MenuItem::ACCESS_-Konstanten
	 *
	 * @see MenuItem
	 * @param string[] $access
	 */
	public function setAccess($access)
	{
		$this->access = (array)$access;
	}

	public function getAccess(): array
	{
		return $this->access;
	}

	public function setNavigation(?string $value)
	{
		$this->navigation = $value;
	}

	public function getNavigation(): ?string
	{
		return $this->navigation;
	}

	public function setEditions(?array $value)
	{
		$this->editions = $value;
	}

	public function getEditions(): ?array
	{
		return $this->editions;
	}

	public function setOrder(?int $orderid)
	{
		$this->order = $orderid;
	}

	public function getOrder(): ?int
	{
		return $this->order;
	}

	public function setEnvs(?array $value)
	{
		$this->envs = $value;
	}

	public function getEnvs(): ?array
	{
		return $this->envs;
	}

	/**
	 * Intern verwendet, um Plugin-Menüpunkte korrekt zu verwalten
	 */
	public function getPluginStateId(): ?string
	{
		return $this->pluginStateId;
	}

	/**
	 * Intern verwendet, um Plugin-Menüpunkte korrekt zu verwalten
	 */
	public function setPluginStateId(?string $id)
	{
		$this->pluginStateId = $id;
	}

	/**
	 * Intern verwendet, um Plugin-Menüpunkte korrekt zu verwalten
	 */
	public function getSubMenu(): ?string
	{
		return $this->submenu;
	}

	/**
	 * Intern verwendet, um Plugin-Menüpunkte korrekt zu verwalten
	 */
	public function setSubMenu(?string $submenu)
	{
		$this->submenu = $submenu;
	}
}
