<?php

namespace App\Component\Menu\Model;
use Generator;

final class MenuGroup
{
	public $name;
	public $order;
	public $items;
	public $icon;

	public function __construct(string $name, ?int $order=null, ?string $icon = null, array $items=[])
	{
		$this->name = $name;
		$this->order = $order;
		$this->icon = $icon;
		$this->items = (array)$items;
	}

	public static function __set_state($arr)
	{
		return new self(
			$arr['name'],
			$arr['order'],
			$arr['icon'],
			$arr['items']
		);
	}

	/**
	 * CSS-Klassen für das Icon
	 */
	public function getIconClasses(): string
	{
		if (!$this->icon) {
			return '';
		}
		elseif (strpos($this->icon, 'fa-') === 0) {
			return 'fa '.$this->icon;
		}
		elseif (strpos($this->icon, 'glyphicon-') === 0) {
			return 'glyphicon '.$this->icon;
		}
		else {
			return $this->icon;
		}
	}

	public function getFirstItem(): ?MenuItem
	{
		foreach ($this->items as $item) {
			if ($item instanceof MenuGroup) {
				$result = $item->getFirstItem();
				if ($result) {
					return $result;
				}
			} else {
				return $item;
			}
		}
		return null;
	}

	/**
	 * @return Generator|MenuItem[]
	 * @phan-return Generator<MenuItem>
	 */
	public function iterateItems()
	{
		foreach ($this->items as $item) {
			if ($item instanceof MenuGroup) {
				yield from $item->iterateItems();
			} else {
				yield $item;
			}
		}
	}

	/**
	 * @param callable(MenuItem):bool $filter
	 * @return self
	 */
	public function filtered(callable $filter): MenuGroup
	{
		$group = clone $this;
		$group->items = [];
		foreach ($this->items as $key => $item) {
			if ($item instanceof MenuGroup) {
				if ($item->items) {
					$item = $item->filtered($filter);
					if ($item->items) {
						if (is_int($key)) {
							$group->items[] = $item;
						} else {
							$group->items[$key] = $item;
						}
					}
				}
			} else {
				if (call_user_func($filter, $item)) {
					if (is_int($key)) {
						$group->items[] = $item;
					} else {
						$group->items[$key] = $item;
					}
				}
			}
		}
		return $group;
	}

	public function activatedByRoute($route): bool
	{
		$route = (string)$route;
		foreach ($this->iterateItems() as $item) {
			if ($item->activatedByRoute($route)) {
				return true;
			}
		}
		return false;
	}
}
