<?php

namespace App\Component;

use App\Exception\ConfigNotReadableException;
use MongoDB\Client;
use MongoDB\Database;

final class MongoDbService
{
	/** @var self $instance */
	private static $instance;

	/** @var Client|null $client */
	private $client = null;

	/** @var Database|null $database */
	private $database = null;

	/**
	 * @return static
	 */
	public static function getInstance(): self
	{
		if (self::$instance === null) {
			self::$instance = new self;
		}
		return self::$instance;
	}

	private function __construct() {}

	/**
	 * @return Client
	 * @throws ConfigNotReadableException
	 */
	public function getClient(): Client
	{
		if (!$this->client) {
			try {
				$this->client = new Client($_ENV['APP_MONGODB_CONNECTION_STRING'] ?? '', ['readPreference' => 'nearest']);
			} catch (\Exception $exception) {
				throw new ConfigNotReadableException($exception->getMessage());
			}
		}

		return $this->client;
	}

	/**
	 * @return Database
	 * @throws ConfigNotReadableException
	 */
	public function getDatabase(): Database
	{
		if (!$this->database) {
			try {
				$this->database = $this->getClient()->selectDatabase($_ENV['APP_MONGODB_DATABASE'] ?? '');
			} catch (\Exception $exception) {
				throw new ConfigNotReadableException($exception->getMessage());
			}
		}

		return $this->database;
	}
}
