<?php

namespace App\Component\OnBoarding;

use \DateTime;

class Cookie
{
	private $data;
	private $sourceUrls;

	public function __construct(array $data, ?array $sourceUrls)
	{
		$this->data = $data;
		$this->sourceUrls = $sourceUrls;
	}

	public function getName(): string
	{
		return str_replace("\0", '\0', $this->data['name']);
	}

	public function getValue(): ?string
	{
		return $this->data['value'] ?? null;
	}

	public function getPurpose(): ?string
	{
		return $this->data['purpose'] ?? null;
	}

	public function isNecessary()
	{
		if(stripos($this->getName(), "csrf") !== false)
		{
			return true;
		}
	}


	public function getDomain(): ?string
	{
		return $this->data['domain'] ?? null;
	}

	public function getPath(): ?string
	{
		return $this->data['path'] ?? null;
	}

	public function getExpireTime(): ?DateTime
	{
		if (!empty($this->data['expires']) && $this->data['expires']) {
			$date = new DateTime;
			$date->setTimestamp($this->data['expires']);
			return $date;
		} else {
			return null;
		}
	}

	public function getLifetime(): string
	{
		if ($this->isSession()) {
			return 'Session';
		}
		$expires = $this->getExpireTime();
		if (!$expires) {
			return '';
		}
		$diff = (new DateTime('now'))->diff($expires);

		if ($diff->y) {
			$years = $diff->y;
			if ($diff->m > 10) {
				$years++;
			} elseif ($diff->m > 1) {
				return ($years*12 + $diff->m).' mon';
			}
			return $years.' y';
		}

		if ($diff->m) {
			$months = $diff->m;
			if ($diff->d > 17) {
				$months++;
			} elseif ($diff->d > 4) {
				$months += 0.5;
			}
			return $months.' mon';
		}

		if ($diff->d) {
			$days = $diff->d;
			if ($diff->h > 21) {
				$days++;
			} elseif ($diff->h > 4) {
				return ($days*24 + $diff->h).' h';
			}
			return $days.' d';
		}

		if ($diff->h) {
			$hours = $diff->h;
			if ($diff->i > 46) {
				$hours++;
			} elseif ($diff->i > 35) {
				$hours += 0.75;
			} elseif ($diff->i > 16) {
				$hours += 0.5;
			} elseif ($diff->i > 7) {
				$hours += 0.25;
			}
			return $hours.' h';
		}

		if ($diff->i) {
			$minutes = $diff->m;
			if ($diff->s > 15) {
				$minutes++;
			}
			return $minutes.' min';
		}

		return $diff->s.' s';
	}

	public function getSize(): ?int
	{
		return $this->data['size'] ?? null;
	}

	public function isHttpOnly(): ?bool
	{
		return $this->data['httpOnly'] ?? null;
	}

	public function isSecure(): ?bool
	{
		return $this->data['secure'] ?? null;
	}

	public function isSession(): ?bool
	{
		return $this->data['session'] ?? null;
	}

	public function isSameParty(): ?bool
	{
		return $this->data['sameParty'] ?? null;
	}

	public function getPriority(): ?string
	{
		return $this->data['priority'] ?? null;
	}

	public function getSourceScheme(): ?string
	{
		return $this->data['sourceScheme'] ?? null;
	}

	public function getSourcePort(): ?int
	{
		return $this->data['sourcePort'] ?? null;
	}

	public function getSourceScripts(): array
	{
		return (array)$this->sourceUrls;
	}
}
