<?php

namespace App\Component\OnBoarding;

use App\Model\Cookie as LegacyCookie;

class ScanData
{
	private $data;

	public function __construct(array $data)
	{
		$this->data = $data;
	}

	public function hasScreenshot(): bool
	{
		return !empty($this->data['screenshot']);
	}

	/**
	 * @return string Base64-kodierter Screenshot
	 */
	public function getScreenshotData(): string
	{
		return $this->data['screenshot'] ?? '';
	}

	/**
	 * @return string Screenshot als data-URI
	 */
	public function getScreenshotAsUri(): string
	{
		$data = $this->getScreenshotData();
		// Die ersten 3 Base64-Zeichen reichen aus um die wichtigsten Bildformate zu unterscheiden
		$head = substr($data, 0, 3);
		switch ($head) {
			case '':
				// Leeres 1x1px GIF, wenn kein Screenshot vorhanden
				return 'data:image/gif;base64,R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==';
			case 'R0l':
				return 'data:image/gif;base64,'.$data;
			case 'iVB':
				return 'data:image/png;base64,'.$data;
			case 'Ukl':
				return 'data:image/webp;base64,'.$data;
			case 'PD9':
				return 'data:image/svg+xml;base64,'.$data;
			case '/9j':
			default:
				return 'data:image/jpeg;base64,'.$data;
		}
	}

	/**
	 * @return Cookie[]
	 */
	public function getCookies(): array
	{
        //Alter Cookie Banner Cookies werden mit den Cookies gemerged
        $this->mergeOldCBCookiesIfExist();

        //Cookies durchgehen
		return array_map(function (array $row) {
			return new Cookie($row, $this->data['cookieUrlMapping'][$row['name']] ?? null);
		}, $this->data['cookies'] ?? []);
	}

    /**
     * Cookies die nicht direkt erkannt wurden sonden aus anderen Bannern, hier derzeit nur CB Data, kommen hier rein
     * @return void
     */
    private function mergeOldCBCookiesIfExist()
    {
        if(!empty($this->data['cbCookiesUndColor']['cookies']))
        {
            //CookieBot Cookies werden mit den Cookies gemerged
            $this->data['cookies'] = $this->merge_arrays_without_duplicates($this->data['cookies'], $this->data['cbCookiesUndColor']['cookies']);
        }
    }


    private function merge_arrays_without_duplicates($array1, $array2) {
        $result = [];
        $names = [];

        // Füge alle Einträge aus Array1 in das Ergebnis-Array ein, wenn der Name noch nicht vorhanden ist
        foreach ($array1 as $item) {
            if (!in_array($item['name'], $names)) {
                $result[] = $item;
                $names[] = $item['name'];
            }
        }

        // Füge alle Einträge aus Array2 in das Ergebnis-Array ein, wenn der Name noch nicht vorhanden ist
        foreach ($array2 as $item) {
            if (!in_array($item['name'], $names)) {
                $result[] = $item;
                $names[] = $item['name'];
            }
        }

        return $result;
    }

	/**
	 * @return string[]
	 */
	public function getCookieNames(): array
	{
		return array_map(function (array $row) {
			return (string)$row['name'];
		}, $this->data['cookies'] ?? []);
	}

	/**
	 * @return Cookie[]
	 */
	public function getLocalStorage(): array
	{
		return array_map(function (array $row) {
			return new Cookie($row, null);
		}, $this->data['localStorage'] ?? []);
	}

	/**
	 * @return Cookie[]
	 */
	public function getSessionStorage(): array
	{
		return array_map(function (array $row) {
			return new Cookie($row, null);
		}, $this->data['sessionStorage'] ?? []);
	}

	/**
	 * @return Resource[]
	 */
	public function getResources(): array
	{
		return array_map(function (array $row) {
			return new Resource($row, $this->data['cookieUrlMapping']);
		}, $this->data['resources'] ?? []);
	}

	public function getStatistics(): array
	{
		return $this->data['shield'] ?? [];
	}

	public function addLegacyCookie(LegacyCookie $cookie): void
	{
		$cookieNames = $this->getCookieNames();
		if (in_array($cookie->getHttpCookieName(), $cookieNames)) {
			return;
		}

		$lifetime = $cookie->getLifetimeHours($sessionFallback=-1, $fallback=0);
		$this->data['cookies'][] = [
			'name' => $cookie->getHttpCookieName(),
			'expires' => ($lifetime > 0) ? time()+(3600*$lifetime) : null,
			'session' => ($lifetime == -1),
		];
	}

    /**
     * cbCookiesUndColor
     * @return Cookie[]
     */
    public function getCbCookiesUndColor(): array
    {
        return $this->data['cbCookiesUndColor'] ?? array();
    }

	public function getCloudDataPageCount(): int
	{
		return $this->data['cloudData']['googlePageCount'] ?? 99;
	}

	public function getCloudDataCMS()
	{
		return $this->data['cloudData']['cmsName'] ?? "";
	}
}
