<?php

namespace App\Component\Plugin;

use Symfony\Component\HttpKernel\Controller\ArgumentValueResolverInterface;
use Symfony\Component\HttpKernel\ControllerMetadata\ArgumentMetadata;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Convert PluginInfo instances from request attribute variable.
 */
class ModelParamConverterService implements ArgumentValueResolverInterface
{
	function supports(Request $request, ArgumentMetadata $metadata): bool
	{
		return $metadata->getType() == PluginInfo::class
			&& !$metadata->isVariadic()
			&& $request->attributes->has($metadata->getName());
	}

	function resolveInner(Request $request, ArgumentMetadata $metadata): ?PluginInfo
	{
		$argumentType = $metadata->getType();
		assert($argumentType == PluginInfo::class);
		$param = $metadata->getName();
        $value = $request->attributes->get($param);

		if ($value === '' or $value === null) {
			if ($metadata->hasDefaultValue()) {
				return $metadata->getDefaultValue();
			}
			return null;
		}

		if (!is_string($value)) {
			return null;
		}

		global $kernel;
		$manager = $kernel->getContainer()->get('plugin.manager');
		return $manager->findInstalledPlugin($value);
	}

	/**
	 * @return array{0:PluginInfo|null}
	 */
	public function resolve(Request $request, ArgumentMetadata $metadata): iterable
	{
		$result = $this->resolveInner($request, $metadata);

		if ($result === null and !$metadata->isNullable()) {
			$param = $metadata->getName();
			throw new NotFoundHttpException("No such $param found");
		}
		return [$result];
	}
}
