<?php

namespace App\Component\Tcf\v2\Model;

use App\Component\Tcf\v2\Repository;
use App\Component\Tcf\v2\ExtendedDisclosureRepository;
use LogicException;

class DeviceStorageDisclosure extends Model
{
	protected static $repositoryFilename = null;
	protected static $repositoryKey = 'disclosures';

	/** @var Repository|null $repository */
	protected static $repository = null;

	/**
	 * Diese Methode stellt die Konfiguration des aktuellen Models bereit und lädt dieses ggf. neu.
	 *
	 * @return ExtendedDisclosureRepository
	 */
	protected static function openRepository(): Repository
	{
		$repoFilename = static::$repositoryFilename;

		// Gewährleisten, dass dem Model ein Dateiname als Repository beiliegt
		if (empty($repoFilename)) {
			throw new LogicException('No repository defined for '.static::class);
		}
		if (empty(static::$repositoryKey)) {
			throw new LogicException('No repository key defined for '.static::class);
		}

		return ExtendedDisclosureRepository::load($repoFilename);
	}

	public static function selectVendor(Vendor $vendor)
	{
		static::$repositoryFilename = 'deviceStorage-'.$vendor->getId().'.json';
		static::$repository = static::openRepository();
	}

	public function getIdentifier(): string
	{
		return (string)$this->get('identifier');
	}

	/** @return string 'cookie'/'web'/'app' */
	public function getType(): string
	{
		return (string)$this->get('type');
	}

	/** @deprecated mit TCFv2.2 sollte getDomains() verwendet werden */
	public function getDomain(): string
	{
		return $this->getDomains()[0] ?? '';
	}

	/**
	 * @return string[]
	 */
	public function getDomains(): array
	{
		$domains = $this->get('domains');
		if ($domains) {
			return $domains;
		} else {
			$domain = $this->get('domain');
			if ($domain) {
				return [$domain];
			} else {
				return [];
			}
		}
	}

	/**
	 * @return Purpose[]
	 */
	public function getPurposes(): array
	{
		return array_filter(array_map(function ($id) {
			return Purpose::find($id);
		}, $this->get('purposes')));
	}


	public function getCookieMaxAgeSeconds(): ?int
	{
		$value = $this->get('maxAgeSeconds');
		return $value === null ? $value : (int)$value;
	}

	public function hasCookieRefresh(): ?bool
	{
		return $this->get('cookieRefresh');
	}

	public function isUsingNonCookieAccess(): ?bool
	{
		return $this->get('usesNonCookieAccess');
	}

	public function getHttpGetLimit(): ?int
	{
		$data = $this->get('overflow');
		return $data['httpGetLimit'] ?? null;
	}

	public function getDeviceStorageDisclosureUrl(): ?string
	{
		return $this->get('deviceStorageDisclosureUrl') ?? null;
	}


	public static function getGvlSpecificationVersion(): int
	{
		return self::getRepository()->get('gvlSpecificationVersion');
	}

	public static function getVendorListVersion(): int
	{
		return self::getRepository()->get('vendorListVersion');
	}

	public static function getTcfPolicyVersion(): int
	{
		return self::getRepository()->get('tcfPolicyVersion');
	}

	public function __toString(): string
	{
		return sprintf('[TCFv2 Device Storage Disclosure "%s"]', $this->getIdentifier());
	}
}
