<?php

namespace App\Component\Tcf\v2\Model\Google;

use App\Component\Tcf\v2\Model\Model;
use App\Component\Tcf\v2\Repository;
use App\Component\Tcf\v2\GoogleRepository;
use \LogicException;

class AdTechProvider extends Model
{
	protected static $repositoryFilename = 'google-additional-consent-providers.json';
	protected static $repositoryKey = 'data';

	/** @var GoogleRepository|null $repository */
	protected static $repository = null;

	/**
	 * Diese Methode stellt die Konfiguration des aktuellen Models bereit und lädt dieses ggf. neu.
	 *
	 * @return GoogleRepository
	 */
	protected static function openRepository(): Repository
	{
		$repoFilename = static::$repositoryFilename;

		// Gewährleisten, dass dem Model ein Dateiname als Repository beiliegt
		if (empty($repoFilename)) {
			throw new LogicException('No repository defined for '.static::class);
		}
		if (empty(static::$repositoryKey)) {
			throw new LogicException('No repository key defined for '.static::class);
		}

		return GoogleRepository::load($repoFilename);
	}

	/**
	 * @override
	 */
	public function getId(): string
	{
		return (string)$this->get('provider_id');
	}

	public function getName(): string
	{
		return (string)$this->get('provider_name');
	}

	public function getPolicyUrl(): string
	{
		return (string)$this->get('policy_url');
	}

	/**
	 * @return string[]
	 */
	public function getDomains(): array
	{
		return $this->get('domains', []);
	}

	public function __toString(): string
	{
		return sprintf('[Google Ad Tech Provider %s: "%s"]', $this->getId(), $this->getName());
	}
}
