<?php

namespace App\Component\Tcf\v2\Model;

use App\Component\Tcf\v2\Repository;
use App\Component\Tcf\v2\Model\Translation\Purpose as Translated;
use App\Exception\ConfigAccessException;

class Purpose extends Model
{
	protected static $repositoryFilename = 'vendor-list.json';
	protected static $repositoryKey = 'purposes';

	/** @var Repository|null $repository */
	protected static $repository = null;

	public function getName(): string
	{
		return $this->get('name', '');
	}

	public function getDescription(): string
	{
		return $this->get('description', '');
	}

	/** @deprecated Seit TCFv2.2 */
	public function getLegalDescription(): string
	{
		return (string)$this->get('descriptionLegal', '');
	}

	/** neu in TCFv2.2
	 * @return string[]
	 */
	public function getIllustrations(): array
	{
		return array_filter($this->get('illustrations', []));
	}

	/**
	 * @param string $langCode Sprachcode
	 * @param bool $fallbackUntranslated Wenn true, wird das unübersetzte Model zurückgegeben,
	 *        wenn die Sprache nicht unterstützt wird. Ansonsten gibt es eine Exception.
	 * @return static|Translated
	 */
	public function getTranslation(string $langCode, bool $fallbackUntranslated=true)
	{
		try {
			$result = Translated::find($langCode, $this->getId());
			return ($result) ? $result : $this;
		}
		catch (ConfigAccessException $e) {
			if ($fallbackUntranslated) {
				return $this;
			} else {
				throw $e;
			}
		}
	}

	public function __toString(): string
	{
		return sprintf('[TCFv2 Purpose %1: "%2"]', $this->getId(), $this->getName());
	}
}
