<?php

namespace App\Component\Tcf\v2;

use App\Component\Tcf\v2\Model\Vendor;
use App\Model\Domain;
use App\Model\EmbeddingBase;

class TcfHelper {
	/**
	 * Diese Methode ermittelt die restlichen Anbieter, die nicht bereits durch `$embeddings` referenziert werden.
	 * @param Domain $domain
	 * @param EmbeddingBase[] $embeddings
	 * @return Vendor[]
	 */
	public static function getExtraTcfVendors(Domain $domain, array $embeddings): array
	{
		// Ermittle Anbieter, die in Embeddings referenziert werden
		$referencedTcfVendorIds = array_reduce($embeddings, function (array $ids, EmbeddingBase $embedding) {
			if ($vendor = $embedding->getTcfVendor()) {
				$ids[$vendor->getId()] = $vendor->getId();
			}
			return $ids;
		}, []);

		$blacklistedTcfVendorIds = array_map(function (Vendor $vendor) {
			return $vendor->getId();
		}, $domain->getTcfVendorBlacklist());

		// Ermittle IAB-Anbieter, die weder referenziert noch geblacklistet sind
		$extraTcfVendors = array_filter(Vendor::all(), function (Vendor $vendor) use ($referencedTcfVendorIds, $blacklistedTcfVendorIds) {
			$vendorId = $vendor->getId();
			return !in_array($vendorId, $referencedTcfVendorIds) && !in_array($vendorId, $blacklistedTcfVendorIds);
		});

		// Sortiere Anbieter nach ihren Namen
		usort($extraTcfVendors, function (Vendor $a, Vendor $b) {
			return strnatcasecmp($a->getName(), $b->getName());
		});

		return $extraTcfVendors;
	}
}
