<?php

namespace App\Component\ThumbnailExtractor;

class OsmThumbnailExtractor extends AbstractThumbnailExtractor
{
	const URL_REGEX = '~^(?:https?:)?//(?:www\.)?openstreetmap\.org/export/embed\.html\?bbox=(?<minlon>[0-9.]{1,20})(?:,|%2C)(?<minlat>[0-9.]{1,20})(?:,|%2C)(?<maxlon>[0-9.]{1,20})(?:,|%2C)(?<maxlat>[0-9.]{1,20})~i';

	public function supportsUrl(string $url): bool
	{
		if (preg_match(static::URL_REGEX, $url)) {
			return true;
		}
		else {
			return false;
		}
	}

	public function fetchEmbedMetadata(string $url): ?EmbedInformation
	{
		$match = [];
		preg_match(static::URL_REGEX, $url, $match);

		// Koordinaten für ein Tile bestimmen, das möglichst nah am gewünschten Bereich und Zoom-Level ist
		$minlon = (float)$match['minlon'];
		$maxlon = (float)$match['maxlon'];
		$minlat = (float)$match['minlat'];
		$maxlat = (float)$match['maxlat'];
		$lon = ($minlon+$maxlon) / 2;
		$lat = ($minlat+$maxlat) / 2;
		$zoom = 19;
		foreach (range(18,1) as $zoom) {
			$minx = $this->getX($minlon, $zoom);
			$miny = $this->getY($minlat, $zoom);
			$maxx = $this->getX($maxlon, $zoom);;
			$maxy = $this->getY($maxlat, $zoom);;
			if (abs($maxx - $minx) < 1.05 and abs($maxy - $miny) < 1.10) {
				break;
			}
		}
		$x = $this->getX($lon, $zoom);
		$y = $this->getY($lat, $zoom);
		$xtile = (int)floor($x);
		$ytile = (int)floor($y);

		$title = '';
		$thumbnailUrl = 'https://tile.openstreetmap.org/'.$zoom.'/'.$xtile.'/'.$ytile.'.png';
		$thumbnailWidth = 256;

		// Daten und erstes Thumbnail zusammenfassen
		$result = new EmbedInformation();
		$result->type = 'map';
		$result->id = $zoom.'_'.$xtile.'_'.$ytile;
		$result->url = $url;
		$result->title = $title;
		$result->providerName = 'OpenStreetMap';
		$result->thumbnails = [
			new ThumbnailInformation([
				'originalUrl' => $thumbnailUrl,
				'width'=> $thumbnailWidth,
			])
		];

		return $result;
	}

	private function getX(float $lon, int $zoom): float
	{
		return (($lon + 180) / 360) * pow(2, $zoom);
	}

	private function getY(float $lat, int $zoom): float
	{
		return (1 - log(tan(deg2rad($lat)) + 1 / cos(deg2rad($lat))) / pi()) /2 * pow(2, $zoom);
	}
}
