<?php

namespace App\Component\ThumbnailExtractor;

class Stream24ThumbnailExtractor extends AbstractThumbnailExtractor
{
	const URL_REGEX = '~^(?:https?:)?//api\.stream24\.net/vod/getVideo\.php\?.*(?<=[?&])id=(?<video_id>[^&]+)~i';

	public function supportsUrl(string $url): bool
	{
		return (bool)preg_match(static::URL_REGEX, $url);
	}

	public function fetchEmbedMetadata(string $url): ?EmbedInformation
	{
		if (!preg_match(static::URL_REGEX, $url, $match)) {
			return null;
		}

		$videoId = $match['video_id'];
		$htmlCode = (string)$this->fetchText($url);

		if (!$htmlCode) {
			return null;
		}

		$title = preg_match('~<title>([^<]+)</title>~', $htmlCode, $match)
			? html_entity_decode($match[1])
			: '';
		$imageUrl = preg_match('~poster: \[\'(?:https?:)?//(?<image_url>media\.stream24\.net/media/[^\']+)~', $htmlCode, $match)
			? 'https://' . str_replace("\'", "'", $match['image_url'])
			: '';

		if (!$imageUrl) {
			return null;
		}

		$result = new EmbedInformation();
		$result->type = 'video';
		$result->id = $videoId;
		$result->url = $url;
		$result->title = $title;
		$result->providerName = 'stream24';
		$result->thumbnails = [
			new ThumbnailInformation([
				'originalUrl' => $imageUrl,
				'width'=> 1280,
			])
		];

		return $result;
	}
}
