<?php

namespace App\Component\Translation;

use App\Component\MongoDbService;
use Symfony\Component\Translation\Dumper\DumperInterface;
use Symfony\Component\Translation\MessageCatalogue;
use MongoDb;

/**
 * Database Loader
 */
class MongoDbDumper implements DumperInterface
{
	const COLLECTION_NAME = 'cm-translations';

	/** @var MongoDbService $service */
	private $service = null;

	public function __construct(MongoDbService $service)
	{
		$this->service = $service;
	}

	public function dumpDomain($assocArray, $locale, $domain, $session=null)
	{
		$collection = $this->service->getDatabase()->selectCollection(self::COLLECTION_NAME);

		// Erstellen der Indices beim ersten Zugriff
		try {
			if ($collection->estimatedDocumentCount() === 0) {
				if (iterator_count($collection->listIndexes()) == 0) {
					$collection->createIndex(['locale'=>1, 'domain'=>1, 'token'=>1]);
				}
			}
		}
		catch (\Exception $exception) {
			// ignore
		}

		// Alte Einträge löschen
		$collection->deleteMany(['domain'=>['$eq', $domain], 'locale'=>['$eq'=>$locale]], ['session'=>$session]);
		// Neue Einträge schreiben
		$collection->insertMany(array_map(function ($key, $value) use ($domain, $locale) {
			return [
				'domain' => $domain,
				'locale' => $locale,
				'token' => $key,
				'content' => $value
			];
		}, array_keys($assocArray), array_values($assocArray)), ['session'=>$session]);
	}

	/**
	 * {@inheritDoc}
	 *
	 * @param MessageCatalogue $messages
	 * @param array<string,mixed> $options
	 */
	public function dump(MessageCatalogue $messages, array $options = []): void
	{
		$session = $this->service->getClient()->startSession();

		try {
			foreach ($messages->getDomains() as $domain) {
				MongoDB\with_transaction($session, function ($session) use ($messages, $domain) {
					$data = $messages->all($domain);
					$this->dumpDomain($data, $messages->getLocale(), $domain, $session);
				});
				$path = $options['path'] ?? '';
				if ($path) {
					file_put_contents(sprintf('%s/%s.%s.database', $path, $domain, $messages->getLocale()), '# autogenerated by CCM19');
				}
			}
		}
		finally {
			$session->endSession();
		}
	}
}
