<?php

namespace App\Component\Translation;

use App\Component\MongoDbService;
use MongoDB\Collection;
use Symfony\Component\Translation\Loader\LoaderInterface;
use Symfony\Component\Translation\MessageCatalogue;

/**
 * Database Loader
 */
class MongoDbLoader implements LoaderInterface
{
	const COLLECTION_NAME = 'cm-translations';

	/** @var Collection|null $collection */
	private $collection = null;

	public function __construct(MongoDbService $service)
	{
		// Nur laden, wenn auch eine Datenbank bekannt ist
		if (!empty($_ENV['APP_MONGODB_DATABASE'])) {
			$this->collection = $service->getDatabase()->selectCollection(self::COLLECTION_NAME);

		}
	}

	/**
	 * {@inheritDoc}
	 */
	public function load($resource, string $locale, string $domain = 'messages'): MessageCatalogue
	{
		$catalogue = new MessageCatalogue($locale);

		if (!$this->collection) {
			return $catalogue;
		}

		// Übersetzungen aus DB holen
		$translations = $this->collection->find(['domain'=>['$eq' => $domain], 'locale'=>['$eq'=>$locale]], ['projection'=>['token'=>1, 'content'=>1]]);

		// Assoziatives Übersetzungs-Array bauen
		$messages = [];
		foreach ($translations as $document) {
			$messages[(string)$document['token']] = (string)$document['content'];
		}

		// Katalog füllen
		$catalogue->add($messages, $domain);
		return $catalogue;
	}
}
