<?php

namespace App\Controller;

use App\Component\Menu\Annotation\Menu;
use App\Model\CsrfToken;
use App\Model\Domain;
use App\Component\Plugin\Manager;
use App\Component\Plugin\PluginInfo;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * @Route("/domains/{_domainId}/available_plugins", name="app_available_plugins")
 */
class AvailablePlugins extends DomainDependantController
{
	/**
	 * @Menu("Available Plugins", group="System", icon="fa-sitemap", editions={"extended"}, order=220)
	 * @Route("", name="")
	 *
	 * @param Request $request
	 * @param Manager $manager
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function index(Request $request, Manager $manager, TranslatorInterface $translator): Response
	{
		$listOfActivatedPlugins = array();
		$domain = Domain::activeDomain();
		$availablePlugins = array_filter(
			$manager->configuredPlugins(),
			function ($plugin) {
				if($plugin->getState()->isAllowedForCurrentUser())
				{
					if(!$plugin->isHidden() && !$plugin->isBroken() && $plugin->isActivatablePerDomain()){
						return true;
					}
				}
			}
		);
		$activatedPluginIds = $domain ? $domain->getActivatedPluginIds() : [];

		//Template ausgeben
		return $this->render('plugins_available/index.html.twig', [
			'availablePlugins' => $availablePlugins,
			'activatedIds' => $activatedPluginIds,
			'csrfToken' => CsrfToken::get(),
		]);
	}

	/**
	 * @Route("/{plugin<.+>}",  name="_configure",  methods={"POST"})
	 * @param PluginInfo|null $plugin
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function configurePlugin(?PluginInfo $plugin, Request $request, TranslatorInterface $translator): Response
	{
		if (!$plugin) {
			return $this->redirectToRoute('app_plugins');
		}
		$action = $request->request->getAlnum('action', '');
		$domain = Domain::activeDomain();
		assert($domain !== null);

		switch ($action) {
			case "deactivate":
				$domain->deactivatePlugin($plugin->getName())->save();
				$this->addFlash('danger',  $translator->trans('The Plugin is now deactivated'));
				break;
			case "activate":
				$domain->activatePlugin($plugin->getName())->save();
				$this->addFlash('success',  $translator->trans('The Plugin is now activated'));
				break;
		}
		return $this->redirectToRoute('app_available_plugins', [], 303);
	}
}
