<?php

namespace App\Controller;

use App\Model\Domain;
use App\Model\User;
use App\Utils;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

class DefaultController extends AbstractController
{
	/**
	 * @Route("/", name="app_root")
	 * @param Request $request
	 * @return Response
	 */
	public function index(Request $request): Response
	{
		$user = User::loggedInUser();
		$domain = Domain::activeDomain();

		if (Utils::isExtendedEdition()) {
			// Wenn Zugriff auf Domainindex, dann dorthin
			if (Utils::mayUserAccessRoute($user, 'app_domains_index', 'GET')) {
				return $this->redirectToRoute('app_domains_index', [], 303);
			}
			// Wenn nicht (z.B. eingeschränkter Subuser), dann zum ersten Domain-Dashboard
			elseif (Domain::count() > 0) {
				$domain = current(Domain::all());
				return $this->redirectToRoute('app_dashboard', ['_domainId'=>$domain->getId()], 303);
			}
			// Wenn auch keine Domains zugeordnet sind, dann zu Benutzerkonto
			elseif (Utils::mayUserAccessRoute($user, 'app_account', 'GET')) {
				return $this->redirectToRoute('app_account', [], 303);
			}
			// Wenn auch das nicht geht, dann leere Inhaltsseite
			else {
				return $this->render('base.html.twig', ['_metaSidebar' => true]);
			}
		}

		if (Utils::isMultiDomainEdition()) {
			// Wenn Zugriff auf Domainindex, dann dorthin
			if (Utils::mayUserAccessRoute($user, 'app_domains_index', 'GET')) {
				return $this->redirectToRoute('app_domains_index', [], 303);
			}
		}

		//Nur falls keine Domain vorhanden ist... Notfall - tritt eigentlich nur auf wenn Verzeichnisse außerhalb gelöscht wurden
		if (Domain::count() == 0) {
			$domain = Domain::createForUser($user->getAccountOwner());
			$domain
				->setName("")
				->setManagementStructurePreLegalConsent(true)
				->setManagementStructure(Domain::MANAGEMENT_STRUCTURE_EMBEDDING)
				->setWhitelabel(false)
				->save();
		}

		return $this->redirectToRoute('app_dashboard', [], 303);
	}

}
