<?php

namespace App\Controller\Hosting\Settings;

use App\Component\Menu\Annotation\Menu;
use App\Config;
use App\Controller\HostingController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * @Route("/hosting/settings/frontend", name="app_hosting_settings_frontend_", methods={"GET", "HEAD"})
 */
class Frontend extends HostingController
{
	/**
	 * @Menu("Frontend", group="System", icon="fa-desktop", order=250)
	 * @Route("", name="index")
	 * @return Response
	 */
	public function frontend(): Response
	{
		return $this->render('hosting/settings/frontend.html.twig');
	}

	/**
	 * @Route("", name="index_post", methods={"POST"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function frontendFormSubmit(Request $request, TranslatorInterface $translator): Response
	{
		Config::getInstance()
			->setShowFrontendWidgetOnlyIfEmbeddingsExist($request->request->getBoolean('showFrontendWidgetOnlyIfEmbeddingsExist'))
			->flush();

		$this->addFlash('success', $translator->trans('The settings were saved successfully!'));

		return $this->redirectToRoute('app_hosting_settings_frontend_index', [], 303);
	}
}
