<?php

namespace App\Controller;

use App\Component\Menu\Annotation\Menu;
use App\Component\ThumbnailExtractor\ThumbnailExtractorService;
use App\Model\Domain;
use App\Model\User;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * @Route("/domains/{_domainId}/iframes", name="app_iframes_")
 */
class Iframes extends DomainDependantController
{
	/**
	 * @Menu("Iframes", group="Cookies & Settings", icon="fa-coffee", order=40)
	 * @Route("", methods={"GET", "HEAD"}, name="list")
	 *
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function index(Request $request, TranslatorInterface $translator): Response
	{
			// Scripte holen
		//$scripts = Script::all();

		//Die Daten aus dem Form
		$enableScriptPostDat = $request->request->get('enableScript');

		return $this->render('iframe/data.html.twig', [
			'iframes' => "",
		]);
	}

	/**
	 * @Route("", methods={"POST"}, name="post")
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @param ThumbnailExtractorService $thumbnails
	 * @return Response
	 */
	public function saveIframeSettings(Request $request, TranslatorInterface $translator, ThumbnailExtractorService $thumbnails): Response
	{
		/** @var User $user */
		$user = User::loggedInUser();
		/** @var Domain $domain */
		$domain = Domain::activeDomain();
		Domain::beginWrite();

		if ($request->request->get('action') == 'clearThumbnails')
		{
			$thumbnails->clearCache($user);
			$this->addFlash('success', $translator->trans('Thumbnail cache was cleared.'));
			return $this->redirectToRoute('app_iframes_list', [], 303);
		}

		// Iframe-Ausnahmen
		$markers = explode("\n", trim((string)$request->request->get('iframeMarkers', '')));
		$markers = array_filter($markers, function ($line) {
			return trim($line) !== '';
		});
		$markers = array_values(array_map(function ($line) {
			return trim($line);
		}, $markers));

		$domain
			->setIframesBlocked((bool)$request->request->get('blockIframes'))
			->setRememberIframeConsentPerDomain((bool)$request->request->get('rememberIframeConsentPerDomain'))
			->setAppendConsentSwitchToIframes($request->request->getBoolean('appendConsentSwitchToIframes'))
			->setIframeBlockMode($request->request->get('iframeBlockMode') ?? "")
			->setIframeMarkers($markers)
			->save();


		$this->addFlash('success', $translator->trans('Settings have been saved.'));

		return $this->redirectToRoute('app_iframes_list', [], 303);
	}

}
