<?php

namespace App\Controller\Settings;

use App\Config;
use App\Component\MailerConfigurator;
use App\Component\Menu\Annotation\Menu;
use App\Controller\AdminController;
use App\Exception\CsrfTokenException;
use App\Component\Mail\MailMessage;
use App\Model\CsrfToken;
use App\Utils;
use Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Contracts\Translation\TranslatorInterface;
use Symfony\Component\Mailer\MailerInterface;

/**
 * @Route("/settings/email", name="app_setting_email")
 */
class Email extends AdminController
{

	/**
	 * @Menu("Email settings", group="System", icon="glyphicon-envelope",
	 *     order=220)
	 * @Route("", name="", methods={"GET","HEAD"})
	 * @phan-suppress PhanTypePossiblyInvalidDimOffset
	 * @param Request $request
	 * @return Response
	 */
	public function showSettings(Request $request): Response
	{
		$config = Config::getInstance();

		$mailer_url = $_ENV['MAILER_URL'] ?? 'null://null';
		$mailer_data = parse_url($mailer_url);
		$mailer_options = [];
		parse_str($mailer_data['query'] ?? '', $mailer_options);
		if ($mailer_data['scheme'] == 'smtp' and !isset($mailer_options['auth_mode'])) {
			$mailer_options['auth_mode'] = 'null';
		}

		if(empty($config->get("sub_crawler_activated_email_betreff")) && empty($config->get("sub_crawler_email_betreff"))){
			$this->setCrawlerMailTexteWennLeer();
		}

		//print_r($config->get("sub_cookie_email_check"));

		//Alle Variablen fürs Template setzen
		$variables = [
			'email' => $config->get('email'),
			'senderEmail' => $config->get('senderEmail'),
			'senderName' => $config->get('senderName'),
			'replyTo' => $config->get('replyTo'),
			'emailScheme' => $mailer_data['scheme'],
			'emailHost' => $mailer_data['host'] ?? '',
			'emailPort' => $mailer_data['port'] ?? '',
			'emailUser' => $mailer_options['username'] ?? '',
			'emailPass' => $mailer_options['password'] ?? '',
			'emailEncryption' => $mailer_options['encryption'] ?? '',
			'emailAuthMode' => $mailer_options['auth_mode'] ?? '',
			'csrfToken' => CsrfToken::get(),
			'sendQuotaCopies' => (bool)$config->get('sendQuotaCopies'),
			'sendUpdateNotifications' => (bool)$config->get('sendUpdateNotifications', false),
			'max_traffic_email_betreff' => $config->get("max_traffic_email_betreff"),
			'max_traffic_email_content' => $config->get("max_traffic_email_content"),
			'sub_max_traffic_email_betreff' => $config->get("sub_max_traffic_email_betreff"),
			'sub_max_traffic_email_content' => $config->get("sub_max_traffic_email_content"),
			'sub_update_email_betreff' => $config->get("sub_update_email_betreff"),
			'sub_update_email_content' => $config->get("sub_update_email_content"),
			'sub_cookie_email_betreff' => $config->get("sub_cookie_email_betreff"),
			'sub_cookie_email_content' => $config->get("sub_cookie_email_content"),
			'sub_cookie_email_check' => $config->get("sub_cookie_email_check"),
			'sub_crawler_activated_email_content' => $config->get("sub_crawler_activated_email_content"),
			'sub_crawler_activated_email_betreff' => $config->get("sub_crawler_activated_email_betreff"),
			'sub_crawler_activated_email_check' => $config->get("sub_crawler_activated_email_check"),
			'sub_crawler_email_content' => $config->get("sub_crawler_email_content"),
			'sub_crawler_email_check' => $config->get("sub_crawler_email_check"),
			'sub_crawler_email_betreff' => $config->get("sub_crawler_email_betreff"),
			'callLimitNotificationSteps' => trim($config->get("callLimitNotificationSteps")),
		];

		return $this->render('settings/email/index.html.twig', $variables);
	}

	/**
	 * @Route("", name="_post", methods={"POST"})
	 * @param Request $request
	 * @param MailerConfigurator $configurator
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function saveSettings(Request $request, MailerConfigurator $configurator, TranslatorInterface $translator): Response
	{
		try {
			(CsrfToken::get())->checkRequest($request);
		} catch (CsrfTokenException $e) {
			$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
			return $this->redirectToRoute('app_setting_email', [], 303);
		}

		$config = Config::getInstance();

		if (Utils::isExtendedEdition()) {
			if (empty($request->request->get('max_traffic_email_betreff')) || empty($request->request->get('max_traffic_email_content'))) {
				$this->addFlash('danger', $translator->trans('Subject or content of a mail template is empty. Please fill out these fields.'));
				return $this->redirectToRoute('app_setting_email', [], 303);
			}
			$config->set('max_traffic_email_betreff', $request->request->get('max_traffic_email_betreff'));
			$config->set('max_traffic_email_content', $request->request->get('max_traffic_email_content'));
			$config->set('sub_max_traffic_email_betreff', $request->request->get('sub_max_traffic_email_betreff'));
			$config->set('sub_max_traffic_email_content', $request->request->get('sub_max_traffic_email_content'));
			$config->set('sub_update_email_betreff', $request->request->get('sub_update_email_betreff'));
			$config->set('sub_update_email_content', $request->request->get('sub_update_email_content'));
			$config->set('sendQuotaCopies', (bool)$request->request->get('sendQuotaCopies'));
			$config->set('sendUpdateNotifications', (bool)$request->request->get('sendUpdateNotifications'));

			$config->set('sub_crawler_email_betreff', $request->request->get('sub_crawler_email_betreff'));
			$config->set('sub_crawler_email_content', $request->request->get('sub_crawler_email_content'));
			$config->set('sub_crawler_email_check', (bool)$request->request->get('sub_crawler_email_check'));

			$config->set('sub_crawler_activated_email_betreff', $request->request->get('sub_crawler_activated_email_betreff'));
			$config->set('sub_crawler_activated_email_content', $request->request->get('sub_crawler_activated_email_content'));
			$config->set('sub_crawler_activated_email_check', (bool)$request->request->get('sub_crawler_activated_email_check'));

			//Call Limit durchgehen und passend setzen, zwischen 80 und 99
			$cl = explode("\n", $request->request->get('callLimitNotificationSteps'));

			if ($cl) {
				$callLimit = "";
				foreach ($cl as $k => $v) {
					if (is_numeric(trim($v)) && $v >= 80 && $v <= 99) {
						$callLimit .= trim($v) . "\n";
					}
				}
			}

			if (empty($callLimit) || strlen($callLimit) < 2) {
				$callLimit = "90\n95\n96";
			}

			$config->set('callLimitNotificationSteps', trim($callLimit));
		}

		//FIx für ?? Warum musste das 0 sein...
		if (empty($request->request->get('sub_cookie_email_check'))) {
			$config->set('sub_cookie_email_check', 0);
		} else {
			$config->set('sub_cookie_email_check', $request->request->get('sub_cookie_email_check'));
		}

		//Immmer anzeigen...
		$config->set('sub_cookie_email_betreff', $request->request->get('sub_cookie_email_betreff'));
		$config->set('sub_cookie_email_content', $request->request->get('sub_cookie_email_content'));


		$scheme = $request->request->get('emailScheme', 'null');
		$host = $request->request->get('emailHost');
		$port = $request->request->get('emailPort');
		$user = $request->request->get('emailUser', '');
		$pass = $request->request->get('emailPass', '');
		$encryption = $request->request->get('emailEncryption', '');
		$auth_mode = $request->request->get('emailAuthMode', '');

		// Absender und Reply-To setzen
		$config->set('senderEmail', $request->request->get('senderEmail'));
		$config->set('senderName', $request->request->get('senderName'));
		$config->set('replyTo', $request->request->get('replyTo'));

		// Mailer-Konfiguration setzen
		$configurator->setMailer($scheme, $host, $port, $encryption, $auth_mode, $user, $pass);
		//print_r($config);exit();
		$config->flush();

		return $this->redirectToRoute('app_setting_email', [], 303);
	}

	private function setCrawlerMailTexteWennLeer(): void
	{
		$config = Config::getInstance();

		$config->set('sub_crawler_email_betreff', "Achtung - auf {{ domainvar }} hat CCM19 neue Einbindungen und Cookies gefunden.");
		$config->set('sub_crawler_email_content', "Sehr geehrte Damen und Herren,

der CCM19 Crawler hat auf Ihrer Domain: {{ domainvar }} neue Cookies oder Skripte gefunden. 
{{ uncontrolledData }}
Bitte überprüfen Sie die Daten in Ihrem CCM19 Account unter dem Menüpunkt Einbindungen.
Gemäß Ihrer Vorgaben wurden die Einbindungen nicht aktiviert - bitte überprüfen Sie die Einbindungen und aktivieren Sie diese, wenn Sie diese benötigen.
Falls Sie diese nicht aktivieren, werden die Cookies und Skripte ohne Consent für Ihre Besucher ausgespielt, was rechtlich problematisch sein könnte.

Bei Fragen wenden Sie sich bitte an unser Support Team!

Mit herzlichen Grüßen,
Ihr Team von ....");

		//Crawler Found Mail wurde aktiviert
		$config->set('sub_crawler_activated_email_betreff', "Achtung - auf {{ domainvar }} hat CCM19 neue Einbindungen und Cookies gefunden.");
		$config->set('sub_crawler_activated_email_content', "Sehr geehrte Damen und Herren,

der CCM19 Crawler hat auf Ihrer Domain: {{ domainvar }} neue Cookies oder Skripte gefunden. 
{{ uncontrolledData }}
Bitte überprüfen Sie die Daten in Ihrem CCM19 Account unter dem Menüpunkt Einbindungen.
Gemäß Ihrer Vorgaben wurden die Einbindungen direkt aktiviert - bitte überprüfen Sie die Einbindungen und deaktivieren Sie diese, falls Sie diese nicht benötigen.

Bei Fragen wenden Sie sich bitte an unser Support Team!

Mit herzlichen Grüßen,
Ihr Team von ....");
		$config->flush();

	}

	/**
	 * @Route("/sendTestEmail", name="_sendmail", methods={"POST"})
	 * @param \Symfony\Component\HttpFoundation\Request $request
	 * @param \Symfony\Component\Mailer\MailerInterface $mailer
	 * @param \Symfony\Contracts\Translation\TranslatorInterface $translator
	 *
	 * @return JsonResponse
	 * @throws \Symfony\Component\Mailer\Exception\TransportExceptionInterface
	 */
	public function sendTestEmail(Request $request, MailerInterface $mailer, TranslatorInterface $translator): JsonResponse
	{
		$config = Config::getInstance();
		$recipient = $request->request->get('recipient');

		if (!$recipient) {
			return $this->json(['error' => 'no recipient'], 400);
		}

		$message = (new MailMessage($translator->trans('CCM19 Test Email')))
			->to($recipient)
			->text($translator->trans("This is a CCM19 test mail, sent to check if emails sent from CCM19 can be delivered."));

		try {
			$mailer->send($message);
		} catch (Exception $e) {
			return $this->json(['status' => 'failed', 'error' => htmlspecialchars($e->getMessage()), 'errno' => $e->getCode()]);
		}
		return $this->json(['status' => 'ok']);
	}

	/**
	 * @Route("/autoconfig", name="_autoconfig", methods={"POST"})
	 * @param \Symfony\Component\HttpFoundation\Request $request
	 * @param \App\Component\MailerConfigurator $configurator
	 *
	 * @return JsonResponse
	 * @phan-suppress PhanTypePossiblyInvalidDimOffset,
	 *     PhanTypeMismatchArgumentInternal
	 */
	public function smtpAutoconfig(Request $request, MailerConfigurator $configurator): JsonResponse
	{
		try {
			(CsrfToken::get())->checkRequest($request);
		} catch (CsrfTokenException $e) {
			return $this->json(['error' => 'invalid csrf token'], 403);
		}

		$result = [];
		$email = $request->request->get('email');
		$host = $request->request->get('host');
		$domain = substr($email, strrpos($email, '@') + 1);

		if (!$domain) {
			return $this->json(['error' => 'no email domain', 400]);
		}

		$urls = [
			'https://autoconfig.%s/mail/config-v1.1.xml?emailaddress=%s',
			'https://%s/.well-known/autoconfig/mail/config-v1.1.xml',
			'https://autoconfig.%s/mail/config-v1.1.xml',
			'http://%s/.well-known/autoconfig/mail/config-v1.1.xml',
			'https://autoconfig.thunderbird.net/v1.1/%s',
		];
		$config = [];
		foreach ($urls as $url) {
			$url = sprintf($url, rawurlencode($domain), rawurlencode($email));
			$config = $configurator->readConfigXml($url);
			if ($config) {
				break;
			}
		}

		if ($config) {
			if (isset($config[strtolower($domain)])) {
				$data = $config[strtolower($domain)][0];
			} else {
				reset($config);
				$data = current($config)[0];
			}

			$host = $configurator->autoconfigReplaceVars($data['hostname'], $email);
			$port = $data['port'];
			$ssl = (strtoupper($data['socketType']) == 'SSL');
			$starttls = (strtoupper($data['socketType']) == 'STARTTLS');
			$username = $configurator->autoconfigReplaceVars($data['username'], $email);
			$testdata = $configurator->testEmailServer($host, $port, $ssl);

			if ($testdata && $testdata['auth']) {
				// Direkten Test und Autoconfig-Datei abgleichen
				if (isset($data['authentication']['password-encrypted']) and isset($testdata['auth']['CRAM-MD5'])) {
					$auth = 'cram-md5';
				} elseif (isset($data['authentication']['secure']) and isset($testdata['auth']['CRAM-MD5'])) {
					$auth = 'cram-md5';
				} elseif (isset($data['authentication']['password-cleartext']) and isset($testdata['auth']['LOGIN'])) {
					$auth = 'login';
				} elseif (isset($data['authentication']['password-cleartext']) and isset($testdata['auth']['PLAIN'])) {
					$auth = 'plain';
				} elseif (isset($data['authentication']['oauth2'])) {
					$auth = 'oauth';
				} elseif (isset($data['authentication']['client-ip-address'])) {
					$auth = 'null';
				} elseif (isset($data['authentication']['none'])) {
					$auth = 'null';
				} else {
					$auth = 'plain';
				}
			} else {
				// Wenn kein direkter Test möglich, beste Variante aus der Autoconfig-Datei raten
				if (isset($data['authentication']['password-encrypted'])) {
					$auth = 'cram-md5';
				} elseif (isset($data['authentication']['secure'])) {
					$auth = 'cram-md5';
				} elseif (isset($data['authentication']['password-cleartext'])) {
					$auth = 'login';
				} elseif (isset($data['authentication']['oauth2'])) {
					$auth = 'oauth';
				} elseif (isset($data['authentication']['client-ip-address'])) {
					$auth = 'null';
				} elseif (isset($data['authentication']['none'])) {
					$auth = 'null';
				} else {
					$auth = 'plain';
				}
			}
			$result = [
				'emailHost' => $host,
				'emailPort' => $port,
				'emailEncryption' => ($ssl ? 'ssl' : ($starttls ? 'tls' : 'null')),
				'emailAuthMode' => $auth,
				'emailUser' => $username,
			];
		} else {
			// E-Mail-Server raten, wenn keine Autoconfig-Informationen
			if ($host) {
				$hosts = [$host, 'mail.' . $domain, $domain];
			} else {
				$hosts = ['mail.' . $domain, $domain];
			}
			$data = null;
			foreach ($hosts as $server) {
				$data = $configurator->testEmailServer($server, 587, false);
				if ($data) {
					$data['host'] = $server;
					$data['port'] = 587;
					$data['ssl'] = false;
					break;
				}
				$data = $configurator->testEmailServer($server, 465, true);
				if ($data) {
					$data['host'] = $server;
					$data['port'] = 465;
					$data['ssl'] = true;
					break;
				}
				$data = $configurator->testEmailServer($server, 25, false);
				if ($data) {
					$data['host'] = $server;
					$data['port'] = 25;
					$data['ssl'] = false;
					break;
				}
			}
			if ($data) {
				// Host zu IP und zurück auflösen, um den *richtigen* Hostnamen zu bekommen
				$resolvedHost = @gethostbyaddr(gethostbyname($data['host']));
				if ($resolvedHost) {
					$data['host'] = $resolvedHost;
				}
				$result = [
					'emailHost' => $data['host'],
					'emailPort' => $data['port'],
					'emailEncryption' => ($data['ssl'] ? 'ssl' : ($data['starttls'] ? 'tls' : 'null')),
					'emailAuthMode' => ($data['auth']['CRAM-MD5'] ?? $data['auth']['LOGIN'] ?? $data['auth']['PLAIN']),
				];
			}
		}

		return $this->json($result, ($result ? 200 : 404));
	}
}
