<?php

namespace App\Controller\Settings;

use App\Component\Menu\Annotation\Menu;
use App\Controller\AdminController;
use App\Exception\CsrfTokenException;
use App\Model\BackendLanguage;
use App\Model\CsrfToken;
use App\Model\Language;
use App\Model\TransLate as TranslateModel;
use App\Utils;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Yaml\Yaml;
use Symfony\Contracts\Translation\TranslatorInterface;

class Translate extends AdminController
{
	/**
	 * @Menu("Translate Base", group="System", icon="fa-flag", order=255, envs={"dev"})
	 * @Route("/settings/app_translate", name="app_translate", methods={"HEAD", "GET"})
	 * @param Request $request
	 * @return Response
	 */
	public function index(Request $request): Response
	{

		$deeplKeyAll = TranslateModel::all();
		if(!empty(array_key_first($deeplKeyAll))) {
			$deeplKey = $deeplKeyAll[array_key_first($deeplKeyAll)];
			$deeplApiKey = $deeplKey->get("deeplAPIKey");
			$googleApiKey = $deeplKey->get("googleAPIKey");
		} else {
			$deeplApiKey = '';
			$googleApiKey = '';
		}
		//print_r($deeplKeyAll);exit();


		//Sprachen holen
		$countLang = $this->getAvailableLangs();

		//Ausgabe der Daten
		return $this->render('settings/translate/index.html.twig', [
			'csrfToken' => CsrfToken::get(),
			'deeplAPIKey' => $deeplApiKey,
			'googleAPIKey' => $googleApiKey,
			'countLang' => $countLang,
		]);
	}

	/**
	 * @Route("/settings/app_translate/translate", name="app_settings_translate_now", methods={"POST"})
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function translateNow(Request $request, TranslatorInterface $translator)
	{
		try {
			CsrfToken::get()->checkRequest($request);
		} catch (CsrfTokenException $e) {
			$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
			return $this->redirectToRoute('app_translate');
		}

		$langId = (string)$request->request->get('id');
		if (!Language::exists($langId)) {
			$this->addFlash('danger', $translator->trans('No Valid Language ID - please Select one.'));
			return $this->redirectToRoute('app_translate');
		}

		//check ob Key vorhanden - sonst raus
		$deeplApiKey = TranslateModel::getDeeplApiKey();
		$googleApiKey = TranslateModel::getGoogleApiKey();

		if(empty($deeplApiKey) && empty($googleApiKey)) {
			$this->addFlash('danger', $translator->trans('No Valid Api Key - please Enter.'));
			return $this->redirectToRoute('app_translate');
		}

		// Fehlende Informationen in /share/skeleton/clickskeks nachtragen
		TranslateModel::complementClickskeksSkeleton();

		//PLugins übersetzen
		TranslateModel::translateJPlugins($deeplApiKey, $langId);

		//zuerst mal den Beifang übersetzen - muss nur initial laufen...
		TranslateModel::translateJsonsAndCo($deeplApiKey, $langId);

		//Dann Sprachdateien übersetzen
		TranslateModel::translateNow($deeplApiKey, $langId);

		$this->addFlash('success', $translator->trans('This Language is now translated.'));
		return $this->redirectToRoute('app_translate', [], 303);
	}

	/**
	 * @return array<string, float>
	 */
	private function getAvailableLangs(): array
	{
		$activeLangs = BackendLanguage::allActive();
		$countLang = [];
		$filesystem = new Filesystem();

		//base = DE
		$filename = Utils::getBaseDir()."/translations/"."messages.de.yaml";
		$yamlBase = Yaml::parseFile($filename);
		$baseCount = count($yamlBase);
		$baseDir = Utils::getBaseDir();

		// Für alle Sprachen Abdeckung berechnen
		foreach ($activeLangs as $language)
		{
			$langKey = $language->getShortCode();
			$langKeyfile = $langKey;

			$filename = $baseDir."/translations/"."messages.".$langKeyfile.".yaml";
			if($filesystem->exists($filename)) {
				$yamlTarget = Yaml::parseFile($filename);
			} else {
				$filesystem->touch($filename);
				$yamlTarget = Yaml::parseFile($filename);
			}

			$diffCount = count(array_diff_key($yamlBase, (array)$yamlTarget));

			//$countLang[$langKey] = round((count($value)/$baseCount*100), 2);
			$countLang[$langKey] = round((100-(($diffCount*100.0)/$baseCount)), 4);
		}
		return $countLang;
	}

	/**
	 * @Route("/settings/app_translate", name="app_settings_translate_settings_save",  methods={"POST"})
	 *
	 * @param Request $request
	 * @param TranslatorInterface $translator
	 * @return Response
	 */
	public function editTranslateDeeplAPI(Request $request, TranslatorInterface $translator): Response
	{
		//csrf valid?
		try {
			CsrfToken::get()->checkRequest($request);
		} catch (CsrfTokenException $e) {
			$this->addFlash('danger', $translator->trans('CSRF token is missing or invalid. Please try again.'));
			return $this->redirectToRoute('app_translate');
		}

		if(empty($request->request->get("deeplAPIKey")) && empty($request->request->get("googleAPIKey")))
		{
			$this->addFlash('danger', $translator->trans('API key is empty - please fill in an valid API key.'));
			return $this->redirectToRoute('app_translate');
		}

		$deeplKeyAll = TranslateModel::all();
		if(!empty(array_key_first($deeplKeyAll)))
		{
			$deeplKey = $deeplKeyAll[array_key_first($deeplKeyAll)];
		}
		else{
			$newTL = TranslateModel::create();
			$newTL->set("deeplAPIKey","");
			$newTL->set("googleAPIKey","");
			$newTL->save();

			$deeplKeyAll = TranslateModel::all();
			$deeplKey = $deeplKeyAll[array_key_first($deeplKeyAll)];
		}
		$deeplKey->set("deeplAPIKey",$request->request->get("deeplAPIKey"));
		$deeplKey->set("googleAPIKey",$request->request->get("googleAPIKey"));

		$deeplKey->save();

		//In Edit Maske springen...
		return $this->redirectToRoute('app_translate', [], 303);
	}

}
