<?php

namespace App\Controller;

use App\Component\Tcf\v2\Model\Vendor;
use App\Model\SystemEmbedding;
use App\Utils;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * Controller zur Ausgabe domain-unabhängiger TCFv2-Daten innerhalb des Backends
 * @Route("/tcf/v2/", name="app_tcf_")
 */
class Tcf extends UniversalController
{
	/**
	 * @Route("vendor/{vendor}/form", name="xhr_form", methods={"GET", "HEAD"})
	 * @Route("vendor/{vendor}/form/system/{embedding}", name="xhr_system_embedding_form", methods={"GET", "HEAD"})
	 *
	 * @return Response
	 */
	public function form(Vendor $vendor, ?SystemEmbedding $embedding): Response
	{
		// Embedding-Daten nur berücksichtigen, wenn die Vendor-ID stimmt
		if ($embedding and $embedding->getTcfVendor() != $vendor) {
			$embedding = null;
		}

		$response = $this->render('tcf/embedding_form.html.twig', [
			'vendor' => $vendor,
			'flexiblePurposeIds' => $vendor->getFlexiblePurposeIds(),
			'embeddingPurposes' => ($embedding) ? $embedding->getTcfPurposes() : null,
			'embeddingSpecialFeatures' => ($embedding) ? $embedding->getTcfSpecialFeatures() : null,
		]);
		if (!$embedding) {
			Utils::resetCacheControl($response);
			$response->setEtag(Vendor::getTcfPolicyVersion().'-'.Vendor::getGvlSpecificationVersion().'-'.Vendor::getVendorListVersion(), true);
			$response->setCache([
				'max_age'=>300,
				'public'=>true,
			]);
		}
		return $response;
	}

	/**
	 * @Route("vendor/datalist", name="vendorlist")
	 *
	 * @return Response
	 */
	public function vendorlist(): Response
	{
		$response = $this->render('tcf/vendorlist.html.twig', [
			'vendors' => Vendor::all()
		]);

		Utils::resetCacheControl($response);
		$response->setCache([
			'max_age'=>900,
			'public'=>true,
		]);
		$response->setEtag(Vendor::getTcfPolicyVersion().'-'.Vendor::getGvlSpecificationVersion().'-'.Vendor::getVendorListVersion(), true);

		return $response;
	}
}

