<?php

namespace App\Event;

use App\Model\Domain;
use App\Model\Model;
use Symfony\Contracts\EventDispatcher\Event;

final class DomainCloneReferencesEvent extends Event
{
	const NAME = 'ccm19.domain.clone.references';

	/** @var array<class-string<Model>,array> $idMappings */
	private $idMappings;

	/** @var Domain $source */
	private $source;

	/** @var Domain $target */
	private $target;

	/**
	 * @param Domain $source
	 * @param Domain $target
	 * @param array<class-string<Model>,array> $idMappings
	 */
	public function __construct(Domain $source, Domain $target, array $idMappings)
	{
		$this->idMappings = $idMappings;
		$this->source = $source;
		$this->target = $target;
	}

	/**
	 * @param class-string<Model> $kind
	 * @return array
	 */
	public function getIdMapping(string $kind): array
	{
		return $this->idMappings[$kind] ?? [];
	}

	/**
	 * @return Domain
	 */
	public function getSource(): Domain
	{
		return $this->source;
	}

	/**
	 * @return Domain
	 */
	public function getTarget(): Domain
	{
		return $this->target;
	}
}
