<?php

namespace App\Event;

use App\Model\Model;
use LogicException;
use Symfony\Contracts\EventDispatcher\Event;

final class DomainDeletePrepareEvent extends Event
{
	const NAME = 'ccm19.domain.delete.prepare';

	/** @var class-string<Model>[] $domainSpecificModels */
	private $domainSpecificModels;

	public function __construct()
	{
		$this->domainSpecificModels = [];
	}

	/**
	 * This method collects models whose data should be deleted when a domain is deleted.
	 * @param class-string<Model> $className
	 * @return $this
	 * @throws LogicException if the class `$className` doesn't exist or isn't a domain specific model.
	 */
	public function addDomainSpecificModel(string $className): self
	{
		if (!is_a($className, 'App\Model\Model', true)) {
			throw new LogicException("Class $className either doesn't exist or isn't of type App\\Model\\Model.");
		}
		elseif (!$className::isDomainSpecificModel()) {
			throw new LogicException("Class $className isn't a domain specific model.");
		}

		$this->domainSpecificModels[$className] = $className;
		return $this;
	}

	/**
	 * @return class-string<Model>[]
	 */
	public function getDomainSpecificModels(): array
	{
		return $this->domainSpecificModels;
	}
}
