<?php
namespace App\Event;

use App\Utils;
use Symfony\Contracts\EventDispatcher\Event;
use Twig\Loader\LoaderInterface;

final class TemplateResolveEvent extends Event
{
	const PREFIX = 'ccm19.template.resolve.';

	/** @var string $view */
	private $view;

	public static function nameForView(string $view): string
	{
		return self::PREFIX.$view;
	}

	public function __construct(string $view)
	{
		$this->view = $view;
	}

	/**
	 * Name des Views/des Templates
	 */
	public function getView(): string
	{
		return $this->view;
	}

	/**
	 * Name des zu rendernden Views/des Templates ändern
	 *
	 * Achtung: Das überschreibt u.u. Änderungen von anderen Plugins.
	 * Im Zweifel lieber extendTemplate() verwenden!
	 *
	 * @return $this
	 */
	public function setView(string $view): self
	{
		$this->view = $view;
		return $this;
	}

	private function quote(string $string): string
	{
		return strtr($string, ["'"=>"\\'", '"'=>'\"', "\\"=>"\\\\"]);
	}

	/**
	 * Erweitert das View über Blöcke aus dem
	 * angegeben View.
	 *
	 * @return $this
	 */
	public function extendTemplate(string $view): self
	{
		global $kernel;
		$dir = Utils::getCacheDir() . '/templates/';
		$filename = hash('sha256', ($this->getView() . '//' . $view)) . '.twig';
		$newView = '@temp/' . $filename;
		$path = $dir . $filename;

		/** @var LoaderInterface $loader */
		$loader = $kernel->getContainer()->get('twig')->getLoader();
		$source = $loader->getSourceContext($view);
		$sourcePath = $source->getPath();

		if (!file_exists($path) or ($sourcePath and @filemtime($sourcePath) > @filemtime($path))) {
			$source = $loader->getSourceContext($view);

			file_put_contents($path, '{% extends "@@orig/'.$this->quote($this->getView()).'" %}'.$source->getCode());
		}
		$this->setView($newView);
		return $this;
	}
}
