<?php
namespace App\Event;

use App\Model\Theme;
use Symfony\Contracts\EventDispatcher\Event;

final class ThemeDisplayEvent extends Event
{
	const NAME = 'ccm19.theme.display';

	/** @var bool $preview */
	private $preview;
	/** @var string|null $explicitId */
	private $explicitId;
	/** @var Theme $theme */
	private $theme;

	public function __construct(Theme $theme, ?string $explicitId, bool $preview)
	{
		$this->theme = $theme;
		$this->explicitId = $explicitId;
		$this->preview = $preview;
	}

	/**
	 * True, wenn eine Theme-Vorschau läuft
	 */
	public function isPreview(): bool
	{
		return $this->preview;
	}

	/**
	 * Theme-ID wenn das Theme explizit durch ?theme=… gesetzt wurde
	 */
	public function getExplicitThemeId(): ?string
	{
		return $this->explicitId;
	}

	/**
	 * Das aktuell zu verwendende Theme
	 */
	public function getTheme(): Theme
	{
		return $this->theme;
	}

	/**
	 * Setzen des aktuellen Themes
	 */
	public function setTheme(Theme $theme): self
	{
		$this->theme = $theme;
		return $this;
	}
}
