<?php

namespace App\Event;

use App\Model\Model;
use LogicException;
use Symfony\Contracts\EventDispatcher\Event;

final class UserDeletePrepareEvent extends Event
{
	const NAME = 'ccm19.user.delete.prepare';

	/** @var class-string<Model>[] $userSpecificModels */
	private $userSpecificModels;

	public function __construct()
	{
		$this->userSpecificModels = [];
	}

	/**
	 * This method collects models whose data should be deleted when a user is deleted.
	 * @param class-string<Model> $className
	 * @return $this
	 * @throws LogicException if the class `$className` doesn't exist or isn't a user specific model.
	 */
	public function addUserSpecificModel(string $className): self
	{
		if (!is_a($className, 'App\Model\Model', true)) {
			throw new LogicException("Class $className either doesn't exist or isn't of type App\\Model\\Model.");
		}
		elseif (!$className::isUserSpecificModel()) {
			throw new LogicException("Class $className isn't a user specific model.");
		}

		$this->userSpecificModels[$className] = $className;
		return $this;
	}

	/**
	 * @return class-string<Model>[]
	 */
	public function getUserSpecificModels(): array
	{
		return $this->userSpecificModels;
	}
}
