<?php

namespace App\EventSubscriber;

use Symfony\Component\Console\ConsoleEvents;
use Symfony\Component\Console\Event\ConsoleCommandEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\HttpFoundation\Session\Storage\MockArraySessionStorage;

class CommandSessionHandler implements EventSubscriberInterface
{
	/** @var RequestStack $stack */
	private $stack;

	/**
	 * @required
	 */
	public function setRequestStack(RequestStack $stack): void
	{
		$this->stack = $stack;
	}

	/**
	 * @override
	 * @return array<string,mixed>
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			ConsoleEvents::COMMAND => ['onCommand', 9000],
		];
	}

	/**
	 * Fake-Request anlegen und Mock-Session starten, wenn kein Request auf dem Stack liegt
	 *
	 * @param ConsoleCommandEvent $event
	 */
	public function onCommand(ConsoleCommandEvent $event): void
	{
		$mainRequest = $this->stack->getMainRequest();
		if (!$mainRequest) {
			$path = urlencode($_SERVER['argv'][1] ?? '');
			$request = Request::create('https://localhost/CONSOLE-CMD/'.$path, 'POST', ['args' => array_slice($_SERVER['argv'] ?? [], 2)]);
			$session = new Session(new MockArraySessionStorage());
			$request->setSession($session);
			$session->start();
			$this->stack->push($request);
		}
	}
}
