<?php

namespace App\EventSubscriber;

use App\Model\BackendLanguage as BackendLanguage;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class LanguageHandler extends AbstractController implements EventSubscriberInterface
{
	/**
	 * @override
	 * @return array<string,mixed>
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			KernelEvents::REQUEST => ['onKernelRequest', 17],
		];
	}

	/**
	 * Am Anfang einer Anfrage wird das Locale aus das aus der Session gesetzt
	 * und das Locale in Templates verfügbar gemacht.
	 *
	 * Wichtig: Die Priority muss > 16 sein, damit das Locale vor dem Laden der
	 * Übersetzungen gesetzt wird.
	 *
	 * @param RequestEvent $event
	 */
	public function onKernelRequest(RequestEvent $event): void
	{
		global $kernel;

		if ($event->isMainRequest() == false) {
			return;
		}

		$request = $event->getRequest();
		$session = ($request->attributes->getBoolean('_stateless')) ? null : $request->getSession();

		if (!$session) {
			$queryLang = (string)$request->query->get('lang');
			if ($queryLang) {
				$request->setLocale($queryLang);
			}
			return;
		}

		// Locale aus Session holen
		try {
			$locale = $session->get('locale');
		} catch (\Exception $ignore) {
			$locale = null;
		}
		if ($locale) {
			$request->setLocale($locale);
		}
		else {
			// Solange nicht angemeldet: Spache über GET-Parameter setzen
			$queryLang = (string)$request->query->get('lang');
			$language = ($queryLang) ? BackendLanguage::byFuzzyLangCode($queryLang) : null;
			try {
				$userId = $session->get('userId', '');
			} catch (\Exception $ignore) {
				$userId = '';
			}
			if ($language and $userId == '') {
				$request->setLocale($queryLang);
			}
			else {
				// Systemweite Standardsprache holen
				$defaultLanguage = BackendLanguage::findDefault();
				if ($defaultLanguage) {
					$request->setLocale($defaultLanguage->getLongCode());
				}
			}
		}

		// Locale in Twig-Variablen schreiben
		$twig = $kernel->getContainer()->get('twig');
		$twig->addGlobal('_locale', $request->getLocale());
	}

}
