<?php

namespace App\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Symfony\Component\HttpKernel\Event\ControllerEvent;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class PluginHooksHandler implements EventSubscriberInterface
{
	/** @var EventDispatcherInterface $dispatcher */
	private $dispatcher;
	/** @var array<string,string> $doPostDispatchFor */
	private $doPostDispatchFor = [];

	public function __construct(EventDispatcherInterface $dispatcher)
	{
		$this->dispatcher = $dispatcher;
	}

	/**
	 * @override
	 * @return array<string,mixed>
	 */
	public static function getSubscribedEvents()
	{
		return [
			KernelEvents::CONTROLLER => ['onKernelController', 1],
			KernelEvents::RESPONSE => ['onKernelResponse', 1],
		];
	}

	/**
	 * Generiert ein Event 'ccm19.controller.request.{route_name}'
	 *
	 * Das Event wird nur erzeugt, wenn der Controller nicht durch einen
	 * vorherigen EventHandler geändert wurde (AuthenticationHandler,
	 * CorsHandler). Authentifizierung ist also schon passiert.
	 *
	 * EventHandler können dieses Event nutzen, um Code for dem
	 * eigentlichen Controller zu injecten, oder um den Controller
	 * mit `$event->setController()` zu ändern.
	 *
	 * @return void
	 * @phan-suppress PhanParamTooMany
	 */
	public function onKernelController(ControllerEvent $event): void
	{
		$attributes = $event->getRequest()->attributes;
		$routeName = $attributes->get('_route');
		/** @var class-string $originalController */
		$originalController = explode('::', $attributes->get('_controller'), 2)[0];

		$controller = (array)$event->getController();
		$controller = reset($controller);

		if ($routeName and class_exists($originalController) and $controller instanceof $originalController) {
			$this->doPostDispatchFor[$routeName] = $routeName;
			$this->dispatcher->dispatch($event, 'ccm19.controller.request.'.$routeName);
		}
	}

	/**
	 * Generiert ein Event 'ccm19.controller.response.{route_name}'
	 *
	 * Das Event wird nur erzeugt, wenn vorher ein entsprechendes
	 * 'request'-Event erzeugt wurde.
	 *
	 * @return void
	 * @phan-suppress PhanParamTooMany
	 */
	public function onKernelResponse(ResponseEvent $event): void
	{
		$attributes = $event->getRequest()->attributes;
		$routeName = $attributes->get('_route');

		if (isset($this->doPostDispatchFor[$routeName])) {
			$this->dispatcher->dispatch($event, 'ccm19.controller.response.'.$routeName);
		}
	}
}
