<?php

namespace App\EventSubscriber;

use App\Utils;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\HttpFoundation\Session\Storage\MockArraySessionStorage;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\EventListener\RouterListener;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\Routing\RequestContext;

class SubRequestHandler implements EventSubscriberInterface
{
	/**
	 * @override
	 * @return array<string,mixed>
	 */
	public static function getSubscribedEvents(): array
	{
		return [
			// Run afterRouterListener::onKernelRequest
			KernelEvents::REQUEST => ['onKernelRequest', 30],
		];
	}

	/**
	 * Sub-Requests erhalten eine temporäre Session, sofern noch keine vorhanden ist.
	 * Außerdem wird der RequestContext an den des Main-Request angepasst.
	 *
	 * Die Methode muss nach `RouterListener::onKernelRequest` ausgeführt werden, da
	 * der RequestContext möglicherweise mit leeren Request-Daten aufgefrischt wird.
	 *
	 * @param RequestEvent $event
	 * @see RouterListener::onKernelRequest
	 */
	public function onKernelRequest(RequestEvent $event): void
	{
		if ($event->isMainRequest()) {
			return;
		}

		/** @var Request $request */
		$request = $event->getRequest();

		// Assign a non-persistent session to the request
		if (!$request->hasSession(true)) {
			$session = new Session(new MockArraySessionStorage());
			$request->setSession($session);
			$session->start();
		}

		/** @var RequestContext $requestContext */
		$requestContext = Utils::getRouter()->getContext();

		// Update the request context if it seems empty
		if ($requestContext->getHost() == '') {
			$requestContext->fromRequest(Utils::getMainRequest() ?? Request::createFromGlobals());
		}
	}
}
