<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;
use App\Utils;

class BaseLayout extends ImmutableModel
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-base-layouts.json';

	/**
	 * @return static
	 */
	public static function default(): self
	{
		$all = array_values(self::all());
		assert(count($all) > 0);
		return array_reduce($all, function (?self $default, self $baseLayout) {
			return $default ?? ($baseLayout->isDefault() ? $baseLayout : $default);
		}) ?? $all[0];
	}

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @return bool
	 */
	public function isDefault(): bool
	{
		return (bool)$this->get('default');
	}

	/**
	 * @return bool
	 */
	public function isEmpty(): bool
	{
		return (bool)$this->get('empty');
	}

	/**
	 * Returns the relative path to the template file if one exists.
	 * @return string|null
	 */
	public function getTemplatePathname(): ?string
	{
		// Allow alphanumeric characters and underscores in between
		// Example: "_foo#@!bar-1()" => "foo_bar_1"
		$slug = trim(preg_replace('~_{2,}~', '_', preg_replace('~[^a-z\d]~', '_', strtolower($this->getId()))), '_');
		$pathname = "external/_base_layouts/$slug.css.twig";
		return is_file(Utils::getBaseDir() . '/templates/' . $pathname) ? $pathname : null;
	}
}
