<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;
use App\Exception\UniqueConstraintException;
use LogicException;

class ClientGroup extends Model
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-client-groups';

	/** @var bool $useRepositorySkeleton */
	protected static $useRepositorySkeleton = false;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
	}

	/**
	 * This method creates a new client group. The changes aren't written, this can be done later
	 * with `ClientGroup::flush`.
	 * @param string|null $name
	 * @return self
	 * @throws LogicException if the name is empty.
	 * @throws UniqueConstraintException if the name is already taken.
	 * @override
	 * @phan-return static
	 * @phan-assert string $name
	 */
	public static function create(?string $name = null): Model
	{
		if ($name === null || $name === '') {
			throw new LogicException('Name may not be empty.');
		}

		self::beginWrite();
		if (self::byName($name)) {
			throw new UniqueConstraintException("Object with name $name already exists.");
		}

		$object = parent::create();
		assert($object instanceof self);

		$object
			->setName($name)
		;

		return $object;
	}

	/**
	 * @param string $name
	 * @return self|null
	 */
	public static function byName(string $name): ?self
	{
		$name = mb_strtolower($name);
		return array_reduce(self::all(), function (?self $carry, self $item) use ($name) {
			return $carry ?? (mb_strtolower($item->getName()) == $name ? $item : $carry);
		}, null);
	}

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @param string $name
	 * @return $this
	 */
	public function setName(string $name): self
	{
		$this->set('name', $name);
		return $this;
	}
}
