<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;

class EmbeddingAsset extends EmbeddingAssetBase
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var bool $domainSpecificRepository */
	protected static $domainSpecificRepository = true;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
	}

	/**
	 * @param LocaleBase $locale
	 * @return string
	 */
	public function getTranslatedDescription(LocaleBase $locale, bool $useFallback=true): string
	{
		return $this->get('translated_description.'.$locale->getName())
			?? $this->get('translated_description.'.$locale->getId())
			?? ($useFallback ? $this->getDescription() : '')
		;
	}

	/**
	 * @param LocaleBase $locale
	 * @param string $description
	 * @return $this
	 */
	public function setTranslatedDescription(LocaleBase $locale, string $description): self
	{
		$key = 'translated_description.'.$locale->getName();
		if ($description !== $this->get($key) and $description !== $this->getDescription()) {
			$this->makingLegallyRelevantChange();
		}
		$this->set($key, $description);
		return $this;
	}

	/**
	 * @param LocaleBase $locale
	 * @return string
	 */
	public function getTranslatedLifetime(LocaleBase $locale, bool $useFallback=true): string
	{
		return $this->get('translated_lifetime.'.$locale->getName())
			?? $this->get('translated_lifetime.'.$locale->getId())
			?? ($useFallback ? $this->getLifetime() : '')
		;
	}

	/**
	 * @param LocaleBase $locale
	 * @param string $lifetime
	 * @return $this
	 */
	public function setTranslatedLifetime(LocaleBase $locale, string $lifetime): self
	{
		$key = 'translated_lifetime.'.$locale->getName();
		if ($lifetime !== $this->get($key) and $lifetime !== $this->getLifetime()) {
			$this->makingLegallyRelevantChange();
		}
		$this->set($key, $lifetime);
		return $this;
	}

	/**
	 * Setzt alle übersetzten Texte, die in $locale noch nie vorher gesetzt
	 * wurden, auf den nicht-übersetzten Wert.
	 *
	 * @param LocaleBase $locale Zu füllendes Locale
	 * @return $this
	 */
	public function fillTranslationsFromDefault(LocaleBase $locale): self
	{
		$this->setTranslatedDescription($locale, $this->getTranslatedDescription($locale));
		$this->setTranslatedLifetime($locale, $this->getTranslatedLifetime($locale));
		return $this;
	}
}
