<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;

class MenuData extends ImmutableModel
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-menu-data.json';

	public function isHidden(): bool
	{
		return (bool)$this->get('hidden', false);
	}

	public function getRoutePrefix(): string
	{
		return (string)$this->get('menuApp', '__unknown');
	}

	public function getName(): string
	{
		return (string)$this->get('menuName', '???');
	}

	/**
	 * Dieses Objekt repräsentiert keinen eigenständigen Menüpunkt, sondern eine
	 * einzelne Funktion.
	 *
	 * Dadurch ist die Rechteverwaltung flexibel erweiterbar, um den Zugriff auf
	 * bestimmte Funktionen eines Bereichs pro Benutzergruppe zu limitieren.
	 *
	 * @return bool
	 */
	public function isMeta(): bool
	{
		return strpos($this->getRoutePrefix(), 'meta__') === 0;
	}

	public function isNotReadOnly(): bool
	{
		return (bool)$this->get('notReadOnly', false);
	}
}
