<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;
use App\Model\LocaleBase as Locale;

class Purpose extends ImmutableModel
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var string $repositoryFilename */
	protected static $repositoryFilename = 'cm-purposes.json';

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
	}

	/**
	 * @param string $name
	 * @return static|null
	 */
	public static function findByName(string $name): ?self
	{
		$filtered = array_values(array_filter(static::all(), function ($item) use ($name) {
			return $name == $item->getName();
		}));

		if ($filtered) {
			return $filtered[0];
		}
		else {
			return null;
		}
	}

	/**
	 * @return string
	 */
	public function getName(): string
	{
		return (string)$this->get('name');
	}

	/**
	 * @param Locale $locale
	 * @param bool $useFallback
	 * @return string
	 */
	public function getTranslatedName(Locale $locale, $useFallback=true): string
	{
		$result = $this->get('translated_name.'.$locale->getName());
		if ($result === null) {
			$result = $this->get('translated_name.'.$locale->getId());
		}
		if ($useFallback and $result === null) {
			return $this->get('name');
		}
		return (string)$result;
	}

	/**
	 * @return string
	 */
	public function getDescription(): string
	{
		return (string)$this->get('description');
	}

	/**
	 * @param Locale $locale
	 * @param bool $useFallback
	 * @return string
	 */
	public function getTranslatedDescription(Locale $locale, $useFallback=true): string
	{
		$result = $this->get('translated_description.'.$locale->getName());
		if ($result === null) {
			$result = $this->get('translated_description.'.$locale->getId());
		}
		if ($useFallback and $result === null) {
			return $this->get('description');
		}
		return (string)$result;
	}

	/**
	 * @return bool
	 */
	public function isMandatory(): bool
	{
		return (bool)$this->get('mandatory');
	}


	/**
	 * Setzt alle übersetzten Texte, die in $locale noch nie vorher gesetzt
	 * wurden, auf den nicht-übersetzten Wert.
	 *
	 * @param Locale $locale Zu füllendes Locale
	 * @return $this
	 */
	public function fillTranslationsFromDefault(Locale $locale): self
	{
		// FIXME: Methoden wurden mit dc88693 entfernt; existiert eine Alternative?
		/*
		$this->setTranslatedName($locale, $this->getTranslatedName($locale));
		$this->setTranslatedDescription($locale, $this->getTranslatedDescription($locale));
		*/
		return $this;
	}
}
