<?php

namespace App\Model\Regulations;

use App\Component\Config\ModelInterface;
use App\Component\EmbeddingDb\Repository;
use App\Exception\ConfigNotReadableException;

class Group extends Model implements ModelInterface
{
	protected static $repositoryFilename = 'regulationgroups';

	/** @var Repository|null $repository */
	protected static $repository = null;

	public function getName(string $langCode = 'en'): string
	{
		return (string)$this->getTranslated('name', $langCode, '');
	}

	public function getDescription(string $langCode = 'en'): string
	{
		return (string)$this->getTranslated('desc', $langCode, '');
	}

	/**
	 * @param int|string $id
	 * @return bool
	 */
	public static function exists($id): bool
	{
		return $id ? is_array(self::getRepository()->get($id)) : false;
	}

	/**
	 * @param int|string $id
	 * @return static|null
	 * @phan-suppress PhanTypeInstantiateAbstractStatic
	 */
	public static function find($id): ?self
	{
		return self::exists($id) ? new static($id, self::getRepository()->get($id)) : null;
	}

	/**
	 * @return static[]
	 * @phan-suppress PhanTypeInstantiateAbstractStatic
	 */
	public static function all(): array
	{
		/** @var static[] $entities */
		$entities = [];

		foreach (self::getRepository()->toArray() as $id => $data) {
			$entities[$id] = new static($id, $data);
		}

		return $entities;
	}

	/**
	 * @return array
	 */
	public static function rawAll(): array
	{
		return self::getRepository()->toArray();
	}

	/**
	 * Zählt die vorhandenen Instanzen dieses Models.
	 * @return int
	 */
	public static function count(): int
	{
		return count(self::rawAll());
	}

	/**
	 * Liest alle Einträge neu von der Festplatte oder ggf. dem Internet ein.
	 * @throws ConfigNotReadableException
	 */
	public static function reload(): void
	{
		self::getRepository()->reload();
	}

	public function __toString(): string
	{
		return sprintf('[Regulation group %s: "%s"]', $this->getId(), $this->getName());
	}
}
