<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;
use DateTimeImmutable;
use Exception;

class StatisticsData extends Model
{
    /** @var RepositoryInterface|null $repository */
    protected static $repository = null;

    /** @var string $repositoryFilename */
    protected static $repositoryFilename = 'cm-statistics-data';

    /** @var bool $repositoryWithCustomIds */
    protected static $repositoryWithCustomIds = true;

    /** @var bool $domainSpecificRepository */
    protected static $domainSpecificRepository = true;

    /**
     * @param string $id
     */
    protected function __construct(string $id)
    {
        parent::__construct($id);
    }

    /**
     * @param $name
     * @param $arguments
     * @return string
     */
    public function __call($name, $arguments)
    {
        return (string)$this->get($name, "");
    }

    /**
     * @param bool $includeArchiv
     * @return bool
     * @throws Exception
     */
    public static function createFullStats(bool $includeArchiv = false): bool
    {
        //Weils was länger dauern könnte
        set_time_limit(300);

        $domain = Domain::activeDomain();
        if (!$domain) {
            return false;
        }
        //Sammel Array
        $sumsPerDay = [];
        //Verwendete Systeme nach denen gruppiert wird
        $sytems = array("Linux ", "Windows", "Mac", "iphone", "android");

        //Ab welchem Zeitpunkt Daten holen
        $lastTimeStamp = 1625130518; //01.07.2021

        //Die Button Namen die verarbeitet werden sollen
        $buttonName = array("acceptAll", "decline");

        //Crawler Bots - in der Statistik auschließen - evtl. mal dynamisch machen...
        $crawlerBots = array("clickskeks", "Deepscan");

        //Archiv wieder einlesen
        if ($includeArchiv) {
            //Das Archiv Protokoll
            $protocol = ProtocolArchive::byDomain($domain);

            //Es gibt Daten - dann verarbeiten
            if (!empty($protocol)) {
                //Daten auslesen und summieren
                $sumsPerDayArchiv = self::getDataFromDeepStatData($protocol, $lastTimeStamp, $sytems, $crawlerBots, $buttonName);
            }
        }
        /**
         * V sind alle items in der Liste - aber pro Besucher können mehrere items einem Besucher gehöhren
         * 1. das erste öffnen = 1 Eintrag
         * 2. der Consent = 2. Eintrag.
         * d.h. um die Anteile auszurechnen braucht man die Anzahl der ersten Öffnungen - also clickedbutton < 3 oder actualOpened = 1
         */

        //aktuelles Protokoll, auch hier die Daten holen...
        $protocol = Protocol::byDomain($domain);

        //Die gesamten Statistik Daten nochmal laden.
        $allStat = self::all();

        //Den letzten Zeitstempel - sonst ist der ja 0
        if (!empty($allStat)) {
            $lastTimeStamp = (strtotime(end($allStat)->getId()));
        }

        //Daten holen und summieren.
        if (!empty($protocol)) {
            $sumsPerDay = self::getDataFromDeepStatData($protocol, $lastTimeStamp, $sytems, $crawlerBots, $buttonName);
        }

        //mergen wenn Daten vorhanden sind, ansonsten so lassen.
        if (!empty($sumsPerDayArchiv)) {
            $sumsPerDay = array_merge($sumsPerDayArchiv, $sumsPerDay);
        }


        //Jetzt noch die views übertragen...
        foreach (Statistic::all() as $key => $statEntry) {
            //print_r($statEntry);
            if (empty($sumsPerDay[$key]["v"])) {
                //Nur view übertragen, wenn es geht - ein Eintrag vorhanden ist
                if (!empty($statEntry->get("v")) && !empty($sumsPerDay[$key])) {
                    $sumsPerDay[$key]["v"] = $statEntry->getViewCount();
                    $sumsPerDay[$key]["c"] = $statEntry->get("c");
                }
                //Kein EIntrag aus Logfile - dann zusammenstellen... aus original Stats
                if (!empty($statEntry->get("v")) && empty($sumsPerDay[$key])) {
                    $sumsPerDay[$key]["v"] = $statEntry->getViewCount();
                    $sumsPerDay[$key]["c"] = $statEntry->get("c");
                    $sumsPerDay[$key]["purposes"] = $statEntry->get("p");
                    $sumsPerDay[$key]["embeddings"] = $statEntry->get("e");
                }
            }
            $sumsPerDay[$key]["c"] = $statEntry->get("c");
        }
        //print_r($sumsPerDay);
        //Nur wenn auch Einträge da sind...
        if (!empty($sumsPerDay)) {
            // Statistiken speichern
            self::beginWrite();
            foreach ($sumsPerDay as $date => $row) {
                $stat = self::findOrCreate($date);

                if ($includeArchiv) {
                    $stat->clearArchivedCounts();
                }

                if (!empty($row['v'])) {
                    $stat->set("v", $row['v']);
                }

                if (!empty($row['c'])) {
                    $stat->set("c", $row['c']);
                }

                if (!empty($row['purposes'])) {
                    foreach ($row['purposes'] as $purposeId => $count) {
                        $stat->setPurposeAcceptCount($purposeId, $count);
                    }
                }

                if (!empty($row['details'])) {
                    //print_r($row['details']);
                    $stat->set("details", $row['details']);
                }

                if (!empty($row['embeddings'])) {
                    foreach ($row['embeddings'] as $embeddingId => $count) {
                        $stat->setEmbeddingAcceptCount($embeddingId, $count);
                    }
                }

                if (!empty($row['cookies'])) {
                    foreach ($row['cookies'] as $cookieId => $count) {
                        if ($count > 1) {
                            $stat->setCookiecceptCount($cookieId, $count);
                        }
                    }
                }

            }
            self::flush();
            //yeah - done
            return true;
        }
        //nope - nix...
        return false;
    }

    /**
     * @param $protocol
     * @param int $startTime Unix-Timestamp
     * @param array $sytems
     * @param array $crawlerBots
     * @param array $buttonName
     * @param bool $archive
     * @return array
     * @throws Exception
     */
    private static function getDataFromDeepStatData($protocol, int $startTime, array $sytems, array $crawlerBots, array $buttonName, bool $archive = false)
    {
        //End Datum immer jetzt
        $stopTime = time();
        $sumsPerDay = [];

        for ($i = $startTime; $i < $stopTime; $i += 259200) {

            //Start und Enddatum im Filter mit der Schleife mit erhöhen, ansonsten sind es zu viele Daten im RAM
            $dateTimeIm = new DateTimeImmutable((string)date("Y-m-d",$i));
            $endTime = new DateTimeImmutable((string)date("Y-m-d",($i+259200)));

            //Uniq Array - immer neu setzen pro Schleife
            $uniq = array();

            //Einträge durchlaufen mit 3 Tages Filter (evtl. mal niedriger machen?)
            foreach ($protocol->filter($dateTimeIm, $endTime) as $item) {

                $item['date'] = date("Y-m-d", $item['timestamp'] ?? 0);
                $continue = false;
                foreach ($crawlerBots as $bot) {
                    //$crawlerBots
                    if (stripos($item['clientUserAgent'], $bot) !== false) {
                        $continue = true;
                    }
                }
                if ($continue) {
                    continue;
                }

                if ($item['clickedButton'] > 3 && !in_array($item['clickedButton'], $buttonName)) {
                    continue;
                }
                $uniq[$item['ucid']][] = $item;
            }


            $currentDate = null;
            foreach ($uniq as $ucItems) {
                //Lesbares Datum
                $date = date("Y-m-d", $ucItems[0]['timestamp'] ?? 0);
                //print_r($ucItems);

                //Array initialisieren
                if ($currentDate !== $date) {
                    $sumsPerDay[$date] = [
                        'purposes' => [],
                        'embeddings' => [],
                        'details' => [
                            "opened" => [],
                            "acceptAll" => [],
                            "decline" => []
                        ]
                    ];
                    $currentDate = $date;
                }

                //Opened Daten finden - alle Öffnungen durch User / Browser
                if (empty($sumsPerDay[$date]['details']['opened']['all'])) {
                    $sumsPerDay[$date]['details']['opened']['all'] = 0;
                }
                $sumsPerDay[$date]['details']['opened']['all']++;

                //Betriebssysteme
                foreach ($sytems as $system) {
                    if (!isset($sumsPerDay[$date]['details']['opened'][$system])) {
                        $sumsPerDay[$date]['details']['opened'][$system] = 0;
                    }
                    if (stripos($ucItems[0]['clientUserAgent'], $system) !== false) {
                        $sumsPerDay[$date]['details']['opened'][$system]++;
                    }
                }

                //mobile / desktop
                if (stripos($ucItems[0]['clientUserAgent'], "iphone") !== false || stripos($ucItems[0]['clientUserAgent'], "android") !== false) {
                    if (!isset($sumsPerDay[$date]['details']['opened']["mobile"])) {
                        $sumsPerDay[$date]['details']['opened']["mobile"] = 0;
                    }
                    $sumsPerDay[$date]['details']['opened']["mobile"]++;
                } else {
                    if (!isset($sumsPerDay[$date]['details']['opened']["desktop"])) {
                        $sumsPerDay[$date]['details']['opened']["desktop"] = 0;
                    }
                    $sumsPerDay[$date]['details']['opened']["desktop"]++;
                }

                //Accept / Decline bestimmen.
                foreach ($ucItems as $item) {
                    if (isset($item['clickedButton'])) {
                        if (!Protocol::isClickedButtonValid((string)$item['clickedButton'])) {
                            continue;
                        }

                        //Set
                        if (!isset($sumsPerDay[$date]['details'][$item['clickedButton']]['all'])) {
                            $sumsPerDay[$date]['details'][$item['clickedButton']]['all'] = 0;
                        }

                        //Count
                        $sumsPerDay[$date]['details'][$item['clickedButton']]['all']++;

                        //Systemdaten
                        if (isset($item['clientUserAgent'])) {
                            if (strlen($item['clientUserAgent']) < 3) {
                                $item['clientUserAgent'] = "none";
                            }

                            //Betriebssysteme
                            foreach ($sytems as $system) {
                                if (!isset($sumsPerDay[$date]['details'][$item['clickedButton']][$system])) {
                                    $sumsPerDay[$date]['details'][$item['clickedButton']][$system] = 0;
                                }
                                if (stripos($item['clientUserAgent'], $system) !== false) {
                                    $sumsPerDay[$date]['details'][$item['clickedButton']][$system]++;
                                }
                            }
                            if (stripos($item['clientUserAgent'], "iphone") !== false || stripos($item['clientUserAgent'], "android") !== false) {
                                if (!isset($sumsPerDay[$date]['details'][$item['clickedButton']]["mobile"])) {
                                    $sumsPerDay[$date]['details'][$item['clickedButton']]["mobile"] = 0;
                                }
                                $sumsPerDay[$date]['details'][$item['clickedButton']]["mobile"]++;
                            } else {
                                if (!isset($sumsPerDay[$date]['details'][$item['clickedButton']]["desktop"])) {
                                    $sumsPerDay[$date]['details'][$item['clickedButton']]["desktop"] = 0;
                                }
                                $sumsPerDay[$date]['details'][$item['clickedButton']]["desktop"]++;
                            }
                        }
                    }
                    unset($item);
                }
                //Hier ab Datum 17022021
                if (strtotime($date) > 1613430098) {
                    if (empty($sumsPerDay[$date]['v'])) {
                        $sumsPerDay[$date]['v'] = 0;
                    }
                    $sumsPerDay[$date]['v']++;
                }
            }
            unset($item);
        }

        return $sumsPerDay;
    }

    /**
     * @param string $cookieId
     * @param int $count
     * @return $this
     */
    public function setCookiecceptCount(string $cookieId, int $count): self
    {
        $this->set('c.' . $cookieId, $count);
        return $this;
    }

    /**
     * @param string $embeddingId
     * @param int $count
     * @return $this
     */
    public function setEmbeddingAcceptCount(string $embeddingId, int $count): self
    {
        $this->set('e.' . $embeddingId, $count);
        return $this;
    }

    /**
     * @param string $purposeId
     * @param int $count
     * @return $this
     */
    public function setPurposeAcceptCount(string $purposeId, int $count): self
    {
        $this->set('p.' . $purposeId, $count);
        return $this;
    }

    /**
     * @param string $date Y-m-d
     * @return static
     */
    public static function findOrCreate(string $date): self
    {
        return self::find($date) ?? self::create($date);
    }

    /**
     * Diese Methode erstellt ein Objekt dieser Klasse
     * @param string|null $date Y-m-d (default: heute)
     * @return static
     * @throws \LogicException
     */
    public static function create(?string $date = null): Model
    {
        if ($date === null) {
            $date = (new DateTimeImmutable('now'))->format('Y-m-d');
        }
        self::beginWrite();
        return new static($date);
    }

    /**
     * @param string $date Y-m-d
     * @return static|null
     */
    public static function find(string $date): ?Model
    {
        return self::exists($date) ? new static($date) : null;
    }

    /**
     * @return static|null
     */
    public static function findToday(): ?Model
    {
        return self::find((new DateTimeImmutable('now'))->format('Y-m-d'));
    }

    /**
     * @return array
     */
    public function getDetails(bool $includeArchived = true): array
    {
        $result = $this->get('details', []);
        if ($includeArchived) {
            $archived = $this->get('adetails', []);
            $this->updateCountArray($archived, $result);
            return $archived;
        } else {
            return $result;
        }
    }

    /**
     * @return int
     */
    public function getPurposeAcceptCount(string $purposeId, bool $includeArchived = true): int
    {
        $result = $this->get('p.' . $purposeId, 0);
        if ($includeArchived) {
            $result += $this->get('ap.' . $purposeId, 0);
        }
        return $result;
    }

    /**
     * @return int
     */
    public function getViewCount(bool $includeArchived = true): int
    {
        $count = $this->get('v', 0);
        return (int)$count;
    }

    /**
     * @return int
     */
    public function getCallCount(): int
    {
        $calls = $this->get('c');
        if (is_array($calls)) {
            return 1;
        } else {
            if ($calls >= 1) {
                return $calls;
            }
            return 1;
        }
    }

    private function updateCountArray(array &$base, array $new)
    {
        foreach ($new as $key => $value) {
            if (!isset($base[$key])) {
                $base[$key] = $value;
            } elseif (is_array($value)) {
                $this->updateCountArray($base[$key], $value);
            } elseif (is_int($value) or is_float($value)) {
                $base[$key] += $value;
            } else {
                $base[$key] = $value;
            }
        }
    }

    /**
     * Archivieren von Zählern bei Protokoll-Rollover
     *
     * @return $this
     */
    public function archiveCounts(): self
    {
        self::beginWrite();
        // Purpose/Embedding
        $embeddings = $this->get('e', []);
        foreach ($embeddings as $embeddingId => $count) {
            $this->increment('ae.' . $embeddingId, $count);
            $this->set('e.' . $embeddingId, 0);
        }
        $purposes = $this->get('p', []);
        foreach ($purposes as $purposeId => $count) {
            $this->increment('ap.' . $purposeId, $count);
            $this->set('p.' . $purposeId, 0);
        }

        //// View-Counts
        //$this->set('av', $this->get('av', 0)+$this->get('v', 0));
        //$this->set('v', 0);

        // Detail-Counts
        $details = $this->get('details', []);
        $archivedDetails = $this->get('adetails', []);
        $this->updateCountArray($archivedDetails, $details);
        if ($archivedDetails) {
            $this->set('adetails', $archivedDetails);
            $this->set('details', []);
        }

        return $this;
    }

    /**
     * @return $this
     */
    public function clearArchivedCounts(): self
    {
        self::beginWrite();
        if ($this->get('adetails')) {
            $this->set('adetails', []);
        }
        if ($this->get('ap')) {
            $this->set('ap', []);
        }
        if ($this->get('ae')) {
            $this->set('ae', []);
        }

        return $this;
    }
}
