<?php

namespace App\Model;

use App\Component\Config\RepositoryInterface;
use App\Component\Media\MediaFileService;
use Symfony\Component\HttpFoundation\File\File;

class SystemTheme extends ThemeBase
{
	/** @var RepositoryInterface|null $repository */
	protected static $repository = null;

	/** @var bool $useRepositorySkeleton */
	protected static $useRepositorySkeleton = false;

	/**
	 * @param string $id
	 */
	protected function __construct(string $id)
	{
		parent::__construct($id);
	}

	/**
	 * @return File|null
	 */
	public function getSettingsIcon(): ?File
	{
		$service = MediaFileService::getInstance();
		$repo = $service->getGlobalRepository('settings_icons');
		$identifier = $this->getSettingsIconIdentifier();
		return ($identifier == '') ? null : $repo->getFile($identifier);
	}

	/**
	 * @param File|null $file
	 * @return $this
	 */
	public function setSettingsIcon(?File $file): ThemeBase
	{
		$service = MediaFileService::getInstance();
		$repo = $service->getGlobalRepository('settings_icons');
		$prevIdentifier = $this->getSettingsIconIdentifier();

		$identifier = $repo->replaceFile($prevIdentifier, $file, $this->getId());
		$this->setSettingsIconIdentifier($identifier);

		return $this;
	}

	/**
	 * @return string
	 */
	public function getPublishedSettingsIconIdentifier(): string
	{
		return (string)$this->get('publishedSettingsIconId', '');
	}

	/**
	 * @param string $mediaId
	 * @return $this
	 */
	protected function setPublishedSettingsIconIdentifier(string $mediaId): self
	{
		$this->set('publishedSettingsIconId', $mediaId);
		return $this;
	}

	/**
	 * @return File|null
	 */
	public function getPublishedSettingsIcon(): ?File
	{
		$service = MediaFileService::getInstance();
		$repo = $service->getGlobalRepository('settings_icons');
		$identifier = $this->getPublishedSettingsIconIdentifier();
		return ($identifier == '') ? null : $repo->getFile($identifier);
	}

	/**
	 * @param File|null $file
	 * @return $this
	 */
	public function setPublishedSettingsIcon(?File $file): self
	{
		$service = MediaFileService::getInstance();
		$repo = $service->getGlobalRepository('settings_icons');
		$prevIdentifier = $this->getPublishedSettingsIconIdentifier();

		$identifier = $repo->replaceFile($prevIdentifier, $file, $this->getId().'-published');
		$this->setPublishedSettingsIconIdentifier($identifier);

		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getPublishedSettingsIconBackgroundColor(): ?string
	{
		return $this->get('publishedSettingsIconBackgroundColor');
	}

	/**
	 * @param string $color
	 * @return $this
	 */
	public function setPublishedSettingsIconBackgroundColor(string $color): self
	{
		$this->set('publishedSettingsIconBackgroundColor', $color);
		return $this;
	}

	/**
	 * @return string|null
	 */
	public function getPublishedSettingsIconForegroundColor(): ?string
	{
		return $this->get('publishedSettingsIconForegroundColor');
	}

	/**
	 * @param string $color
	 * @return $this
	 */
	public function setPublishedSettingsIconForegroundColor(string $color): self
	{
		$this->set('publishedSettingsIconForegroundColor', $color);
		return $this;
	}
}
