(function () {
	"use strict";
	/* Polyfill: Array.from() */
	if ('from' in Array == false) {
		/**
		 * @param {*} collection
		 * @return {Array}
		 */
		var toArray = function (collection) {
			return Array.prototype.slice.call(collection);
		};
	}
	else {
		var toArray = Array.from;
	}
	/* Polyfill: […].includes() */
	if ('includes' in Array.prototype == false) {
		Object.defineProperty(Array.prototype, 'includes', {
			configurable: true,
			enumerable: false,
			value: function Array_includes (needle, offset) {
				offset = parseInt(offset);
				if (isNaN(offset)) {
					offset = 0;
				} else {
					offset = Math.min(Math.max(0, (offset >= 0) ? offset : this.length + offset), this.length - 1);
				}
				var haystack = Array.prototype.slice.call(this, offset);
				return haystack.reduce(function (found, currentValue) {
					return found || currentValue === needle;
				}, false);
			}
		});
	}
	/* Polyfill: "…".includes() */
	if ('includes' in String.prototype == false) {
		Object.defineProperty(String.prototype, 'includes', {
			configurable: true,
			enumerable: false,
			value: function String_includes (needle, offset) {
				offset = parseInt(offset);
				if (isNaN(offset)) {
					offset = 0;
				} else {
					offset = Math.min(Math.max(0, (offset >= 0) ? offset : this.length + offset), this.length - 1);
				}
				return this.indexOf(needle, offset) !== -1;
			}
		});
	}

	// Polyfill: addEventListener/removeEventListener für alte IEs
	if (!('addEventListener' in window) && 'attachEvent' in window) {
		EventTarget.prototype.addEventListener = function (event, fn, opt) {
			if (typeof opt == 'object' && opt.once) {
				var obj = this;
				var origFn = fn;
				fn = function () {
					var result;
					try {
						result = origFn.apply(this, arguments);
					} catch (e) {
						obj.detachEvent('on'+event, fn);
						throw e;
					}
					obj.detachEvent('on'+event, fn);
					return result;
				}
			}
			this.attachEvent('on'+event, fn);
		}
		EventTarget.prototype.removeEventListener = function (event, fn, opt) {
			this.detachEvent('on'+event, fn);
		}
	}

	if (typeof window.CustomEvent != 'function') {
		window.CustomEvent = function (event, params) {
			params = params || { bubbles: false, cancelable: false, detail: null };
			var evt = document.createEvent( 'CustomEvent' );
			evt.initCustomEvent( event, params.bubbles, params.cancelable, params.detail );
			return evt;
		};
		window.CustomEvent.prototype = window.Event.prototype;
	}

	if ('padStart' in String.prototype == false) {
		String.prototype.padStart = function padStart(len, padding) {
			len = len | 0;
			padding = (padString === undefined) ? padding+"" : " ";
			if (this.length > len) {
				return this;
			} else {
				len = len - this.length;
				if (len > len.length) {
					padding += padding.repeat(len / padding.length);
				}
				return padding.slice(0, len) + this;
			}
		};
	}

	// Store references to native browser functions
	var antiInterception = {
		XMLHttpRequest: XMLHttpRequest,
		xhr: {
			send: XMLHttpRequest.prototype.send,
		},
	};

	/**
	 * @typedef {Object} CategorizedBaseDomain
	 * @property {string} domain
	 * @property {boolean} sameAccount
	 * @property {boolean} sameDomain
	 */

	/**
	 * @typedef {Object} GcmInfo
	 * @property {boolean} consentModeUsed
	 * @property {Object} consentMode
	 * @property {Object} eventData
	 */

	/**
	 * @typedef {Object} JsApiFeatureList
	 * @property {boolean} embeddingConsentState
	 */

	/**
	 * @typedef {Object} AssetInfo
	 * @property {string} id
	 * @property {string} name
	 * @property {string} type
	 * @property {boolean} glob
	 * @property {string} lifetime
	 * @property {?number} hours
	 */

	/**
	 * @typedef {Object} ConsentState
	 * @property {Object<string, boolean>} googleAdTechProviders
	 * @property {Object<string, boolean>} tcfLegIntPurposes
	 * @property {Object<string, boolean>} tcfLegIntVendors
	 * @property {Object<string, boolean>} tcfPurposes
	 * @property {Object<string, boolean>} tcfSpecialFeatures
	 * @property {Object<string, boolean>} tcfVendors
	 */

	/**
	 * @typedef {Object} EmbeddingInfo
	 * @property {string} id
	 * @property {string} name
	 * @property {string} vendor
	 * @property {string} code
	 * @property {boolean} consent Defined at runtime.
	 * @property {Array.<string>} iframeBlockingStrings
	 * @property {boolean} hideIframesUntilConsent
	 * @property {boolean} dispatchLoadEventForIframes
	 * @property {string} scriptLoaderGroup
	 * @property {Array.<string>} excludeUrls
	 * @property {string} excludeUrlsMode
	 * @property {?string} purpose
	 * @property {boolean} mandatory
	 * @property {Array.<AssetInfo>} assets
	 * @property {Array.<TcfInfo>} tcf
	 */

	/**
	 * @typedef {Object} PluginInfo
	 * @property {string} name
	 * @property {string} script
	 */

	/**
	 * @typedef {Object} TcfInfo
	 * @property {Array.<number>} p
	 * @property {Array.<number>} sf
	 */

	// FNV-1a hash map
	/** @type {Array.<string>} */
	var hl = [];
	for(var i=0; i < 256; i++){
		hl[i] = ((i >> 4) & 15).toString(16) + (i & 15).toString(16);
	}

	var Utils = {
		/**
		 * @param {Element} element
		 * @param {string} className
		 * @param {?boolean=} forceState
		 * @return {boolean}
		 */
		toggleClass: function (element, className, forceState) {
			if (!element || !element.classList) {
				if (CCM_DEBUG) {
				console.warn('[CCM19] Cannot toggle class %s on %s', className, element === null ? element : typeof element);
				}
				return false;
			}

			var newState = typeof forceState == 'boolean'
				? forceState
				: element.classList.contains(className) == false;

			newState ? element.classList.add(className) : element.classList.remove(className);
			return newState;
		},
		/**
		 * @param {Element} target
		 * @param {Element} source
		 */
		applyAttributes: function (target, source) {
			toArray(source.attributes).forEach(function (attr) {
				target.setAttribute(attr.name, attr.value);
			});
		},
		/**
		 * Check if the node is visible inside the DOM.
		 * @param {Node} node
		 * @return {boolean}
		 */
		isVisibleDOMNode: function (node) {
			if (node.ownerDocument !== document) {
				return false;
			}

			while (node !== document.documentElement && node.parentElement) {
				node = node.parentElement;
			}

			return node === document.documentElement;
		},
		/**
		 * Detects whether the document is ready to be modified.
		 * @return {boolean}
		 */
		isDocumentReady: function () {
			// Es ist aufgetreten, dass CCM19 selbst von anderer Software blockiert
			// und später nachgeladen wurde. Dabei passierte es, dass DOMContentLoaded
			// bereits gefeuert wurde, document.readyState aber auf "loading" stand
			// und auch readystatechange nicht mehr für "interactive" feuerte.
			// Obwohl CCM19 als erstes Skript im <head> zu laden ist, gibt es hiermit
			// nun einen Workaround für diesen Fall, damit das Widget trotzdem lädt.
			return document.body && typeof document.body == 'object' && 'appendChild' in document.body;
		},
		/**
		 * @param {Array.<boolean>} values
		 * @return {boolean|'partial'} Returns `true` if all values evaluate to `true`; `'partial'` if any value does; and `false` if none do or array is empty.
		 */
		partial: function (values) {
			return values.reduce(function (newState, value) {
				if (newState === true) {
					newState = !!value || 'partial';
				}
				else {
					newState = !!value ? 'partial' : newState;
				}

				return newState;
			}, values.length ? !!values[0] : false);
		},
		/**
		 * @param {Array} array
		 * @return {Array}
		 */
		uniqueValues: function (array) {
			return array.filter(function (value, index) {
				return array.indexOf(value) == index;
			});
		},
		/**
		 * @param {Array} a
		 * @param {Array} b
		 * @return {Array}
		 */
		diff: function (a, b) {
			return a.filter(function (value) {
				return !b.includes(value);
			});
		},
		/**
		 * @param {Array} a
		 * @param {Array} b
		 * @return {Array}
		 */
		intersect: function (a, b) {
			return a.filter(function (value) {
				return b.includes(value);
			});
		},
		/**
		 * @param {Element} element
		 * @return {void}
		 */
		enableButtons: function (element) {
			toArray(element.getElementsByTagName('BUTTON')).forEach(function (button) {
				button.disabled = false;
			});
		},
		/**
		 * @param {Element} element
		 * @return {void}
		 */
		disableButtons: function (element) {
			toArray(element.getElementsByTagName('BUTTON')).forEach(function (button) {
				button.disabled = true;
			});
		},
		/**
		 * @param {string} url
		 * @return {string}
		 */
		extractDomain: function (url) {
			var match = url.match(/^(?:https?:)?\/\/([^\/]+)/);
			return match ? match[1] : '';
		},
		/**
		 * @param {EventTarget} node
		 * @param {string} name
		 * @param {*=} detail
		 * @return {void}
		 */
		triggerCustomEvent: function (node, name, detail) {
			var evt;
			try {
				evt = new CustomEvent(name, { detail: detail });
			}
			catch (_) {
				evt = document.createEvent('CustomEvent');
				// noinspection JSDeprecatedSymbols
				evt.initCustomEvent(name, true, false, detail);
			}
			node.dispatchEvent(evt);
		},
		/**
		 * @param {string} name
		 * @param {Object} data
		 * @return {void}
		 */
		triggerGTMEvent: function (name, data) {
			data = data || {};
			data.event = name;
			window.dataLayer = window.dataLayer || [];
			window.dataLayer.push(data);
		},
		/**
		 * @param {string} type
		 * @return {boolean}
		 */
		hasGTMLayer: function (type) {
			var layers = window.dataLayer || [];
			for (var i = 0; i < layers.length; ++i) {
				var layer = layers[i];
				if (layer[0] === type || layer[type] !== undefined) {
					return true;
				}
			}
			return false;
		},
		/**
		 * @param {...*}
		 * @return {void}
		 */
		sendGTag: function () {
			if (typeof window.gtag == 'function') {
				window.gtag.apply(null, arguments);
			}
			else {
				window.dataLayer = window.dataLayer || [];
				window.dataLayer.push(arguments);
			}
		},
		/**
		 * @param {HTMLScriptElement} script
		 * @return {boolean}
		 */
		isJavaScript: function (script) {
			return (!script.type || /(application|text)\/(x-)?(java|ecma|j|live)script(1.[0-5])?/.test(script.type.toLowerCase()) || script.type.toLowerCase() == 'module');
		},
		/**
		 * @returns {boolean}
		 */
		isIE: function () {
			return 'userAgent' in navigator && /\b(Trident|MSIE)\b/.test(navigator.userAgent);
		},
		/**
		 * @param {Node} node
		 * @return {?Node}
		 */
		getContainingModal: function getContainingModal(node) {
			if (!node || !('classList' in node)) {
				return null;
			}
			else if (node.classList.contains('ccm-modal')) {
				return node;
			}
			return getContainingModal(node.parentNode);
		},
		/**
		 * @param {?Element=} root
		 * @return {Array.<Element>}
		 */
		focusableElements: function (root) {
			if (!root) {
				root = document.body;
			}
			return toArray(root.querySelectorAll('button, a[href], map area[href], input:not(:disabled), select, textarea, [tabindex]:not([tabindex="-1"])'))
				.filter(function (el) { return el.offsetHeight !== 0 && el.offsetWidth !== 0; });
		},
		/**
		 * @param {MouseEvent} event
		 * @returns {boolean}
		 */
		isManipulatedClick: function (event) {
			if (event instanceof MouseEvent == false || event.type != 'click') {
				return false;
			}

			if ('isTrusted' in event && event.isTrusted == false) {
				if (CCM_DEBUG) {
					console.warn('[CCM19] Manipulated click detected, will probably ignore.');
				}
				return true;
			}

			return false;
		},
		assign: ('assign' in Object) ? Object.assign : function (target, source) {
			for (var attrname in source) { target[attrname] = source[attrname]; }
			return target;
		},
		filterObject: function (obj, callback) {
			var target = {};
			for (var attrname in obj) {
				if (callback(obj[attrname], attrname)) {
					target[attrname] = obj[attrname];
				}
			}
			return target;
		},
		mapObject: function (obj, callback) {
			var target = {};
			for (var attrname in obj) { target[attrname] = callback(obj[attrname], attrname); }
			return target;
		},
		/**
		 * Implicit polyfill for `Object.values`.
		 * @param {Object} obj
		 * @return {Array}
		 */
		objectValues: function (obj) {
			return ('values' in Object) ? Object.values(obj) : Object.keys(obj).map(function (key) { return obj[key]; });
		},
		/**
		 * Similar to `function.bind(self)`, but implemented with a closure instead.
		 *
		 * Can only bind `this`, not other parameters! Intended as a workaround for browser bugs.
		 * @param {Object} obj
		 * @return {Array}
		 */
		bindThis: function (self, func) {
			return function (/*...*/) { return func.apply(self, arguments); };
		},
		/**
		 * @param {string} str
		 * @return {string}
		 */
		hash: function (str) {
			/* FNV-1a Hash (64 bit) from fnv-plus by Travis Webb <me@traviswebb.com> (MIT Licence) */
			var c,i,l=str.length,t0=0,v0=0x2325,t1=0,v1=0x8422,t2=0,v2=0x9ce4,t3=0,v3=0xcbf2;

			for (i = 0; i < l; i++) {
				c = str.charCodeAt(i);
				if(c < 128){
					v0^=c;
				}else if(c < 2048){
					v0^=(c>>6)|192;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=(c&63)|128;
				}else if(((c&64512)==55296)&&(i+1)<l&&((str.charCodeAt(i+1)&64512)==56320)){
					c=65536+((c&1023)<<10)+(str.charCodeAt(++i)&1023);
					v0^=(c>>18)|240;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=((c>>12)&63)|128;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=((c>>6)&63)|128;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=(c&63)|128;
				}else{
					v0^=(c>>12)|224;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=((c>>6)&63)|128;
					t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
					t2+=v0<<8;t3+=v1<<8;
					t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
					v0^=(c&63)|128;
				}
				t0=v0*435;t1=v1*435;t2=v2*435;t3=v3*435;
				t2+=v0<<8;t3+=v1<<8;
				t1+=t0>>>16;v0=t0&65535;t2+=t1>>>16;v1=t1&65535;v3=(t3+(t2>>>16))&65535;v2=t2&65535;
			}

			return hl[v3>>8]+hl[v3&255]+hl[v2>>8]+hl[v2&255]+hl[v1>>8]+hl[v1&255]+hl[v0>>8]+hl[v0&255];
		},
		/**
		 * @param {Element|Location|string} element
		 * @param {boolean=} forceCopy
		 * @return {Location|HTMLHyperlinkElementUtils}
		 */
		getUrl: function (element, forceCopy) {
			var url = null;
			if (!forceCopy && typeof element == 'object' && 'hostname' in element && 'hash' in element) {
				url = element;
			}
			else {
				url = document.createElement('a');
				if (typeof element == 'string' || element instanceof String) {
					url.href = element;
				}
				else if ('href' in element) {
					url.href = element.href;
				}
				else if ('action' in element) {
					url.href = element.action;
				}
			}
			return url;
		},
		/**
		 * @param {Element|Location} element
		 * @param {HTMLHyperlinkElementUtils|Location|string} url
		 * @return {void}
		 */
		setUrl: function (element, url) {
			if (element !== url) {
				if ('href' in url) {
					url = url.href;
				}

				if ('href' in element) {
					element.href = url;
				}
				else if ('action' in element) {
					element.action = url;
				}
			}
			return url;
		},
		/**
		 * Subset von fnmatch-Patternmatching
		 *
		 * "*": Beliebige Anzahl Zeichen
		 * "?": Ein beliebiges Zeichen
		 * "\": Escaped * und ?, aber mehrere Backslashes geben die
		 *      Sonderbedeutung der Zeichen nicht wieder frei
		 *
		 * @param {string} pattern
		 * @param {string} string
		 * @return {boolean}
		 */
		fnmatch: function(pattern, string) {
			pattern = pattern.replace(/[.+^${}()|[\]\\]/g, '\\$&').replace(/\*/g, '.*').replace(/\\\\\.\*/g, '\*').replace(/\?/g, '.').replace(/\\\\\.\*/g, '\?');
			return (new RegExp("^"+pattern+"$", "")).test(string);
		},
		/**
		 * Prüfen, ob die aktuelle Seite in einem Top-Level-Context läuft
		 *
		 * @return {boolean} false, wenn in einem IFrame o.ä.
		 */
		isTopLevel: function() {
			try {
				var top = window.top;
				return (window === top || window.self === top);
			} catch (e) {
				return false;
			}
		},
		/**
		 * Property mit eigenem Getter/Setter wrappen
		 * (Proxy-Objekt lite)
		 *
		 * @param {Object} object
		 * @param {string} name
		 * @param {Object.<string, Function>} options {get, set, value}
		 * @return {boolean} Erfolg
		 */
		wrapProperty: function(object, name, options) {
			// Original Property-Descriptor holen
			var descriptor = null;
			var prototype = object;
			for (var i = 0; (i < 10) && prototype; ++i) {
				descriptor = Object.getOwnPropertyDescriptor(prototype, name);
				if (descriptor !== undefined) {
					break;
				}
				prototype = Object.getPrototypeOf(prototype);
			}
			if (!descriptor) {
				return false;
			}

			// Getter wrappen
			if (typeof options.get == 'function') {
				var origGetter = descriptor.get;
				descriptor.get = function () {
					return options.get.call(this, origGetter);
				}
			}

			// Setter wrappen
			if (typeof options.set == 'function') {
				var origSetter = descriptor.set;
				descriptor.set = function (value) {
					return options.set.call(this, value, origSetter);
				}
			}

			// Wert wrappen
			if (typeof options.value == 'function') {
				var origValue = descriptor.value;
				descriptor.value = function () {
					return options.value.apply(this, [origValue].concat(toArray(arguments)));
				};
			}

			try {
				// Consider that [value or writable] and [get or set] is illegal and may throw an exception
				Object.defineProperty(object, name, descriptor);
				if (CCM_DEBUG) {
					console.info('[CCM19] Property %s.%s wrapped', object, name, descriptor);
				}
				return true;
			} catch (e) {
				if (CCM_DEBUG) {
					console.warn('[CCM19] Cannot wrap property %s. %s: %s', name, e.name, e.message);
				}
				return false;
			}
		},
		skipAnimationFrame: function (callback) {
			window.requestAnimationFrame(function () {
				window.requestAnimationFrame(callback);
			});
		},
	};

	Object.defineProperties(Utils, {
		/**
		 * Implicit polyfill for `Date.now`.
		 * @return {number}
		 */
		now: {
			enumerable: true,
			get: function () {
				return ('now' in Date) ? Date.now() : new Date().getTime();
			},
		},
	});

	function CookieConsentManagement(config, urlConsent) {
		delete config.init;
		Utils.assign(this, config);
		this.urlConsent = urlConsent;
		this.init();
	}

	/**
	 * @property {string} domainId
	 * @property {string} versionId
	 * @property {string} versionName
	 * @property {Object.<string, string>} consts
	 * @property {?string} locale
	 * @property {string} fallbackLocale
	 * @property {boolean} prioritizeLangAttribute
	 * @property {Object.<string, string>} detailsUrls {locale: url, …}
	 * @property {Object.<string, string>} widgetUrls {locale: url, …}
	 * @property {Object.<string, Object<string, string>>} l10n {locale: {name: translation, …}, …}
	 * @property {string} contentBlockedUrl
	 * @property {string} cookieDeclarationUrl
	 * @property {string} cronUrl
	 * @property {string} pageCheckUrl
	 * @property {string} statisticsUrl
	 * @property {string} origin
	 * @property {Array.<string>} cssFiles
	 * @property {Array.<string>} jsFiles
	 * @property {Array.<PluginInfo>} plugins
	 * @property {string} consentStorageMethod
	 * @property {string} storageNamespace
	 * @property {string} storageNamespaceUrlParameter
	 * @property {number} cookieLifetime
	 * @property {boolean} cookieUseSecure
	 * @property {boolean} isActive
	 * @property {boolean} isPreview
	 * @property {number} widgetDelayInMilliseconds
	 * @property {boolean} showFrontendWidgetOnlyIfEmbeddingsExist
	 * @property {number} nonCmpEmbeddingCount
	 * @property {boolean} triggeringExtGtmEvents
	 * @property {boolean} deleteAllCookiesConfig
	 * @property {boolean} deleteAllCookiesReload
	 * @property {boolean} runMandatoryEmbeddingsWithoutConsent
	 * @property {number} consentCookieReset
	 * @property {number} consentLifetime
	 * @property {?number} consentLifetimeForPartialConsent
	 * @property {Array.<string>} consentShareDomains
	 * @property {boolean} consentCrossDomain
	 * @property {Array.<string>} accountWideConsentSharingDomains
	 * @property {boolean} accountWideConsentSharing
	 * @property {boolean} settingsIconEnabled
	 * @property {string} settingsIconTarget
	 * @property {Object.<string, EmbeddingInfo>} repository
	 * @property {Object} tcfData
	 * @property {Object} behavior
	 * @property {JsApiFeatureList} jsApiFeatures
	 * @property {boolean} partOfAdAndTrackingFreeNetwork
	 * @property {boolean} evalInlineScripts
	 * @property {boolean} blockIframes
	 * @property {boolean} rememberIframeConsent
	 * @property {boolean} appendConsentSwitchToIframes
	 * @property {string} iframeBlockMode
	 * @property {boolean} blockNewScripts
	 * @property {boolean} blockSameDomainScripts
	 * @property {boolean} blockInlineScripts
	 * @property {boolean} detectCloakedScripts
	 * @property {boolean} manipulationPrevention
	 * @property {Object.<string, Array.<string>>} blockEmbeddingScriptMarkers
	 * @property {Array.<string>} deniedScripts
	 * @property {Array.<string>} allowedScripts
	 * @property {Array.<string>} allowedScriptMarkers
	 * @property {Array.<string>} deniedScriptMarkers
	 * @property {Array.<string>} iframeMarkers
	 * @property {?string} country
	 * @property {Array.<string>} exemptUrls
	 * @property {boolean} scripTracker
	 * @property {boolean} mustMatchDomains
	 * @property {string} mainMatchDomain
	 * @property {string} selfUrlPattern
	 * @property {number} cronProbability
	 * @property {number} reportProbability
	 * @property {?Object} clickskeksLegacy
	 */
	CookieConsentManagement.prototype = {
		constructor: CookieConsentManagement,
		initialized: false,
		widgetInitialized: false,
		shouldReport: false,
		container: null,
		controlPanel: null,
		urlConsent: null,
		widget: null,
		widgetUrl: null,
		ownHeadElements: [],
		highPriorityEventHandlers: {},
		internalEventHandlers: {},
		cookies: {
			consent: 'ccm_consent',
		},
		/** @type {Object.<string, string>} Property names should correspond to values for `clickedButton`. */
		accountWideConsentType: {
			acceptAll: '1',
			decline: '0',
		},
		ucid: Utils.hash(String(Utils.now) + String(Math.random()).substring(1)),
		/** @type {?string} ('acceptAll', 'decline', 'save') */
		clickedButton: null,
		/** @type {ConsentState} */
		consentState: {},
		/** @type {boolean} Whether consent settings are currently being saved, i.e. waiting for skipAnimationFrame. */
		saving: false,
		saveSettingsFunctionQueue: [],
		tcfDependantFunctionQueue: [],
		embeddingsEmbedded: [],
		loadingModal: false,
		forceOpenWidget: false,
		requiresConsentUpdate: false,
		consentShareChannels: [],
		dynamicStorageNamespace: false,
		/** @type MutationObserver */
		domWatcher: null,
		iframes: [],
		iframeConsentDomains: [],
		iframeConsentSwitchMap: {},
		blockableScripts: [],
		inertedElements: [],
		scriptLoaderQueue: [],
		scriptLoaderRunning: false,
		foundScriptCookies: {},
		inEval: false,
		focusStack: [],
		tasksRunning: {},

		appendCrossDomainConsent: function (element) {
			var currentDomain = this.getBaseDomain(window.location.hostname);
			var origUrl = Utils.getUrl(element, true);

			var url = this.withCrossDomainConsent(Utils.getUrl(element, true));
			if (url.hash !== origUrl.hash) {
				// Geänderte URL setzen
				Utils.setUrl(element, url);
				// Und danach wiederherstellen
				window.setTimeout(function () { Utils.setUrl(element, origUrl); }, 100);
			}
		},
		withCrossDomainConsent: function (url) {
			var currentDomain = this.getBaseDomain(window.location.hostname);
			var origUrl = url;
			url = Utils.getUrl(url, false);

			// Wenn keine URL oder noch kein Consent erteilt, abbrechen
			if (!url || !this.consentGiven()) {
				return origUrl;
			}

			// Wenn nicht eine der Cross-Sharing-Domains, abbrechen
			var targetDomain = this.getBaseDomain(url.hostname);
			if (!targetDomain || targetDomain === currentDomain) {
				return origUrl;
			}
			// Wenn CCM-Part schon in URL, abbrechen
			if (/(^#?|&)CCM19consent=/.test(url.hash)) {
				return origUrl;
			}

			if (url.hash) {
				url.hash += '&'+this.buildCrossDomainConsent();
			} else {
				url.hash = '#'+this.buildCrossDomainConsent();
			}

			// Geänderte URL zurückgeben
			return (typeof origUrl === 'string') ? url.href : url;
		},
		buildCrossDomainConsent: function () {
			var consentData = this.getConsentStorage();
			var data = '';
			if (consentData && 'updated' in consentData) {
				// Zeitpunkt als Sekunden seit 2021-01-01 übertragen, damit die URL kürzer bleibt.
				var updated = ((consentData.updated / 10 - 1609459200) | 0);
				// Embedding-IDs bestimmen
				var embeddings = ('embeddings' in consentData) ? consentData.embeddings : [];

				// Signal "accept all" or "decline" for account wide consent sharing
				if (this.accountWideConsentSharing && this.clickedButton in this.accountWideConsentType) {
					embeddings = [this.accountWideConsentType[consentData.clickedButton]];
				}

				data = (Utils.now / 1000 / 60 | 0).toString(36)+'|'+updated.toString(36)+'|'+this.getUniqueCookieId()+'|'+(embeddings.join('|'));
			}
			return 'CCM19consent='+data;
		},
		/**
		 * @param {string} hostname
		 * @return {CategorizedBaseDomain}
		 */
		getCategorizedBaseDomain: function (hostname) {
			var domains = this.consentShareDomains;
			var accountDomains = (this.accountWideConsentSharing) ? this.accountWideConsentSharingDomains : [];

			var hostname = hostname.replace(/\.+$/, '');
			for (var i = 0; i < domains.length; ++i) {
				var domain = domains[i];
				if (hostname == domain || (hostname).indexOf('.'+domain, hostname.length-domain.length-1) != -1) {
					return { domain: domain, sameDomain: true, sameAccount: true };
				}
			}
			for (var i = 0; i < accountDomains.length; ++i) {
				var domain = accountDomains[i];
				if (hostname == domain || (hostname).indexOf('.'+domain, hostname.length-domain.length-1) != -1) {
					return { domain: domain, sameDomain: false, sameAccount: true };
				}
			}

			return { domain: domain, sameDomain: false, sameAccount: false };
		},
		/**
		 * Returns the base domain, if it is valid for consent sharing.
		 *
		 * Only returns a value for account-wide consent sharing, if `this.clickedButton` is valid
		 * for account-wide sharing. Use `getCategorizedBaseDomain()` instead if you need more
		 * details.
		 *
		 * @param {string} hostname
		 * @return {?string}
		 */
		getBaseDomain: function (hostname) {
			var data = this.getCategorizedBaseDomain(hostname);

			if (data.sameDomain) {
				return data.domain;
			}
			if (this.accountWideConsentSharing && this.clickedButton in this.accountWideConsentType) {
				if (data.sameAccount) {
					return data.domain;
				}
			}
			return null;
		},
		/**
		 * @param {string} [domain=window.location.hostname]
		 * @return {boolean}
		 */
		isConsentShareDomain: function (domain) {
			domain = typeof domain == 'string' ? domain : window.location.hostname;
			return !!this.getBaseDomain(domain);
		},
		/**
		 * @param {string} urlString
		 * @return {boolean}
		 */
		checkIsExternal: function (urlString) {
			if (!urlString) {
				return false;
			}
			// Relative Pfade
			var dblSlashIdx = urlString.search('//');
			if (dblSlashIdx < 0 || dblSlashIdx > 16) {
				return false;
			}

			// URL parsen
			var url = Utils.getUrl(urlString);

			// data:-URLs
			if (url.protocol === 'data:') {
				return false;
			}
			// Aktueller Hostname
			if (url.hostname === location.hostname) {
				return false;
			}

			// Assoziierte Domains
			if (this.getBaseDomain(url.hostname)) {
				return false;
			}

			// Hauptdomain bestimmen (TODO: Sonderfälle .co.uk etc.)
			var mainDomain = location.hostname;
			mainDomain = mainDomain.replace(/\.$/, '');
			mainDomain = mainDomain.replace(/^.*\.([^.]+\.[^.]+)$/, '$1');

			// Subdomain der gleichen Hauptdomain
			var domainRegExp = new RegExp('\\b'+mainDomain.replace(/([.\\])/g, '\\$1')+'$');
			if (url.hostname.search(domainRegExp) !== -1) {
				return false;
			}

			// CCM-Script ignorieren
			if (Utils.fnmatch(this.selfUrlPattern, urlString)) {
				return false;
			}

			return true;
		},
		/**
		 * Detect scripts that use consent notation specific to other CMPs.
		 * @param {HTMLScriptElement} script
		 * @return {boolean}
		 */
		checkIsCloakedScript: function (script) {
			return this.detectCloakedScripts && (
				script.type.toLowerCase() == 'text/plain' && script.dataset.cookieconsent // Cookiebot
				|| script.type.toLowerCase() == 'text/plain' && script.dataset.usercentrics // Usercentrics
				|| script.type.toLowerCase() == 'text/plain' && script.classList.contains('cmplazyload') && (
					script.dataset.cmpPurpose || script.dataset.cmpVendor
				) // ConsentManager
			);
		},
		/**
		 * @param {HTMLScriptElement} script
		 * @return {boolean}
		 */
		checkIsBlockableScript: function (script) {
			return Utils.isJavaScript(script)
				|| script.type.toLowerCase() == 'text/x-ccm-loader' && !script.dataset.ccmLoaderGroup
				|| script.type.toLowerCase() == 'text/x-magento-init'
				|| this.checkIsCloakedScript(script)
			;
		},
		/**
		 * This method checks for manipulated clicks if manipulation prevention is enabled.
		 * @param {Event} event
		 * @return {boolean}
		 */
		checkIsManipulatedClick: function (event) {
			// Disable manipulation check if consent update is possible through JavaScript API
			if (this.jsApiFeatures.embeddingConsentState) {
				return false;
			}

			return this.manipulationPrevention && Utils.isManipulatedClick(event);
		},
		/**
		 * @return {Array.<Object.<string,string>>}
		 */
		getCookies: function () {
			return document.cookie.split(';')
				.filter(function (chunk) {
					return chunk.trim().length > 0;
				})
				.map(function (cookie) {
					var parts = cookie.trim().split('=');
					return {
						name: parts[0],
						value: parts[1],
					};
				});
		},
		// Nur im Zusammenhang mit intern gesetzten Cookies (setCookie) verwenden!
		/**
		 * @param {string} name
		 * @param {string=} storageMethod
		 * @return {?}
		 */
		getCookie: function (name, storageMethod) {
			// In Themevorschau immer von einem leeren Speicher ausgehen
			if (this.isPreview && name == this.cookies.consent) {
				return {};
			}

			var self = this;
			var result = null;

			var method = (storageMethod !== undefined) ? storageMethod : this.consentStorageMethod;

			// Session-Storage (entweder durch Einstellung oder für temporären Consent, z.B. durch Schließen-Button
			if ('sessionStorage' in window && 'getItem' in window.sessionStorage) {
				result = window.sessionStorage.getItem(name);
				if (result !== null) {
					// Unterdrücke Fehlermeldungen, wenn der Wert manipuliert wurde
					try {
						return JSON.parse(result);
					} catch (error) {
						if (method == 'sessionStorage') {
							return {};
						}
					}
				}
			}

			if (method == 'localStorage' && 'localStorage' in window && 'getItem' in window.localStorage) {
				result = window.localStorage.getItem(name);
				if (result !== null) {
					// Unterdrücke Fehlermeldungen, wenn der Wert manipuliert wurde
					try {
						return JSON.parse(result);
					} catch (error) {
						return {};
					}
				}
			}

			if (method == 'cookie' || !('localStorage' in window) || !('sessionStorage' in window)) {
				if (name == this.cookies.consent) {
					this.tidyConsentStorage();
				}

				return this.getCookies().reduce(function (value, cookie) {
					// Unterdrücke Fehlermeldungen, wenn der Wert manipuliert wurde
					try {
						return cookie.name === name ? self.decodeCookieValue(cookie.value) : value;
					} catch (error) {
						return {};
					}
				}, null);
			}

			return {};
		},
		/**
		 * @param {string} name
		 * @param {*} value
		 * @return {?Object=} options
		 */
		setCookie: function (name, value, options) {
			if (name === undefined || value === undefined) {
				throw new Error('Undefined name or value not allowed for cookies.');
			}

			options = typeof options === 'object' ? options : {};

			var method = ('method' in options) ? options.method : this.consentStorageMethod;

			if (method == 'sessionStorage' && 'sessionStorage' in window && 'setItem' in window.sessionStorage) {
				window.sessionStorage.setItem(name, JSON.stringify(value));
			}
			else if (method == 'localStorage' && 'localStorage' in window && 'setItem' in window.localStorage) {
				window.localStorage.setItem(name, JSON.stringify(value));
			}
			else {
				var chunks = [
					name + '=' + this.encodeCookieValue(value),
				];

				var domain = this.getBaseDomain(location.hostname);
				if (domain) {
					chunks.push('domain=' + domain);
				}

				if (options.path) {
					chunks.push('path=' + options.path);
				}
				if (options.expires) {
					chunks.push('expires=' + options.expires);
				}
				if (options.sameSite) {
					chunks.push('SameSite=' + options.sameSite);
				}
				else {
					chunks.push('SameSite=Lax');
				}
				if (options.secure) {
					chunks.push('Secure');
				}
				if (options.partitioned) {
					chunks.push('Partitioned');
				}

				document.cookie = chunks.join(';');
			}
		},
		/**
		 * @param {*} value
		 * @return {string}
		 */
		encodeCookieValue: function (value) {
			return encodeURIComponent(btoa(JSON.stringify(value)));
		},
		/**
		 * @param {string} value
		 * @return {?}
		 */
		decodeCookieValue: function (value) {
			return JSON.parse(atob(decodeURIComponent(value)));
		},
		/**
		 * @return {boolean}
		 */
		isBotBrowser: function () {
			return /(bot\b|BOT\b|Bot\b|BingPreview| \(\+http)|[a-z]-Google\b/.test(navigator.userAgent);
		},
		/**
		 * @return {boolean}
		 */
		isDoNotTrackEnabledInBrowser: function () {
			return window.doNotTrack && window.doNotTrack == 1
				|| navigator.doNotTrack && [1, '1', 'yes'].includes(navigator.doNotTrack)
				|| navigator.msDoNotTrack && navigator.msDoNotTrack == 1
				|| typeof window.external === 'object' && 'msTrackingProtectionEnabled' in window.external && window.external.msTrackingProtectionEnabled()
				|| false // redundant, so what...
			;
		},
		/**
		 * @param {string} url
		 * @param {?Object=} options
		 */
		ajax: function (url, options) {
			options = typeof options === 'object' ? options : {};

			var body = options.body;
			var async = typeof options.async === 'boolean' ? options.async : true;
			var method = options.method || 'GET';

			var xhr = new antiInterception.XMLHttpRequest();
			var self = this;

			xhr.onreadystatechange = function () {
				if (this.readyState == 4) {
					(this.status >= 200 && this.status <= 205)
						? typeof options.success === 'function' && options.success.call(self, this.responseText, this.status, xhr)
						: typeof options.failure === 'function' && options.failure.call(self, this.status, xhr);
					typeof options.done === 'function' && options.done.call(self);
				}
			};

			xhr.open(method, url, async);

			if (['POST', 'PUT', 'PATCH'].includes(method)) {
				var contentType = options.contentType || 'application/x-www-form-urlencoded';

				if (contentType === 'application/json') {
					body = JSON.stringify(body);
				}

				xhr.setRequestHeader('Content-Type', contentType);
			}

			if ('headers' in options) {
				for (var key in options.headers) {
					xhr.setRequestHeader(key, options.headers[key]+'');
				}
			}

			// There were scripts that intercepted the request truncating the
			// state handlers, thus, for example, preventing the site from
			// unblocking after a resource (e.g. widget) had been loaded.
			antiInterception.xhr.send.call(xhr, body);
		},
		/**
		 * @return {void}
		 */
		injectCss: function () {
			var self = this;
			this.cssFiles.forEach(function (url) {
				var link = document.createElement('LINK');
				link.setAttribute('href', url);
				link.setAttribute('rel', 'stylesheet');
				link.setAttribute('type', 'text/css');
				self.ownHeadElements.push(link);
				document.head.appendChild(link);
			});
		},
		/**
		 * @return {void}
		 */
		injectJs: function () {
			this.jsFiles.forEach(function (url) {
				var script = document.createElement('SCRIPT');
				script.async = true;
				script.setAttribute('src', url);
				script.setAttribute('data-ccm-injected', 'true');
				document.head.appendChild(script);
			});
		},
		initTcfStub: function () {
			/// TCF-Stub
			if (this.tcfData.enabled && window.__tcfapi === undefined) {
				// Google AdSense pausieren, bis Consent erteilt
				if (!('adsbygoogle' in window)) {
					window.adsbygoogle = [];
				}
				window.adsbygoogle.pauseAdRequests=1;

				var queue = [];
				var gdprApplies = this.isConsentRequired();
				var stub = function stub() {
					if (arguments.length == 0) {
						return queue.splice(0);
					} else if (arguments[0] === 'ping') {
						var result = {
							gdprApplies: gdprApplies,
							cmpLoaded: false,
							apiVersion: '2.2',
							cmpStatus: 'stub',
							displayStatus: 'hidden',
						};
						if (arguments[2]) {
							arguments[2](result, true);
						} else {
							return result;
						}
					} else {
						queue.push(toArray(arguments));
					}
				}

				var locatorIFrame = document.createElement("iframe");
				locatorIFrame.id = 'ccm-tcf-locator';
				locatorIFrame.style.display = 'none';
				locatorIFrame.style.visibility = 'hidden';
				locatorIFrame.style.width = '0';
				locatorIFrame.style.height = '0';
				locatorIFrame.name = '__tcfapiLocator';

				function insertLocatorFrame() {
					if (document.body) {
						document.body.append(locatorIFrame);
					} else {
						window.addEventListener('DOMContentLoaded', function () { document.body.append(locatorIFrame); });
					}
				}

				var otherCmpFound = !!window.frames['__tcfapiLocator'];
				if (!otherCmpFound) {
					// Stub einfügen
					window.__tcfapi = stub;

					// TCF-Iframe-Message-Handler
					window.addEventListener('message', function (evt) {
						try {
							var payload = typeof event.data === 'string' ? JSON.parse(event.data) : event.data;
						} catch (ignore) {
							payload = null;
						}

						if (payload && typeof payload == 'object' && '__tcfapiCall' in payload && typeof payload.__tcfapiCall == 'object') {
							var call = payload.__tcfapiCall;
							window.__tcfapi(call.command, call.version, function (data, success) {
								if (success === undefined && typeof data === 'boolean') {
									success = data;
									data = undefined;
								}
								evt.source.postMessage({
									__tcfapiReturn: {
										returnValue: data,
										success: success,
										callId: call.callId
									}
								}, evt.origin);
							}, call.parameter);
						}
					}, false);

					// TCF-Iframe-Marker setzen
					insertLocatorFrame();
				}
			}
		},
		injectPlugins: function () {
			var self = this;
			window.Ccm19Plugins = window.Ccm19Plugins || {};

			// Provide plugins with access to the internal event system
			var eventRegistry = Object.create(null, {
				/**
				 * @param {String} name
				 * @param {Function} callback
				 */
				addEventListener: {
					enumerable: true,
					value: function (name, callback) {
						if (typeof callback != 'function') {
							throw new TypeError('[CCM19] Wrong data type, function expected.');
						}
						self.internalEventHandlers[name] = self.internalEventHandlers[name] || [];
						self.internalEventHandlers[name].push(callback);

						// Dispatch the event retroactively if the widget already has been initialized
						if (name == 'widgetInitialized' && self.widgetInitialized) {
							callback();
						}
					}
				},
			});

			// Provide access to some of the internal functions
			var pluginApi = Object.create(null, {
				/**
				 * @type {boolean}
				 */
				initialized: {
					enumerable: true,
					get: function () {
						return self.widgetInitialized;
					},
				},
				/**
				 * @return {void}
				 */
				closeAllModals: {
					enumerable: true,
					value: this.closeAllModals.bind(this),
				},
				/**
				 * @return {void}
				 */
				saveSettings: {
					enumerable: true,
					value: function () {
						if (this.initialized) {
							self.saveSettings();
						}
					},
				},
				/**
				 * @param {String} embeddingId
				 * @param {boolean} state
				 */
				setEmbeddingConsentState: {
					enumerable: true,
					value: function (embeddingId, state) {
						var embedding = self.repository[embeddingId];
						if (embedding) {
							embedding.consent = state || embedding.mandatory;
							self.selectEmbedding(embedding);
						}
					},
				},
				/**
				 * @param {String} embeddingId
				 */
				showEmbeddingDetails: {
					enumerable: true,
					value: function (embeddingId) {
						if (this.initialized) {
							self.showEmbeddingDetails(embeddingId);
						}
					},
				},
				/**
				 * @param {string} embeddingId
				 * @return {boolean}
				 */
				isEmbeddingActiveForCurrentLocation: {
					enumerable: true,
					value: function (embeddingId) {
						return self.repository.propertyIsEnumerable(embeddingId)
							&& self.isEmbeddingActiveForCurrentLocation(self.repository[embeddingId]);
					},
				},
				/**
				 * @param {String} code
				 */
				executeJavascriptCode: {
					enumerable: true,
					value: function (code) {
						var script = document.createElement('SCRIPT');
						script.textContent = code;
						script.async = false;
						self.executeScriptQueue([script]);
					},
				},
			});

			// Load and instantiate available plugins
			this.plugins.forEach(function (plugin) {
				if (!plugin.script) {
					plugin.loaded = true;
					return;
				}

				// Load plugins in the language that is currently active
				var url = plugin.script;
				url = url.replace(/&lang=[^&]+/, '&lang=' + encodeURIComponent(self.locale));

				var script = document.createElement('SCRIPT');
				script.async = true;
				script.setAttribute('src', url);
				script.setAttribute('data-ccm-injected', 'true');
				script.addEventListener('error', function () {
					plugin.loaded = true;
				});
				script.addEventListener('load', function () {
					var constructor = window.Ccm19Plugins[plugin.name];
					if (typeof constructor == 'function') {
						try {
							plugin.instance = new constructor(pluginApi, eventRegistry);
						} catch (e) {
							console.error('[CCM19] Error while initializing plugin "%s": %s', plugin.name, e);
						}
						// Prevent other scripts from accessing the plugin's constructor
						delete window.Ccm19Plugins[plugin.name];
					}
					plugin.loaded = true;
				});
				document.head.appendChild(script);
			});
		},
		/**
		 * Whether all plugins have been loaded.
		 * @return {boolean}
		 */
		isAllPluginsLoaded: function () {
			return this.plugins.reduce(function (loaded, plugin) {
				return loaded && !!plugin.loaded;
			}, true);
		},
		/**
		 * Dispatch CCM19-internal events; primarily for plugins.
		 * @param name
		 * @param data
		 */
		dispatchInternalEvent: function (name, data) {
			var handlers = this.internalEventHandlers[name] || [];
			handlers.forEach(function (handler) {
				handler(data);
			});
		},
		initPlatformDependentComponents: function () {
			// Determine scrollbar dimensions and store as css variable
			var dummy = document.createElement('DIV');
			dummy.style.boxSizing = 'border-box';
			dummy.style.display = 'block';
			dummy.style.position = 'fixed';
			dummy.style.top = '-9999px';
			dummy.style.left = '-9999px';
			dummy.style.width = '320px';
			dummy.style.height = '240px';
			dummy.style.visibility = 'hidden';
			dummy.style.overflowX = 'scroll';
			dummy.style.overflowY = 'scroll';

			document.body.appendChild(dummy);

			var scrollbarWidth = dummy.offsetWidth - dummy.clientWidth;
			var scrollbarHeight = dummy.offsetHeight - dummy.clientHeight;

			document.body.removeChild(dummy);

			document.documentElement.style.setProperty('--ccm--scrollbar-width', scrollbarWidth + 'px');
			document.documentElement.style.setProperty('--ccm--scrollbar-height', scrollbarHeight + 'px');
		},
		/**
		 * @param {Array.<HTMLScriptElement>} scriptNodes
		 * @param {string=} component Optional component name to control internal logic.
		 */
		executeScriptQueue: function (scriptNodes, component) {
			var self = this;
			var interrupt = false;

			scriptNodes.forEach(function (script, index) {
				if (interrupt) {
					return;
				}

				var original = script.ccmOriginalNode || script;
				var src = (script.type == 'text/x-ccm-loader' && script.dataset.ccmLoaderSrc) || script.src;

				// Clear the script type to make the script executable.
				if (script.type == 'text/x-ccm-loader' || self.checkIsCloakedScript(script)) {
					// Only the clone must be executable; Chrome and Firefox showed different behavior
					// when the attributes (type|src) were updated on the original script element.
					var tmpScript = script.cloneNode(true);
					tmpScript.removeAttribute('type');
					tmpScript.async = script.hasAttribute('async');

					// Resolve manual blocking code following the ConsentManager notation
					if (script.type.toLowerCase() == 'text/plain' && script.classList.contains('cmplazyload')) {
						if (!src && script.dataset.cmpSrc) {
							src = script.dataset.cmpSrc;
						}
						tmpScript.classList.remove('cmplazyload');
						tmpScript.removeAttribute('data-cmp-purpose');
						tmpScript.removeAttribute('data-cmp-src');
						tmpScript.removeAttribute('data-cmp-vendor');
					}

					script = tmpScript;

					if (src) {
						script.removeAttribute('data-ccm-loader-src');
						script.setAttribute('src', src);
					}
				}

				if (CCM_DEBUG) {
					var name = src || script.textContent.trim().replace(/\s{2,}/g, ' ').substring(0, 40);
					console.info('[CCM19] Executing script %s', name);
				}
				if (script.hasAttribute('src') || self.evalInlineScripts == false) {
					// Use createElement over cloneNode so that the browser will definitely run the script
					var clone = document.createElement('SCRIPT');
					clone.setAttribute('data-ccm-injected', 'true');

					Utils.applyAttributes(clone, script);

					// Copy content for inline scripts
					if (!src) {
						clone.textContent = script.textContent;
					}

					// Preserve async value to enforce correct execution order.
					// @see https://developer.mozilla.org/en-US/docs/Web/HTML/Element/script#compatibility_notes
					clone.async = script.async;

					if (src) {
						if (original !== script) {
							/** @param {Event} event */
							var onScriptFinished = function (event) {
								// Send the event to the original script element. It may have a listener assigned to it.
								Utils.triggerCustomEvent(original, event.type);
							};
							clone.addEventListener('error', onScriptFinished);
							clone.addEventListener('load', onScriptFinished);
						}

						// Delay further script execution until the current script injection is complete.
						if (!script.async) {
							var resumeScriptQueue = function () {
								self.executeScriptQueue(scriptNodes.slice(index + 1), component);
							};
							clone.addEventListener('error', resumeScriptQueue);
							clone.addEventListener('load', resumeScriptQueue);
							interrupt = true;
						}
					}

					// Check if the original node is inside the public DOM.
					if (Utils.isVisibleDOMNode(original)) {
						original.parentElement.insertBefore(clone, original);
						// Keep the original inside the DOM because it could be referenced elsewhere
						original.type = 'text/x-unblocked-script';
					}
					else if (document.body) {
						document.body.appendChild(clone);
					}
					else {
						document.head.appendChild(clone);
					}
				}
				else {
					try {
						self.inEval = true;
						// Wenn es Javascript ist, per Eval ausführen
						if (Utils.isJavaScript(script)) {
							eval.call(window, script.textContent);
						}
						self.inEval = false;
					} catch (error) {
						self.inEval = false;
						console.error('[CCM19] Script execution: (' + error.name + ') ' + error.message);
					}
				}
			});

			if (!interrupt && component == 'ScriptLoader') {
				this.scriptLoaderRunning = false;
				this.executeScriptLoaderQueue();
			}
		},
		executeScriptLoaderQueue: function () {
			if (this.scriptLoaderRunning || this.scriptLoaderQueue.length == 0) {
				return;
			}
			this.scriptLoaderRunning = true;

			var scriptQueue = this.scriptLoaderQueue;
			this.scriptLoaderQueue = [];
			this.executeScriptQueue(scriptQueue, 'ScriptLoader');
		},
		/**
		 * @param {EmbeddingInfo} embedding
		 * @return {boolean}
		 */
		isEmbeddingActiveForCurrentLocation: function (embedding) {
			return embedding.excludeUrls.reduce(function (block, url) {
				var isWildcard = url.charAt(url.length - 1) == '*';
				url = isWildcard ? url.substr(0, url.length - 1) : url;
				return block || location.href.indexOf(url) == 0 && (isWildcard || location.href.length == url.length);
			}, false) == (embedding.excludeUrlsMode == this.consts.MODE_WHITELIST);
		},
		/**
		 * @param {EmbeddingInfo} embedding
		 */
		embedEmbedding: function (embedding) {
			var self = this;

			if (this.embeddingsEmbedded.includes(embedding.id)) {
				return;
			}
			this.embeddingsEmbedded.push(embedding.id);

			if (this.isEmbeddingActiveForCurrentLocation(embedding) == false) {
				if (CCM_DEBUG) {
					console.info('[CCM19] Embedding blocked for current url: ' + embedding.name);
				}
				return;
			}

			if (CCM_DEBUG) {
				console.info('[CCM19] Load embedding: ' + embedding.name);
			}

			var container = document.createElement('DIV');
			container.innerHTML = embedding.code;

			/** @type Array.<HTMLScriptElement> */
			var scriptQueue = [];

			while (container.children.length > 0) {
				var child = container.removeChild(container.firstElementChild);

				if (child.tagName === 'SCRIPT') {
					if (child.src == '' && Utils.isJavaScript(child)) {
						child.appendChild(document.createTextNode("\n//# sourceURL=#embedding:"+encodeURIComponent(embedding.name)));
					}
					// Consider that async may be initialized with a different value depending on the browser
					child.async = child.hasAttribute('async');
					scriptQueue.push(child);
				}
				else {
					document.body.appendChild(child);
				}
			}

			Utils.triggerCustomEvent(window, 'ccm19EmbeddingAccepted', {
				name: embedding.name,
				code: embedding.code,
				purpose: embedding.purpose,
				mandatory: embedding.mandatory,
			});

			if (this.triggeringExtGtmEvents) {
				// Event für zugelassenes Embedding an GTM senden
				Utils.triggerGTMEvent('CCM19.embeddingAccepted', {
					id: embedding.id,
					name: embedding.name,
				});
			}

			// Blockierte Skripte ggf. freigeben
			this.blockableScripts.forEach(function (node) {
				if (!node.ccmEmbeddings || !node.ccmEmbeddings.includes(embedding.id)) {
					return;
				}

				// Prüfe, ob alle Embeddings, deren Blacklist das Skript blockiert, freigegeben wurden
				var shouldActivate = node.ccmEmbeddings.reduce(function (result, embeddingId) {
					return result && self.embeddingsEmbedded.includes(embeddingId);
				}, true);

				// Aktiviere das zuvor blockierte Skript, sofern relevante Embeddings freigegeben sind
				if (shouldActivate) {
					scriptQueue.push(node);
				}
			});

			// Blockierte Iframes ggf. freigeben
			var selectedEmbeddingIds = this.getEmbeddingsWithConsent().map(function (embedding) { return embedding.id; });
			this.iframes.forEach(function (iframe) {
				var blockedByEmbeddings = iframe.ccm19BlockedByEmbeddings || [];
				if (blockedByEmbeddings.length > 0) {
					var restoreIframe = blockedByEmbeddings.reduce(function (restore, embeddingId) {
						return restore && selectedEmbeddingIds.includes(embeddingId);
					}, true);
					if (restoreIframe) {
						self.restoreIframe(iframe);
					}
				}
			});

			// Execute scripts having attributes async or defer last
			scriptQueue = scriptQueue.filter(function (script) { return !script.async && !script.defer; })
				.concat(scriptQueue.filter(function (script) { return !script.async && script.defer; }))
				.concat(scriptQueue.filter(function (script) { return script.async; }))
			;

			this.executeScriptQueue(scriptQueue);

			// Dispatch event to internal code and plugins
			this.dispatchInternalEvent('embeddingLoaded', embedding.id);
		},
		/**
		 * @param {string} id
		 * @param {?Event} event
		 */
		jumpTo: function (id, event) {
			event = event || window.event;
			event && event.preventDefault();

			// Warte einen weiteren Frame, da das Zielelement in einem bis dahin verstecktem Modal liegen könnte
			window.requestAnimationFrame(function () {
				var el = document.getElementById(id);
				if (el) {
					// Zur korrekten Stelle scrollen
					el.scrollIntoView && el.scrollIntoView();

					// Element bzw. möglichst erstes Kind-Element (i.d.R. die Überschrift) fokussieren.
					var focusEl = el;
					if (el.firstElementChild && ['P', 'DIV'].indexOf(el.firstElementChild.tagName) != -1) {
						focusEl = el.firstElementChild;
					}
					var temporaryFocus = (focusEl.tabIndex == -1);
					if (temporaryFocus) {
						focusEl.tabIndex = 0;
					}
					focusEl.focus({ preventScroll: true });
					if (temporaryFocus) {
						focusEl.tabIndex = -1;
					}
				}
			});
		},
		/**
		 * Handler für Fokus-Änderungen. Hält den Fokus im Widget, wenn aktiviert.
		 *
		 * @param {Event} event
		 * @return {void}
		 */
		onFocusIn: function (event) {
			// Offenen Dialog bestimmen
			var openModals = document.querySelectorAll('.ccm-root > .ccm-modal.ccm-show');
			var container = openModals[openModals.length-1];
			if (!container) {
				return;
			}
			// Während der Fokus im Tab-Keydown-Handler bearbeitet wird, nicht reinpfuschen
			if (this._ignoreFocused && this._ignoreFocused === event.target) {
				return;
			}
			// Ggf. Fokus-Helper im DOM umordnen, wenn sie von anderem JS verschoben wurden
			var topFocusHelper = document.getElementById('ccm-focus-helper--top');
			var bottomFocusHelper = document.getElementById('ccm-focus-helper--bottom');
			if (topFocusHelper.previousSibling) {
				document.body.insertBefore(topFocusHelper, document.body.firstChild);
			}
			if (bottomFocusHelper.nextSibling) {
				document.body.appendChild(bottomFocusHelper);
			}

			// Bestimmen, ob der Container den Fokus verlieren würde
			var fromContainer = Utils.getContainingModal(event.relatedTarget);
			var nextContainer = Utils.getContainingModal(event.target);
			if (nextContainer !== container) {
				// Bestimmen, wo der Fokus zuletzt war
				var focusable = Utils.focusableElements(container);
				var innerContainer = container.querySelector('.ccm-modal-inner');
				var firstFocusable = focusable[0];
				var lastFocusable = focusable[focusable.length - 1];
				// Wenn Fokus am Ende des Dialogs war, oder zum Anfang der Seite ging, zum Anfang des Dialogs
				if (lastFocusable == event.relatedTarget || event.target === topFocusHelper) {
					firstFocusable.focus();
					// Versuchen den Inner-Container vom Modal zu selecten (falls eine Scrollbar da ist)
					if (innerContainer) {
						innerContainer.focus();
					}
					event.preventDefault();
				}
				// Wenn Fokus am Anfang des Dialogs war, oder zum Ende der Seite ging, zum Ende des Dialogs
				else if (firstFocusable == event.relatedTarget || event.relatedTarget === innerContainer || event.target === bottomFocusHelper) {
					lastFocusable.focus();
					event.preventDefault();
				}
				// Ansonsten, wenn der Fokus aus dem Dialog ging, zum Punkt zurück wo er war
				else if (fromContainer === container) {
					event.relatedTarget.focus();
					event.preventDefault();
				}
				// Ansonsten zum Anfang des Dialogs
				else {
					firstFocusable.focus({ preventScroll: true });
					event.preventDefault();
				}
			}
		},
		/**
		 * Handler für Tab bei Fokuswechseln. Ergänzt onFocusIn indem dafür gesorgt
		 * wird, dass der User auch zu den Browser-Controls tabben kann, und nicht
		 * komplett im Dialog eingesperrt ist.
		 *
		 * @param {Event} event
		 * @return {void}
		 */
		onTabKeyDown: function (event) {
			// Nur Tab-Key beachten
			if (event.keyCode !== 9) {
				return;
			}
			// Offenen Dialog bestimmen
			var openModals = document.querySelectorAll('.ccm-root > .ccm-modal.ccm-show');
			var container = openModals[openModals.length-1];
			if (!container) {
				return;
			}
			// Fokussierbare Elemente sammeln
			var innerContainer = container.querySelector('.ccm-modal-inner');
			var focusable = Utils.focusableElements(container);

			// Wenn Shift+Tab und am Anfang des Dialogs, zum Seitenanfang springen
			// damit bei Keyup der Benutzer bei den Browser-Controls landet
			if (event.shiftKey && (event.target == focusable[0] || event.target == innerContainer)) {
				var newTarget = document.getElementById('ccm-focus-helper--top');
				this._ignoreFocused = newTarget;
				newTarget.focus({ preventScroll: true });
				this._ignoreFocused = null;
			}
			// Wenn Tab und am Ende des Dialogs, zum Seitenende springen
			// damit bei Keyup der Benutzer bei den Browser-Controls landet
			else if (!event.shiftKey && event.target == focusable[focusable.length-1]) {
				var newTarget = document.getElementById('ccm-focus-helper--bottom');
				this._ignoreFocused = newTarget;
				newTarget.focus({ preventScroll: true });
				this._ignoreFocused = null;
			}
		},
		/**
		 * Handler für Window-Resize-Events während ein Dialog offen ist
		 *
		 * @return {void}
		 */
		onWindowResize: function onWindowResize () {
			var self = this;
			requestAnimationFrame(function (ts) {
				var height = window.innerHeight;
				var width = window.innerWidth;
				if (
					onWindowResize.lastTs !== undefined && onWindowResize.lastTs >= ts
				) {
					return;
				}
				onWindowResize.lastTs = ts;
			});
		},
		/**
		 * This method invokes the API action that is referenced by the target.
		 *
		 * In this method, `this` refers not to the element linked to the event, but to
		 * the `CookieConsentManagement` instance. This is because `init` binds `this`.
		 * @param {Event} event
		 * @param {HTMLAnchorElement} target The real target is made available through the high priority event listener.
		 * @return {void}
		 */
		onJavaScriptApiLinkClicked: function (event, target) {
			try {
				var methodName = target.href.replace(/^.*#CCM\./, '');
				if (methodName in window.CCM && typeof window.CCM[methodName] == 'function') {
					window.CCM[methodName]();
					event.preventDefault();
				}
			} catch (e) {}
		},
		/**
		 * @param {Element} element
		 * @return {void}
		 */
		registerJavaScriptApiHandler: function (element) {
			if (element instanceof HTMLAnchorElement && element.hash.indexOf('#CCM.') == 0) {
				this.addHighPriorityEventListener('click', element, this.onJavaScriptApiLinkClicked);
			}
		},
		/**
		 * @return {void}
		 */
		registerJavaScriptApiHandlers: function () {
			var self = this;
			toArray(document.body.querySelectorAll('a[href^="#CCM."]')).forEach(function (element) {
				self.addHighPriorityEventListener('click', element, self.onJavaScriptApiLinkClicked);
			});
		},
		/**
		 * @param {?Event} event
		 */
		onSettingsIconClicked: function (event) {
			event = event || window.event;
			event.preventDefault();
			event.stopImmediatePropagation();

			this.hideSettingsIcon();

			switch (this.settingsIconTarget) {
				case 'main':
					this.openWidget();
					break;
				case 'purpose':
				default:
					this.openControlPanel();
					break;
			}
		},
		/**
		 * @return {void}
		 */
		showSettingsIcon: function () {
			if (this.settingsIconEnabled == false
				|| this.isActive == false
				|| this.widgetInitialized == false
				|| this.partOfAdAndTrackingFreeNetwork
				|| this.isModalActive()
			) {
				return;
			}

			// Resolve lazy-loaded resources
			toArray(this.container.querySelectorAll('.ccm-settings-summoner img[data-ccm-lazy-src]')).forEach(function (image) {
				var src = image.dataset.ccmLazySrc || '';
				if (src) {
					image.setAttribute('src', src);
					image.hidden = false;
				}
				image.removeAttribute('data-ccm-lazy-src');
			});

			toArray(this.container.querySelectorAll('.ccm-settings-summoner')).forEach(function (element) {
				element.classList.remove('ccm-hide');
				element.classList.add('ccm-show');
				element.hidden = false;
			});
		},
		/**
		 * @return {void}
		 */
		hideSettingsIcon: function () {
			if (this.settingsIconEnabled == false) {
				return;
			}

			toArray(this.container.querySelectorAll('.ccm-settings-summoner')).forEach(function (element) {
				element.classList.remove('ccm-show');
				element.classList.add('ccm-hide');
				element.hidden = true;
			});
		},
		/**
		 * @return {void}
		 */
		updateBlocking: function () {
			// May occur when called from init (opt-out)
			if (!Utils.isDocumentReady()) {
				return;
			}

			var activeModals = this.getActiveModals();
			var isBlocking = activeModals.reduce(function (isBlocking, modal) {
				return isBlocking || modal.classList.contains('ccm--is-blocking');
			}, false);
			var self = this;

			Utils.toggleClass(document.documentElement, 'ccm-blocked', isBlocking);
			Utils.toggleClass(document.body, 'ccm-blocked', isBlocking);

			// Alles außer dem aktuellen Dialog auf 'inert' setzen
			if (isBlocking) {
				toArray(document.body.children).forEach(function (el) {
					if (el.id.indexOf('ccm-focus-helper') !== 0 && !el.classList.contains('ccm-root')) {
						if (el.inert === false) {
							self.inertedElements.push(el);
							el.inert = true;
						}
					}
				});
			} else {
				this.inertedElements.forEach(function (el) {
					el.inert = false;
				});
				this.inertedElements = [];
			}
			if (activeModals.length) {
				activeModals.slice(0, -1).forEach(function (el) { el.inert = isBlocking; });
				activeModals[activeModals.length - 1].inert = false;
			}

			activeModals.length > 0 ? this.hideSettingsIcon() : this.showSettingsIcon();

			// Wenn Blocking-Dialog, Fokus-Handler setzen, falls noch nicht passiert
			if (document.body.classList.contains('ccm-blocked')) {
				this.captureFocus();
			}
			// Wenn kein Blocking-Dialog mehr offen, Fokus-Handler entfernen, falls vorhanden
			else {
				this.endCaptureFocus();
			}
		},
		/**
		 * @return {boolean}
		 */
		isModalActive: function () {
			return this.getActiveModals().length > 0;
		},
		/**
		 * @return {Array.<Element>}
		 */
		getActiveModals: function () {
			return toArray(this.container.getElementsByClassName('ccm-modal')).filter(function (modal) {
				return modal.classList.contains('ccm-show');
			});
		},
		/**
		 * @param {Element} target
		 * @param {boolean=} [updateBlocking=true]
		 * @param {?number=} [focusIndex=undefined]
		 * @return {void}
		 */
		showModal: function (target, updateBlocking, focusIndex) {
			if (target == null || target.classList.contains('ccm-modal') == false) {
				return;
			}

			this.focusStack.push(window.event ? window.event.currentTarget : window.activeElement);

			toArray(target.querySelectorAll('div.ccm--h1')).forEach(function (el) {
				el.setAttribute('role', 'heading');
				el.setAttribute('aria-level', '1');
			});
			toArray(target.querySelectorAll('div.ccm--h2')).forEach(function (el) {
				el.setAttribute('role', 'heading');
				el.setAttribute('aria-level', '2');
			});
			toArray(target.querySelectorAll('.ccm--h3')).forEach(function (el) {
				el.setAttribute('role', 'heading');
				el.setAttribute('aria-level', '3');
			});

			updateBlocking = typeof updateBlocking == 'boolean' ? updateBlocking : true;

			if (target.classList.contains('ccm-details')) {
				var ucidShort = this.getUniqueCookieId().substr(0, 16);

				toArray(target.getElementsByClassName('ccm-user-info--ucid--value')).forEach(function (element) {
					element.textContent = ucidShort;
				});
			}

			// Event beim Öffnen des ersten Modals
			if (this.isModalActive() == false) {
				Utils.triggerCustomEvent(window, 'ccm19WidgetOpened');

				// Resizing-Event abfangen
				window.addEventListener('resize', this.onWindowResize);

				// Banner-Einblendung festhalten
				this.logOpenData();
			}

			// Ggf. Buttons bzw. Button-Texte ändern, wenn Dialog mit bestehendem Consent geöffnet wird
			if (this.consentGiven()) {
				toArray(target.querySelectorAll('button[data-ccm-reopen-text]')).forEach(function (el) {
					var text = el.innerText;
					var newText = el.getAttribute('data-ccm-reopen-text');
					if (newText) {
						el.innerText = el.getAttribute('data-ccm-reopen-text');
						el.removeAttribute('data-ccm-reopen-text');
						el.setAttribute('data-ccm-original-text', text);
					}
				});
				toArray(target.querySelectorAll('.ccm--reopen-only')).forEach(function (el) {
					el.classList.remove('ccm--reopen-only');
				});
			}

			target.classList.add('ccm-show');
			target.removeAttribute('hidden');
			target.style.display = '';
			Utils.enableButtons(target);

			if (updateBlocking) {
				this.updateBlocking();
			}

			// Mindestens 1x pro Dialog-Öffnung onWindowResize ausführen
			this.onWindowResize();

			// Tastatur-Fokus setzen
			if (focusIndex !== -1) {
				var hashMatch = location.hash.match(/^#[a-z][a-z0-9-]*$/);
				var btns = target.getElementsByTagName('button');
				var primaryElement = (
					(hashMatch ? target.querySelector(hashMatch) : null)
					|| (focusIndex !== null && focusIndex !== undefined ? Utils.focusableElements(target)[focusIndex] : null)
					|| target.querySelector('[role="document"][tabindex]')
					|| (btns.length === 1 ? btns[0] : null)
				);
				if (!primaryElement) {
					primaryElement = target;
				}
				if (primaryElement.tabIndex == -1 && primaryElement.children.length > 0) {
					//primaryElement = primaryElement.children[0];
				}
				if (primaryElement.tabIndex == -1) {
					primaryElement.tabIndex = 0;
					primaryElement.focus({ preventScroll: true });
					primaryElement.tabIndex = -1;
				} else {
					primaryElement.focus({ preventScroll: true });
				}
			}
		},
		/**
		 *
		 * @param {String} id
		 * @param {boolean} [show=true] Show modal after it's been loaded.
		 * @param {String=} jumpTo Relevant if `show` equals `true` only.
		 */
		loadModal: function (id, show, jumpTo) {
			show = typeof show == 'boolean' ? show : true;

			var target = document.getElementById(id);
			if (target) {
				if (show) {
					this.showModal(target);
					if (jumpTo) {
						window.requestAnimationFrame(function () {
							var el = document.getElementById(jumpTo);
							el && el.scrollIntoView && el.scrollIntoView();
						});
					}
				}
				return;
			}

			var url = {
				'ccm-details': this.detailsUrls[this.locale],
			}[id];

			if (!url) {
				if (CCM_DEBUG) {
					console.warn('[CCM19] URL not available to load modal "' + id + '".');
				}
				return;
			}

			if (this.loadingModal) {
				return;
			}

			this.loadingModal = true;
			document.documentElement.classList.add('ccm--is-loading');

			this.ajax(url, {
				method: 'GET',
				success: function (response) {
					var temp = document.createElement('DIV');
					temp.innerHTML = response;

					while (temp.children.length > 0) {
						this.container.appendChild(temp.removeChild(temp.firstElementChild));
					}

					this.registerControls();
					this.updateConsentState();

					var target = document.getElementById(id);
					if (target && show) {
						this.showModal(target);

						if (jumpTo) {
							this.jumpTo(jumpTo)
						}
					}
				},
				failure: function (code) {
					console.warn('[CCM19] Error while loading modal "' + id + '". (' + code + ')');
				},
				done: function () {
					this.loadingModal = false;
					document.documentElement.classList.remove('ccm--is-loading');
				},
			});
		},
		/**
		 * @param {Element} target
		 * @param {boolean=} updateBlocking
		 * @return {void}
		 */
		hideModal: function (target, updateBlocking) {
			if (target == null || target.classList.contains('ccm-modal') == false) {
				return;
			}

			updateBlocking = typeof updateBlocking == 'boolean' ? updateBlocking : true;

			var wasOpen = target.classList.contains('ccm-show');
			target.classList.remove('ccm-show');
			target.setAttribute('hidden', '');
			target.style.display = 'none';
			Utils.disableButtons(target);

			// Event beim Schließen des letzten Modals
			if (wasOpen && this.isModalActive() == false) {
				// Resizing-Event nicht mehr benötigt
				window.removeEventListener('resize', this.onWindowResize);

				this.callAsSoonAsSavingSettingsIsDone(function () {
					Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
				});
			}

			toArray(target.querySelectorAll('div.ccm--h1,div.ccm--h2,div.ccm--h3')).forEach(function (el) {
				el.removeAttribute('role');
				el.removeAttribute('aria-level');
			});

			if (updateBlocking) {
				this.updateBlocking();
			}

			try {
				if (this.focusStack.length) {
					var el = this.focusStack.pop();
					var activeEl = window.activeElement;
					if (el && 'tabIndex' in el) {
						el.focus({ preventScroll: true });
					} else if (this.focusStack.length == 0 && activeEl && this.container.contains(activeEl)) {
						var focusHelper = document.createElement('div');
						focusHelper.style.position = 'fixed';
						focusHelper.style.left = '0';
						focusHelper.style.right = '0';
						focusHelper.style.height = '0';
						focusHelper.style.maxHeight = '0';
						focusHelper.background = 'transparent';
						focusHelper.border = '0 none transparent';
						focusHelper.tabIndex = 0;
						focusHelper.style.top = '0';
						focusHelper.id = 'ccm-focus-helper';
						focusHelper.ariaHidden = true;
						document.body.insertBefore(focusHelper, document.body.firstChild);
						focusHelper.focus({ preventScroll: true });
						requestAnimationFrame(function () {
							focusHelper.blur();
							document.body.removeChild(focusHelper);
						});
					}
				}
			} catch (e) {
				// ignore
			}
		},
		/**
		 * @return {void}
		 */
		closeAllModals: function () {
			var self = this;

			toArray(this.container.getElementsByClassName('ccm-modal')).forEach(function (modal) {
				self.hideModal.call(self, modal, false);
			});

			this.updateBlocking();
		},
		/**
		 * @param {String} embeddingId
		 */
		showEmbeddingDetails: function (embeddingId) {
			this.openDetailsWindow('ccm-cookie-details-' + embeddingId);
		},
		/**
		 * @param {String} purposeId
		 * @return {void}
		 */
		showPurposeDetails: function (purposeId) {
			this.openDetailsWindow('ccm-purpose-details-' + purposeId);
		},
		/**
		 * @param {String=} jumpTo
		 * @return {void}
		 */
		openDetailsWindow: function (jumpTo) {
			this.loadModal('ccm-details', true, jumpTo);
		},
		/**
		 * @return {void}
		 */
		closeDetailsWindow: function () {
			var modal = this.container.querySelector('.ccm-modal.ccm-details.ccm-show');
			modal && this.hideModal(modal);
		},
		/**
		 * @return {void}
		 */
		openControlPanel: function () {
			this.showModal(this.controlPanel);
		},
		/**
		 * @return {void}
		 */
		closeControlPanel: function () {
			this.hideModal(this.controlPanel);
		},
		/**
		 * @param {?number=} [focusIndex=null]
		 * @return {void}
		 */
		openWidget: function (focusIndex) {
			//nur wenn aktiv gesetzt
			if (this.isActive) {
				this.showModal(this.widget, true, focusIndex);
			}
		},
		/**
		 * @return {void}
		 */
		closeWidget: function () {
			this.hideModal(this.widget);
		},
		/**
		 * @param {?Event} event
		 */
		onModalCloseButtonClicked: function (event) {
			event = event || window.event;
			var target = event.target;

			while (target !== null && target.classList.contains('ccm-modal') == false) {
				target = target.parentElement;
			}

			this.hideModal(target);
		},
		/**
		 * @param {string} locale
		 * @return {void}
		 */
		switchLocale: function (locale) {
			if (locale in this.widgetUrls) {
				var self = this;
				this.locale = locale;
				this.widgetUrl = this.widgetUrls[locale];
				if (this.widget) {
					this.widget.classList.add('ccm-widget--loading');
				}
				var openWidget = this.isModalActive();

				// Bei offenem Dialog: Bestimmen, welches Element aktuell den Fokus hat
				var focusIndex = null;
				if (document.activeElement && openWidget) {
					var candidates = Utils.focusableElements(this.widget);
					var focusEl = document.activeElement;
					for (var i = 0; i < candidates.length; ++i) {
						if (candidates[i] === focusEl) {
							focusIndex = i;
							break;
						}
					}
				}

				// Dialog neu bauen und ggf. mit korrektem Fokus neu öffnen
				this.build(function () {
					if (openWidget) {
						self.openWidget(focusIndex);
					}
				});
			}
			else {
				console.error('[CCM19] Could not switch to unknown locale "'+locale+'"');
			}
		},
		/**
		 * This method reads the `lang` attribute of the `html` element and tries
		 * to find and enable a locale that fits.
		 * @return {void}
		 */
		setLocaleUsingLangAttribute: function () {
			var lang = String(document.documentElement.lang).toLowerCase();

			var variants = {};
			Object.keys(this.widgetUrls).forEach(function (localeName) {
				var variantName = localeName.replace(/_/g, '-').toLowerCase();
				variants[variantName] = localeName;
				variants[variantName.split('-')[0]] = localeName;
			});

			var locale = variants[lang] || variants[lang.split('-')[0]] || null;

			if (locale) {
				this.locale = locale;
				this.widgetUrl = this.widgetUrls[locale];
			}
		},
		/**
		 * This method reads the `languages` navigator property and tries
		 * to find and enable a locale that fits.
		 * @return {void}
		 */
		setLocaleFromBrowserPreference: function () {
			var languages = ('languages' in navigator) ? navigator.languages : [(navigator.userLanguage || navigator.language), navigator.language];

			var variants = {};
			Object.keys(this.widgetUrls).forEach(function (localeName) {
				var variantName = localeName.replace(/_/g, '-').toLowerCase();
				var shortVariantName = variantName.split('-')[0]
				variants[variantName] = localeName;
				if (!(shortVariantName in variants)) {
					variants[shortVariantName] = localeName;
				}
			});

			var locale = null;
			for (var i = 0; i < languages.length; ++i) {
				var lang = languages[i];
				locale = (variants[lang] || variants[lang.split('-')[0]] || null);
				if (locale !== null) {
					break;
				}
			}

			if (locale) {
				this.locale = locale;
				this.widgetUrl = this.widgetUrls[locale];
			}
		},
		/**
		 * @param {string} name
		 * @return {string}
		 */
		getLocalizedString: function (name) {
			return this.l10n[this.locale][name] || '';
		},
		/**
		 * @param {boolean} [logSettings=true]
		 * @return {void}
		 */
		dismissTemporarily: function (logSettings) {
			logSettings = typeof logSettings == 'boolean' ? logSettings : true;

			if (!this.consentGiven()) {
				var embeddingIds = [];
				var tcf = {};
				this.saveSettingsInternal(this.getUniqueCookieId(), true, embeddingIds, tcf, 'temporary');

				if (logSettings) {
					this.clickedButton = "decline";
					this.logConsentSettings();
				}

			}
			this.closeAllModals();
		},
		/**
		 * @param {boolean} [hasConsent=true]
		 * @return {Array.<EmbeddingInfo>}
		 */
		getEmbeddingsWithConsent: function (hasConsent) {
			hasConsent = typeof hasConsent == 'boolean' ? hasConsent : true;
			return this.repository.filter(function (embedding) {
				return embedding.consent == hasConsent;
			});
		},
		getPurposeCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-purpose]')).filter(function (item) {
				// Workaround for an IE bug when using ":not([data-embedding])" inside the selector
				return !item.dataset.embedding;
			});
		},
		getEmbeddingCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-embedding]'));
		},
		/**
		 * @return {void}
		 */
		selectAllPurposes: function () {
			this.setAllPurposes(true);
		},
		selectPurpose: function (purposeId, newState) {
			newState = newState == 'partial' ? newState : (typeof newState == 'boolean' ? newState : true);
			var purposes = this.getPurposeCheckboxes();

			// Update every purpose checkbox
			purposes.forEach(function (purpose) {
				if (purpose.dataset.purpose == purposeId) {
					purpose.checked = !!newState;
					purpose.indeterminate = newState == 'partial';
				}
			});

			// Update embeddings if purpose has been either selected or deselected completely
			if (typeof newState == 'boolean') {
				this.getEmbeddingCheckboxes().forEach(function (embedding) {
					if (embedding.dataset.purpose == purposeId) {
						embedding.checked = newState;
						embedding.indeterminate = false;
					}
				});
			}
		},
		selectEmbedding: function (embedding) {
			// Update relevant embedding checkboxes
			this.getEmbeddingCheckboxes().forEach(function (checkbox) {
				if (checkbox.dataset.embedding == embedding.id) {
					checkbox.checked = embedding.consent;
				}
			});

			// Determine new state for linked purpose
			var newPurposeState = this.repository.reduce(function (newState, currentEmbedding) {
				if (currentEmbedding.purpose == embedding.purpose) {
					if (newState === true) {
						newState = currentEmbedding.consent || 'partial';
					}
					else {
						newState = currentEmbedding.consent ? 'partial' : newState;
					}
				}

				return newState;
			}, embedding.consent);

			// Update purpose
			this.selectPurpose(embedding.purpose, newPurposeState);
		},
		/**
		 * @return {Array}
		 */
		selectedPurposes: function () {
			return Utils.uniqueValues(this.getEmbeddingsWithConsent().map(function (embedding) {
				return embedding.purpose;
			}));
		},
		/**
		 * Finds all purposes and those with legitimate interest.
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfPurposesCategoryCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="tcf-p-collection-"]'));
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfPurposesCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="tcf-p--"]'));
		},
		selectAllTcfPurposes: function () {
			for (var id in this.consentState.tcfPurposes) {
				this.consentState.tcfPurposes[id] = true;
			}
			this.getTcfPurposesCheckboxes().forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
			this.updateTcfPurposeGroups();
		},
		resetAllTcfPurposes: function () {
			for (var id in this.consentState.tcfPurposes) {
				this.consentState.tcfPurposes[id] = false;
			}
			this.getTcfPurposesCheckboxes().forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
			this.updateTcfPurposeGroups();
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedTcfPurposes: function () {
			var self = this;
			return Object.keys(this.consentState.tcfPurposes)
				.filter(function (id) {
					return self.consentState.tcfPurposes[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetTcfPurposes: function () {
			var tcf = this.getTcfConsentInfo();

			this.resetAllTcfPurposes();

			var self = this;
			tcf.p.forEach(function (purposeId) {
				self.consentState.tcfPurposes[purposeId] = true;
			});

			this.updateTcfPurposes();
		},
		updateTcfPurposes: function () {
			var self = this;

			var checkboxes = this.getTcfPurposesCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var purposeId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.tcfPurposes[purposeId];
			});

			this.updateTcfPurposeGroups();
		},
		updateTcfPurposeGroups: function () {
			var self = this;

			this.getTcfPurposesCategoryCheckboxes().forEach(function (checkbox) {
				var purposeId = parseInt(checkbox.value);

				var states = [];
				if (purposeId in self.consentState.tcfPurposes) {
					states.push(!!self.consentState.tcfPurposes[purposeId]);
				}
				if (purposeId in self.consentState.tcfLegIntPurposes) {
					states.push(!!self.consentState.tcfLegIntPurposes[purposeId]);
				}

				var newGroupState = Utils.partial(states);

				checkbox.checked = !!newGroupState;
				checkbox.indeterminate = newGroupState == 'partial';
			});
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfLegIntPurposesCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="tcf-li-p--"]'));
		},
		selectAllTcfLegIntPurposes: function () {
			for (var id in this.consentState.tcfLegIntPurposes) {
				this.consentState.tcfLegIntPurposes[id] = true;
			}
			this.getTcfLegIntPurposesCheckboxes().forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
			this.updateTcfPurposeGroups();
		},
		resetAllTcfLegIntPurposes: function () {
			for (var id in this.consentState.tcfLegIntPurposes) {
				this.consentState.tcfLegIntPurposes[id] = false;
			}
			this.getTcfLegIntPurposesCheckboxes().forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
			this.updateTcfPurposeGroups();
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedTcfLegIntPurposes: function () {
			var self = this;
			return Object.keys(this.consentState.tcfLegIntPurposes)
				.filter(function (id) {
					return self.consentState.tcfLegIntPurposes[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetTcfLegIntPurposes: function () {
			var tcf = this.getTcfConsentInfo();

			this.resetAllTcfLegIntPurposes();

			var self = this;
			tcf.lip.forEach(function (purposeId) {
				self.consentState.tcfLegIntPurposes[purposeId] = true;
			});

			this.updateTcfLegIntPurposes();
		},
		updateTcfLegIntPurposes: function () {
			var self = this;

			var checkboxes = this.getTcfLegIntPurposesCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var purposeId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.tcfLegIntPurposes[purposeId];
			});

			// There is no analogous method for legitimate interest purposes
			this.updateTcfPurposeGroups();
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfSpecialFeatureCategoryCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group="tcf-sf"]'));
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfSpecialFeatureCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group-parent="tcf-sf"]'));
		},
		selectAllTcfSpecialFeatures: function () {
			for (var id in this.consentState.tcfSpecialFeatures) {
				this.consentState.tcfSpecialFeatures[id] = true;
			}
			this.getTcfSpecialFeatureCheckboxes().concat(this.getTcfSpecialFeatureCategoryCheckboxes()).forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
		},
		resetAllTcfSpecialFeatures: function () {
			for (var id in this.consentState.tcfSpecialFeatures) {
				this.consentState.tcfSpecialFeatures[id] = false;
			}
			this.getTcfSpecialFeatureCheckboxes().concat(this.getTcfSpecialFeatureCategoryCheckboxes()).forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedTcfSpecialFeatures: function () {
			var self = this;
			return Object.keys(this.consentState.tcfSpecialFeatures)
				.filter(function (id) {
					return self.consentState.tcfSpecialFeatures[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetTcfSpecialFeatures: function () {
			var tcf = this.getTcfConsentInfo();

			this.resetAllTcfSpecialFeatures();

			var self = this;
			tcf.sf.forEach(function (featureId) {
				self.consentState.tcfSpecialFeatures[featureId] = true;
			});

			this.updateTcfSpecialFeatures();
		},
		updateTcfSpecialFeatures: function () {
			var self = this;

			var checkboxes = this.getTcfSpecialFeatureCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var featureId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.tcfSpecialFeatures[featureId];
			});

			var newGroupState = Utils.partial(Utils.objectValues(this.consentState.tcfSpecialFeatures));
			this.getTcfSpecialFeatureCategoryCheckboxes().forEach(function (checkbox) {
				checkbox.checked = !!newGroupState;
				checkbox.indeterminate = newGroupState == 'partial';
			});
		},
		/**
		 * Finds all "third party" vendor switches.
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfVendorThirdPartyCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group="tcf-third-party"]'));
		},
		/**
		 * Finds all "consent" vendor switches.
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfVendorCategoryCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group="tcf-gvl"]'));
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfVendorCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="tcf-gvl--"]'));
		},
		selectAllTcfVendors: function () {
			for (var id in this.consentState.tcfVendors) {
				this.consentState.tcfVendors[id] = true;
			}
			this.getTcfVendorCheckboxes().forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
			this.updateTcfVendorGroup();
		},
		resetAllTcfVendors: function () {
			for (var id in this.consentState.tcfVendors) {
				this.consentState.tcfVendors[id] = false;
			}
			this.getTcfVendorCheckboxes().forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
			this.updateTcfVendorGroup();
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedTcfVendors: function () {
			var self = this;
			return Object.keys(this.consentState.tcfVendors)
				.filter(function (id) {
					return self.consentState.tcfVendors[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetTcfVendors: function () {
			var tcf = this.getTcfConsentInfo();

			this.resetAllTcfVendors();

			var self = this;
			tcf.v.forEach(function (vendorId) {
				self.consentState.tcfVendors[vendorId] = true;
			});

			this.updateTcfVendors();
		},
		updateTcfVendors: function () {
			var self = this;

			var checkboxes = this.getTcfVendorCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var vendorId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.tcfVendors[vendorId];
			});

			this.updateTcfVendorGroup();
		},
		/**
		 * Updates "third party", "consent" and "legitimate interest" switches.
		 * @return {void}
		 */
		updateTcfVendorGroup: function () {
			if (this.tcfData.legIntEnabled) {
				var visibleVendorStates = this.consentState.tcfVendors;
			}
			else {
				// If legitimate interest is disabled, there are no switches for TCF vendors for embeddings.
				// Disregard these vendors to complete the "checked" condition for the parent switches.
				var embeddingTcfVendorIds = Array.prototype.concat.apply([],
					this.repository.map(function (embedding) { return Object.keys(embedding.tcf || {}); })
				);

				visibleVendorStates = Utils.filterObject(this.consentState.tcfVendors, function (state, id) {
					return embeddingTcfVendorIds.includes(id) == false;
				});
			}

			// Update "third party" switches
			var newGroupState = Utils.partial(Utils.objectValues(visibleVendorStates)
				.concat(Utils.objectValues(this.consentState.tcfLegIntVendors))
				.concat(Utils.objectValues(this.consentState.googleAdTechProviders))
			);
			this.getTcfVendorThirdPartyCheckboxes().forEach(function (checkbox) {
				checkbox.checked = !!newGroupState;
				checkbox.indeterminate = newGroupState == 'partial';
			});

			// Update "consent" switches
			newGroupState = Utils.partial(Utils.objectValues(visibleVendorStates)
				.concat(Utils.objectValues(this.consentState.googleAdTechProviders))
			);
			this.getTcfVendorCategoryCheckboxes().forEach(function (checkbox) {
				checkbox.checked = !!newGroupState;
				checkbox.indeterminate = newGroupState == 'partial';
			});

			// Update "legitimate interest" switches
			newGroupState = Utils.partial(Utils.objectValues(this.consentState.tcfLegIntVendors));
			this.getTcfLegIntVendorCategoryCheckboxes().forEach(function (checkbox) {
				checkbox.checked = !!newGroupState;
				checkbox.indeterminate = newGroupState == 'partial';
			});
		},
		/**
		 * Finds all "legitimate interest" vendor switches.
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfLegIntVendorCategoryCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group="tcf-li-gvl"]'));
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getTcfLegIntVendorCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="tcf-li-gvl--"]'));
		},
		selectAllTcfLegIntVendors: function () {
			for (var id in this.consentState.tcfLegIntVendors) {
				this.consentState.tcfLegIntVendors[id] = true;
			}
			this.getTcfLegIntVendorCheckboxes().forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
			this.updateTcfVendorGroup();
		},
		resetAllTcfLegIntVendors: function () {
			for (var id in this.consentState.tcfLegIntVendors) {
				this.consentState.tcfLegIntVendors[id] = false;
			}
			this.getTcfLegIntVendorCheckboxes().forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
			this.updateTcfVendorGroup();
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedTcfLegIntVendors: function () {
			var self = this;
			return Object.keys(this.consentState.tcfLegIntVendors)
				.filter(function (id) {
					return self.consentState.tcfLegIntVendors[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetTcfLegIntVendors: function () {
			var tcf = this.getTcfConsentInfo();

			this.resetAllTcfLegIntVendors();

			var self = this;
			tcf.liv.forEach(function (vendorId) {
				self.consentState.tcfLegIntVendors[vendorId] = true;
			});

			this.updateTcfLegIntVendors();
		},
		updateTcfLegIntVendors: function () {
			var self = this;

			var checkboxes = this.getTcfLegIntVendorCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var vendorId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.tcfLegIntVendors[vendorId];
			});

			// There is no analogous method for legitimate interest vendors
			this.updateTcfVendorGroup();
		},
		/**
		 * @return {Array.<HTMLInputElement>}
		 */
		getGoogleProviderCheckboxes: function () {
			return toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group^="gad--"]'));
		},
		selectAllGoogleProviders: function () {
			for (var id in this.consentState.googleAdTechProviders) {
				this.consentState.googleAdTechProviders[id] = true;
			}
			this.getGoogleProviderCheckboxes().forEach(function (checkbox) {
				checkbox.checked = true;
				checkbox.indeterminate = false;
			});
		},
		resetAllGoogleProviders: function () {
			for (var id in this.consentState.googleAdTechProviders) {
				this.consentState.googleAdTechProviders[id] = false;
			}
			this.getGoogleProviderCheckboxes().forEach(function (checkbox) {
				checkbox.checked = false;
				checkbox.indeterminate = false;
			});
		},
		/**
		 * @return {Array.<number>}
		 */
		selectedGoogleProviders: function () {
			var self = this;
			return Object.keys(this.consentState.googleAdTechProviders)
				.filter(function (id) {
					return self.consentState.googleAdTechProviders[id];
				})
				.map(function (id) { return parseInt(id); })
				.sort(function (a, b) { return a - b; })
			;
		},
		presetGoogleProviders: function () {
			var tcf = this.getTcfConsentInfo();
			var gad = ('gad' in tcf && tcf.gad) ? tcf.gad : [];

			this.resetAllGoogleProviders();

			var self = this;
			gad.forEach(function (providerId) {
				self.consentState.googleAdTechProviders[providerId] = true;
			});

			this.updateGoogleProviders();
		},
		updateGoogleProviders: function () {
			var self = this;

			var checkboxes = this.getGoogleProviderCheckboxes();
			checkboxes.forEach(function (checkbox) {
				var providerId = parseInt(checkbox.value);
				checkbox.checked = !!self.consentState.googleAdTechProviders[providerId];
			});

			// There is no analogous method for additional providers
			this.updateTcfVendorGroup();
		},
		/**
		 * @param {Array.<EmbeddingInfo>} allowedEmbeddings
		 * @return {GcmInfo}
		 */
		gatherGcmInfo: function (allowedEmbeddings) {
			var consentModeUsed = false;
			var consentMode = {};
			var eventData = {
				initialConsent: !this.consentGiven(),
			};

			this.repository.forEach(function (embedding) {
				var selected = allowedEmbeddings.reduce(function (selected, allowedEmbedding) {
					return selected || embedding.id === allowedEmbedding.id;
				}, false);

				eventData['ccm19_' + embedding.name] = selected;

				if (embedding.gcm) {
					consentModeUsed = true;
					for (var i = 0; i < embedding.gcm.length; i++) {
						if (selected) {
							consentMode[embedding.gcm[i]] = 'granted';
						}
						else if (consentMode[embedding.gcm[i]] === undefined) {
							consentMode[embedding.gcm[i]] = 'denied';
						}
					}
				}
			});

			return {
				consentModeUsed: consentModeUsed,
				consentMode: consentMode,
				eventData: eventData,
			};
		},
		/**
		 * @param {boolean} [limitToTechnicallyNecessary=false]
		 * @return {void}
		 */
		loadScripts: function (limitToTechnicallyNecessary) {
			limitToTechnicallyNecessary = typeof limitToTechnicallyNecessary == 'boolean' ? limitToTechnicallyNecessary : false;

			// Nichts tun, wenn technisch notwendige Einbindungen voriger Zustimmung bedürfen
			if (limitToTechnicallyNecessary && this.runMandatoryEmbeddingsWithoutConsent == false) {
				return;
			}

			var self = this;
			var selectedEmbeddings = this.getEmbeddingsWithConsent();

			// Filter out optional embeddings if necessary
			if (limitToTechnicallyNecessary) {
				selectedEmbeddings = selectedEmbeddings.filter(function (embedding) {
					return embedding.mandatory;
				});
			}

			// Update GCM parameters before loading the selected embeddings
			var gcmInfo = this.gatherGcmInfo(selectedEmbeddings);
			if (gcmInfo.consentModeUsed) {
				Utils.sendGTag('consent', 'update', gcmInfo.consentMode);
			}

			// Skripte anhand ausgewählter Embeddings nachladen
			selectedEmbeddings.forEach(function (embedding) {
				self.embedEmbedding(embedding);
			});

			// Process repeatedly toggled embeddings without reloading the website
			this.repository
				.filter(function (embedding) { return self.embeddingsEmbedded.includes(embedding.id); })
				.forEach(function (embedding) {
					self.blockOrRestoreIframesForEmbedding(embedding);
				});

			// Keine Events bei erzwungener Einbindung technisch notwendiger Ressourcen
			if (limitToTechnicallyNecessary) {
				return;
			}

			// Gebündeltes Event über den Zustand aller Embeddings an GTM senden
			Utils.triggerGTMEvent('CCM19.consentStateChanged', gcmInfo.eventData);

			this.cleanUpClientStorage(true);

			this.showSettingsIcon();
		},
		/**
		 * @return {string}
		 */
		getUniqueCookieId: function () {
			var storage = this.getConsentStorage();
			return storage.ucid || this.ucid;
		},
		/**
		 * @return {string}
		 */
		getConsentLanguage: function () {
			var storage = this.getConsentStorage();
			return storage.lang || '';
		},
		/**
		 * @param {Array.<number>} ranges [Start-ID-1, Ende-ID-1, Start-ID-2, …]
		 * @return {Array.<number>} ID-Liste
		 */
		expandIdRanges: function (ranges) {
			var result = [];
			var l = ranges.length;
			for (var i = 1; i < l; i+=2) {
				var start = ranges[i-1];
				var end = ranges[i];
				for (var j = start; j <= end; ++j) {
					result.push(j);
				}
			}
			return result;
		},
		/**
		 * @param {Array.<number>} ids Sortierte ID-Liste
		 * @return {Array.<number>} [Start-ID-1, Ende-ID-1, Start-ID-2, …]
		 */
		makeIdRanges: function (ids) {
			if (ids.length == 0) {
				return [];
			}
			var result = [];
			var l = ids.length;

			var currentStart = ids[0];
			var current = ids[0];
			for (var i = 1; i <= l; ++i) {
				var id = ids[i];
				if (id == current+1) {
					current = id;
				} else {
					result.push(currentStart);
					result.push(current);
					currentStart = current = id;
				}
			}
			result.push(currentStart);
			result.push(current);
			return result;
		},
		/**
		 * @return {string}
		 */
		getTcfConsentInfo: function () {
			var storage = this.getConsentStorage();
			var selectedLegIntPurposes = Object.keys(this.consentState.tcfLegIntPurposes).map(function(x){return parseInt(x);});
			var selectedLegIntVendors = Object.keys(this.consentState.tcfLegIntVendors).map(function(x){return parseInt(x);});

			var tcf = storage.tcf || {};
			tcf.p = Array.isArray(tcf.p) ? tcf.p : [];
			tcf.lip = Array.isArray(tcf.lip) ? tcf.lip : selectedLegIntPurposes;
			tcf.sf = Array.isArray(tcf.sf) ? tcf.sf : [];
			if (Array.isArray(tcf.vr)) {
				tcf.v = this.expandIdRanges(tcf.vr);
			} else {
				tcf.v = Array.isArray(tcf.v) ? tcf.v : [];
			}
			if (Array.isArray(tcf.livr)) {
				tcf.liv = this.expandIdRanges(tcf.livr);
			} else {
				tcf.liv = Array.isArray(tcf.liv) ? tcf.liv : selectedLegIntVendors;
			}
			tcf.gad = Array.isArray(tcf.gad) ? tcf.gad : [];
			tcf.created = 'created' in storage ? storage.created : null;
			tcf.updated = 'updated' in storage ? storage.updated : null;

			// Disregard consent for resources that are irrelevant to the current configuration
			tcf.gad = Utils.intersect(tcf.gad, Object.keys(this.consentState.googleAdTechProviders).map(function(x){return parseInt(x);}));
			tcf.p = Utils.intersect(tcf.p, Object.keys(this.consentState.tcfPurposes).map(function(x){return parseInt(x);}));
			tcf.lip = Utils.intersect(tcf.lip, selectedLegIntPurposes);
			tcf.sf = Utils.intersect(tcf.sf, Object.keys(this.consentState.tcfSpecialFeatures).map(function(x){return parseInt(x);}));
			tcf.v = Utils.intersect(tcf.v, Object.keys(this.consentState.tcfVendors).map(function(x){return parseInt(x);}));
			tcf.liv = Utils.intersect(tcf.liv, selectedLegIntVendors);

			return tcf;
		},
		/**
		 * This method calls the provided function as soon as the consent settings
		 * have been saved. If, currently, no consent settings are being saved, the
		 * function is called immediately.
		 * @param {Function} [callback] The function to call, or empty to just process the queue.
		 */
		callAsSoonAsSavingSettingsIsDone: function (callback) {
			// If a callback function is provided, add it to the queue
			if (typeof callback == 'function') {
				this.saveSettingsFunctionQueue.push(callback);
			}

			// If settings are currently being saved, wait for the next frame
			if (this.saving) {
				window.requestAnimationFrame(this.callAsSoonAsSavingSettingsIsDone.bind(this));
				return;
			}

			// Otherwise, execute all queued functions
			this.saveSettingsFunctionQueue.forEach(function (callback) {
				callback();
			});
			this.saveSettingsFunctionQueue = [];
		},
		/**
		 * This method calls the provided function as soon as `__tcfapi` is available
		 * or if TCF is disabled. It will wait for TCF to be available and build a
		 * queue if it is called multiple times to maintain the proper order of calls.
		 * @param {Function} callback
		 */
		callAsSoonAsTcfApiIsAvailableOrIfDisabled: function (callback) {
			if (typeof callback == 'function') {
				this.tcfDependantFunctionQueue.push(callback);
			}

			function processQueue() {
				this.tcfDependantFunctionQueue.forEach(function (callback) {
					callback();
				});
				this.tcfDependantFunctionQueue = [];
			}

			if ('__tcfapi' in window) {
				window.__tcfapi('getTCData', 2, processQueue.bind(this));
			}
			else {
				processQueue.call(this);
			}
		},
		/**
		 * @return {void}
		 */
		logOpenData: function () {
			this.ajax(this.statisticsUrl, {
				method: 'POST',
				contentType: 'application/json',
				body: {
					ucid: this.getUniqueCookieId(),
					lang: this.locale,
					clientUserAgent: navigator.userAgent,
					clientOs: navigator.platform,
					clientLang: navigator.language,
					actualUrl: window.location.href,
					actualRef: document.referrer,
					actualOpened: 1
				}
			});
		},
		/**
		 * @return {void}
		 */
		logConsentSettings: function () {
			var hasConsent = this.consentGiven();
			var storage = hasConsent ? this.getConsentStorage() : {};
			var tcf = this.getTcfConsentInfo();
			var self = this;
			tcf.created = tcf.created ? Math.round(tcf.created / 10) : null;
			tcf.updated = tcf.updated ? Math.round(tcf.updated / 10) : null;

			this.ajax(this.statisticsUrl, {
				method: 'POST',
				contentType: 'application/json',
				body: {
					ucid: this.getUniqueCookieId(),
					consent: hasConsent,
					clickedButton: this.clickedButton,
					purposes: this.selectedPurposes(),
					embeddings: this.getEmbeddingsWithConsent().map(function (embedding) { return embedding.id; }),
					iframeConsentDomains: ('iframeConsentDomains' in storage) ? storage.iframeConsentDomains : [],
					tcf: tcf, // someday we'll be able to use shorthand property name syntax
					lang: this.locale,
					clientUserAgent: navigator.userAgent,
					clientOs: navigator.platform,
					clientLang: navigator.language,
					clientWidth: window.innerWidth,
					clientHeight: window.innerHeight,
					actualUrl: window.location.href,
					actualRef: document.referrer,
					actualOpened: 0
				},
				done: function () {
					var message = {
						realm: 'CCM19',
						type: 'ConsentSharing',
						action: 'consent',
						ucid: self.getUniqueCookieId(),
						updated: ('updated' in storage) ? storage.updated : null,
						consent: hasConsent,
						embeddings: ('embeddings' in storage) ? storage.embeddings : [],
						tcf: ('tcf' in storage) ? tcf : null,
						domain: self.domainId,
						clickedButton: self.clickedButton,
					};
					self.consentShareChannels.forEach(function (item) {
						try {
							var origin = (item.origin !== undefined) ? item.origin : null;
							item.channel.postMessage(message, origin);
						}
						catch (e) {
							// ignore
						}
					});
				},
			});
		},
		/**
		 * @return {void}
		 */
		setStorageNamespaceUsingUrlParameter: function () {
			if (this.storageNamespaceUrlParameter.length == 0) {
				return;
			}

			var self = this;

			var storageNamespace = location.search.slice(1).split('&').reduce(function (carry, param) {
				param = param.split('=', 2).map(function (x) { return decodeURIComponent(x); });
				var name = param[0];
				var value = param[1] || null;
				return name == self.storageNamespaceUrlParameter ? value : carry;
			}, null);

			if (storageNamespace) {
				this.storageNamespace = storageNamespace;
				this.dynamicStorageNamespace = true;
			}
		},
		/**
		 * @returns {Object}
		 */
		getConsentStorage: function () {
			var storage = this.getCookie(this.cookies.consent) || {};
			var hasNamespace = Object.keys(storage).reduce(function (carry, property) {
				return carry || String(property).indexOf('/') == 0;
			}, false);

			if (storage[this.storageNamespace]) {
				// Use storage from current namespace
				storage = storage[this.storageNamespace];
			}
			else if (hasNamespace) {
				// Don't use legacy mode if namespaces exist
				storage = {};
			}

			return typeof storage == 'object' ? storage : {};
		},
		/**
		 * @returns {void}
		 */
		tidyConsentStorage: function () {
			var self = this;
			var cookies = this.getCookies().filter(function (cookie) { return cookie.name == self.cookies.consent; });

			// Multiple cookies may be possible when switching from or to consent sharing
			if (cookies.length > 1) {
				if (this.isConsentShareDomain()) {
					// Delete the cookie for the hostname of the current document location (excluding subdomains)
					document.cookie = this.cookies.consent + '=;path=/;expires=Thu, 01 Jan 1970 00:00:00 UTC';
				}

				cookies = this.getCookies().filter(function (cookie) { return cookie.name == self.cookies.consent; });
			}

			if (cookies.length > 1) {
				console.warn('[CCM19] The cookie %s exists for multiple domains, which can result in unexpected behavior.', this.cookies.consent);
			}
		},
		/**
		 * @param {boolean} [logSettings=true]
		 * @param {boolean} [closeModalWindows=true]
		 * @return {void}
		 */
		saveSettings: function (logSettings, closeModalWindows) {
			var self = this;
			logSettings = typeof logSettings == 'boolean' ? logSettings : true;
			closeModalWindows = typeof closeModalWindows == 'boolean' ? closeModalWindows : true;

			// Set the saving flag to bridge the gap till skipAnimationFrame is done
			this.saving = true;

			Utils.skipAnimationFrame((function () {
				var embeddingIds = self.getEmbeddingsWithConsent().map(function (embedding) { return embedding.id; });
				var tcf = {
					p: self.selectedTcfPurposes(),
					lip: self.selectedTcfLegIntPurposes(),
					sf: self.selectedTcfSpecialFeatures(),
					v: self.selectedTcfVendors(),
					liv: self.selectedTcfLegIntVendors(),
					gad: self.selectedGoogleProviders(),
				};

				var consent = self.consentGiven() || closeModalWindows;
				self.saveSettingsInternal(self.getUniqueCookieId(), consent, embeddingIds, tcf);

				if (logSettings) {
					self.logConsentSettings();
				}

				// Call all functions that are waiting for the settings to be saved
				self.saving = false;
				self.callAsSoonAsSavingSettingsIsDone();
			}));

			if (closeModalWindows) {
				this.closeAllModals();
			}
		},
		/**
		 * @param {string} ucid
		 * @param {boolean} consent
		 * @param {Array.<string>} embeddingIds
		 * @param {Object} [tcf]
		 * @param {Array.<number>} tcf.p Allowed purposes
		 * @param {Array.<number>} tcf.lip Allowed legitimate interest purposes
		 * @param {Array.<number>} tcf.sf Allowed special features
		 * @param {Array.<number>} tcf.v Allowed vendors
		 * @param {Array.<number>} tcf.liv Allowed legitimate interest vendors
		 * @param {string} [mode='permanent'] ('permanent', 'temporary')
		 * @param {number=} updatedTs Update timestamp in Deciseconds (Default: now)
		 * @return {void}
		 */
		saveSettingsInternal: function (ucid, consent, embeddingIds, tcf, mode, timestamp) {
			mode = typeof mode == 'string' ? mode : 'permanent';

			var storage = this.getConsentStorage();
			var options = {
				path: '/',
			};

			tcf = typeof tcf == 'object' && tcf !== null && 'sf' in tcf ? tcf : {
				p: [],
				sf: [],
				v: [],
			};

			// TCF-Vendor-Liste möglicht mit minimalem Speicherplatz speichern, um Cookie-Größenlimits nicht zu überschreiten
			var vr = this.makeIdRanges(tcf.v);
			if (vr.length < tcf.v.length) {
				tcf.vr = vr;
				delete tcf.v;
			}
			else if ('vr' in tcf) {
				delete tcf.vr;
			}
			if (Array.isArray(tcf.liv)) {
				var livr = this.makeIdRanges(tcf.liv);
				if (livr.length < tcf.liv.length) {
					tcf.livr = livr;
					delete tcf.liv;
				}
				else if ('livr' in tcf) {
					delete tcf.livr;
				}
			}

			if (this.cookieLifetime > 0) {
				options.expires = new Date(Utils.now + (this.cookieLifetime * 86400000)).toUTCString();
			}
			options.secure = (this.cookieUseSecure && location.protocol == 'https:');
			options.sameSite = (options.secure) ? 'None' : 'Lax';

			// Zehntelsekunden
			if (timestamp === undefined || timestamp === null) {
				timestamp = Math.round(Utils.now/100);
			}

			if (mode == 'temporary') {
				options.method = 'sessionStorage';
			}
			else if (this.consentStorageMethod != 'sessionStorage' && 'sessionStorage' in window && 'removeItem' in window.sessionStorage) {
				window.sessionStorage.removeItem(this.cookies.consent);
			}

			storage.gen = 2;
			storage.ucid = ucid;
			storage.consent = consent;
			storage.embeddings = embeddingIds;
			storage.created = ('created' in storage) ? storage.created : timestamp;
			storage.updated = timestamp;
			storage.clickedButton = this.clickedButton;
			storage.iframeConsentDomains = this.iframeConsentDomains;
			storage.tcf = tcf;
			storage.lang = this.locale;

			var fullStorage = this.getCookie(this.cookies.consent) || {};
			fullStorage[this.storageNamespace] = storage;

			this.setCookie(this.cookies.consent, fullStorage, options);
			this.requiresConsentUpdate = !consent;
			this.forceOpenWidget = this.forceOpenWidget ? this.requiresConsentUpdate : false;

			if (this.widgetInitialized) {
				this.setPurposesAndEmbeddings(embeddingIds);
				this.loadScripts();
				this.updateCookieDeclaration();
			}

			if ('clickskeksLegacy' in this) {
				var clickskeksCategoriesConsentState = window.Clickskeks.getCurrentAllowedConfig()
				this.clickskeksLegacy.allowedConfigChangeCallbacks.forEach(function (callback) {
					callback(clickskeksCategoriesConsentState);
				});
			}
		},
		/**
		 * This method checks all available consent options.
		 * @param {?boolean=} Also opt-in to all legitimate interest (default: true)
		 */
		grantAllPrivileges: function (optInLegitimateInterest) {
			this.selectAllPurposes();
			this.selectAllTcfPurposes();
			if (optInLegitimateInterest !== false) {
				this.selectAllTcfLegIntPurposes();
				this.selectAllTcfLegIntVendors();
			}
			this.selectAllTcfSpecialFeatures();
			this.selectAllGoogleProviders();
			this.selectAllTcfVendors();
		},
		/**
		 * This method unchecks all available consent options excluding mandatory ones.
		 * @param {?boolean=} Also opt-out of all legitimate interest (default: false)
		 */
		revokeAllPrivileges: function (optOutLegitimateInterest) {
			this.setAllPurposes(false);
			this.resetAllTcfPurposes();
			if (optOutLegitimateInterest === true) {
				this.resetAllTcfLegIntPurposes();
				this.resetAllTcfLegIntVendors();
			}
			this.resetAllTcfSpecialFeatures();
			this.resetAllGoogleProviders();
			this.resetAllTcfVendors();
		},
		/**
		 * @param {?Event} event
		 */
		onConsentButtonClicked: function (event) {
			event = event || window.event;
			var button = event.currentTarget || event.target;
			// Disregard manipulated clicks if manipulation protection is enabled
			if (this.checkIsManipulatedClick(event)) {
				console.warn('[CCM19] Manipulated click on consent button detected and ignored.');
				return;
			}

			if (button.dataset.fullConsent == 'true') {
				this.clickedButton = 'acceptAll';
				this.grantAllPrivileges();
			} else {
				this.clickedButton = 'save';
				// Reset remebered iframes on non "full consent" save.
				this.iframeConsentDomains = [];
			}

			this.saveSettings();
		},
		/**
		 * @param {?Event} event
		 */
		onDeclineButtonClicked: function (event) {
			this.clickedButton = "decline";

			this.iframeConsentDomains = [];
			this.revokeAllPrivileges();
			this.saveSettings();
		},
		onActionButtonClicked: function (event) {
			event = event || window.event;
			var button = event.currentTarget || event.target;

			var actions = {
				'enableEverything': this.grantAllPrivileges,
				'disableEverything': this.revokeAllPrivileges,
			};
			var action = button.dataset.ccmAction;

			typeof actions[action] == 'function' && actions[action].call(this, true);
		},
		onTreeNodeToggleClicked: function (event) {
			event = event || window.event;
			var button = event.currentTarget || event.target;

			var treeNode = button.parentElement;
			while (treeNode && treeNode.classList.contains('ccm--tree-node') == false) {
				treeNode = treeNode.parentElement;
			}

			if (!treeNode) {
				return;
			}

			var isOpen = Utils.toggleClass(treeNode, 'ccm--tree-node--open');
			button.ariaExpanded = isOpen;

			toArray(treeNode.querySelectorAll('.ccm--collapsable')).forEach(function (element) {
				var parentTreeNode = element.parentElement;
				while (parentTreeNode && parentTreeNode.classList.contains('ccm--tree-node') == false) {
					parentTreeNode = parentTreeNode.parentElement;
				}

				// Only handle immediate descendants (children) of the clicked tree node
				if (parentTreeNode === treeNode) {
					element.setAttribute('aria-hidden', isOpen ? 'false' : 'true');
				}
			});
		},
		/**
		 * This method updates non-recursively; only single-level parents and/or children.
		 * @param {Event} event
		 */
		onSwitchStateChanged: function (event) {
			var checkbox = event.target;
			var switches = toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group]'));
			var descendants = toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group-parent]'));

			var switchGroup = switches.filter(function (item) {
				return item.dataset.switchGroup == checkbox.dataset.switchGroup;
			});
			var switchGroupParents = switches.filter(function (item) {
				var parentName = checkbox.dataset.switchGroupParent;
				return parentName && item.dataset.switchGroup == parentName;
			});
			var switchGroupSiblings = switches.filter(function (item) {
				var parentName = checkbox.dataset.switchGroupParent;
				return parentName && item.dataset.switchGroupParent == parentName;
			});
			var switchGroupChildren = switches.filter(function (item) {
				var parentName = checkbox.dataset.switchGroup;
				return item.dataset.switchGroupParent == parentName;
			});

			var newState = checkbox.checked;

			var updateConsentState = function (checkbox) {
				var category = checkbox.dataset.consentStateCategory;

				if (this.consentState[category] && checkbox.value in this.consentState[category]) {
					this.consentState[category][checkbox.value] = checkbox.checked;
				}
				else if (CCM_DEBUG) {
					console.warn('[CCM19] Unknown consent state trigger or id in view, try clearing browser cache. (' + category + ', ' + checkbox.value + ')');
				}
			}.bind(this);

			var updateConsentStateForGroups = function (checkbox) {
				var self = this;

				checkbox.dataset.consentStateCategoryGroups.split(',').forEach(function (category) {
					if (self.consentState[category]) {
						for (var id in self.consentState[category]) {
							self.consentState[category][id] = checkbox.checked;
						}
					}
					else if (CCM_DEBUG) {
						console.warn('[CCM19] Unknown consent state category in view, try clearing browser cache. (' + category + ')');
					}
				});
			}.bind(this);

			// Update every checkbox in the same group
			switchGroup.forEach(function (checkbox) {
				checkbox.checked = newState;
				checkbox.indeterminate = false;

				if (checkbox.dataset.consentStateCategory) {
					updateConsentState(checkbox);
				}

				// Update consent state for every child, e.g. TCF vendors and Google ATPs when "third parties" has changed
				if (checkbox.dataset.consentStateCategoryGroups) {
					updateConsentStateForGroups(checkbox);
				}
			});

			// Update every switch group parent
			if (switchGroupParents.length > 0) {
				// Determine new state for group parent
				var newParentState = switchGroupSiblings.reduce(function (newState, checkbox) {
					if (newState === true) {
						newState = checkbox.checked || 'partial';
					}
					else {
						newState = checkbox.checked ? 'partial' : newState;
					}

					return newState;
				}, newState);

				switchGroupParents.forEach(function (checkbox) {
					checkbox.checked = !!newParentState;
					checkbox.indeterminate = newParentState == 'partial';
				});
			}

			// Update every switch group child
			switchGroupChildren.forEach(function (checkbox) {
				checkbox.checked = newState;
				checkbox.indeterminate = false;
				updateConsentState(checkbox);
			});

			// Paint updated consent state
			this.updateConsentState();
		},
		/**
		 * @param {Event} event
		 */
		onPurposeToggled: function (event) {
			var checkbox = event.target;

			var purposeId = checkbox.dataset.purpose;
			var newState = checkbox.checked; // Store state because it's likely to be modified in the upcoming routine
			var self = this;

			this.repository.forEach(function (embedding) {
				if (embedding.purpose == purposeId) {
					embedding.consent = newState || embedding.mandatory;
					self.selectEmbedding(embedding);
				}
			});
		},
		/**
		 * @param {Event} event
		 */
		onEmbeddingToggled: function (event) {
			var checkbox = event.target;

			var embedding = this.repository[checkbox.dataset.embedding];

			if (embedding) {
				embedding.consent = checkbox.checked || embedding.mandatory;
				this.selectEmbedding(embedding);
			}
			else if (CCM_DEBUG) {
				console.warn('[CCM19] Unknown embedding in view, try clearing browser cache. (' + embedding.id + ')');
			}
		},
		/**
		 * @param {Event} event
		 */
		onTcfDescriptionExpandClicked: function (event) {
			function insertTcfText(container, text) {
				var text = text.trim().split('\n');
				var curEl;
				for (var i = 0; i < text.length; ++i) {
					if (!text[i]) {
						curEl = null;
					}
					else if (text[i][0] === '*' && text[i][1] === ' ') {
						if (curEl && curEl.tagName.toLowerCase() != 'ul') {
							curEl = null;
						}
						if (!curEl) {
							curEl = document.createElement('ul');
							container.appendChild(curEl);
						}

						var li = document.createElement('li');
						li.innerText = text[i].substr(2);
						curEl.appendChild(li);
					}
					else {
						if (curEl) {
							if (curEl.tagName.toLowerCase() == 'p') {
								curEl.appendChild(document.createElement('br'));
							} else {
								curEl = null;
							}
						}
						if (!curEl) {
							curEl = document.createElement("p");
							container.appendChild(curEl);
						}
						curEl.appendChild(document.createTextNode(text[i]));
					}
				}
			}

			var span = (event.currentTarget || event.target);
			var li = span.parentNode;
			var targetSpec = li.getAttribute('data-ccm-tcf').split(':');
			li.classList.toggle('ccm-expanded');
			if (li.classList.contains('ccm-expanded')) {
				var container = li.getElementsByClassName('ccm-tcf-description')[0];
				if (!container) {
					var hiddenInfo = toArray(li.getElementsByClassName('ccm--hidden'));
					var container = document.createElement('div');
					var topContainer = li.parentNode;
					while (topContainer.parentNode && !topContainer.id) {
						topContainer = topContainer.parentNode;
					}
					var id = 'ccm-pdesc--'+topContainer.id.replace(/[^a-zA-Z0-9-]/g, '-')+'--'+li.getAttribute('data-ccm-tcf').replace(/[^a-zA-Z0-9-]/g, '-');
					container.className = 'ccm-tcf-description';
					container.id = id;
					span.setAttribute('aria-controls', id);
					var innerContainer = document.createElement('div');
					for (var i = 0; i < hiddenInfo.length; ++i) {
						hiddenInfo[i].classList.remove('ccm--hidden');
						container.appendChild(hiddenInfo[i]);
					}
					container.appendChild(innerContainer);
					var dataContainer = document.getElementById('ccm-tcf-descriptions');
					var descriptionData = JSON.parse(dataContainer.innerHTML)[targetSpec[0]][targetSpec[1]];

					insertTcfText(innerContainer, descriptionData.desc);

					if (descriptionData.legal) {
						insertTcfText(innerContainer, descriptionData.legal);
					}
					span.ariaExpanded = true;
					li.appendChild(container);
				} else {
					span.ariaExpanded = true;
					container.classList.remove('ccm--hidden');
				}
			} else {
				span.ariaExpanded = false;
				var element = li.getElementsByClassName('ccm-tcf-description')[0];
				if (element) {
					element.classList.add('ccm--hidden');
				}
			}
		},
		/**
		 * @param {Event} event
		 */
		onTcfExtendedDisclosureExpandClicked: function (event) {
			var title = (event.currentTarget || event.target);
			var parent = title.parentNode;
			var vendorId = parseInt(parent.getAttribute('data-ccm-tcf'));
			parent.classList.toggle('ccm-expanded');
			if (parent.classList.contains('ccm-expanded')) {
				var url = this.tcfData.disclosureUrlTemplate.replace(':VENDOR', vendorId).replace(':LANG', this.locale);
				this.ajax(url, {
					method: 'GET',
					success: function (data) {
						var container = document.createElement('div');
						var id = 'ccm-extdiscl--'+parent.getAttribute('data-ccm-tcf').replace(/[^a-zA-Z0-9-]/g, '-');
						container.className = 'ccm-tcf-description';
						container.id = id;
						title.setAttribute('aria-controls', id);
						container.innerHTML = data;
						title.ariaExpanded = true;
						parent.appendChild(container);
					},
					failure: function () {
						parent.classList.remove('ccm-expanded');
					}
				});
			} else {
				title.ariaExpanded = false;
				parent.removeChild(parent.getElementsByClassName('ccm-tcf-description')[0]);
			}
		},
		/**
		 * @param {Event} event
		 */
		onTcfPermissionVendorListExpandClicked: function (event) {
			function getVendorContainer(refElement) {
				var el = refElement;
				while ((el = el.parentNode)) {
					if ('classList' in el && (el.classList.contains('ccm-details--content--item') || el.classList.contains('ccm-details--content--cookie'))) {
						return el;
					}
				}
				return el;
			}

			var span = (event.currentTarget || event.target);
			var outerContainer = span.parentNode;
			var targetSpec = outerContainer.getAttribute('data-ccm-tcf').replace(/[^a-zA-Z0-9_:-]/g, '');
			outerContainer.classList.toggle('ccm-expanded');
			if (outerContainer.classList.contains('ccm-expanded')) {
				var container = outerContainer.parentNode.getElementsByClassName('ccm--tcf-vendors-list')[0];
				if (!container) {
					container = document.createElement('ul');
					var id = 'ccm-vlist--'+outerContainer.getAttribute('data-ccm-tcf').replace(/[^a-zA-Z0-9-]/g, '-');
					container.id = id;
					span.setAttribute('aria-controls', id);
					container.classList.add('ccm--tcf-vendors-list');
					var references = toArray(document.getElementById('ccm-details').querySelectorAll('.ccm-details--embedding--tcf li[data-ccm-tcf="'+targetSpec+'"]'));
					for (var i = 0; i < references.length; ++i) {
						var vendorBox = getVendorContainer(references[i]);
						if (!vendorBox) {
							continue;
						}
						var name;
						var idEl = vendorBox.getElementsByClassName('ccm-details--content--cookie--title')[0];
						// TCF-Vendor in Embedding:
						if (idEl) {
							var vendorNameEl = vendorBox.querySelector('data.ccm--tcf-vendor');
							name = vendorNameEl.value.replace(/^[0-9]*:/, '');
						}
						// Drittanbieter TCF-Vendor:
						else {
							idEl = vendorBox;
							name = vendorBox.querySelector('.ccm--tcf-permission--title').innerText.trim();
						}

						var li = document.createElement('li');
						var a = document.createElement('a');
						a.href = '#'+idEl.id;
						a.innerText = name;
						li.appendChild(a);
						container.appendChild(li);
					}
					outerContainer.appendChild(container);
				} else {
					container.classList.remove('ccm--hidden');
				}
				span.ariaExpanded = true;
			} else {
				span.ariaExpanded = false;
				var element = outerContainer.getElementsByClassName('ccm--tcf-vendors-list')[0];
				if (element) {
					element.classList.add('ccm--hidden');
				}
			}
		},
		/**
		 * Presets controls with the stored consent state.
		 * @return {void}
		 */
		presetConsentState: function () {
			if (this.consentGiven()) {
				this.presetPurposesAndEmbeddings();
				this.presetTcfPurposes();
				this.presetTcfLegIntPurposes();
				this.presetTcfSpecialFeatures();
				this.presetTcfVendors();
				this.presetTcfLegIntVendors();
				this.presetGoogleProviders();
			}
			else if (this.isConsentRequired()) {
				this.revokeAllPrivileges();
				this.selectAllTcfLegIntPurposes();
				this.selectAllTcfLegIntVendors();
			}
			else {
				this.grantAllPrivileges();
				// Save settings to update the logic in conjunction with the UI
				if (!this.forceOpenWidget) {
					this.saveSettings();
				}
			}
		},
		/**
		 * Sets controls to the current selection state across all windows.
		 * @return {void}
		 */
		updateConsentState: function () {
			this.updatePurposesAndEmbeddings();
			this.updateTcfPurposes();
			this.updateTcfLegIntPurposes();
			this.updateTcfSpecialFeatures();
			this.updateTcfVendors();
			this.updateTcfLegIntVendors();
			this.updateGoogleProviders();
		},
		/**
		 * @return {void}
		 */
		presetPurposesAndEmbeddings: function () {
			var storage = this.getConsentStorage();
			var allowedEmbeddings = storage.embeddings || [];
			this.setPurposesAndEmbeddings(allowedEmbeddings);
		},
		/**
		 * @return {void}
		 */
		updatePurposesAndEmbeddings: function () {
			var allowedEmbeddings = this.repository
				.filter(function (embedding) { return embedding.consent; })
				.map(function (embedding) { return embedding.id; });
			this.setPurposesAndEmbeddings(allowedEmbeddings);
		},
		/**
		 * @return {void}
		 */
		setPurposesAndEmbeddings: function (allowedEmbeddings) {
			var self = this;

			this.repository.forEach(function (embedding) {
				embedding.consent = allowedEmbeddings.includes(embedding.id) || embedding.mandatory;
				if (self.container) {
					self.selectEmbedding(embedding);
				}
			});
		},
		/**
		 * @param {boolean} [newState=false]
		 * @return {void}
		 */
		setAllPurposes: function (newState) {
			newState = typeof newState == 'boolean' ? newState : false;
			var allowedEmbeddings = newState
				? this.repository.map(function (embedding) { return embedding.id; })
				: [];

			this.setPurposesAndEmbeddings(allowedEmbeddings);
		},
		/**
		 *
		 */
		captureFocus: function () {
			if (this._focusCaptured) {
				return;
			}
			var self = this;
			this._focusCaptured = true;
			var focusHelperTop = document.createElement('div');
			focusHelperTop.style.position = 'fixed';
			focusHelperTop.style.left = '0';
			focusHelperTop.style.right = '0';
			focusHelperTop.style.height = '0';
			focusHelperTop.style.maxHeight = '0';
			focusHelperTop.background = 'transparent';
			focusHelperTop.border = '0 none transparent';
			focusHelperTop.ariaHidden = true;
			focusHelperTop.tabIndex = 0;
			var focusHelperBottom = focusHelperTop.cloneNode();
			focusHelperTop.style.top = '0';
			focusHelperTop.id = 'ccm-focus-helper--top';
			document.body.insertBefore(focusHelperTop, document.body.firstChild);
			focusHelperBottom.style.bottom = '0';
			focusHelperBottom.id = 'ccm-focus-helper--bottom';
			document.body.appendChild(focusHelperBottom);

			document.body.addEventListener('focusin', self.onFocusIn, {capture: true, passive: false});
			document.body.addEventListener('keydown', self.onTabKeyDown, {capture: true, passive: true});
		},
		/**
		 *
		 */
		endCaptureFocus: function () {
			if (!this._focusCaptured) {
				return;
			}
			var self = this;
			this._focusCaptured = false;
			var topHelper = document.getElementById('ccm-focus-helper--top');
			var bottomHelper = document.getElementById('ccm-focus-helper--bottom');
			if (topHelper) {
				topHelper.parentNode.removeChild(topHelper);
			}
			if (bottomHelper) {
				bottomHelper.parentNode.removeChild(bottomHelper);
			}

			document.body.removeEventListener('focusin', self.onFocusIn, {capture: true, passive: false});
			document.body.removeEventListener('keydown', self.onTabKeyDown,{capture: true, passive: true});
		},
		/**
		 * @return {boolean}
		 */
		consentGiven: function () {
			if (this.requiresConsentUpdate) {
				return false;
			}
			var storage = this.getConsentStorage();
			return (storage.consent && storage.gen == 2) || false;
		},
		/**
		 * @return {boolean}
		 */
		isFullConsentGiven: function () {
			if (!this.consentGiven()) {
				return false;
			}
			else if (this.clickedButton === 'acceptAll') {
				return true;
			}

			var storage = this.getConsentStorage();
			var tcf = this.getTcfConsentInfo();

			return Utils.diff(Object.keys(this.repository), storage.embeddings || []).length == 0
				&& Utils.diff(Object.keys(this.consentState.googleAdTechProviders).map(function(x){return parseInt(x);}), tcf.gad).length == 0
				&& Utils.diff(Object.keys(this.consentState.tcfPurposes).map(function(x){return parseInt(x);}), tcf.p).length == 0
				&& Utils.diff(Object.keys(this.consentState.tcfLegIntPurposes).map(function(x){return parseInt(x);}), tcf.lip).length == 0
				&& Utils.diff(Object.keys(this.consentState.tcfSpecialFeatures).map(function(x){return parseInt(x);}), tcf.sf).length == 0
				&& Utils.diff(Object.keys(this.consentState.tcfVendors).map(function(x){return parseInt(x);}), tcf.v).length == 0
				&& Utils.diff(Object.keys(this.consentState.tcfLegIntVendors).map(function(x){return parseInt(x);}), tcf.liv).length == 0
			;
		},
		/**
		 * Register handlers for CCM19 dialog interactions
		 * @return {void}
		 */
		registerControls: function () {
			var self = this;
			var initializedControls = [];

			toArray(this.container.querySelectorAll('.ccm-settings-summoner > .ccm-settings-summoner--link')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				self.addHighPriorityEventListener('click', element, function (event) { return self.onSettingsIconClicked(event); });
			});

			toArray(this.container.querySelectorAll('[data-jump-to]')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function (event) { self.jumpTo(element.dataset.jumpTo, event); });
			});

			toArray(this.container.querySelectorAll('[data-ccm-modal]')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function () { self.loadModal(element.dataset.ccmModal, true, element.dataset.jumpTo); });
			});

			var actionButtonClickedHandler = function (event) { return self.onActionButtonClicked(event); };
			toArray(this.container.querySelectorAll('button[data-ccm-action], .button[data-ccm-action]')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', actionButtonClickedHandler);
			});

			toArray(this.container.querySelectorAll('.ccm--tree-node .ccm--tree-node-toggle')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.ariaExpanded = false;
				element.addEventListener('click', self.onTreeNodeToggleClicked);
			});

			toArray(this.container.getElementsByClassName('ccm-modal--close')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function (event) { return self.onModalCloseButtonClicked(event); });
			});

			toArray(this.container.getElementsByClassName('ccm--save-settings')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function (event) { return self.onConsentButtonClicked(event); });
			});

			toArray(this.container.getElementsByClassName('ccm--decline-cookies')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function (event) { return self.onDeclineButtonClicked(event); });
			});

			toArray(this.container.getElementsByClassName('ccm-widget--language-select')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('change', function () {self.switchLocale(this.value);});
			});

			toArray(this.container.getElementsByClassName('ccm-dismiss-button')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', function () { self.dismissTemporarily(); });
			});

			toArray(this.container.querySelectorAll('input[type="checkbox"][data-switch-group]')).forEach(function (checkbox) {
				if (checkbox.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(checkbox);
				checkbox.addEventListener('change', function (event) { return self.onSwitchStateChanged(event); });
			});

			var tcfDescriptionExpandClicked = function (event) { return self.onTcfDescriptionExpandClicked(event); };
			var tcfExtendedDisclosureExpandClicked = function (event) { return self.onTcfExtendedDisclosureExpandClicked(event); };
			var tcfPermissionVendorListExpandClicked = function (event) { return self.onTcfPermissionVendorListExpandClicked(event); };
			var tcfDescriptionExpandKeyPress = function (event) {
				if (event.key == 'Enter' || event.key == 'Spacebar' || event.key == ' ' || event.key == 'NumpadEnter') {
					event.preventDefault();
					this.click();
					event.stopPropagation();
				}
			};
			toArray(this.container.querySelectorAll('li[data-ccm-tcf] > span')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', tcfDescriptionExpandClicked);
				element.addEventListener('keypress', tcfDescriptionExpandKeyPress);
				element.ariaExpanded = false;
				element.role = 'button';
			});
			toArray(this.container.querySelectorAll('div[data-ccm-tcf] > strong')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', tcfExtendedDisclosureExpandClicked);
				element.addEventListener('keypress', tcfDescriptionExpandKeyPress);
				element.ariaExpanded = false;
				element.role = 'button';
			});

			toArray(this.container.querySelectorAll('.ccm--tcf-vendors[data-ccm-tcf] > span')).forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('click', tcfPermissionVendorListExpandClicked);
				element.addEventListener('keypress', tcfDescriptionExpandKeyPress);
				element.ariaExpanded = false;
				element.role = 'button';
			});


			this.getPurposeCheckboxes().forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('change', function (event) { return self.onPurposeToggled(event); });
			});
			this.getEmbeddingCheckboxes().forEach(function (element) {
				if (element.classList.contains('ccm--ctrl-init')) {
					return;
				}
				initializedControls.push(element);
				element.addEventListener('change', function (event) { return self.onEmbeddingToggled(event); });
			});

			// Mark processed controls as initialized
			initializedControls.forEach(function (element) {
				element.classList.add('ccm--ctrl-init');
			});
		},
		useRemoteConsent: function (ucid, consent, embeddingIds, tcfData, domainId, source, updated, clickedButton, callback) {
			this.ucid = ucid;
			// Same-domain sharing
			if (this.domainId === domainId) {
				this.clickedButton = 'save';
				this.saveSettingsInternal(ucid, consent, embeddingIds, tcfData, 'permanent', updated);
			}
			// Account-wide sharing of 'Accept all'/'Decline all'
			else if (this.accountWideConsentSharing) {
				// Reject all
				if (clickedButton === 'decline') {
					if (this.clickedButton !== 'decline' || this.requiresConsentUpdate) {
						this.clickedButton = 'decline';
						this.iframeConsentDomains = [];
						this.revokeAllPrivileges();
						this.saveSettings();
					}
				}
				// Accept all
				else if (clickedButton === 'acceptAll') {
					if (this.clickedButton !== 'acceptAll' || this.requiresConsentUpdate) {
						this.clickedButton = 'acceptAll';
						this.grantAllPrivileges();
						this.saveSettings();
					}
				}
			}

			if (callback) {
				// Wait for settings to be saved, then call the callback
				this.callAsSoonAsSavingSettingsIsDone(callback);
			}
		},
		/**
		 * @param {string} url
		 * @return {boolean}
		 */
		urlIsExempt: function (url)
		{
			var hashlessUrl = url.replace(/#.*$/, '');
			return this.exemptUrls.reduce(function (isMatch, item) {
				if (item.indexOf('//') === 0) {
					item = location.protocol + item;
				}
				else if (item.indexOf('/') === 0) {
					item = location.protocol + '//' + location.host + item;
				}

				var isWildcard = item.charAt(item.length - 1) == '*';
				item = isWildcard ? item.substr(0, item.length - 1) : item;

				if (item.includes('#')) {
					return isMatch || url.indexOf(item) == 0 && (isWildcard || url.length == item.length);
				} else {
					return isMatch
						|| hashlessUrl.indexOf(item) == 0 && (isWildcard || hashlessUrl.length == item.length)
						|| url.indexOf(item+'#') === 0;
				}
			}, false);
		},
		/**
		 * @param {boolean} [ignoreDnt=false]
		 * @return {boolean} Returns `true` if the client is in the EU or "do not track"-rules apply, `false` otherwise.
		 */
		isConsentRequired: function (ignoreDnt) {
			ignoreDnt = typeof ignoreDnt == 'boolean'? ignoreDnt : false;
			var inEu = !this.behavior.noConsentRequired;
			var dnt = !ignoreDnt && this.behavior.respectDoNotTrack && this.isDoNotTrackEnabledInBrowser();
			return inEu || dnt;
		},
		/**
		 * Build CCM19 into the DOM
		 * @param {?function=} onBuildFinished Callback after the build is finished.
		 * @return {void}
		 */
		build: function (onBuildFinished) {
			var self = this;

			if (!this.widgetUrl) {
				this.widgetUrl = this.widgetUrls[this.locale];
			}

			/**
			 * @param {boolean} firstInitialization
			 */
			function finishWidgetBuild(firstInitialization)
			{
				if (onBuildFinished) {
					onBuildFinished.call(self, firstInitialization);
				}
				Utils.triggerCustomEvent(window, 'ccm19WidgetLoaded');
			}

			function onWidgetHtmlLoadedInner() {
				var self = this;
				var firstInitialization = !this.widgetInitialized;
				if (firstInitialization) {
					if (!Utils.isDocumentReady()) {
						window.requestAnimationFrame(onWidgetHtmlLoaded);
						return;
					}
				}

				// Before any further processing, mark the widget as initialized
				this.widgetInitialized = true;

				this.insertCookieDeclaration();
				this.closeAllModals();
				this.registerControls();
				this.registerJavaScriptApiHandlers();
				this.presetConsentState();

				var currentMinute = (Utils.now / 1000 / 60 | 0);
				var accountWideUrlConsent = this.urlConsent ? Utils.objectValues(this.accountWideConsentType).includes(this.urlConsent[5]) : false;
				var urlConsentApplied = false;

				this.initPlatformDependentComponents();

				// Prefetch für Details-Dialog
				if (firstInitialization) {
					var detailsUrl = this.detailsUrls[this.locale];
					if (detailsUrl) {
						var link = document.getElementById('ccm-prefetch--detail-dialog');
						if (link) {
							link.parentNode.removeChild(link);
						} else {
							link = document.createElement('LINK');
						}
						link.rel = 'prefetch';
						link.as = 'fetch';
						link.crossOrigin = 'anonymous';
						link.href = detailsUrl;
						link.id = 'ccm-prefetch--detail-dialog';
						document.head.appendChild(link);
					}
				}

				// Bei IFrames: Consent von übergeordnetem Fenster anfragen
				if (!Utils.isTopLevel()) {
					var messaged = [], context;
					['parent', 'top', 'opener'].forEach(function (key) {
						try {
							context = window[key];
							if (context !== null && !messaged.includes(context)) {
								context.postMessage({
									realm: 'CCM19',
									type: 'ConsentSharing',
									action: 'requestConsent'
								}, '*');
								messaged.push(context);
							}
						}
						catch (e) {
							// ignore
						}
					});
				}

				if (firstInitialization) {
					this.dispatchInternalEvent('widgetInitialized');
					this.wrapHistoryApi();
					window.addEventListener('popstate', function (event) { self.onHistoryStateChanged(event); });
					var focusHelper = document.getElementById('ccm-focus-helper--bottom');
					if (focusHelper) {
						document.body.insertBefore(this.container, focusHelper);
					} else {
						document.body.appendChild(this.container);
					}
				}

				// Cross-Domain übertragenen Consent verwenden
				if ((this.consentCrossDomain && !accountWideUrlConsent || this.accountWideConsentSharing && accountWideUrlConsent)
					&& this.urlConsent && Math.abs(parseInt(this.urlConsent[2], 36) - currentMinute) < 2
				) {
					var updated = (parseInt(this.urlConsent[3], 36) + 1609459200)*10;
					var storage = this.getConsentStorage();
					if (!storage || !('updated' in storage) || updated > storage.updated) {
						if (CCM_DEBUG) {
							console.info("[CCM19] Cross-domain consent applied.")
						}

						if (accountWideUrlConsent) {
							var embeddingIds = (this.urlConsent[5] === '1') ? ['*'] : [];
						} else {
							var embeddingIds = this.urlConsent[5].split('|');
						}

						this.useRemoteConsent(
							this.urlConsent[4],
							true,
							embeddingIds,
							null,
							(accountWideUrlConsent) ? null : this.domainId,
							'URL',
							updated,
							(accountWideUrlConsent) ? ((this.urlConsent[5] === '1') ? 'acceptAll' : 'decline') : 'save',
							function () { finishWidgetBuild.call(self, firstInitialization); }
						);
						urlConsentApplied = true;
					}
				}

				if (!urlConsentApplied) {
					// Ansonsten Widget direkt fertigstellen
					finishWidgetBuild.call(this, firstInitialization);
				}

			}
			var onWidgetHtmlLoaded = function () { onWidgetHtmlLoadedInner.call(self); };

			function onWidgetHtmlFailureInner() {
				if (!Utils.isDocumentReady()) {
					window.requestAnimationFrame(onWidgetHtmlFailure);
					return;
				}
				this.closeAllModals();
				this.container.innerHTML = '<div class="ccm-widget">Error while loading resource, please try again later.</div>';
			}
			var onWidgetHtmlFailure = function () { onWidgetHtmlFailureInner.call(self); };

			this.ajax(this.widgetUrl, {
				method: 'GET',
				success: function (response) {
					this.container.innerHTML = response;
					this.controlPanel = this.container.querySelector('#ccm-control-panel');
					this.widget = this.container.querySelector('#ccm-widget');

					// Wait for the save that occurs with opt-out initialization
					this.callAsSoonAsSavingSettingsIsDone(onWidgetHtmlLoaded);
				},
				failure: onWidgetHtmlFailure,
			});

		},
		/**
		 * @param {HTMLIFrameElement} iframe
		 * @return {void}
		 */
		restoreIframe: function (iframe) {
			/** @type {HTMLInputElement} */
			var consentSwitch = (this.iframeConsentSwitchMap[iframe.ccm19IframeConsentSwitchId] || {}).checkbox || null;
			if (consentSwitch) {
				consentSwitch.checked = true;
			}

			var sibling = iframe.ccm19IframePlaceholder || iframe;

			// Raus, wenn der Klon nicht mehr im DOM steckt
			if (sibling.parentElement === null) {
				return;
			}

			// Originales Iframe nach Möglichkeit wiederherstellen
			if (iframe.ccm19OriginalIframe) {
				iframe.ccm19OriginalIframe.ccm19ConsentGranted = true;
				iframe.ccm19OriginalIframe.src = iframe.ccm19OriginalIframe.dataset.ccmSrc;
				delete iframe.ccm19OriginalIframe.dataset.ccmLoaderGroup;
				delete iframe.ccm19OriginalIframe.dataset.ccmLoaderSrc;
				delete iframe.ccm19OriginalIframe.dataset.ccmSrc;
				sibling.parentElement.insertBefore(iframe.ccm19OriginalIframe, sibling);
				sibling.parentElement.removeChild(sibling);
				return;
			}

			// Auf ursprüngliche Iframe-URL im Klon wechseln
			if (iframe.dataset.ccmSrc !== '') {
				iframe.ccm19ConsentGranted = true;
				iframe.src = iframe.dataset.ccmSrc;
				iframe.dataset.ccmSrc = '';

				// Revert iframe to initial height
				var inlineCss = iframe.dataset.initialHeightInline == 'true';
				iframe.style.height = inlineCss ? iframe.dataset.initialHeight : iframe.dataset.initialHeight + 'px';
				iframe.style.maxHeight = '';
				iframe.dataset.initialHeightInline = '';
				iframe.dataset.initialHeight = '';

				if (inlineCss == false) {
					window.requestAnimationFrame(function () {
						iframe.style.height = '';
					});
				}
			}
		},
		/**
		 * @param {HTMLIFrameElement} iframe
		 * @return {void}
		 */
		blockIframe: function (iframe) {
			this.processIframeForBlocking(iframe, true);
		},
		/**
		 * @param {EmbeddingInfo} embedding
		 * @return {void}
		 */
		blockOrRestoreIframesForEmbedding: function (embedding) {
			var self = this;

			this.iframes.forEach(function (clone) {
				if (clone.ccm19BlockedByEmbeddings[0] == embedding.id) {
					if (embedding.consent) {
						self.restoreIframe(clone);
					}
					else {
						self.blockIframe(clone.ccm19OriginalIframe);
					}
				}
			});
		},
		/**
		 * @param {HTMLIFrameElement} iframe
		 * @returns {HTMLIFrameElement|null}
		 */
		findExistingIframeClone: function (iframe) {
			return this.iframes.reduce(function (clone, item) {
				return item.ccm19OriginalIframe === iframe ? item : clone;
			}, null);
		},
		/**
		 * @param {HTMLIFrameElement} iframe
		 * @param {boolean} [initialState=false]
		 */
		appendConsentSwitchToIframe: function (iframe, initialState) {
			initialState = typeof initialState == 'boolean' ? initialState : false;

			var self = this;
			var clone = this.findExistingIframeClone(iframe);

			var consentSwitchContainer = this.iframeConsentSwitchMap[clone.ccm19IframeConsentSwitchId]
				? this.iframeConsentSwitchMap[clone.ccm19IframeConsentSwitchId].container
				: null;

			if (!consentSwitchContainer) {
				var switchId = 'ccm--consent-switch-' + Utils.hash(String(Utils.now) + String(Math.random() || Math.random()).slice(1));
				clone.ccm19IframeConsentSwitchId = switchId;

				consentSwitchContainer = document.createElement('DIV');
				consentSwitchContainer.classList.add('ccm--consent-switch');

				var consentSwitchCheckbox = document.createElement('INPUT');
				consentSwitchCheckbox.id = switchId;
				consentSwitchCheckbox.type = 'checkbox';
				consentSwitchCheckbox.addEventListener('change', function (event) { return self.onIframeConsentSwitchStateChanged(event); });
				var consentSwitchLabel = document.createElement('LABEL');
				consentSwitchLabel.setAttribute('for', switchId);
				consentSwitchLabel.textContent = this.getLocalizedString('blockedContentShowExternalContent');

				consentSwitchContainer.appendChild(consentSwitchCheckbox);
				consentSwitchContainer.appendChild(consentSwitchLabel);

				var embedding = this.repository[clone.ccm19BlockedByEmbeddings[0]];
				if (embedding) {
					consentSwitchLabel.textContent = this.getLocalizedString('blockedContentAllowEmbedding').replace(':embedding', embedding.name);
					var consentSwitchEmbeddingLink = document.createElement('A');
					consentSwitchEmbeddingLink.href = 'javascript:';
					consentSwitchEmbeddingLink.textContent = '(' + this.getLocalizedString('blockedContentProviderDetails') + ')';
					consentSwitchEmbeddingLink.addEventListener('click', function () { self.loadModal('ccm-details', true, 'ccm-cookie-details-' + embedding.id); });
					consentSwitchContainer.appendChild(consentSwitchEmbeddingLink);
				}

				this.iframeConsentSwitchMap[switchId] = {
					original: iframe,
					clone: clone,
					checkbox: consentSwitchCheckbox,
					container: consentSwitchContainer,
				};
			}

			this.iframeConsentSwitchMap[clone.ccm19IframeConsentSwitchId].checkbox.checked = initialState;

			var switchSibling = initialState ? iframe : (clone.ccm19IframePlaceholder || clone);
			if (consentSwitchContainer.previousElementSibling !== switchSibling) {
				switchSibling.nextElementSibling
					? switchSibling.parentElement.insertBefore(consentSwitchContainer, switchSibling)
					: switchSibling.parentElement.appendChild(consentSwitchContainer);
			}
		},
		/**
		 * @param {Event} event
		 */
		onIframeConsentSwitchStateChanged: function (event) {
			/** @type {HTMLInputElement} */
			var checkbox = event.currentTarget || event.target;

			var self = this;
			var clone = this.iframeConsentSwitchMap[checkbox.id].clone;
			var original = this.iframeConsentSwitchMap[checkbox.id].original;
			var embedding = this.repository[clone.ccm19BlockedByEmbeddings[0]];

			// Toggle the embedding on and off and therby the linked iframes
			if (embedding) {
				embedding.consent = checkbox.checked || embedding.mandatory;
				this.selectEmbedding(embedding);
				this.saveSettings(true, false);
			}

			// Toggle the single iframe that is linked to the switch
			else {
				if (checkbox.checked) {
					this.restoreIframe(clone);
				}
				else {
					this.blockIframe(original);
				}

				var iframeDomain = original.dataset.ccmDomain;
				this.iframeConsentDomains = this.iframeConsentDomains.filter(function (domain) {
					return domain !== iframeDomain;
				});
				if (checkbox.checked) {
					this.iframeConsentDomains.push(iframeDomain);
				}
			}
		},
		/**
		 * @param {MessageEvent} event
		 * @return {void}
		 */
		onPostMessage: function (event) {
			// Nur CCM19-Events behandeln
			if (typeof event.data !== 'object' || event.data.realm !== 'CCM19') {
				return;
			}

			// Alle CCM19-Events werden hier abschließend bearbeitet
			event.stopImmediatePropagation();

			switch (event.data.type) {
				case 'ContentBlocker':
					return this.onMessageFromContentBlocker(event)
				case 'ConsentSharing':
					return this.onMessageFromConsentSharing(event)
				default:
					break;
			}
		},
		/**
		 * @param {MessageEvent} event
		 * @return {void}
		 */
		onMessageFromContentBlocker: function (event) {
			var self = this;
			if (event.origin != this.origin) {
				return;
			}

			var iframe = this.iframes.reduce(function (iframe, currentItem) {
				if (event.source === currentItem.contentWindow) {
					return currentItem;
				}

				return iframe;
			}, null);

			if (iframe == null) {
				return;
			}

			var action = typeof event.data.action == 'string' ? event.data.action : '';
			switch (action) {
				// Update iframe height to match inner content
				case 'updateHeight':
					if (typeof event.data.innerHeight == 'number') {
						// Prevent iframe's height from being updated after consent has been given
						if (iframe.ccm19ConsentGranted) {
							return;
						}

						if (event.data.initialUpdate) {
							// Make sure to persistently restore height in case of inline css
							iframe.dataset.initialHeightInline = iframe.style.height ? 'true' : 'false';
							iframe.dataset.initialHeight = iframe.style.height || iframe.clientHeight;
							iframe.dataset.initialClientHeight = iframe.clientHeight;
						}

						iframe.style.height = Math.max(event.data.innerHeight, iframe.dataset.initialClientHeight) + 'px';
						iframe.style.maxHeight = '100%';
					}
					break;
				// Update placeholder with actual resource's domain
				case 'resourceDomain':
					var domain = iframe.dataset.ccmDomain;
					iframe.contentWindow.postMessage({
						realm: 'CCM19',
						type: 'ContentBlocker',
						action: action,
						domain: domain,
					}, this.origin);

					var embedding = this.repository[iframe.ccm19BlockedByEmbeddings[0]];
					if (embedding) {
						if (embedding.dispatchLoadEventForIframes) {
							// Emulate load event to add support for lightboxes, preloaders and the like
							Utils.triggerCustomEvent(iframe.ccm19OriginalIframe, 'load');
						}
					}

					break;
				// Consent given, load external resource
				case 'thirdPartyContentConsentGranted':
					// Domain bestimmen
					var domain = iframe.dataset.ccmDomain;

					// Diesen Iframe aktivieren
					this.restoreIframe(iframe);

					// Alle Iframes der selben Domain aktivieren
					this.iframes.forEach(function (iframe) {
						if (iframe.dataset.ccmDomain == domain) {
							self.restoreIframe(iframe);
						}
					});

					// Zugelassene Domain merken und im Protokoll festhalten
					if (this.iframeConsentDomains.includes(domain) == false) {
						this.iframeConsentDomains.push(domain);
						this.saveSettings(true, false);
					}

					break;
				default:
					break;
			}
		},
		/**
		 * @param {MessageEvent} event
		 * @return {void}
		 */
		onMessageFromConsentSharing: function (event) {
			if (this.dynamicStorageNamespace) {
				return;
			}

			var originData = this.getCategorizedBaseDomain(event.origin.replace(/^[^:\/]*:\/\//, '').replace(/:[0-9]+$/, ''));
			var originDomain = originData.domain;
			// Abbrechen, wenn weder von eigenem Origin noch einer der Consent-Share-Domains.
			if (event.origin !== window.origin && !originData.sameAccount) {
				return;
			}

			var self = this;
			var data = event.data;

			switch (data.action) {
				// Request to inform the source if consent is given
				case 'requestConsent':
					var hasConsent = this.consentGiven();
					var storage = hasConsent ? this.getConsentStorage() : {};
					// Aktuellen Stand des Consents direkt übermitteln
					event.source.postMessage({
						realm: 'CCM19',
						type: 'ConsentSharing',
						action: 'consent',
						ucid: this.getUniqueCookieId(),
						updated: ('updated' in storage) ? storage.updated : null,
						consent: hasConsent,
						embeddings: ('embeddings' in storage) ? storage.embeddings : [],
						tcf: ('tcf' in storage) ? storage.tcf : null,
						domain: this.domainId,
						clickedButton: this.clickedButton,
					}, event.origin);
					// Vormerken, um einen erteilten Consent später weiterzureichen
					this.consentShareChannels.push({channel: event.source, origin: event.origin});
					break;
				// Consent überprüfen und ggf. übernehmen und alle CCM-Fenster schließen
				case 'consent':
					if (data.consent) {
						var storage = this.getConsentStorage();
						if (typeof storage.updated !== 'number' || (typeof data.updated === 'number' && data.updated > storage.updated)) {
							this.useRemoteConsent(data.ucid, data.consent, data.embeddings, data.tcf, data.domain, event.origin, data.updated, data.clickedButton, function () {
								if (self.consentGiven()) {
									self.closeAllModals();
								}
							});
						}
					} else {
						self.closeAllModals();
					}
					if (data.domain == this.domainId) {
						try {
							if (event.source === window.parent || event.source === window.top) {
								this.hideSettingsIcon();
							}
						} catch (e) {
							console.debug("[CCM19] Failed to check consent source window. Sandbox?", e);
						}
					}
					break;
				default:
					break;
			}
		},
		/**
		 * @return {Array}
		 */
		getAllowedEmbeddingIds: function () {
			if (this.requiresConsentUpdate) { return []; }
			return this.getConsentStorage().embeddings || [];
		},
		/**
		 * @return {Array}
		 */
		getAllowedAndMandatoryEmbeddingIds: function () {
			// Nichts zulassen, wenn Zustimmung fehlt und technisch notwendige Einbindungen zurückzuhalten sind
			if (this.consentGiven() == false && this.runMandatoryEmbeddingsWithoutConsent == false) {
				return [];
			}

			var allowedEmbeddingIds = this.getAllowedEmbeddingIds();

			for (var embeddingId in this.repository) {
				var embedding = this.repository[embeddingId];
				if (allowedEmbeddingIds.includes(embedding.id) == false && embedding.mandatory) {
					allowedEmbeddingIds.push(embedding.id);
				}
			}

			return allowedEmbeddingIds;
		},
		/**
		 * @param {HTMLScriptElement} node
		 * @return {boolean}
		 */
		shouldBlockScriptAccordingToBlacklist: function(node) {
			if (
				this.checkIsBlockableScript(node) == false
				&& node.type.toLowerCase() != 'text/x-ccm-loader' // Block every script that originates from the source code
				|| node.hasAttribute('data-ccm-injected')
			) {
				return false;
			}

			var text = String(node.outerHTML);
			return this.deniedScriptMarkers.reduce(function (isMatch, item) {
				// Consider that the src attribute may contain a relative URL; explicitly compare against src property
				return isMatch || text.includes(item) || node.src.includes(item);
			}, false);
		},
		/**
		 * @param {HTMLScriptElement} node Script node
		 * @return {boolean}
		 */
		shouldBlockScript: function (node)
		{
			if (this.shouldBlockScriptAccordingToBlacklist(node)) {
				return true;
			}

			// Alles was kein JS ist oder vom CCM stammt is ok.
			if (this.inEval || !this.checkIsBlockableScript(node) || node.hasAttribute('data-ccm-injected')) {
				return false;
			}

			var src = (node.type == 'text/x-ccm-loader' && node.dataset.ccmLoaderSrc) || node.src;

			// Eingebundene Skripte
			if (src) {
				return (this.blockSameDomainScripts || this.checkIsExternal(src));
			}
			// Inline-Skripte
			else {
				var text = String(node.outerHTML);
				var exception = this.allowedScriptMarkers.reduce(function (result, item) {
					return (result || text.includes(item));
				}, false);
				return (!exception && this.blockInlineScripts);
			}
		},
		/**
		 * @param {HTMLScriptElement} node
		 * @param {Array.<string>} embeddingIds
		 * @return {Array.<string>} IDs aller Embeddings, die dieses Skript blockieren
		 */
		embeddingsBlockingScript: function (node, embeddingIds)
		{
			var self = this;

			if (node.type == 'text/x-ccm-loader') {
				var loaderGroup = node.dataset.ccmLoaderGroup;
				if (loaderGroup) {
					// Ignore permitted embeddings to differ between non-associated group and greenlit script.
					var associatedEmbeddingIds = this.repository
						.filter(function (embedding) { return embedding.scriptLoaderGroup == loaderGroup; })
						.filter(function (embedding) { return self.isEmbeddingActiveForCurrentLocation(embedding); })
						.map(function (embedding) { return embedding.id; })
					;

					if (associatedEmbeddingIds.length == 0) {
						if (CCM_DEBUG) {
							console.warn('[CCM19] Loader group "%s" is not associated with any embedding; corresponding scripts won\'t be loaded.', loaderGroup);
						}
						throw new Error('ScriptLoaderGroupNotAssociatedWithAnyEmbeddingException');
					}

					return associatedEmbeddingIds.filter(function (id) { return !embeddingIds.includes(id); });
				}
			}

			// Nur relevant, wenn es Skripte gibt, die so geblockt werden sollen
			if (!this.blockEmbeddingScriptMarkers) {
				return [];
			}
			// Alles was kein JS ist oder vom CCM stammt is ok.
			if (this.inEval || !this.checkIsBlockableScript(node) || node.hasAttribute('data-ccm-injected')) {
				return [];
			}

			var text = String(node.outerHTML);

			var embeddingIdsBlockingScript = [];
			for (var embeddingId in this.blockEmbeddingScriptMarkers) {
				var embedding = this.repository[embeddingId];
				if (!embedding || this.isEmbeddingActiveForCurrentLocation(embedding) == false) {
					continue;
				}
				if (!embeddingIds.includes(embeddingId)) {
					var isBlocking = this.blockEmbeddingScriptMarkers[embeddingId].reduce(function (isBlocking, marker) {
						// Consider that the src attribute may contain a relative URL; explicitly compare against src property
						return isBlocking || text.includes(marker) || node.src.includes(marker);
					}, false);
					if (isBlocking) {
						embeddingIdsBlockingScript.push(embeddingId);
					}
				}
			}
			return embeddingIdsBlockingScript;
		},
		/**
		 * @param {HTMLScriptElement} node
		 * @return {void}
		 */
		processScriptForBlocking: function (node) {
			var self = this;
			var blocked = false;
			var embeddingIds = self.getAllowedAndMandatoryEmbeddingIds();
			// Prüfen, ob Script von einem Cookie in einem nicht-ausgewählten
			// Zweck geblockt werden soll
			var blockedByEmbeddings = [];
			var handledByEmbeddings = [];
			var allowScriptLoader = true;
			try {
				blockedByEmbeddings = self.embeddingsBlockingScript(node, embeddingIds);
				handledByEmbeddings = self.embeddingsBlockingScript(node, []);
			} catch (e) {
				allowScriptLoader = false;
			}
			var shouldBlock = self.shouldBlockScript(node);

			if ((blockedByEmbeddings.length || shouldBlock) && !node.hasAttribute('data-ccm-injected')) {
				//Hash erzeugen - muss nicht besonders krytografisch wertvoll sein - geht nur darum
				//eine singuläre ID für das Skript dieser Seite zu erstellen...
				// Dabei Fragment und Query-String wegschneiden
				var contentToHash;
				var src = (node.type == 'text/x-ccm-loader' && node.dataset.ccmLoaderSrc) || node.src;
				if (src) {
					contentToHash = String(src)
						.trim()
						.replace(/#.*$/i, "")
						.replace(/\?.*/i, "?")
						.replace(/\/(intl\/|releases\/|viewthroughconversion\/|www-widgetapi-)[^\/]+\//, '/$1…/');
				}
				else {
					contentToHash = String(node.innerHTML).trim().substring(1,1024).replace(/\s+/ig, ' ');
				}

				if (node.id) {
					contentToHash += '#'+node.id.trim();
				}
				if (node.className) {
					contentToHash += '.'+node.className.trim();
				}
				if (node.async) {
					contentToHash += ':async';
				}
				if (node.defer) {
					contentToHash += ':defer';
				}
				if (node.type == 'text/x-ccm-loader') {
					contentToHash += ':loader';
				}
				if (self.checkIsCloakedScript(node)) {
					contentToHash += ':cloaked';
				}

				var hash = Utils.hash(contentToHash);

				var blacklisted = this.shouldBlockScriptAccordingToBlacklist(node);
				var scriptIsAllowed = self.allowedScripts.includes(hash);
				var scriptIsDenied = self.deniedScripts.includes(hash);
				if (blockedByEmbeddings.length || !scriptIsAllowed || blacklisted) {

					// Kopie des Script-Knotens speichern
					var clone = node.cloneNode(true);
					// Preserve async value to determine correct execution order.
					// @see https://developer.mozilla.org/en-US/docs/Web/HTML/Element/script#compatibility_notes
					clone.async = node.async;
					clone.ccm_id = hash;
					self.blockableScripts.push(clone);

					// Wenn Blocking aktiv: Blocken
					if (self.blockNewScripts || scriptIsDenied || (self.isActive && blockedByEmbeddings.length) || blacklisted) {
						node.setAttribute('data-ccm-id', hash);

						if (CCM_DEBUG) {
							if (src) {
								console.debug('[CCM19] Blocked script %s', src);
							}
							else {
								console.debug('[CCM19] Blocked inline script %s…', node.innerText.trim().replace(/\s\s+/g, ' ').substring(0, 40));
							}
						}

						node.setAttribute('data-ccm-type', node.type);
						node.type = 'text/x-blocked-script';

						blocked = true;

						// Wenn nur durch Cookie-Auswahl geblockt, markieren, damit später aktiviert werden kann
						if (blockedByEmbeddings.length && !(shouldBlock && this.blockNewScripts && !scriptIsAllowed || blacklisted)) {
							if (CCM_DEBUG) {
								console.group('[CCM19] Script may be unblocked by accepting the following embeddings');
							}
							blockedByEmbeddings.forEach(function (embeddingId) {
								var embedding = self.repository[embeddingId];
								if (CCM_DEBUG) {
									embedding
									? console.info('%s (%s)', embedding.name, embedding.id)
									: console.warn('Unknown embedding, try clearing browser cache. (%s)', embeddingId);
								}
							});
							if (CCM_DEBUG) {
								console.groupEnd();
							}

							clone.ccmEmbeddings = blockedByEmbeddings;
							clone.ccmOriginalNode = node;
						}
						else if (handledByEmbeddings.length && blacklisted) {
							if (CCM_DEBUG) {
								console.info('[CCM19] While the script could be reactivated via an embedding, it\'s blocked by the general blocklist. Therefore, the script is inaccessible to the runtime.');
							}
						}
					}

				}
			}

			if ((node.type == 'text/x-ccm-loader' || this.checkIsCloakedScript(node)) && allowScriptLoader && !blocked) {
				this.scriptLoaderQueue.push(node);
				this.executeScriptLoaderQueue();
			}
		},
		/**
		 * @param {HTMLIFrameElement} node
		 * @param {boolean} [overrideConsent=false]
		 */
		processIframeForBlocking: function (node, overrideConsent) {
			overrideConsent = typeof overrideConsent == 'boolean' ? overrideConsent : false;

			// Check for existing parent elements to skip previously processed nodes.
			// Also, skip iframes whose src attributes are empty.
			if (this.blockIframes == false
				|| !node.getAttribute('src') && !node.dataset.ccmLoaderSrc
				|| !Utils.isVisibleDOMNode(node)
				|| node.ccm19ConsentGranted && !overrideConsent
			) {
				return;
			}

			var self = this;

			// Block iframe by default
			var blockIframe = this.iframeBlockMode == 'whitelist';
			var contentBlockedUrl = this.contentBlockedUrl;
			contentBlockedUrl = contentBlockedUrl.replace(/&lang=[^&]+/, '&lang=' + encodeURIComponent(this.locale));
			var alreadyBlocked = !!node.dataset.ccmSrc && node.src.toLowerCase().indexOf(contentBlockedUrl.toLowerCase().split('?', 1)[0]) == 0;
			var tamperedWith = alreadyBlocked && node.src.toLowerCase().indexOf(contentBlockedUrl.toLowerCase()) != 0;

			// If blocked iframe was cloned by a third-party script, reset to block again
			if (alreadyBlocked && this.iframes.includes(node) == false) {
				alreadyBlocked = false;
				node.src = node.dataset.ccmSrc;
			}
			else if (tamperedWith && this.iframes.includes(node)) {
				var newSrc = contentBlockedUrl + '&url=' + encodeURIComponent(node.dataset.ccmSrc.replace(/[#].*$/, ''));
				if ((node.ccm19BlockedByEmbeddings || []).length > 0) {
					newSrc += '&embedding=' + encodeURIComponent(node.ccm19BlockedByEmbeddings[0]);
				}
				node.src = newSrc;
			}

			// Nothing to do
			if (alreadyBlocked) {
				return;
			}

			var nodeHtml = String(node.outerHTML);
			var blockedByEmbeddings = [];
			var hideIframesUntilConsent = false;
			var loaderGroupOrphaned = false;

			var loaderGroup = node.dataset.ccmLoaderGroup;
			if (loaderGroup) {
				// Disregard whether embeddings have consent to detect orphaned groups
				blockedByEmbeddings = this.repository
					.filter(function (embedding) { return embedding.scriptLoaderGroup == loaderGroup; })
					.filter(function (embedding) { return self.isEmbeddingActiveForCurrentLocation(embedding); })
					.map(function (embedding) { return embedding.id; })
				;

				if (blockedByEmbeddings.length == 0) {
					if (CCM_DEBUG) {
						console.warn('[CCM19] Loader group "%s" is not associated with any embedding.', loaderGroup);
					}
					loaderGroupOrphaned = true;
				}
			}
			else {
				// Check if an embedding contains rules that match the iframe
				blockedByEmbeddings = this.repository
					.filter(function (embedding) {
						var isBlocking = embedding.iframeBlockingStrings.reduce(function (isMatch, needle) {
							// Consider that the src attribute may contain a relative URL; explicitly compare against src property
							return isMatch || nodeHtml.includes(needle) || node.src.includes(needle);
						}, false);

						return isBlocking;
					})
					.filter(function (embedding) { return self.isEmbeddingActiveForCurrentLocation(embedding); })
					.map(function (embedding) { return embedding.id; })
				;
			}

			// If embedding specific filters match, take precedence over global iframe markers
			if (blockedByEmbeddings.length > 0) {
				var allowedEmbeddingIds = this.getAllowedAndMandatoryEmbeddingIds();

				blockIframe = blockedByEmbeddings.reduce(function (isBlocking, embeddingId) {
					return isBlocking || allowedEmbeddingIds.includes(embeddingId) == false;
				}, false);

				hideIframesUntilConsent = blockedByEmbeddings.reduce(function (hide, embeddingId) {
					return hide || self.repository[embeddingId].hideIframesUntilConsent;
				}, hideIframesUntilConsent);
			}

			// Don't block if outside the EU and DNT disabled
			else if (this.isConsentRequired() == false) {
				blockIframe = false;
			}

			// Block if a loader group is defined but isn't referenced by an embedding
			else if (loaderGroupOrphaned) {
				blockIframe = true;
			}

			// If filters are defined, block or allow iframe based on configured block mode
			else if (this.iframeMarkers.length > 0) {
				var isMatch = this.iframeMarkers.reduce(function (isMatch, item) {
					// Consider that the src attribute may contain a relative URL; explicitly compare against src property
					return isMatch || nodeHtml.includes(item) || node.src.includes(item);
				}, false);

				blockIframe = this.iframeBlockMode == 'whitelist' ? !isMatch : isMatch;
			}

			var src = node.src || node.dataset.ccmLoaderSrc;

			if (src.substring(0, 5) == 'data:') {
				blockIframe = false;
			}
			else if (src.substring(0, 6) == 'about:') {
				blockIframe = false;
			}
			else if (this.rememberIframeConsent && this.iframeConsentDomains.includes(Utils.extractDomain(src))) {
				blockIframe = false;
			}

			if (overrideConsent) {
				blockIframe = true;
			}

			delete node.ccm19ConsentGranted;
			node.dataset.ccmSrc = src;
			node.dataset.ccmDomain = Utils.extractDomain(src);

			var newSrc = contentBlockedUrl + '&url=' + encodeURIComponent(src.replace(/[#].*$/, ''));
			if (blockedByEmbeddings.length > 0) {
				newSrc += '&embedding=' + encodeURIComponent(blockedByEmbeddings[0]);
			}

			var existingClone = this.findExistingIframeClone(node);

			var substitute = null;

			if (existingClone) {
				existingClone.dataset.ccmSrc = src;
				existingClone.dataset.ccmDomain = Utils.extractDomain(src);
				existingClone.src = newSrc;
				substitute = existingClone.ccm19IframePlaceholder || existingClone;
			}
			else {
				var tmpClone = node.cloneNode();
				tmpClone.src = newSrc;
				tmpClone.setAttribute('scrolling', 'auto');
				tmpClone.style.opacity = '';
				tmpClone.style.visibility = '';
				var clone = tmpClone.cloneNode();
				clone.ccm19BlockedByEmbeddings = blockedByEmbeddings;
				clone.ccm19OriginalIframe = node;
				if (hideIframesUntilConsent) {
					var placeholder = document.createElement('SPAN');
					placeholder.classList.add('ccm--iframe-placeholder');
					substitute = clone.ccm19IframePlaceholder = placeholder;
				}
				else {
					substitute = clone;
				}

				this.iframes = this.iframes.filter(function (iframe) {
					return iframe !== node;
				});
				this.iframes.push(clone);
				existingClone = clone;
			}

			if (blockIframe) {
				node.src = newSrc;

				// Temporarily disable iframe blocking to prevent weird behavior in some web browsers
				this.blockIframes = false;

				node.parentElement.insertBefore(substitute, node);
				node.parentElement.removeChild(node);

				// Reactivate iframe blocking
				this.blockIframes = true;

				if (CCM_DEBUG && blockedByEmbeddings.length > 0 && 'groupCollapsed' in console) {
					console.groupCollapsed('[CCM19] Iframe may be unblocked by accepting the following embeddings');
					blockedByEmbeddings.forEach(function (embeddingId) {
						var embedding = self.repository[embeddingId];
						embedding
							? console.info('%s (%s)', embedding.name, embedding.id)
							: console.warn('Unknown embedding, try clearing browser cache. (%s)', embeddingId);
					});
					console.groupEnd();
				}
			}
			else {
				delete node.dataset.ccmLoaderGroup;
				delete node.dataset.ccmLoaderSrc;
				delete node.dataset.ccmSrc;
				node.ccm19ConsentGranted = true;

				// Load resource from the data-ccm-loader-src attribute
				if (!node.src) {
					node.src = src;
					node.parentElement.insertBefore(node, node.nextElementSibling);
				}
			}

			if (this.appendConsentSwitchToIframes) {
				this.appendConsentSwitchToIframe(node, !blockIframe);
			}
		},
		/**
		 * @param {string} url
		 */
		preprocessMacroUrl: function (url) {
			if (this.tcfData.adTagsAutoAppend && url.match(/https?:\/\/www\.awin1\.com\/[cpa][a-z]{4,9}\.php\?/i)) {
				url += '&gdpr=${GDPR}&gdpr_consent=${GDPR_CONSENT_907}';
			}
			return url;
		},
		/**
		 * @param {HTMLAnchorElement} node
		 */
		processLinkForMacros: function (node) {
			// Skip previously processed nodes.
			if ('ccmHref' in node.dataset) {
				return;
			}
			var url = this.preprocessMacroUrl(node.href);
			// Skip links without placeholders
			if (!url.includes('${GDPR')) {
				return;
			}

			var handler;
			if ('__tcfapi' in window) {
				handler = window.__tcfapi;
			} else {
				handler = function (_a, _b, callback, _d) {
					callback(null, false);
				}
			}

			var matches = url.match(/\$\{GDPR_CONSENT_[0-9]{1,10}\}/g);
			var ids = [];
			if (matches) {
				ids = matches.map(function (match) {
					return parseInt(match.substr(15));
				});
			}

			var self = this;
			node.dataset.ccmHref = url;
			handler('getTCData', 2, function (tcData) {
				if (tcData) {
					node.href = url.replace(
						/\$\{GDPR(_CONSENT_[0-9]{1,10})?\}/g,
						function (match, want_consent_string) {
							if (want_consent_string) {
								return tcData.tcString;
							} else {
								return (tcData.gdprApplies) ? '0' : '1';
							}
						}
					);
				} else {
					// Wenn kein TCF verfügbar:
					node.href = url.replace(
						/\$\{GDPR(_CONSENT_([0-9]{1,10}))?\}/g,
						function (match, want_consent_string) {
							if (want_consent_string) {
								return '';
							} else {
								return (self.behavior.noConsentRequired) ? '0' : '1';
							}
						}
					);
				}
			}, ids);
		},
		/**
		 * @param {HTMLLinkElement} node
		 */
		processImageForMacros: function (node) {
			// Skip previously processed nodes.
			if ('ccmSrc' in node.dataset) {
				return;
			}
			var url = this.preprocessMacroUrl(node.src);
			// Skip images without placeholders
			if (!url.includes('${GDPR')) {
				return;
			}

			var handler;
			if ('__tcfapi' in window) {
				handler = window.__tcfapi;
			} else {
				handler = function (_a, _b, callback, _d) {
					callback(null, false);
				}
			}

			var matches = url.match(/\$\{GDPR_CONSENT_[0-9]{1,10}\}/g);
			var ids = [];
			if (matches) {
				ids = matches.map(function (match) {
					return parseInt(match.substr(15));
				});
			}

			var self = this;
			node.dataset.ccmSrc = url;
			handler('getTCData', 2, function (tcData) {
				if (tcData) {
					node.href = url.replace(
						/\$\{GDPR(_CONSENT_[0-9]{1,10})?\}/g,
						function (match, want_consent_string) {
							if (want_consent_string) {
								return tcData.tcString;
							} else {
								return (tcData.gdprApplies) ? '0' : '1';
							}
						}
					);
				} else {
					if (self.tcfData.enabled) {
						// Wenn gerade kein TCF verfügbar, aber prinzipiell aktiv: erstmal blockieren
						node.src = "data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg'/%3E";
					} else {
						// Wenn TCF komplett deaktiviert, Standardwerte einfügen
						node.src = url.replace(
							/\$\{GDPR(_CONSENT_([0-9]{1,10}))?\}/g,
							function (match, want_consent_string) {
								if (want_consent_string) {
									return '';
								} else {
									return (self.behavior.noConsentRequired) ? '0' : '1';
								}
							}
						);
					}
				}
			}, ids);
		},
		/**
		 * Deactivate elements dynamically inserted into the document if necessary.
		 * @param {Node|HTMLElement} node
		 */
		performBlockingActionsOnNode: function (node) {
			if (node instanceof HTMLIFrameElement) {
				this.processIframeForBlocking(node);
			}
			else if (node instanceof HTMLScriptElement) {
				this.processScriptForBlocking(node);
			}
			else if ('classList' in node && node.classList.contains('ccm-cookie-declaration') && node.children.length == 0) {
				this.insertCookieDeclaration();
			}
			else {
				if (this.tcfData.adTags) {
					if (node instanceof HTMLImageElement) {
						this.processImageForMacros(node);
					}
					else if (node instanceof HTMLAnchorElement) {
						if (node.hasAttribute('href')) {
							this.processLinkForMacros(node);
						}
					}
				}

				// Check for potential API handlers and register them
				if (node.nodeType == Node.ELEMENT_NODE) {
					this.registerJavaScriptApiHandler(node);
				}

				var children = 'children' in node ? node.children : node.childNodes;
				for (var i = 0; i < children.length; i++) {
					this.performBlockingActionsOnNode(children[i]);
				}
			}
		},
		/**
		 * Install mutation observers for script and iframe blocking and set
		 * "beforescriptexecute" handler for script blocking in Firefox.
		 * @return {void}
		 */
		observeDocument: function () {
			if (this.domWatcher) {
				return;
			}

			// Firefox script blocking
			function onBeforeScriptExecute(event) {
				var node = event.target;
				// Prevent only marked scripts from executing
				if (node.type == 'text/x-blocked-script') {
					node.setAttribute('data-ccm-type', node.type);
					event.preventDefault();
				}
			}
			if (this.blockNewScripts || this.deniedScripts || this.blockEmbeddingScriptMarkers) {
				window.addEventListener('beforescriptexecute', onBeforeScriptExecute, true);
			}

			var self = this;

			/**
			 * @param {MutationRecord[]} mutations
			 * @param {MutationObserver} observer
			 */
			function callback(mutations, observer) {
				mutations.forEach(function (mutation) {
					if (mutation.type == 'attributes') {
						if (mutation.target instanceof HTMLIFrameElement) {
							// Handle iframes that obtain their real src at runtime; e.g. site
							// loads src attribute with 'data:' and later on is dynamically
							// changed to some web resource.
							if (mutation.attributeName == 'src') {
								self.processIframeForBlocking(mutation.target);
							}
						}
					}

					toArray(mutation.addedNodes).forEach(function (node) {
						// Duplicate handling of nodes may occur here. This is necessary
						// to process every child node to find, for instance, iframes in
						// privacy policy container in ccm19 widget.
						self.performBlockingActionsOnNode(node);

						// Detect hydration of the document
						if (node instanceof HTMLHtmlElement || node instanceof HTMLBodyElement) {
							if (self.widgetInitialized) {
								// Restore the banner and its assets after a dynamic DOM refresh
								self.ownHeadElements.forEach(function (node) {
									if (document.head && node.parentElement !== document.head) {
										document.head.appendChild(node);
									}
								});
								if (document.body && self.container && self.container.parentElement !== document.body) {
									document.body.appendChild(self.container);
									// Make sure the bottom focus helper stays below the CCM19 container.
									var focusHelper = document.getElementById('ccm-focus-helper--bottom');
									if (focusHelper) {
										document.body.appendChild(focusHelper);
									}
								}
							}
						}

						if (node.nodeType == Node.ELEMENT_NODE) {
							self.dispatchInternalEvent('domElementNodeAdded', node);
						}
					});

					toArray(mutation.removedNodes).forEach(function (node) {
						// Detect if the widget has been removed
						if (self.container && node === self.container) {
							window.setTimeout(function () {
								// Prevent infinite remove/add iterations
								if (node.parentElement !== document.body) {
									document.body.appendChild(node);
									// Make sure the bottom focus helper stays below the CCM19 container.
									var focusHelper = document.getElementById('ccm-focus-helper--bottom');
									if (focusHelper) {
										document.body.appendChild(focusHelper);
									}
								}
							}, 1000);
						}
						// Re-insert head elements
						else if (self.ownHeadElements.includes(node)) {
							// Prevent infinite remove/add iterations
							if (node.parentElement !== document.head) {
								document.head.appendChild(node);
							}
						}
					});
				});
			};

			/** @type MutationObserverInit */
			var options = {
				attributes: true,
				childList: true,
				subtree: true,
				characterData: true
			};

			this.domWatcher = new MutationObserver(callback);
			this.domWatcher.observe(document, options);

			window.addEventListener('message', function (event) { return self.onPostMessage(event); });

			if (this.consentCrossDomain || this.accountWideConsentSharing) {
				document.addEventListener('click', (function (event) {
					var targets = ('composedPath' in event) ? event.composedPath() : [event.target];
					var link = targets[0];
					for (var i = 0; i < targets.length; ++i) {
						if ('href' in targets[i]) {
							link = targets[i];
							break;
						}
					}

					while (link && 'href' in link == false) {
						link = link.parentElement;
					}

					if (link) {
						self.appendCrossDomainConsent(link);
					}
				}), true);

				document.addEventListener('submit', (function (event) {
					if ('action' in event.target) {
						self.appendCrossDomainConsent(event.target);
					}
				}), true);

				// Handle HTMLFormElement.prototype.submit calls that don't dispatch a submit event.
				var origSubmit = HTMLFormElement.prototype.submit;
				Object.defineProperty(HTMLFormElement.prototype, 'submit', {
					configurable: true,
					enumerable: false,
					writable: true,
					value: function () {
						self.appendCrossDomainConsent(this);
						origSubmit.call(this);
					},
				});
			}
		},
		/**
		 * Depending on the current URL and the visibility state of the banner, this method
		 * opens or closes the banner when the URL of the document has changed.
		 * @return {void}
		 */
		handleHistoryStateChange: function () {
			this.callAsSoonAsSavingSettingsIsDone(function () {
				if (this.isActive == false
					|| this.partOfAdAndTrackingFreeNetwork
					|| this.isBotBrowser()
					|| this.consentGiven()
				) {
					return;
				}

				var isModalActive = this.isModalActive();
				var urlIsExempt = this.urlIsExempt(location.href);

				if (isModalActive && urlIsExempt) {
					this.closeAllModals();
					this.revokeAllPrivileges();
				}
				else if (!isModalActive && !urlIsExempt) {
					this.openWidget();
				}
			}.bind(this));
		},
		/**
		 * @param {PopStateEvent} event
		 * @return {void}
		 */
		onHistoryStateChanged: function (event) {
			this.handleHistoryStateChange();
		},
		checkDomainMatch: function (currentHost) {
			var matches = false;
			var currentHost = currentHost.replace(/\.$/, '');
			var mainDomain = this.mainMatchDomain.replace(/^https?:\/\//, '').replace(/[/ ?].*/, '').replace(/\.$/, '').replace(/^www\./, '');
			if (currentHost.replace(/^www\./, '') == mainDomain) {
				return true;
			}
			if (this.getBaseDomain(currentHost) !== null) {
				return true;
			}
			return false;
		},
		/**
		 * Initialize CCM19
		 * @return {void}
		 */
		init: function () {
			if (this.initialized) {
				return;
			}

			this.consentState = {
				googleAdTechProviders: this.tcfData.enabled ? Utils.mapObject(this.tcfData.googleATPs, function () { return false; }) : {},
				tcfLegIntPurposes: this.tcfData.enabled ? Utils.mapObject(this.tcfData.legIntPurposes, function () { return false; }) : {},
				tcfLegIntVendors: this.tcfData.enabled && this.tcfData.legIntEnabled
					? Utils.mapObject(
						Utils.filterObject(this.tcfData.vendor, function (vendor) {
							return vendor.pL.length > 0;
						}),
						function () { return false; }
					)
					: {},
				tcfPurposes: this.tcfData.enabled ? Utils.mapObject(this.tcfData.purposes, function () { return false; }) : {},
				tcfSpecialFeatures: this.tcfData.enabled ? Utils.mapObject(this.tcfData.specialFeatures, function () { return false; }) : {},
				tcfVendors: this.tcfData.enabled ? Utils.mapObject(this.tcfData.vendor, function () { return false; }) : {},
			};

			this.onFocusIn = Utils.bindThis(this, this.onFocusIn);
			this.onTabKeyDown = Utils.bindThis(this, this.onTabKeyDown);
			this.onWindowResize = Utils.bindThis(this, this.onWindowResize);
			this.onJavaScriptApiLinkClicked = Utils.bindThis(this, this.onJavaScriptApiLinkClicked);

			// PermaScan Report
			var rand = Math.random();
			this.shouldReport = (this.reportProbability === 0) ? false : (rand <= this.reportProbability);
			if (this.shouldReport) {
				this.wrapCookieSetter();
			}

			var self = this;

			// Check ob Domain passt, falls nötig
			var matchesDomain = undefined;
			if (this.isActive && this.mustMatchDomains && !this.isPreview) {
				matchesDomain = this.checkDomainMatch(location.hostname);
				if (!matchesDomain) {
					this.shouldReport = false;
					this.isActive = false;
				}
			}

			// Array-Methoden für das Repository-Objekt bereitstellen
			[
				'filter',
				'forEach',
				'map',
				'reduce',
			].forEach(function (methodName) {
				Object.defineProperty(self.repository, methodName, {
					configurable: false,
					enumerable: false,
					writable: false,
					value: function () {
						var array = Utils.objectValues(self.repository);
						if (methodName in array && typeof array[methodName] == 'function') {
							return array[methodName].apply(array, arguments);
						}
					},
				});
			});

			// Basiselement aller Widget-Komponenten bauen
			this.container = document.createElement('DIV');
			this.container.classList.add('ccm-root');
			Utils.toggleClass(this.container, 'ccm--tcf-enabled', this.tcfData.enabled);
			Utils.toggleClass(this.container, 'ccm--is-ie', Utils.isIE());
			// Grundlegende Barrierefreiheits-Fixups:
			this.container.addEventListener('keyup', function (evt) {
				// Tastenkombinationen hier nicht beachten, nur einfache Tastendrücke
				if (evt.shiftKey || evt.metaKey || evt.ctrlKey || evt.altKey || evt.isComposing) {
					return;
				}
				// a[role=button] muss sich auch mit Leertaste aktivieren lassen.
				if (evt.key == ' ' && evt.target.tagName == 'A' && evt.target.role == 'button') {
					evt.preventDefault();
					evt.target.click();
				}
				// Dialoge mit "Close"-Button lassen sich auch mit Escape schließen
				else if (evt.key == 'Escape') {
					var dialog = evt.target;
					while (dialog && dialog.role != "dialog") {
						if (dialog == self.container) { return; }
						dialog = dialog.parentNode;
					}
					if (dialog) {
						var btn = dialog.getElementsByClassName('ccm-modal--close')[0];
						if (!btn) {
							btn = dialog.getElementsByClassName('ccm-dismiss-button')[0];
						}
						if (btn && !btn.disabled) {
							evt.preventDefault();
							evt.stopPropagation();
							btn.click();
						}
					}
				}

			});

			if (this.isActive == false) {
				if (matchesDomain === false) {
					console.info('[CCM19] Frontend widget is not active on this domain (%s).', window.location.hostname);
				} else {
					if (this.showFrontendWidgetOnlyIfEmbeddingsExist && this.nonCmpEmbeddingCount) {
						console.info('[CCM19] Frontend widget has no relevant embeddings.');
					} else {
						console.info('[CCM19] Frontend widget is disabled.');
					}
				}
				this.initialized = true;
				return;
			}

			if (!this.locale) {
				if (this.prioritizeLangAttribute) {
					this.setLocaleUsingLangAttribute();
				} else {
					this.setLocaleFromBrowserPreference();
				}
				if (!this.locale) {
					this.locale = this.fallbackLocale;
					this.widgetUrl = this.widgetUrls[this.fallbackLocale];
					if (this.widgetUrl === undefined) {
						this.locale = Object.keys(this.widgetUrls)[0];
						this.widgetUrl = this.widgetUrls[this.locale];
					}
				}
			}

			// Prüfe auf benutzerdefinierten Consent-Storage-Namensraum
			this.setStorageNamespaceUsingUrlParameter();

			var storage = this.getConsentStorage();

			// Prüfe, ob noch keine endgültige Consent-Konfiguration vorliegt
			if (storage.consent === false && storage.gen === 2) {
				// Temporär zugelassene Einbindungen (Iframe-Blocker) wieder zurücksetzen
				if ((storage.embeddings || []).length > 0) {
					this.saveSettingsInternal(this.getUniqueCookieId(), false, []);
				}
			}

			// Zustimmung pro Komponente initialisieren
			this.presetConsentState();

			this.clickedButton = storage.clickedButton || null;
			this.iframeConsentDomains = typeof storage.iframeConsentDomains == 'object' && storage.iframeConsentDomains instanceof Array
				? storage.iframeConsentDomains
				: [];

			this.observeDocument();
			this.wrapElementInsertion();
			this.protectNativeFunctions();
			this.initHighPriorityEventListeners();

			/* Google Consent Mode: wenn verwendet und kein Standard vorher gesetzt, dann Standard denied */
			if (!Utils.hasGTMLayer('consent')) {
				for (var embeddingId in this.repository) {
					if (this.repository[embeddingId].gcm) {
						Utils.sendGTag("consent", "default", {
							ad_storage: "denied",
							analytics_storage: "denied",
							functionality_storage: "denied",
							personalization_storage: "denied",
							security_storage: "denied",
							// BEGIN GCMv2
							ad_personalization: "denied",
							ad_user_data: "denied",
							// END GCMv2
							wait_for_update: 1000,
						});
						break;
					}
				}
			}

			this.injectCss();
			this.injectJs();
			this.initTcfStub();
			this.injectPlugins();

			var consentLastModified = Math.round((storage.updated || 0) / 10);
			var consentReset = this.consentCookieReset;
			var consentLifetime = (this.consentLifetimeForPartialConsent === null || this.consentLifetimeForPartialConsent === this.consentLifetime)
				? this.consentLifetime
				: (this.isFullConsentGiven() ? this.consentLifetime : this.consentLifetimeForPartialConsent);

			if (this.consentGiven() && !this.urlIsExempt(window.location.href)) {
				if ((consentLifetime > 0 && (Utils.now/1000) - consentLastModified) > (consentLifetime * 86400)) {
					this.forceOpenWidget = true;
					this.requiresConsentUpdate = true;
				}
				else if (consentLastModified < consentReset && consentReset <= (Utils.now/1000)) {
					this.forceOpenWidget = true;
					this.requiresConsentUpdate = true;
				}
			}

			// Widget nicht öffnen, Consent-Konfiguration stattdessen dem Netzwerk überlassen
			if (this.partOfAdAndTrackingFreeNetwork) {
				this.forceOpenWidget = false;

				// TCF-Komponente mitteilen, dass die Consent-Konfiguration aussteht
				if (this.consentGiven() == false || this.requiresConsentUpdate) {
					this.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
						Utils.triggerCustomEvent(window, 'ccm19WidgetOpened');

						// Banner-Einblendung festhalten
						self.logOpenData();
					});
				}
			}

			self.build(function (firstInitialization) {
				var limitToTechnicallyNecessary = false;
				var showWidget = false;

				if (this.forceOpenWidget) {
					this.forceOpenWidget = false;
					limitToTechnicallyNecessary = true;
					showWidget = true;
				}
				else if (this.isBotBrowser() || (this.urlIsExempt(location.href) && !this.consentGiven())) {
					this.revokeAllPrivileges();
					this.settingsIconTarget = 'main';
				}
				else if (this.behavior.noConsentRequired || this.partOfAdAndTrackingFreeNetwork) {
					this.presetConsentState();
				}
				else if (this.consentGiven() == false) {
					limitToTechnicallyNecessary = true;
					showWidget = true;
				}

				// Fokus nur holen, wenn nicht in einem Iframe.
				var focusIndex = (Utils.isTopLevel() ? undefined : -1);

				if (showWidget) {
					if (firstInitialization && this.widgetDelayInMilliseconds > 0) {
						window.setTimeout(function () { self.openWidget(focusIndex); }, this.widgetDelayInMilliseconds);
					}
					else {
						this.openWidget(focusIndex);
					}
				}

				if (document.body) {
					this.loadScripts(limitToTechnicallyNecessary);
				}
				else {
					window.addEventListener('DOMContentLoaded', function () { self.loadScripts(limitToTechnicallyNecessary); });
				}
			});

			this.initialized = true;
		},
		/**
		 * Send page check report
		 * @return {void}
		 */
		pageCheckReport: function () {
			var report = {};

			// List all cookie names
			report.c = this.getCookies().map(function (cookie) {
				return cookie.name;
			});

			// List external scripts
			var scripts = document.getElementsByTagName('script');
			report.s = [];
			for (var i = 0; i < scripts.length; ++i) {
				var script = scripts[i];
				if (this.checkIsExternal(script.src) && !script.hasAttribute('data-ccm-injected')) {
					report.s.push(script.src);
				}
			}

			// List external objects/embeds/iframes
			var objects = document.getElementsByTagName('object');
			report.o = [];
			for (var i = 0; i < objects.length; ++i) {
				var object = objects[i];
				if (this.checkIsExternal(object.data)) {
					report.o.push(object.data);
				}
			}
			objects = document.getElementsByTagName('embed');
			for (var i = 0; i < objects.length; ++i) {
				object = objects[i];
				if (this.checkIsExternal(object.src)) {
					report.o.push(object.src);
				}
			}
			objects = document.getElementsByTagName('iframe');
			for (var i = 0; i < objects.length; ++i) {
				object = objects[i];
				if (this.checkIsExternal(object.src)) {
					report.o.push(object.src);
				}
			}

			// List LocalStorage names
			try {
				report.ls = [];
				var storage = window.localStorage;
				for (var i = 0; i < storage.length; ++i) {
					report.ls.push(storage.key(i));
				}
			} catch (e) {
				// ignore
			}

			// List SessionStorage names
			try {
				report.ss = [];
				var storage = window.sessionStorage;
				for (var i = 0; i < storage.length; ++i) {
					report.ss.push(storage.key(i));
				}
			} catch (e) {
				// ignore
			}


			// List all scripts
			try {
				report.as = {};
				if (this.scripTracker) {
					if (CCM_DEBUG) {
						console.info("[CCM19] Script Tracking active");
					}
					var foundScriptCookies = this.foundScriptCookies;
					this.blockableScripts.forEach(function (node) {
						if (node.type == 'text/x-ccm-loader' && node.dataset.ccmLoaderGroup) {
							return;
						}

						var id = node.ccm_id;
						report.as[id] = {
							s: btoa(node.outerHTML.trim().replace(/\s+\n/g, '\n').replace(/\n\n+/g, '\n\n').substring(0, 500)),
							c: ((node.src in foundScriptCookies) ? Object.keys(foundScriptCookies[node.src]) : [])
						};
					});
				}
			} catch (e) {
				// ignore
			}


			// Referrer URL
			report.url = location.href.replace(/[?#].*$/, '');

			// Send report
			this.ajax(this.pageCheckUrl, {
				contentType: 'application/json',
				body: report,
				method: 'POST'
			});
		},
		/**
		 * Trigger CCM19 cronjobs
		 * @return {void}
		 */
		triggerCron: function () {
			this.ajax(this.cronUrl, {
				method: 'POST',
			});
		},
		/**
		 * Cookies löschen
		 * @param {boolean} [byUserSelection=false]
		 * @return {void}
		 */
		cleanUpClientStorage: function (byUserSelection) {
			byUserSelection = typeof byUserSelection == 'boolean' ? byUserSelection : false;

			var self = this;

			var knownlist = {
				cookies: {},
				localStorage: {},
				sessionStorage: {},
			};
			var whitelist = {
				cookies: {},
				localStorage: {},
				sessionStorage: {},
			};

			var cookies = this.getCookies().map(function (cookie) { return cookie.name; });
			var lsProperties = Object.getOwnPropertyNames(window.localStorage);
			var ssProperties = Object.getOwnPropertyNames(window.sessionStorage);

			var resourceRemoved = false;

			function evaluateEmbedding(embedding, list) {
				embedding.assets.forEach(function (asset) {
					var collection = [];
					var storageType = null;
					switch (asset.type) {
						case self.consts.TYPE_COOKIE:
							collection = cookies;
							storageType = 'cookies';
							break;
						case self.consts.TYPE_LOCAL_STORAGE:
							collection = lsProperties;
							storageType = 'localStorage';
							break;
						case self.consts.TYPE_SESSION_STORAGE:
							collection = ssProperties;
							storageType = 'sessionStorage';
							break;
						default:
							break;
					}

					if (!storageType) {
						return;
					}

					if (asset.glob) {
						collection.forEach(function (name) {
							if (Utils.fnmatch(asset.name, name)) {
								list[storageType][name] = asset.name;
							}
						});
					}
					else {
						list[storageType][asset.name] = asset.name;
					}
				});
			}

			// Alle CCM19 bekannten Cookies etc. sammeln
			for (var embeddingId in this.repository) {
				evaluateEmbedding(this.repository[embeddingId], knownlist);
			}

			// Benutzerauswahl berücksichtigen
			if (byUserSelection) {
				this.getEmbeddingsWithConsent().forEach(function (embedding) {
					// Sämtliche EmbeddingAssets als zugelassen vormerken
					evaluateEmbedding(embedding, whitelist);
				});
			}
			else {
				// Ansonsten alle bekannten Cookies vorerst zulassen
				whitelist = knownlist;
			}

			//so wird die ccm19 (Login)Session nicht geloescht
			whitelist.cookies.ccm_login_session = 'ccm_login_session';
			whitelist.cookies.ccm_consent = 'ccm_consent';
			whitelist.localStorage.ccm_consent = 'ccm_consent';
			whitelist.sessionStorage.ccm_consent = 'ccm_consent';

			var domains = [window.location.hostname];

			var secondLevelDomainMatch = window.location.hostname.match(/\.([^.]+\.[^.]+)$/);
			if (secondLevelDomainMatch) {
				// Seite läuft auf Subdomain, Cookie wurde aber u. U. auf .example.com registriert
				domains.push(secondLevelDomainMatch[1]);
			}

			// Clean up cookies
			cookies.forEach(function(cookieName) {
				// Is current cookie part of managed cookies or allowed purposes
				var isAllowedCookie = !!whitelist.cookies[cookieName];
				// Match current cookie against all entries maintained with ccm19
				var isUnknownCookie = !knownlist.cookies[cookieName];

				if (isAllowedCookie || (isUnknownCookie && self.deleteAllCookiesConfig == false)) {
					return false;
				}

				domains.forEach(function (domain) {
					document.cookie = cookieName + "=" +
						";domain="+ domain +
						";path=/"+
						";expires=Thu, 01 Jan 1970 00:00:00 UTC";
				});

				// Cookie nicht nur für z. B. ".www.domain.xyz", sondern auch für "www.domain.xyz" löschen
				document.cookie = cookieName + '=;path=/;expires=Thu, 01 Jan 1970 00:00:00 UTC';

				resourceRemoved = true;
			});

			// Clean up local storage
			lsProperties.forEach(function (property) {
				// Is property part of managed assets or allowed embeddings
				var isAllowed = !!whitelist.localStorage[property];
				// Match property against all entries maintained with CCM19
				var isUnknown = !knownlist.localStorage[property];

				if (isAllowed || (isUnknown && self.deleteAllCookiesConfig == false)) {
					return false;
				}

				window.localStorage.removeItem(property);

				resourceRemoved = true;
			});

			// Clean up session storage
			ssProperties.forEach(function (property) {
				// Is property part of managed assets or allowed embeddings
				var isAllowed = !!whitelist.sessionStorage[property];
				// Match property against all entries maintained with CCM19
				var isUnknown = !knownlist.sessionStorage[property];

				if (isAllowed || (isUnknown && self.deleteAllCookiesConfig == false)) {
					return false;
				}

				window.sessionStorage.removeItem(property);

				resourceRemoved = true;
			});

			//nur wenn der Schalter gesetzt ist neuladen
			if (this.deleteAllCookiesReload) {
				if (resourceRemoved) {
					window.location.reload(true);
				}
			}
		},
		/**
		 * Wrap Node.appendChild and Node.insertBefore to catch
		 * dynamically inserted scripts before execution.
		 * Also, wrap `Element.prototype` methods that insert nodes.
		 * @return {void}
		 */
		wrapElementInsertion: function () {
			var self = this;
			var origAppendChild = Node.prototype.appendChild;
			var origInsertBefore = Node.prototype.insertBefore;

			Node.prototype.appendChild = function appendChild(aChild) {
				self.performBlockingActionsOnNode(aChild);
				return origAppendChild.call(this, aChild);
			};
			Node.prototype.insertBefore = function insertBefore(newNode, referenceNode) {
				self.performBlockingActionsOnNode(newNode);
				return origInsertBefore.call(this, newNode, referenceNode);
			};

			// Wrap Element.prototype methods
			[
				'after', 'before',
				'append', 'prepend',
			].filter(function (property) {
				return property in Element.prototype && typeof Element.prototype[property] == 'function';
			}).forEach(function (methodName) {
				var origMethod = Element.prototype[methodName];

				Element.prototype[methodName] = function methodWrapper() {
					toArray(arguments)
						.filter(function (argument) { return argument instanceof Node; })
						.forEach(function (node) { self.performBlockingActionsOnNode(node); });

					// Passthrough void return value (undefined)
					return origMethod.apply(this, arguments);
				};
			});
		},
		/**
		 * Wrap document.cookie to trace scripts that set cookies.
		 * @return {void}
		 */
		wrapCookieSetter: function () {
			var self = this;

			if (!('getOwnPropertyDescriptor' in Object)) {
				if (CCM_DEBUG) {
					console.debug('[CCM19] Browser incompatible; cannot trace cookie setting scripts');
				}
				return;
			}

			if (!('stack' in Error.prototype)) {
				if (CCM_DEBUG) {
					console.debug('[CCM19] Browser incompatible; cannot trace cookie setting scripts');
				}
				return;
			}

			// document.cookie wrappen
			Utils.wrapProperty(HTMLDocument.prototype, 'cookie', {
				set: function (value, origSetter) {
					try {
						var key = value.split('=', 1)[0];
						var stack = ((new Error()).stack.trim().split('\n'));
						stack.shift();
						if (stack.length > 1 && /\/(js|jquery)\.cookie[._]/.test(stack[0])) {
							stack.shift(); // Cookie-Wrapper-Librarys ignorieren
						}
						if (stack.length > 0) {
							// Script-URL aus Trace holen
							var m = stack[0].match(/\b((?:https?|data):.*):[0-9]+:[0-9]+\b/);
							if (m) {
								if (!(m[1] in self.foundScriptCookies)) {
									self.foundScriptCookies[m[1]] = {}
								}
								self.foundScriptCookies[m[1]][key] = key;
							}
						}
					}
					catch (e) {
						console.error('[CCM19] Error while tracing cookie in script: %s', e);
					}
					return origSetter.call(this, value);
				},
			});
		},
		/**
		 * Wrap history.pushState and history.replaceState to handle
		 * dynamic page navigation.
		 * @return {void}
		 */
		wrapHistoryApi: function () {
			var self = this;
			var wrapper = function (fn) {
				fn.apply(this, toArray(arguments).slice(1));
				self.handleHistoryStateChange();
			};

			// Wrap calls to history.pushState and history.replaceState
			Utils.wrapProperty(History.prototype, 'pushState', { value: wrapper });
			Utils.wrapProperty(History.prototype, 'replaceState', { value: wrapper });
		},
		/**
		 * This method locks native browser functionality to prevent certain
		 * third-party scripts from overwriting it with non-standard behavior.
		 * @return {void}
		 */
		protectNativeFunctions: function () {
			var protectees = [{
				object: Array.prototype,
				properties: [
					'reduce',
				],
			}];

			protectees.forEach(function (protectee) {
				protectee.properties.forEach(function (property) {
					var descriptor = Object.getOwnPropertyDescriptor(protectee.object, property);
					if (descriptor && descriptor.configurable) {
						descriptor.configurable = false;
						descriptor.writable = false;
						Object.defineProperty(protectee.object, property, descriptor);
					}
				});
			});
		},
		/**
		 * Registers event listeners for specific events and uses `useCapture` to
		 * ensure that the event is handled before any other event listeners.
		 * This method is only called once when CCM19 is initialized.
		 * @return {void}
		 */
		initHighPriorityEventListeners: function () {
			var self = this;

			function handleEvent(event) {
				var eventName = event.type;

				// Disregard manipulated clicks if manipulation protection is enabled
				if (self.checkIsManipulatedClick(event)) {
					return;
				}

				// Iterate over all event handlers as long as propagation is not stopped
				var handlers = self.highPriorityEventHandlers[eventName] || [];
				handlers.forEach(function (handler) {
					// Check if the event target is a child of the handler target
					var target = event.target;
					while (target && target !== handler.target) {
						target = target.parentElement;
					}

					// If the handler target equals the event target, call the handler
					if (target === handler.target) {
						handler.handler(event, target);
					}
				});
			};

			// Set up high priority event listeners
			[
				'click',
			].forEach(function (eventName) {
				self.highPriorityEventHandlers[eventName] = [];
				document.addEventListener(eventName, handleEvent, {capture: true});
			});
		},
		/**
		 * Add a high priority event listener. CCM19 tries to handle the event
		 * before any other event listener. This is useful to prevent certain
		 * third-party scripts from executing before CCM19 has processed the event.
		 * @param {string} eventName
		 * @param {HTMLElement} target
		 * @param {Function} handler
		 * @return {void}
		 */
		addHighPriorityEventListener: function (eventName, target, handler) {
			if (eventName in this.highPriorityEventHandlers) {
				if (this.hasHighPriorityEventListener(eventName, target, handler)) {
					return;
				}
				this.highPriorityEventHandlers[eventName].push({
					handler: handler,
					target: target,
				});
			}
			else {
				throw new Error('High priority event not available: ' + eventName);
			}
		},
		/**
		 * Checks whether a high priority event listener is registered.
		 * @param {string} eventName
		 * @param {HTMLElement} target
		 * @param {Function} handler
		 * @return {boolean}
		 */
		hasHighPriorityEventListener: function (eventName, target, handler) {
			return (this.highPriorityEventHandlers[eventName] || []).some(function (item) {
				return item.handler === handler && item.target === target;
			});
		},
		/**
		 * Update dynamic information for cookie declaration table in `container`
		 * @param {Element|DocumentFragment} container
		 * @param {string} language
		 * @return {void}
		 */
		_updateCookieDeclarationInner: function (container, language) {
			if (!language) { lang = navigator.language; }
			var storage = this.getConsentStorage();
			toArray(container.querySelectorAll('.ccm-cookie-declaration--consent-id')).forEach(function (el) {
				el.textContent = ('ucid' in storage) ? storage.ucid : '—';
			});
			toArray(container.querySelectorAll('.ccm-cookie-declaration--consent-date')).forEach(function (el) {
				if ('updated' in storage) {
					var date = new Date(storage.updated * 100);
					el.dateTime = date.toISOString();
					el.textContent = date.toLocaleString(language);
				} else {
					el.textContent = '—';
				}
			});
			toArray(container.querySelectorAll('.ccm-cookie-declaration--consent-state')).forEach(function (el) {
				toArray(el.querySelectorAll('span[data-ccm-state]')).forEach(function (el) {
					var state = storage.clickedButton;
					if (!state) {
						state = 'none';
					}
					if (state === el.getAttribute('data-ccm-state')) {
						el.hidden = false;
						el.style.display = 'inline';
					} else {
						el.hidden = true;
						el.style.display = 'none';
					}
				});
			});
		},
		/**
		 * Update dynamic information in cookie declaration tables
		 * @param {Element|DocumentFragment} container
		 * @param {string} language
		 * @return {void}
		 */
		updateCookieDeclaration: function () {
			var self = this;
			toArray(document.getElementsByClassName('ccm-cookie-declaration')).forEach(function (el) {
				self._updateCookieDeclarationInner(el, el.getAttribute('lang'));
			});
		},
		/**
		 * Insert cookie declaration tables into marked containers
		 * @return {void}
		 */
		insertCookieDeclaration: function () {
			if (this.tasksRunning.insertCookieDeclaration) {
				return;
			}

			var self = this;
			var containers = toArray(document.getElementsByClassName('ccm-cookie-declaration'));
			if (!containers.length) {
				return;
			}

			// Prevent an infinite recursion loop by never running this task in parallel
			this.tasksRunning.insertCookieDeclaration = true;

			// Prevent block elements inside of <p> elements by replacing them with <div> elements
			containers = containers.map(function (container) {
				if (container instanceof HTMLParagraphElement) {
					var substitute = document.createElement('DIV');
					Utils.applyAttributes(substitute, container);
					container.parentElement.insertBefore(substitute, container);
					container.parentElement.removeChild(container);
					return substitute;
				}
				else {
					return container;
				}
			});

			var containersByLang = containers.reduce(function (result, container) {
				var requestedLanguage = container.getAttribute('data-lang');
				requestedLanguage = (requestedLanguage) ? requestedLanguage.replace('-', '_') : self.locale;
				if (!(requestedLanguage in result)) {
					result[requestedLanguage] = [container];
				} else {
					result[requestedLanguage].push(container);
				}
				return result;
			}, Object.create(null));

			var baseUrl = this.cookieDeclarationUrl+'&lang=';
			var languages = Object.keys(containersByLang);
			var completedRequests = 0;
			languages.forEach(function (lang) {
				var containers = containersByLang[lang];
				var url = baseUrl + encodeURIComponent(lang);

				self.ajax(url, {
					method: 'GET',
					success: function (response, text, xhr) {
						var language = xhr.getResponseHeader('Content-Language');
						try {
							var range = document.createRange();
							range.selectNode(containers[0]);
							var fragment = range.createContextualFragment(response);
							self._updateCookieDeclarationInner(fragment, language);
						} catch (e) {
							console.error("[CCM19] Failed to insert current consent state to cookie declaration table", e);
						}

						containers.forEach(function (container) {
							container.setAttribute('lang', language);
							container.classList.remove('ccm-cookie-declaration--error');
							container.classList.add('ccm-cookie-declaration--loaded');
							container.innerHTML = '';
							container.appendChild(fragment.cloneNode(true));
							var linkContainer = container.getElementsByClassName('ccm-cookie-declaration--change-consent')[0];
							if (linkContainer) {
								linkContainer.getElementsByTagName('a')[0].addEventListener("click", function (event) { return self.onSettingsIconClicked(event); });
							}
							else if (CCM_DEBUG) {
								console.warn('[CCM19] Inserting cookie declaration table might have failed.', container);
							}
						});
					},
					failure: function () {
						containers.forEach(function (container) {
							container.classList.remove('ccm-cookie-declaration--loaded');
							container.classList.add('ccm-cookie-declaration--error');
						});
						console.error("[CCM19] Cooke declaration table for locale '%s' could not be loaded", lang);
					},
					done: function () {
						if (++completedRequests == languages.length) {
							this.tasksRunning.insertCookieDeclaration = false;

							// New elements may have been added to the DOM during loading
							if (document.querySelectorAll('.ccm-cookie-declaration:not([class*="ccm-cookie-declaration--"])').length > 0) {
								this.insertCookieDeclaration();
							}
						}
					},
				});

			})
		},
	};

	// CCM19 public interface
	if (window.CCM === undefined) {
		(function () {
			var urlConsent;

			if (window.location.hash.indexOf("CCM19consent=") >= 0) {
				var urlConsentRE = /(^#?|&)CCM19consent=([0-9a-z]+)\|([0-9a-z]+)\|([0-9a-f]+)\|([0-9a-f|]*)/;
				urlConsent = urlConsentRE.exec(window.location.hash);
				if (!urlConsent) {
					urlConsent = urlConsentRE.exec(decodeURI(window.location.hash));
				}
				if (!urlConsent) {
					urlConsent = urlConsentRE.exec(decodeURI(decodeURI(window.location.hash)));
				}
				// Consent-Zeug aus URL entfernen
				if (urlConsent) {
					var url = Utils.getUrl(window.location, true);
					url.hash = url.hash.replace(/(^#?|&)CCM19consent=[a-z0-9A-F%|]*/, '');
					if (url.hash == '' && 'replaceState' in history) {
						history.replaceState(history.state, '', window.location.pathname + window.location.search);
					}
					else {
						window.location.replace(url.href);
					}
				}
			}

			var ccm = new CookieConsentManagement(_CCM19_CONFIG_, urlConsent);

			if (ccm.isActive) {
				//einmal zum start - alles raushauen...
				ccm.cleanUpClientStorage();

				// Nicht zugelassene Cookies nach dem Laden des Widgets versuchen zu entfernen
				window.addEventListener('ccm19WidgetLoaded', function () {
					ccm.cleanUpClientStorage(true);
				});

				var ccmProperties = {
					version: {
						get: function () {
							return ccm.versionName;
						},
						enumerable: true,
					},
					consent: {
						get: function () {
							return ccm.consentGiven();
						},
						enumerable: true,
					},
					fullConsentGiven: {
						get: function () {
							return ccm.isFullConsentGiven();
						},
						enumerable: true,
					},
					ucid: {
						get: function () {
							return this.consent ? ccm.getUniqueCookieId() : null;
						},
						enumerable: true,
					},
					acceptedCookies: {
						get: function () {
							var cookieNames = [];

							ccm.getAllowedEmbeddingIds().forEach(function (embeddingId) {
								var embedding = ccm.repository[embeddingId];
								if (!embedding) {
									return;
								}
								embedding.assets.forEach(function (asset) {
									if (cookieNames.includes(asset.name)) {
										return;
									}
									else if (asset.type != 'cookie') {
										return;
									}
									cookieNames.push(asset.name);
								});
							});

							return this.consent ? cookieNames : [];
						},
						enumerable: true,
					},
					acceptedEmbeddings: {
						get: function () {
							var embeddings = [];

							ccm.getAllowedEmbeddingIds().forEach(function (embeddingId) {
								var embedding = ccm.repository[embeddingId];
								if (!embedding) {
									return;
								}
								embeddings.push(Object.create({}, {
									id: { value: embedding.id, writable: true, enumerable: true },
									name: { value: embedding.name, writable: true, enumerable: true },
									_tcf: { value: embedding.tcf, writable: true, enumerable: false },
								}));
							});

							return this.consent ? embeddings : [];
						},
						enumerable: true,
					},
					consentLanguage: {
						get: function () {
							return ccm.getConsentLanguage();
						},
					},
					primaryCountry: {
						get: function () {
							return ccm.country;
						},
					},
					crossDomainConsentString: {
						get: function () {
							return ccm.consentGiven() ? ccm.buildCrossDomainConsent() : '';
						},
						enumerable: true,
					},
					consentRequired: {
						get: function () {
							return !ccm.behavior.noConsentRequired;
						},
						enumerable: true,
					},
					openWidget: {
						value: function () {
							ccm.openWidget();
						},
						enumerable: true,
					},
					closeWidget: {
						value: function () {
							ccm.closeWidget();
						},
						enumerable: true,
					},
					openControlPanel: {
						value: function () {
							ccm.openControlPanel();
						},
						enumerable: true,
					},
					closeControlPanel: {
						value: function () {
							ccm.closeControlPanel();
						},
						enumerable: true,
					},
					openDetailsWindowAtEmbedding: {
						/**
						* @param {string} embeddingId
						*/
						value: function (embeddingId) {
							if (typeof embeddingId != 'string') {
								throw new TypeError('embeddingId is not a string');
							}
							else if (embeddingId in ccm.repository) {
								ccm.showEmbeddingDetails(embeddingId);
							}
							else {
								throw new Error('Embedding "' + embeddingId + '" does not exist');
							}
						},
						enumerable: true,
					},
					openDetailsWindowAtPurpose: {
						/**
						* @param {string} purposeId
						*/
						value: function (purposeId) {
							var availablePurposeIds = Utils.uniqueValues(ccm.repository.map(function (embedding) { return embedding.purpose; }));
							if (typeof purposeId != 'string') {
								throw new TypeError('purposeId is not a string');
							}
							else if (availablePurposeIds.includes(purposeId)) {
								ccm.showPurposeDetails(purposeId);
							}
							else {
								throw new Error('Purpose "' + purposeId + '" either does not exist or isn\'t available in this configuration');
							}
						},
						enumerable: true,
					},
					openDetailsWindow: {
						value: function () {
							ccm.openDetailsWindow();
						},
						enumerable: true,
					},
					closeDetailsWindow: {
						value: function () {
							ccm.closeDetailsWindow();
						},
						enumerable: true,
					},
					navigate: {
						/**
						* @param {string} url
						* @param {boolean=} replace Ob der aktuelle History-Eintrag ersetzt werden soll (Standard: false)
						* @param {boolean=} navigateTop Ob - bei Frames - die Hauptseite statt dem aktuellen Frame navigiert werden soll (Standard: false)
						* @return {void}
						*/
						value: function (url, replace, navigateTop) {
							url = ccm.withCrossDomainConsent(url);
							var location = (navigateTop) ? window.top.location : window.location;
							if (replace) {
								location.replace(url);
							} else {
								location.assign(url);
							}
						},
						enumerable: true,
					},
					availableLocales: {
						/**
						* @return {Array.<string>}
						*/
						value: function availableLocales () {
							return Object.keys(ccm.widgetUrls);
						},
						enumerable: true,
					},
					switchLocale: {
						/**
						* @param {string} language
						* @return {void}
						*/
						value: function switchLocale (language) {
							ccm.switchLocale(language);
						},
						enumerable: true,
					},
					_tcfInfo: {
						get: function () {
							return {
								'consent': ccm.getTcfConsentInfo(),
								'repository': ccm.tcfData,
							}
						},
					},
					toString: {
						value: function () {
							return '[namespace CCM]';
						}
					},
				};

				if (ccm.partOfAdAndTrackingFreeNetwork) {
					Utils.assign(ccmProperties, {
						denyAllPrivileges: {
							value: function () {
								this.revokeAllPrivileges();
							},
							enumerable: true,
						},
						grantAllPrivileges: {
							value: function () {
								ccm.clickedButton = 'acceptAll';
								ccm.grantAllPrivileges();
								ccm.saveSettings();
								ccm.callAsSoonAsSavingSettingsIsDone(function () {
									ccm.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
										Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
									});
								});
							},
							enumerable: true,
						},
						revokeAllPrivileges: {
							value: function () {
								ccm.clickedButton = 'decline';
								ccm.iframeConsentDomains = [];
								ccm.revokeAllPrivileges();
								ccm.saveSettings();
								ccm.callAsSoonAsSavingSettingsIsDone(function () {
									ccm.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
										Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
									});
								});
							},
							enumerable: true,
						},
						requiresConsentUpdate: {
							get: function () {
								return this.consent == false || ccm.requiresConsentUpdate;
							},
							enumerable: true,
						},
						embeddings: {
							enumerable: true,
							get: function () {
								return ccm.repository.map(function (embedding) {
									var result = {
										id: embedding.id,
										name: embedding.name,
										vendor: embedding.vendor,
										l10n: JSON.parse(JSON.stringify(embedding.l10n)), // Deep copy
										assets: embedding.assets.map(function (asset) {
											return {
												name: asset.name,
												type: asset.type,
												l10n: JSON.parse(JSON.stringify(asset.l10n)), // Deep copy
											};
										}),
									};

									// Add fallbackLocale to result.l10n
									result.l10n._ = result.l10n[ccm.fallbackLocale];
									// Add fallbackLocale to result.assets[].l10n
									result.assets.forEach(function (asset) {
										asset.l10n._ = asset.l10n[ccm.fallbackLocale];
									});

									return result;
								});
							},
						},
						tcf: {
							value: Object.create(null, {
								enabled: {
									enumerable: true,
									value: ccm.tcfData.enabled,
								},
								stacks: {
									enumerable: true,
									get: function () {
										if (this.enabled == false) {
											return [];
										}
										return ccm.tcfData.stacks;
									},
								},
								vendors: {
									enumerable: true,
									get: function () {
										if (this.enabled == false) {
											return [];
										}
										return Object.keys(ccm.tcfData.vendor).map(function (vendorId) {
											var vendor = ccm.tcfData.vendor[vendorId];
											return {
												id: parseInt(vendorId),
												name: vendor.n,
												flexiblePurposes: vendor.pF,
												legitimateInterestPurposes: vendor.pL,
												purposes: vendor.p,
												specialPurposes: vendor.sp,
											};
										}, []);
									},
								},
								customVendors: {
									enumerable: true,
									get: function () {
										return Utils.uniqueValues(ccm.repository.reduce(function (vendors, embedding) {
											if (!embedding.tcf) {
												vendors.push(embedding.vendor);
											}
											return vendors;
										}, [])).sort(function (a, b) { return a.localeCompare(b) });
									},
								},
							}),
							enumerable: true,
						}
					});
				}

				if (ccm.jsApiFeatures.embeddingConsentState) {
					Utils.assign(ccmProperties, {
						setEmbeddingConsentState: {
							/**
							 * @param {Object.<string, boolean>} state `{embeddingId: consent, …}`
							 */
							value: function (state) {
								// Track whether the consent state changes
								var stateChanged = false;

								// Apply the new consent state
								for (var embeddingId in state) {
									var embedding = ccm.repository[embeddingId];
									if (embedding) {
										var consent = !!state[embeddingId] || embedding.mandatory;
										if (embedding.consent !== consent) {
											embedding.consent = consent;
											ccm.selectEmbedding(embedding);
											stateChanged = true;
										}
									}
								}

								// Save settings if the consent state has changed
								if (stateChanged) {
									ccm.clickedButton = 'save';
									ccm.saveSettings();
									ccm.callAsSoonAsSavingSettingsIsDone(function () {
										ccm.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
											Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
										});
									});
								}
							},
							enumerable: true,
						},
					});
				}

				window.CCM = Object.create(null, ccmProperties);

				if ('clickskeksLegacy' in ccm) {
					// Polyfills for https://www.clickskeks.at/so-gehts/expertenfunktionen/api
					window.Clickskeks = Object.create(null, {
						getInfo: {
							/**
							* @return {String}
							*/
							value: function () {
								return window.CCM.version;
							},
							enumerable: true,
						},
						showConsent: {
							value: function () {
								window.CCM.openWidget();
							},
							enumerable: true,
						},
						acceptCategories: {
							/**
							* @param {Array.<String>} categories
							*/
							value: function (categories) {
								var purposeMap = {
									statistics: 'a717ff5',
									preference: '7c19e32',
									marketing: '6cd2721',
									unclassified: '15c61c3',
								};
								categories.forEach(function (category) {
									var purposeId = purposeMap[category] || null;
									if (purposeId) {
										ccm.selectPurpose(purposeId, true);
									}
								});
								ccm.saveSettings();
								ccm.callAsSoonAsSavingSettingsIsDone(function () {
									ccm.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
										Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
									});
								});
							},
							enumerable: true,
						},
						getCurrentAllowedConfig: {
							/**
							* @return {Object.<String, boolean>}
							*/
							value: function () {
								var hasFullConsent = function (purposeId) {
									var purposeHasEmbeddings = ccm.repository.reduce(function (carry, embedding) {
										return carry || embedding.purpose == purposeId;
									}, false);

									if (purposeHasEmbeddings == false) {
										return false;
									}

									return ccm.repository.reduce(function (consent, embedding) {
										return consent && (embedding.purpose != purposeId || embedding.consent);
									}, true);
								};
								return {
									necessary: true,
									statistics: hasFullConsent('a717ff5'),
									preference: hasFullConsent('7c19e32'),
									marketing: hasFullConsent('6cd2721'),
									unclassified: hasFullConsent('15c61c3'),
								};
							},
							enumerable: true,
						},
						optOut: {
							value: function () {
								ccm.iframeConsentDomains = [];
								ccm.revokeAllPrivileges();
								ccm.saveSettings();
								ccm.callAsSoonAsSavingSettingsIsDone(function () {
									ccm.callAsSoonAsTcfApiIsAvailableOrIfDisabled(function () {
										Utils.triggerCustomEvent(window, 'ccm19WidgetClosed');
									});
								});
							},
							enumerable: true,
						},
						buildDisclaimer: {
							/**
							* @param {String} elementId
							*/
							value: function (elementId) {
								var element = document.getElementById(elementId);
								if (element) {
									element.classList.add('ccm-cookie-declaration');
									ccm.insertCookieDeclaration();
								}
							},
							enumerable: true,
						},
						buildDisclaimerWithClass: {
							/**
							* @param {String} classNames
							*/
							value: function (classNames) {
								var elements = toArray(document.getElementsByClassName(classNames));
								if (elements.length > 0) {
									elements.forEach(function (element) {
										element.classList.add('ccm-cookie-declaration');
									});
									ccm.insertCookieDeclaration();
								}
							},
							enumerable: true,
						},
						addAllowedConfigChangedCallback: {
							/**
							* @param {Function} allowedConfigChangedCallback
							* @param {boolean} [callbackWithCurrentData=false]
							*/
							value: function (allowedConfigChangedCallback, callbackWithCurrentData) {
								if (typeof allowedConfigChangedCallback != 'function') {
									console.error('Invalid argument, callback function expected');
									return;
								}

								if (callbackWithCurrentData) {
									allowedConfigChangedCallback(this.getCurrentAllowedConfig());
								}

								ccm.clickskeksLegacy.allowedConfigChangeCallbacks.push(allowedConfigChangedCallback);
							},
							enumerable: true,
						},
						setAllowedConfigChangedCallback: {
							/**
							* @param {Function} allowedConfigChangedCallback
							* @param {boolean} [callbackWithCurrentData=false]
							*/
							value: function (allowedConfigChangedCallback, callbackWithCurrentData) {
								if (typeof allowedConfigChangedCallback != 'function') {
									console.error('Invalid argument, callback function expected');
									return;
								}

								this.removeAllAllowedConfigChangedCallbacks();
								ccm.clickskeksLegacy.allowedConfigChangeCallbacks.push(allowedConfigChangedCallback);
							},
							enumerable: true,
						},
						removeAllowedConfigChangedCallback: {
							/**
							* @param {Function} allowedConfigChangedCallback
							*/
							value: function (allowedConfigChangedCallback) {
								ccm.clickskeksLegacy.allowedConfigChangeCallbacks = ccm.clickskeksLegacy.allowedConfigChangeCallbacks
									.filter(function (callback) {
										return callback !== allowedConfigChangedCallback;
									});
							},
							enumerable: true,
						},
						removeAllAllowedConfigChangedCallbacks: {
							value: function () {
								ccm.clickskeksLegacy.allowedConfigChangeCallbacks = [];
							},
							enumerable: true,
						},
						toString: {
							/**
							* @return {String}
							*/
							value: function () {
								return '[namespace Clickskeks]';
							},
						},
					});
				}
				}

			if (ccm.cronProbability > 0.0) {
				var rand = Math.random();
				if (rand <= ccm.cronProbability) {
					ccm.triggerCron();
				}
			}

			if (ccm.shouldReport) {
				window.addEventListener('load', function () {
					window.setTimeout(function () { ccm.pageCheckReport(); }, 501);
				});
			}
		})();
	}
})();
